/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 **  This class encapsulates the JacORB CORBA::ORB singleton object.
 **
 **  @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 **  @version 0.2
 **/

public abstract class TheORB
       extends TheORBSingleton
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    TheORB()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Remove ORB arguments.
     **
     **  @param args The command-line arguments.
     **  
     **  @return The command line arguments without OpenORB options.
     **/
    protected static String[]
    remove_ORB_arguments(String[] args)
    {
	int nb = 0;

	// Copy arguments without ORB ones.
	String[] tmp = new String[args.length];
	for(int i=0; i<args.length; i++)
	{
            // System.out.println ("* args["+i+"] = "+args[i]); 
	    if(args[i].startsWith("-ORBInitRef"))
	    {
		i++;

		String arg = args[i];

		int index = arg.indexOf('=');
		String name = arg.substring(0, index);
		String ior = arg.substring(index+1);
		((org.openorb.CORBA.ORB)getORB())
		    .addInitialReference(name, string_to_object(ior));
	    }
	    else if(!args[i].startsWith("-ORB"))
	    {
		tmp[nb] = args[i];
		nb++;
	    }
	}

	// Construct the returned array.
	args = new String[nb];
	for(int i=0; i<nb; i++)
	{
	    args[i] = tmp[i];
	}

	// Return arguments without ORB ones.
	return args;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Initialize the ORBacus CORBA::ORB singleton object.
     **
     **  @param args The command line arguments.
     **
     **  @return The command line arguments without ORBacus options.
     **/
    public static String[]
    initialize(String[] args)
    {
	// Define system properties.
	java.util.Properties properties = System.getProperties();
        properties.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
        properties.put("org.omg.CORBA.ORBSingletonClass",
                       "org.jacorb.orb.ORBSingleton");

	// Initialize the CORBA::ORB.
	setORB( org.omg.CORBA.ORB.init(args, properties) );

	return remove_ORB_arguments(args) ;
    }

    /**
     **  Destroy the CORBA::ORB singleton object.
     **/
    public static void
    destroy()
    {
        getORB().destroy();
    }
}
