/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Romain Rouvoy_________________________.

====================================================================*/

package org.objectweb.ccm.demo1;

/**
 * The demo1 application deployment.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</A>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</A>
 */

public class Demo1
{
    /**
     * The bootstrap of the demo1 application.
     */
    public static void
    main(String[] args)
    throws Exception
    {
       String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();
       
       // Init the ORB.
       System.out.println("Initializing the ORB...");
       
       // TODO: Need to report this ORB.init() problem to OpenORB developers!
       // Due to an OpenORB problem, the following line:
       //   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
       // are replaced by the next line.
				
       // Init the OpenCCM Components Runtime.
       org.omg.CORBA.ORB orb = org.objectweb.ccm.Components.Runtime.init(args);
       
       // Obtain the Name Service.
       System.out.println("Obtaining the Name Service...");
       org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
       org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);
       
//        String[] ids = orb.list_initial_services();
//        System.out.println("Rfrences initiales disponibles : ");
//        for (int i=0;i<ids.length;i++)
// 	 System.out.println("-> "+ids[i]);
       
       org.omg.CosTransactions.Current current = null;
       if (_OTS.equals("yes"))
	 {
	    System.out.println("Obtaining the Transaction Service...");
	    org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
	    current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
	 }
       
       try
	 {
	    if ((current!=null)&&(_OTS.equals("yes")))
	      {
		 System.out.println("Status = "+current.get_status());
		 System.out.println("Beginning the transaction...");
		 current.begin();
	      }
	    
	    // Obtain the component servers.
	    System.out.println("Obtaining Component Servers...");
	    org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
	    ncomp[0] = new org.omg.CosNaming.NameComponent("ComponentServer1", "");
	    obj = nc.resolve(ncomp);
	    org.objectweb.ccm.Deployment.Server server1 = org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
	    ncomp[0].id = "ComponentServer2";
	    obj = nc.resolve(ncomp);
	    org.objectweb.ccm.Deployment.Server server2 = org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
	    
	    // Obtain the container homes and archive installators.
	    org.omg.Components.Deployment.ComponentServer server1_cs = server1.provide_component_server();
	    org.omg.Components.Deployment.ComponentInstallation server1_inst = server1.provide_install();
	    org.omg.Components.Deployment.ComponentServer server2_cs = server2.provide_component_server();
	    org.omg.Components.Deployment.ComponentInstallation server2_inst = server2.provide_install();
	    
	    // Install archives.
	    System.out.println("Installing archives...");
	    server1_inst.install("demo1", "./archives/demo1.jar");
	    server2_inst.install("demo1", "./archives/demo1.jar");
	    // install some plugins already provided by OpenCCM.
	    server1_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
	    server2_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
	    
	    // declaring the container types
	    //
	    //  Uncomment this part for a minimal configuration
	    //  and comment the following one.
	    //
	    /**
	     String cont_config = 
	     "container = CONTAINER.container ; " +
	     "config_home = container.create_system_home("+
	     "\"openccm_plugins\", " +
	     "\"EmptyConfig\", "+
	     "\"org.objectweb.ccm.Containers.Plugins.EmptyConfigurationHome.create\") ;"+
	     "config = config_home.create_component(JAVA.null) ;" +
	     "container.create_system_home("+
	     "\"openccm_plugins\", " +
	     "\"EmptyCoordinatorHome\", "+
	     "\"org.objectweb.ccm.Containers.Plugins.EmptyCoordinatorHome.create\") ;"+
	     "container.create_system_home("+
	     "\"openccm_plugins\", " +
	     "\"EmptyControllerHome\", "+
	     "\"org.objectweb.ccm.Containers.Plugins.EmptyControllerHome.create\") ;"+
	     "container.set_container_configuration(config) ;";
	     **/
	    
	    String cont_config = 
	      // retrieve the container reference.
	      "container = CONTAINER.container ; " +
	      // create aliases on PropertySetImpl and StringPropertyImpl class.
	      "JAVA.declare(\"org.objectweb.ccm.Containers\") ; " +
	      "PropertySet = org.objectweb.ccm.Containers.PropertySetImpl ;" +
								"StringProperty = org.objectweb.ccm.Containers.StringPropertyImpl ;" +
	      // create an empty configuration with it's alias.
	      "config_home = container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"EmptyConfig\", "+
	      "\"org.objectweb.ccm.Containers.Plugins.EmptyConfigurationHome.create\") ;"+
	      "config = config_home.create_component(JAVA.null) ;" +
	      // create an empty coordinator system home with it's alias
	      "container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"EmptyCoordinatorHome\", "+
	      "\"org.objectweb.ccm.Containers.Plugins.EmptyCoordinatorHome.create\") ;"+
	      // create an empty controller system home with it's alias
	      "container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"EmptyControllerHome\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyControllerHome.create\") ;"+
	      // create a trace controller system home with it's alias.
	      "container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"Trace\", " +
	      "\"org.objectweb.ccm.Containers.Plugins.TraceControllerHome.create\") ; "+
	      // create a list coordinator system home with it's alias.
	      "container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"ListCoord\", " +
	      "\"org.objectweb.ccm.Containers.Plugins.ListCoordinatorHome.create\") ; "+
	      // create an port specific config.
	      "container.create_system_home("+
	      "\"openccm_plugins\", " +
	      "\"PortConfig\", " +
	      "\"org.objectweb.ccm.Containers.Plugins.PortSpecificConfigurationHome.create\") ; " +
	      // set this configuration as the container default configuration.
	      "container.set_container_configuration(config) ;";
	    
	    // instantiate a container on each server.
	    org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[1];
	    config[0] = new org.objectweb.ccm.Components.ConfigValueImpl();
	    config[0].name = "container_script";
	    org.omg.CORBA.TypeCode string_tc =
	      org.objectweb.ccm.CORBA.TheORB.getORB().get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
	    org.omg.CORBA.Any value = 
	      org.objectweb.ccm.CORBA.TheDynamicAnyFactory.getFactory().
	      create_dyn_any_from_type_code(string_tc).to_any();
	    value.insert_string(cont_config);
	    config[0].value = value;
	    
	    
	    org.omg.Components.Deployment.Container server1_cont =
	      //            server1_cs.create_container(config);
	      server1_cs.create_container(new org.omg.Components.ConfigValue[0]);
	    org.omg.Components.Deployment.Container server2_cont =
	      //            server2_cs.create_container(config);
	      server2_cs.create_container(new org.omg.Components.ConfigValue[0]);
	    
	    
	    // Install homes.
	    System.out.println("Instantiating homes...");
	    
	    // define a container configuration.
	    config[0].name = "home_script";
	    
	    //
	    //  Uncomment this part for a minimal configuration
	    //  and comment the following one.
	    //
	    /**
	     java.lang.String home_config = 
	     "container.set_home_configuration(container.get_container_configuration()) ;" +
	     "container.set_component_configuration(container.get_container_configuration()) ;";
	     **/
	    
	    java.lang.String home_config = 
	      // set the container config as the home config
	      // i.e. an empty configuration.
	      "container.set_home_configuration(container.get_container_configuration()) ;" +
	      // create a port specific config
	      "config = container.find_system_home(\"PortConfig\").create_component(JAVA.null) ;" +
	      // set the component ::CORBA::Repository of the component.
	      "config.setComponentUId(\"IDL:ccm.objectweb.org/demo1/Server:1.0\") ;" +
	      // set a list coordinator for the "for_clients" facet of the "Server" component
	      "config.setCallCoordinator(\"for_clients\", \"ListCoord\") ;" +
	      // set a trace controller for the "for_clients" facet of the "Server" component
	      "config.setCallControllers(\"for_clients\", [ \"Trace\" ]) ;" +
	      // create and set the trace configuration for this facet
	      "trace_config = PropertySet(\"trace\")" +
	      "trace_config.add(StringProperty(\"filename\", \"./trace_Server.txt\"))" +
	      "config.setCallControllersConfig(\"for_clients\", [ trace_config ])" +
	      // set the container config as the parent config.
	      "config.set_parent_configuration(container.get_container_configuration()) ;" +
	      // set this config as the component config.
	      "container.set_component_configuration(config) ;";
	    
	    // install the Server home with this container configuration.
	    value.insert_string(home_config);
	    config[0].value = value;
	    
	    //        org.omg.Components.CCMHome h = server1_cont.install_home("demo1", "org.objectweb.ccm.demo1.ServerHomeImpl.create_home", config);
	    org.omg.Components.CCMHome h = server1_cont.install_home("demo1", "org.objectweb.ccm.demo1.ServerHomeImpl.create_home", new org.omg.Components.ConfigValue[0]);
	    ServerHome sh = ServerHomeHelper.narrow(h);
	    
	    // define a container configuration.
	    //
	    //  Uncomment this part for a minimal configuration
	    //  and comment the following one.
	    //
	    /**
	     **/
	    
	    home_config = 
	      // set the container config as the home config
	      // i.e. an empty configuration.
	      "container.set_home_configuration(container.get_container_configuration()) ;" +
	      // create a port specific config
	      "config = container.find_system_home(\"PortConfig\").create_component(JAVA.null) ;" +
	      // set the component ::CORBA::Repository of the component.
	      "config.setComponentUId(\"IDL:ccm.objectweb.org/demo1/Client:1.0\") ;" +
	      // set a list coordinator for the "to_server" receptacle of the "Client" component
	      "config.setCallCoordinator(\"to_server\", \"ListCoord\") ;" +
	      // set a trace controller for the "to_server" receptacle of the "Client" component
	      "config.setCallControllers(\"to_server\", [ \"Trace\" ]) ;" +
	      // create and set the trace configuration for this receptacle
	      "trace_config = PropertySet(\"trace\")" +
	      "trace_config.add(StringProperty(\"filename\", \"./trace_Clients.txt\"))" +
	      "trace_config.add(StringProperty(\"singleton\", \"to_server\"))" +
	      "config.setCallControllersConfig(\"to_server\", [ trace_config ])" +
	      // set the container config as the parent config.
	      "config.set_parent_configuration(container.get_container_configuration()) ;" +
	      // set this config as the component config.
	      "container.set_component_configuration(config) ;";
	    
	    // install the Client home with this container configuration.
	    value.insert_string(home_config);
	    config[0].value = value;
	    
	    //        h = server2_cont.install_home("demo1", "org.objectweb.ccm.demo1.ClientHomeImpl.create_home", config);
	    h = server2_cont.install_home("demo1", "org.objectweb.ccm.demo1.ClientHomeImpl.create_home", new org.omg.Components.ConfigValue[0]);
	    ClientHome ch = ClientHomeHelper.narrow(h);
	    
	    
	    // Create components.
	    System.out.println("Instantiating components...");
	    Server s = sh.create();
	    Client c1 = ch.create();
	    Client c2 = ch.create();
	    Client c3 = ch.create();
	    
	    // Configure components.
	    System.out.println("Configuring components...");
	    s.the_name("The Server");
	    c1.the_name("Mathieu");
	    c2.the_name("Raphael");
	    c3.the_name("Philippe");
	    
	    // Connect clients and server.
	    System.out.println("Interconnecting components...");
	    Display for_clients = s.provide_for_clients();
	    c1.connect_to_server(for_clients);
	    c2.connect_to_server(for_clients);
	    c3.connect_to_server(for_clients);
	    
	    // Configuration completion.
	    System.out.println("Configuration completion...");
	    s.configuration_complete();
	    c1.configuration_complete();
	    c2.configuration_complete();
	    c3.configuration_complete();
	    
	 } catch (Exception e) {
	    if ((current!=null)&&(_OTS.equals("yes")))
	      {
		 System.out.println("Error during deployment :");
		 e.printStackTrace();
		 System.out.print("Rolling Back ... ");
		 current.rollback();
		 System.out.println("Done");
		 // Force to exit.
		 System.exit(0);
	      }
	 }
       
       if ((current!=null)&&(_OTS.equals("yes")))
	 {
	    System.out.print("Do you want to commit the Deployment ? [Y/n] ");
	    
	    java.io.BufferedReader _buffer
	      = new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
	    char _answer = (char) _buffer.read();
	    
	    if ((_answer=='n')||(_answer=='N'))
	      {
		 System.out.print("Rolling Back ... ");
		 current.rollback();
		 System.out.println("Done");
	      }	else {
		 System.out.print("Committing ... ");
		 current.commit(false);
		 System.out.println("Done");
	      }
	 }
       
       // Force to exit.
       System.exit(0);
    }
}







