/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.demo3;

/**
 * The demo3 application deployment.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</A>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</A>
 */

public class Demo3
{
    /**
     * The bootstrap of the demo3 application.
     */
    public static void
    main(String[] args)
    throws Exception
    {
				String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();

        // Init the ORB.
        System.out.println("Initializing the ORB...");

				// TODO: Need to report this ORB.init() problem to OpenORB developers!
				// Due to an OpenORB problem, the following lines:
				//   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
        //   org.objectweb.ccm.Components.Runtime.init(orb);
        // are replaced by the next line.
				
				// Init the OpenCCM Components Runtime.
				org.omg.CORBA.ORB orb =
						org.objectweb.ccm.Components.Runtime.init(args);
				
        // Obtain the Name Service.
        System.out.println("Obtaining the Name Service...");
        org.omg.CORBA.Object obj =
            orb.resolve_initial_references("NameService");
        org.omg.CosNaming.NamingContext nc =
            org.omg.CosNaming.NamingContextHelper.narrow(obj);

				org.omg.CosTransactions.Current current = null;
				if (_OTS.equals("yes"))
				{
						System.out.println("Obtaining the Transaction Service...");
						org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
						current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
				}

				try
				{
						if ((current!=null)&&(_OTS.equals("yes")))
						{
								System.out.println("Beginning the transaction...");
								current.begin();
						}
						
						// Obtain the component servers.
						System.out.println("Obtaining Component Servers...");
						org.omg.CosNaming.NameComponent[] ncomp =
								new org.omg.CosNaming.NameComponent[1];
						ncomp[0] = new org.omg.CosNaming.NameComponent("ComponentServer1", "");
						obj = nc.resolve(ncomp);
						org.objectweb.ccm.Deployment.Server server1 =
								org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
						ncomp[0].id = "ComponentServer2";
						obj = nc.resolve(ncomp);
						org.objectweb.ccm.Deployment.Server server2 =
								org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
						
						// Obtain the container homes and archive installators.
						org.omg.Components.Deployment.ComponentServer server1_cs = server1.provide_component_server();
						org.omg.Components.Deployment.ComponentInstallation server1_inst = server1.provide_install();
						org.omg.Components.Deployment.ComponentServer server2_cs = server2.provide_component_server();
						org.omg.Components.Deployment.ComponentInstallation server2_inst = server2.provide_install();
						
						// Install archives.
						System.out.println("Installing archives...");
						server1_inst.install("demo3", "./archives/demo3.jar");
						server2_inst.install("demo3", "./archives/demo3.jar");
						server1_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
						server2_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
						
						// declaring the container types
						String cont_config = 
								// retrieve the container reference.
								"container = CONTAINER.container ; " +
								// create aliases on PropertySetImpl and StringPropertyImpl class.
								"JAVA.declare(\"org.objectweb.ccm.Containers\") ; " +
								"PropertySet = org.objectweb.ccm.Containers.PropertySetImpl ;" +
								"StringProperty = org.objectweb.ccm.Containers.StringPropertyImpl ;" +
								// create an empty configuration with it's alias.
								"config_home = container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyConfig\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyConfigurationHome.create\") ;"+
								"config = config_home.create_component(JAVA.null) ;" +
								// create an empty coordinator system home with it's alias
								"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyCoordinatorHome\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyCoordinatorHome.create\") ;"+
								// create an empty controller system home with it's alias
								"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyControllerHome\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyControllerHome.create\") ;"+
								// create a trace controller system home with it's alias.
								"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"Trace\", " +
                "\"org.objectweb.ccm.Containers.Plugins.TraceControllerHome.create\") ; "+
								// create a list coordinator system home with it's alias.
								"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"ListCoord\", " +
                "\"org.objectweb.ccm.Containers.Plugins.ListCoordinatorHome.create\") ; "+
								// create an port specific config.
								"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"PortConfig\", " +
                "\"org.objectweb.ccm.Containers.Plugins.PortSpecificConfigurationHome.create\") ; " +
								// set this configuration as the container default configuration.
								"container.set_container_configuration(config) ;";
						

						// instantiate a container on each server.
						org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[1];
						config[0] = new org.objectweb.ccm.Components.ConfigValueImpl();
						config[0].name = "container_script";
						org.omg.CORBA.TypeCode string_tc =
								org.objectweb.ccm.CORBA.TheORB.getORB().get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
						org.omg.CORBA.Any value = 
								org.objectweb.ccm.CORBA.TheDynamicAnyFactory.getFactory().
                create_dyn_any_from_type_code(string_tc).to_any();
						value.insert_string(cont_config);
						config[0].value = value;
						

						org.omg.Components.Deployment.Container server1_cont =
								server1_cs.create_container(config);
						org.omg.Components.Deployment.Container server2_cont =
								server2_cs.create_container(config);
						
						// Install homes.
						System.out.println("Instantiating homes...");
						
						// define a container configuration.
						config[0].name = "home_script";
						String home_config = 
								// set the container config as the home config
								// i.e. an empty configuration.
								"container.set_home_configuration(container.get_container_configuration()) ;" +
								// create a port specific config
								"config = container.find_system_home(\"PortConfig\").create_component(JAVA.null) ;" +
								// set the component ::CORBA::Repository of the component.
								"config.setComponentUId(\"IDL:ccm.objectweb.org/demo3/Client:1.0\") ;" +
								// set a list coordinator for the "the_service" receptacle of the "Client" component
								"config.setCallCoordinator(\"the_service\", \"ListCoord\") ;" +
								// set a trace controller for the "the_service" receptacle of the "Client" component
								"config.setCallControllers(\"the_service\", [ \"Trace\" ]) ;" +
								// create and set the trace configuration for this receptacle
								"trace_config = PropertySet(\"trace\")" +
								"trace_config.add(StringProperty(\"filename\", \"./trace_Clients.txt\"))" +
								"trace_config.add(StringProperty(\"singleton\", \"the_service\"))" +
								"config.setCallControllersConfig(\"the_service\", [ trace_config ])" +
								// set the container config as the parent config.
								"config.set_parent_configuration(container.get_container_configuration()) ;" +
								// set this config as the component config.
								"container.set_component_configuration(config) ;";
						
						// install the Client home with this container configuration.
						value.insert_string(home_config);
						config[0].value = value;
						
						org.omg.Components.CCMHome h = null;
						h = server1_cont.install_home("demo3", "org.objectweb.ccm.demo3.ClientHomeImpl.create_home", config);
						ClientHome ch = ClientHomeHelper.narrow(h);

						// define a container configuration.
						home_config = 
								// set the container config as the home config
								// i.e. an empty configuration.
								"container.set_home_configuration(container.get_container_configuration()) ;" +
								// create a port specific config
								"config = container.find_system_home(\"PortConfig\").create_component(JAVA.null) ;" +
								// set the component ::CORBA::Repository of the component.
								"config.setComponentUId(\"IDL:ccm.objectweb.org/demo3/Server:1.0\") ;" +
								// set a list coordinator for the "the_service" facet of the "Server" component
								"config.setCallCoordinator(\"the_service\", \"ListCoord\") ;" +
								// set a trace controller for the "the_service" facet of the "Server" component
								"config.setCallControllers(\"the_service\", [ \"Trace\" ]) ;" +
								// create and set the trace configuration for this facet
								"trace_config = PropertySet(\"trace\")" +
								"trace_config.add(StringProperty(\"filename\", \"./trace_Server_Service.txt\"))" +
								"config.setCallControllersConfig(\"the_service\", [ trace_config ])" +
								// set a list coordinator for the "to_consumers" event source of the "Server" component
								"config.setCallCoordinator(\"to_consumers\", \"ListCoord\") ;" +
								// set a trace controller for the "to_consumers" event source of the "Server" component
								"config.setCallControllers(\"to_consumers\", [ \"Trace\" ]) ;" +
								// create and set the trace configuration for this event source
								"trace_config = PropertySet(\"trace\")" +
								"trace_config.add(StringProperty(\"filename\", \"./trace_Server_TextEvent.txt\"))" +
								"config.setCallControllersConfig(\"to_consumers\", [ trace_config ])" +
								// set the container config as the parent config.
								"config.set_parent_configuration(container.get_container_configuration()) ;" +
								// set this config as the component config.
								"container.set_component_configuration(config) ;";
						
						// install the Server home with this container configuration.
						value.insert_string(home_config);
						config[0].value = value;
						
						h = server2_cont.install_home("demo3", "org.objectweb.ccm.demo3.ServerHomeImpl.create_home", config);
						ServerHome sh = ServerHomeHelper.narrow(h);
						
						// define a container configuration.
						home_config = 
								// set the container config as the home config
								// i.e. an empty configuration.
								"container.set_home_configuration(container.get_container_configuration()) ;" +
								// create a port specific config
								"config = container.find_system_home(\"PortConfig\").create_component(JAVA.null) ;" +
								// set the component ::CORBA::Repository of the component.
								"config.setComponentUId(\"IDL:ccm.objectweb.org/demo3/Consumer:1.0\") ;" +
								// set a list coordinator for the "from_servers" event sink of the "Consumer" component
								"config.setCallCoordinator(\"from_servers\", \"ListCoord\") ;" +
								// set a trace controller for the "from_servers" event sink of the "Consumer" component
								"config.setCallControllers(\"from_servers\", [ \"Trace\" ]) ;" +
								// create and set the trace configuration for this event sink
								"trace_config = PropertySet(\"trace\")" +
								"trace_config.add(StringProperty(\"filename\", \"./trace_Consumers.txt\"))" +
								"trace_config.add(StringProperty(\"singleton\", \"from_servers\"))" +
								"config.setCallControllersConfig(\"from_servers\", [ trace_config ])" +
								// set the container config as the parent config.
								"config.set_parent_configuration(container.get_container_configuration()) ;" +
								// set this config as the component config.
								"container.set_component_configuration(config) ;";
						
						// install the Server home with this container configuration.
						value.insert_string(home_config);
						config[0].value = value;

						h = server1_cont.install_home("demo3", "org.objectweb.ccm.demo3.ConsumerHomeImpl.create_home", config);
						ConsumerHome csh = ConsumerHomeHelper.narrow(h);
						
						// Create components.
						System.out.println("Creating components...");
						Server s = sh.create();
						Client c1 = ch.create();
						Client c2 = ch.create();
						Client c3 = ch.create();
						Consumer cs1 = csh.create();
						Consumer cs2 = csh.create();
						Consumer cs3 = csh.create();
						
						// Configure components.
						System.out.println("Configuring components...");
						c1.name("Mathieu");
						c2.name("Raphael");
						c3.name("Philippe");
						s.name("The Server");
						cs1.name("Mathieu");
						cs2.name("Raphael");
						cs3.name("Philippe");
						
						// Connecting clients and consumers to server.
						System.out.println("Interconnecting components...");
						Service the_service = s.provide_the_service();
						c1.connect_the_service(the_service);
						c2.connect_the_service(the_service);
						c3.connect_the_service(the_service);
						s.subscribe_to_consumers(cs1.get_consumer_from_servers());
						s.subscribe_to_consumers(cs2.get_consumer_from_servers());
						s.subscribe_to_consumers(cs3.get_consumer_from_servers());
						
						// Configuration completion.
						System.out.println("Configuration completion...");
						cs1.configuration_complete();
						cs2.configuration_complete();
						cs3.configuration_complete();
						s.configuration_complete();
						c1.configuration_complete();
						c2.configuration_complete();
						c3.configuration_complete();
						
				} catch (Exception e) {
						if ((current!=null)&&(_OTS.equals("yes")))
								{
								System.out.println("Error during deployment :");
								e.printStackTrace();
								System.out.print("Rolling Back ... ");
								current.rollback();
								System.out.println("Done");
								// Force to exit.
								System.exit(0);
						}
				}

				if ((current!=null)&&(_OTS.equals("yes")))
				{
						System.out.print("Do you want to commit the Deployment ? [Y/n] ");

						java.io.BufferedReader _buffer
								= new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
						char _answer = (char) _buffer.read();

						if ((_answer=='n')||(_answer=='N'))
						{
								System.out.print("Rolling Back ... ");
								current.rollback();
								System.out.println("Done");
						}	else {
								System.out.print("Committing ... ");
								current.commit(false);
								System.out.println("Done");
						}
				}

				// force to exit
				System.exit(0);
				}
}


