# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2002 USTL - LIFL - GOAL
# Contact: openccm-team@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Philippe Merle, Sylvain Leblanc.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  OMG IDLscript for the philosopher dinner application deployment.
#
#  Note: To do before starting this script
#    ir_load ${OpenCCM_HOMEDIR}/idl/Components.idl
#    ir_load ${OpenCCM_HOMEDIR}/idl/Deployment.idl
#    ir_load ${OpenCCM_HOMEDIR}/idl/OpenCCM.idl
#    ir_load dinner.idl
#
# ====================================================================

println("Obtaining the Name Service...")

ns = CORBA.ORB.resolve_initial_references("NameService")

# ====================================================================

println("Obtaining the Java Component Servers...")

server1 = ns.resolve([["ComponentServer1",""]])
server2 = ns.resolve([["ComponentServer2",""]])

server1_cs = server1.provide_component_server()
server1_inst = server1.provide_install()

server2_cs = server2.provide_component_server()
server2_inst = server2.provide_install()

# ====================================================================

println("Installing archives...")

server1_inst.install("dinner", "./archives/DiningPhilosophers.jar")
server2_inst.install("dinner", "./archives/DiningPhilosophers.jar")

server1_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar")
server2_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar")

# ====================================================================

println("Creating containers...")

server1_cont = server1_cs.create_container([]);
server2_cont = server1_cs.create_container([]);

# ====================================================================

println("Instantiating homes...")

oh = server1_cont.install_home("dinner", "DiningPhilosophers.ObserverHomeImpl.create_home", [])
ph = server2_cont.install_home("dinner", "DiningPhilosophers.PhilosopherHomeImpl.create_home", [])
fh = server2_cont.install_home("dinner", "DiningPhilosophers.ForkHomeImpl.create_home", [])

# ====================================================================

println("Instantiating components...")

p1 = ph.new("Mathieu")
p2 = ph.new("Raphael")
p3 = ph.new("Philippe")
p4 = ph.new("Sylvain")
f1 = fh.create()
f2 = fh.create()
f3 = fh.create()
f4 = fh.create()
o = oh.create()

# ====================================================================

println("Configuring components...")

# ====================================================================

println("Interconnecting components...")

fork = f1.provide_the_fork()
p1.connect_right(fork)
p2.connect_left(fork)

fork = f2.provide_the_fork()
p2.connect_right(fork)
p3.connect_left(fork)

fork = f3.provide_the_fork()
p3.connect_right(fork)
p4.connect_left(fork)

fork = f4.provide_the_fork()
p4.connect_right(fork)
p1.connect_left(fork)

consumer = o.get_consumer_info()
p1.subscribe_info(consumer)
p2.subscribe_info(consumer)
p3.subscribe_info(consumer)
p4.subscribe_info(consumer)

# ====================================================================

println("Configuration completion...")

f1.configuration_complete()
f2.configuration_complete()
f3.configuration_complete()
f4.configuration_complete()
o.configuration_complete()
p1.configuration_complete()
p2.configuration_complete()
p3.configuration_complete()
p4.configuration_complete()

# ====================================================================
