/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * The philosopher dinner application deployment.
 * 
 * <p>
 * This application is composed of four philosopher 
 * components, four fork manager components and one observer 
 * component distributed on two servers. 
 * These servers must be named "ComponentServer1" and "ComponentServer2". 
 * The OpenCCM_plugins.jar archive must be 
 * in the current directory and DinningPhilosophers.jar archive 
 * must be in <code>./archives</code> directory.
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 */

public class Deployment_PC
{
    /**
     * The bootstrap of the dinner application.
     */
    public static void
    main(String[] args)
    throws Exception
    {
	String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();

        // Init the ORB.
        System.out.println("Initializing the ORB...");

	// TODO: Need to report this ORB.init() problem to OpenORB developers!
	// Due to an OpenORB problem, the following lines:
	//   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
        //   fr.lifl.goal.OpenCCM.Components.Runtime.init(orb);
        // are replaced by the next line.
				
	// Init the OpenCCM Components Runtime.
	org.omg.CORBA.ORB orb = org.objectweb.ccm.Components.Runtime.init(args);
				
        // Obtain the Name Service.
        System.out.println("Obtaining the Name Service...");
        org.omg.CORBA.Object obj =
            orb.resolve_initial_references("NameService");
        org.omg.CosNaming.NamingContext ns =
            org.omg.CosNaming.NamingContextHelper.narrow(obj);
        obj = orb.resolve_initial_references("CommonNameService");
        org.omg.CosNaming.NamingContext cns =
            org.omg.CosNaming.NamingContextHelper.narrow(obj);

	org.omg.CosTransactions.Current current = null;
	if (_OTS.equals("yes"))
	{
            System.out.println("Obtaining the Transaction Service...");
            org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
            current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
	}

	try
	{
            if ((current!=null)&&(_OTS.equals("yes")))
	    {
		System.out.println("Beginning the transaction...");
		current.begin();
	    }
						
 	    // Obtain the component servers.
	    System.out.println("Obtaining Component Servers...");
	    String cs1_name = System.getProperty("org.objectweb.ccm.DiningPhilosophers.CS1");
	    String cs2_name = System.getProperty("org.objectweb.ccm.DiningPhilosophers.CS2");

            org.omg.CosNaming.NameComponent[] ncomp =
		new org.omg.CosNaming.NameComponent[1];
	    ncomp[0] = new org.omg.CosNaming.NameComponent(cs1_name, "");
	    obj = ns.resolve(ncomp);
	    org.objectweb.ccm.Deployment.Server server1 =
		org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
	    ncomp[0].id = cs2_name;
	    obj = ns.resolve(ncomp);
	    org.objectweb.ccm.Deployment.Server server2 =
	    org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);

	    // Obtain the home factories and archive installators.
	    org.omg.Components.Deployment.ComponentServer server1_cs =
		server1.provide_component_server();
	    org.omg.Components.Deployment.ComponentInstallation server1_inst =
		server1.provide_install();
	    org.omg.Components.Deployment.ComponentServer server2_cs = 
		server2.provide_component_server();
	    org.omg.Components.Deployment.ComponentInstallation server2_inst =
		server2.provide_install();

	    // Install archives.
	    System.out.println("Installing archives...");
	    String http_server = System.getProperty("org.objectweb.ccm.DiningPhilosophers.http");
	    server1_inst.install("dinner", http_server + "DiningPhilosophers.jar");
	    server2_inst.install("dinner", http_server + "DiningPhilosophers.jar");
	    // install some plugins already provided by OpenCCM.
	    server1_inst.install("openccm_plugins", http_server + "DiningPhilosophers.jar");
	    server2_inst.install("openccm_plugins", http_server + "DiningPhilosophers.jar");

	    // declaring the container types
	    String cont_config = 
		"container = CONTAINER.container ; " +
		"config_home = container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyConfig\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyConfigurationHome.create\") ;"+
		"config = config_home.create_component(JAVA.null) ;" +
		"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyCoordinatorHome\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyCoordinatorHome.create\") ;"+
		"container.create_system_home("+
                "\"openccm_plugins\", " +
                "\"EmptyControllerHome\", "+
                "\"org.objectweb.ccm.Containers.Plugins.EmptyControllerHome.create\") ;"+
		"container.set_container_configuration(config) ;";

	   // instantiate a container on each server.
	   org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[1];
	   config[0] = new org.objectweb.ccm.Components.ConfigValueImpl();
	   config[0].name = "container_script";
	   org.omg.CORBA.TypeCode string_tc =
		org.objectweb.ccm.CORBA.TheORB.getORB().get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
	   org.omg.CORBA.Any value = 
		org.objectweb.ccm.CORBA.TheDynamicAnyFactory.getFactory().
					create_dyn_any_from_type_code(string_tc).to_any();
	   value.insert_string(cont_config);
	   config[0].value = value;

	   org.omg.Components.Deployment.Container server1_cont =
		server1_cs.create_container(config);
	   org.omg.Components.Deployment.Container server2_cont =
		server2_cs.create_container(config);

	   // Create homes.
	   System.out.println("Instantiating homes...");

	   // define a container configuration.
	  java.lang.String home_config = 
		"container.set_home_configuration(container.get_container_configuration()) ;" +
		"container.set_component_configuration(container.get_container_configuration()) ;";

	   config[0].name = "home_script";
	   value.insert_string(home_config);
	   config[0].value = value;

	   // install homes with this container configuration.

	   // On laptop
	   org.omg.Components.CCMHome h = server2_cont.install_home(
               "dinner", "DiningPhilosophers.PhilosopherHomeImpl.create_home", config);
	   PhilosopherHome ph = PhilosopherHomeHelper.narrow(h);

	   h = server2_cont.install_home("dinner", "DiningPhilosophers.ForkHomeImpl.create_home", config);  
	   ForkHome fh = ForkHomeHelper.narrow(h);
	   h = server2_cont.install_home("dinner", "DiningPhilosophers.ObserverHomeImpl.create_home", config);  
	   ObserverHome oh = ObserverHomeHelper.narrow(h);

	   // Register homes in the name service

	   // creating OpenCCM Context 
	   org.omg.CosNaming.NameComponent[] nc = 
		new org.omg.CosNaming.NameComponent[1];
	   nc[0] = new org.omg.CosNaming.NameComponent();
	   nc[0].id = "OpenCCM";
	   nc[0].kind = "";

	   try {
	      ns = cns.bind_new_context(nc);            
	   } catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ex) {
		ns = org.omg.CosNaming.NamingContextHelper.narrow(cns.resolve(nc));
	   }

	   nc[0].id = "PhilosopherHome";
	   ns.rebind(nc, ph);

	   nc[0].id = "ForkHome";
	   ns.rebind(nc, fh);

	   nc[0].id = "ObserverHome";
	   ns.rebind(nc, oh);
	
        } catch (Exception e) {
            if ((current!=null)&&(_OTS.equals("yes")))
	    {
		System.out.println("Error during deployment :");
		e.printStackTrace();
		System.out.print("Rolling Back ... ");
		current.rollback();
		System.out.println("Done");
		// Force to exit.
		System.exit(0);
	     }
	}

	if ((current!=null)&&(_OTS.equals("yes")))
	{
 	    System.out.print("Do you want to commit the Deployment ? [Y/n] ");

	    java.io.BufferedReader _buffer
		= new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
	    char _answer = (char) _buffer.read();

	    if ((_answer=='n')||(_answer=='N'))
	    {
		System.out.print("Rolling Back ... ");
		current.rollback();
		System.out.println("Done");
	    } else {
		System.out.print("Committing ... ");
		current.commit(false);
		System.out.println("Done");
 	    }
	}

	// Force to exit.
	System.exit(0);
    }
}





