/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * This is an AWT Panel GUI used to show a philosopher state.
 * This panel show the philosopher name, the philosopher 
 * forks and the philosopher status.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 */

public class PhilosopherPdaPanel 
    extends java.awt.Panel 
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * The progress bar used to show philosopher's thicks 
     * since last meal.
     */
    //protected javax.swing.JProgressBar hungryness_;

    /**
     * The check box used to show if philosopher has 
     * his right fork.
     */
    protected MyCheckBox right_fork_;

    /**
     * The check box used to show if philosopher has 
     * his left fork.
     */
    protected MyCheckBox left_fork_;

    /** The label used to show the philosopher name. */      
    protected java.awt.Label name_;

    /** The label used to show the philosopher status. */      
    protected java.awt.Label state_;

    /** The philosopher possible states as string. */
    protected String[] stateAsStrings_ = {
        "EATING", "THINKING", "HUNGRY", "STARVING", "DEAD"
    };

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Initializes the philosopher panel using a StatusInfo 
     * event.
     *
     * @param status The StatusInfo event used to initialize 
     *               the philosopher panel.
     */ 
    public PhilosopherPdaPanel(StatusInfo status) 
    {
        /* this panel has a border layout */
        super(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));


        java.net.URL url;
        java.awt.Image my_fork, no_fork;

        /* setting hungryness progress bar in two pannels 
           to center it in the main panel */
        //        hungryness_ = new javax.swing.JProgressBar(0, 40);
        //        hungryness_.setSize(new java.awt.Dimension(150, 30));
        //        hungrynessSubPanel.add(hungryness_);
        //        hungrynessPanel.add(new javax.swing.JLabel(" "), java.awt.BorderLayout.NORTH);
        //        hungrynessPanel.add(hungrynessSubPanel, java.awt.BorderLayout.CENTER);

        /* create and configure the name label */
        name_ = new java.awt.Label(status.name);
        name_.setSize(new java.awt.Dimension(100, name_.getHeight()));

        /* create check boxes */
        right_fork_ = new MyCheckBox();
        left_fork_ = new MyCheckBox();

        /* create state label */
        state_ = new java.awt.Label();
        state_.setSize(new java.awt.Dimension(100, state_.getHeight()));

        /* Setting fork icons (icons must be located in the dinner jar file) */
        url = PhilosopherPdaPanel.class.getResource("/images/myfork.gif");
        my_fork = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        right_fork_.setSelectedIcon(my_fork);
         left_fork_.setSelectedIcon(my_fork);
         url = PhilosopherPdaPanel.class.getResource("/images/nofork.gif");
         no_fork = java.awt.Toolkit.getDefaultToolkit().getImage(url);
         right_fork_.setIcon(no_fork);
         left_fork_.setIcon(no_fork);
        right_fork_.setSelected(false);
        left_fork_.setSelected(false);
        right_fork_.setSize(50, 50);
        left_fork_.setSize(50, 50);


        /* add name to left side of the panel */
         add(name_);

        /* creates a new sub panel for forks and state */
        java.awt.Panel forks = new java.awt.Panel(new java.awt.FlowLayout());
        add(right_fork_);
        add(state_);
        add(left_fork_);
        
        /* adding progress bar panel to the east side of the panel */
        //add(hungrynessPanel, java.awt.BorderLayout.EAST);

        /* initializes panel values */
        updatePanel(status);
        validate();

        /* Configure the panel */
        setVisible(true);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Update the philosopher's panel using a StatusInfo event.
     *
     * @param event The event used to update the panel.
     */
    public void updatePanel(StatusInfo event)
    {
        /* update the status label */
        state_.setText(stateAsStrings_[event.state.value()]);
        /* update the hungryness progress bar */
        //hungryness_.setForeground(getFG(event.ticks_since_last_meal));
        //hungryness_.setValue(event.ticks_since_last_meal);
        /* update forks check boxes */
        left_fork_.setSelected(event.has_left_fork);
        right_fork_.setSelected(event.has_right_fork);
    }
}

/**
 * An internal check box GUI for PDA (based on AWT).
 */
class MyCheckBox extends java.awt.Panel 
{

    /** used icons */
    protected java.awt.Image selectedIcon_, icon_;

    /** check box internal state */
    protected boolean selected_;

    /**
     * Default constructor.
     */
    public MyCheckBox() {
        selected_ = false;
    };

    /**
     * Set the icon to use when check box has been selected.
     */
    public void setSelectedIcon(java.awt.Image icon) 
    {
        selectedIcon_ = icon;
    }

    /**
     * Set the icon to use when check box hasn't been selected.
     */
    public void setIcon(java.awt.Image icon) 
    {
        icon_ = icon;
    }

    /**
     * Set selection on the check box.
     */
    public void setSelected(boolean b)
    {
        selected_ = b;
        repaint();
    }

    /**
     * Overrides java.awt.Component.getPreferredSize 
     * in order to have a nice display of icons.
     */
    public java.awt.Dimension getPreferredSize()
    {
        return new java.awt.Dimension(50, 50);
    }

    /**
     * Overrides the component paint method to draw 
     * icons.
     */
     public void paint(java.awt.Graphics g)
     {
         if (selected_){
             g.drawImage(selectedIcon_, 0, 0, this);
         } else {
             g.drawImage(icon_, 0, 0, this);
         }
     }
}
