/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * The philosopher dinner application deployment.
 * 
 * <p>
 * This application is composed of four philosopher 
 * components, four fork manager components and one observer 
 * component distributed on two servers. 
 * These servers must be named "ComponentServer1" and "ComponentServer2". 
 * The OpenCCM_plugins.jar archive must be 
 * in the current directory and DinningPhilosophers.jar archive 
 * must be in <code>./archives</code> directory.
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 */

public class Run
{
    /**
     * The bootstrap of the dinner application.
     */
    public static void
    main(String[] args)
    throws Exception
    {
        String _OTS = System.getProperties().
                      getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();

        // Init the ORB.
        System.out.println("Initializing the ORB...");

	// TODO: Need to report this ORB.init() problem to OpenORB developers!
	// Due to an OpenORB problem, the following lines:
	//   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
        //   fr.lifl.goal.OpenCCM.Components.Runtime.init(orb);
        // are replaced by the next line.

	// Init the OpenCCM Components Runtime.
	org.omg.CORBA.ORB orb =
	    org.objectweb.ccm.Components.Runtime.init(args);

        // Obtain the Name Service.
        System.out.println("Obtaining the Name Service...");
        org.omg.CORBA.Object obj =
            orb.resolve_initial_references("CommonNameService");
        org.omg.CosNaming.NamingContext ns =
            org.omg.CosNaming.NamingContextHelper.narrow(obj);

				org.omg.CosTransactions.Current current = null;
				if (_OTS.equals("yes"))
				{
						System.out.println("Obtaining the Transaction Service...");
						org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
						current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
				}

				try
				{
						if ((current!=null)&&(_OTS.equals("yes")))
						{
								System.out.println("Beginning the transaction...");
								current.begin();
						}
						
         // resolving OpenCCM Context 
        org.omg.CosNaming.NameComponent[] nc = 
            new org.omg.CosNaming.NameComponent[1];
        nc[0] = new org.omg.CosNaming.NameComponent();
        nc[0].id = System.getProperty("org.objectweb.ccm.DiningPhilosophers.name");;
        nc[0].kind = "";

        ns = org.omg.CosNaming.NamingContextHelper.narrow(ns.resolve(nc));

        // resolving homes
        nc[0].id = "PhilosopherHome";
        obj = ns.resolve(nc);
        PhilosopherHome ph = PhilosopherHomeHelper.narrow(obj);
        
        nc[0].id = "ForkHome";
        obj = ns.resolve(nc);
        ForkHome fh = ForkHomeHelper.narrow(obj);
        
        nc[0].id = "ObserverHome";
        obj = ns.resolve(nc);
        ObserverHome oh = ObserverHomeHelper.narrow(obj);
        
        // Create components.
        System.out.println("Creating components...");
        Philosopher p1 = ph._new("Mathieu");
        Philosopher p2 = ph._new("Raphael");
        Philosopher p3 = ph._new("Philippe");
        Philosopher p4 = ph._new("Sylvain");
        ForkManager f1 = fh.create();
        ForkManager f2 = fh.create();
        ForkManager f3 = fh.create();
        ForkManager f4 = fh.create();
        Observer o = oh.create();

        // Configure components.
        System.out.println("Configuring components...");

        // Connecting components.
        System.out.println("Interconnecting components...");

        Fork fork = f1.provide_the_fork();
        p1.connect_right(fork);
        p2.connect_left(fork);

        fork = f2.provide_the_fork();
        p2.connect_right(fork);
        p3.connect_left(fork);

        fork = f3.provide_the_fork();
        p3.connect_right(fork);
        p4.connect_left(fork);

        fork = f4.provide_the_fork();
        p4.connect_right(fork);
        p1.connect_left(fork);

        StatusInfoConsumer consumer = o.get_consumer_info();
        p1.subscribe_info(consumer);
        p2.subscribe_info(consumer);
        p3.subscribe_info(consumer);
        p4.subscribe_info(consumer);

        // Configuration completion.
        System.out.println("Configuration completion...");
        f1.configuration_complete();
        f2.configuration_complete();
        f3.configuration_complete();
        f4.configuration_complete();
        o.configuration_complete();
        p1.configuration_complete();
        p2.configuration_complete();
        p3.configuration_complete();
        p4.configuration_complete();

				} catch (Exception e) {
						if ((current!=null)&&(_OTS.equals("yes")))
						{
								System.out.println("Error during deployment :");
								e.printStackTrace();
								System.out.print("Rolling Back ... ");
								current.rollback();
								System.out.println("Done");
								// Force to exit.
								System.exit(0);
						}
            throw e;
				}

				if ((current!=null)&&(_OTS.equals("yes")))
				{
						System.out.print("Do you want to commit the Deployment ? [Y/n] ");
								
						java.io.BufferedReader _buffer
								= new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
						char _answer = (char) _buffer.read();

						if ((_answer=='n')||(_answer=='N'))
						{
								System.out.print("Rolling Back ... ");
								current.rollback();
								System.out.println("Done");
						}	else {
								System.out.print("Committing ... ");
								current.commit(false);
								System.out.println("Done");
						}
				}

        System.exit(0);
    }
}
