/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.hello;

/**
 * The hello application deployment.
 */
public class Install
{
  /**
   * The usage message.
   */
  public static void
  usage()
  {
    System.err.println("usage: Install <component_server_name> <NS_name>");
    System.err.println("");
    System.err.println("  <component_server_name>    " + 
                       "The component server name on which the " + 
                       "ServerHome will be deployed.");
    System.err.println("  <NS_name>                  " + 
                       "The name used to register the ServerHome " + 
                       "instance.");
  }
  
  /**
   * The bootstrap of the demo1 application.
   */
  public static void
  main(String[] args)
    throws Exception
  {
    String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();
    
    // Init the ORB.
    System.out.println("Initializing the ORB...");
    
    // TODO: Need to report this ORB.init() problem to OpenORB developers!
    // Due to an OpenORB problem, the following line:
    //   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
    // are replaced by the next line.
    
    // Init the OpenCCM Components Runtime.
    org.omg.CORBA.ORB orb = org.objectweb.ccm.Components.Runtime.init(args);
    
    if (args.length < 2) {
      usage();
      System.exit(1);
    }
    
    // Obtain the Name Service.
    System.out.println("Obtaining the Name Service...");
    org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
    org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);
    
    org.omg.CosTransactions.Current current = null;
    if (_OTS.equals("yes")) {
      System.out.println("Obtaining the Transaction Service...");
      org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
      current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
    }
    
    try {
      if ((current!=null)&&(_OTS.equals("yes"))) {
        System.out.println("Beginning the transaction...");
        current.begin();
      }
      
      // Obtain the component servers.
      System.out.println("Obtaining Component Servers...");
      org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
      ncomp[0] = new org.omg.CosNaming.NameComponent(args[0], "");
      obj = nc.resolve(ncomp);
      org.objectweb.ccm.Deployment.Server server = org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
      
      // Obtain the container homes and archive installators.
      org.omg.Components.Deployment.ComponentServer server_cs = server.provide_component_server();
      org.omg.Components.Deployment.ComponentInstallation server_inst = server.provide_install();
      
      // Install archives.
      System.out.println("Installing archives...");
      server_inst.install("hello", "./archives/hello.jar");
      
      // install some plugins already provided by OpenCCM.
      server_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
      
      
      // instantiate a container on each server.
      org.omg.Components.Deployment.Container server_cont =
        server_cs.create_container(new org.omg.Components.ConfigValue[0]);
      
      // Install homes.
      System.out.println("Instantiating home...");
      org.omg.Components.CCMHome h = server_cont.install_home("hello", "org.objectweb.ccm.hello.ServerHomeImpl.create_home", new org.omg.Components.ConfigValue[0]);
      ServerHome sh = ServerHomeHelper.narrow(h);
      
      // register the home in the name service with the specified name
      ncomp[0] = new org.omg.CosNaming.NameComponent(args[1], "");
      nc.rebind(ncomp, sh);
      System.out.println("ServerHome instance register under \'" + 
                         args[1] + " \' in the name service");
      
    } catch (Exception e) {
      if ((current!=null)&&(_OTS.equals("yes"))) {
        System.out.println("Error during deployment :");
        e.printStackTrace();
        System.out.print("Rolling Back ... ");
        current.rollback();
        System.out.println("Done");
        // Force to exit.
        System.exit(0);
      }
    }
    
    if ((current!=null)&&(_OTS.equals("yes"))) {
      System.out.print("Do you want to commit the Deployment ? [Y/n] ");
      
      java.io.BufferedReader _buffer
        = new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
      char _answer = (char) _buffer.read();
      
      if ((_answer=='n')||(_answer=='N'))
      {
        System.out.print("Rolling Back ... ");
        current.rollback();
        System.out.println("Done");
      }	else {
        System.out.print("Committing ... ");
        current.commit(false);
        System.out.println("Done");
        System.out.println(args[0] + " is ready to be used...");
      }
    } else {
      System.out.println(args[0] + " is ready to be used...");
    }
    
    System.exit(0);
  }
}






