/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.hello;

/**
 * The hello client script.
 */

public class Run
{
  
  /**
   * The usage message.
   */
  public static void
  usage()
  {
    System.err.println("usage: Install <NS_name>");
    System.err.println("");
    System.err.println("  <NS_name>                  " + 
                       "The name of the registered ServerHome " + 
                       "instance.");
  }
  
  /**
   * The bootstrap of the demo1 client application.
   */
  public static void
  main(String[] args)
    throws Exception
  {
    String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();
    
    // Init the ORB.
    System.out.println("Initializing the ORB...");
    
    // TODO: Need to report this ORB.init() problem to OpenORB developers!
    // Due to an OpenORB problem, the following line:
    //   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
    // are replaced by the next line.
    
    // Init the OpenCCM Components Runtime.
    org.omg.CORBA.ORB orb = org.objectweb.ccm.Components.Runtime.init(args);
    
    if (args.length < 1) {
      usage();
      System.exit(1);
    }
    
    // Obtain the Name Service.
    System.out.println("Obtaining the Name Service...");
    org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
    org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);
    
    // Resolving the hello::ServerHome instance
    System.out.println("Obtaining the " + args[0] + " ServerHome instance...");
    org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
    ncomp[0] = new org.omg.CosNaming.NameComponent(args[0], "");
    obj = nc.resolve(ncomp);
    ServerHome home = ServerHomeHelper.narrow(obj);
    
    org.omg.CosTransactions.Current current = null;
    if (_OTS.equals("yes")) {
      System.out.println("Obtaining the Transaction Service...");
      org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
      current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
    }
    
    try {
      if ((current!=null)&&(_OTS.equals("yes"))) {
        System.out.println("Beginning the transaction...");
        current.begin();
      }
      
      // Creates a new Server component instance
      System.out.println("Creates a new Server component instance...");
      Server server = home.create();
      
      // Retrieve the hello::HelloWorld facet
      System.out.println("Retrieve the facet 'for_clients'...");
      HelloWorld for_clients = server.provide_for_clients();
      
      // Call the 'say_hello' method
      System.out.println("Call the 'say_hello' method...");
      for_clients.say_hello();
      
      // Destroy the server component
      System.out.println("Destroy the server component instance...");
      server.remove();
      
      System.out.println("Bye !!");
    } catch (Exception e) {
      if ((current!=null)&&(_OTS.equals("yes"))) {
        System.out.println("Error during deployment :");
        e.printStackTrace();
        System.out.print("Rolling Back ... ");
        current.rollback();
        System.out.println("Done");
        // Force to exit.
        System.exit(0);
      }
    }
    
    if ((current!=null)&&(_OTS.equals("yes"))) {
      System.out.print("Do you want to commit the Deployment ? [Y/n] ");
      
      java.io.BufferedReader _buffer
        = new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
      char _answer = (char) _buffer.read();
      
      if ((_answer=='n')||(_answer=='N'))
      {
        System.out.print("Rolling Back ... ");
        current.rollback();
        System.out.println("Done");
      }	else {
        System.out.print("Committing ... ");
        current.commit(false);
        System.out.println("Done");
        System.out.println(args[0] + " is ready to be used...");
      }
    } else {
      System.out.println(args[0] + " is ready to be used...");
    }
    
    System.exit(0);
  }
}





















