/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class provides the main structure of a CORBA client application.
 *
 * Subclasses must defined the run() method.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public abstract class Application
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    Application()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Report an exception stack.
     **
     **  @param exception The exception.
     **/
    protected void
    report_exception(Exception exception)
    {
       System.err.print("Please report the following exception trace");
       System.err.println(" to openccm@objectweb.org");
       exception.printStackTrace();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Start the application, i.e.
     **  initialize the CORBA::ORB,
     **  run the application,
     **  destroy the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **/
    public int
    start(String[] args)
    {
	try
	{
            // Initialize the CORBA::ORB.
            args = TheORB.initialize(args);

            // Run the application.
            int status = run(args);

            // Destroy the CORBA::ORB.
            TheORB.destroy();

	    return status;
	}
	catch(UserExceptionWrapper exc)
	{
	    report_exception(exc.getUserException());
	}
	catch(Exception exc)
	{
	    report_exception(exc);
	}
	return -1;
    }

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public abstract int
    run(String[] args);
}
