/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class provides the main structure of a CORBA server application.
 *
 * Subclasses must defined the before_run() and after_run() methods.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public abstract class ApplicationServer
       extends Application
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    ApplicationServer()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Run the application, i.e.
     **  initialize the Root POA,
     **  active it,
     **  call the before_run() method,
     **  run the ORB,
     **  call the after_run() method.
     **
     **  @param args The command line arguments.
     **
     **  @return The status returned by the after_run() method or zero.
     **/
    public int
    run(String[] args)
    {
	// Initialize the Root POA.
        org.omg.PortableServer.POA rootPOA = TheRootPOA.getRootPOA();

        // Activate the Root POA.
        try
	{
	    rootPOA.the_POAManager().activate();
	}
	catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive exc)
	{
	    // Wrap the PortableServer::POAManager::AdapterInactive exception.
	    throw new UserExceptionWrapper(exc);
	}

	// Before running the ORB.
	int status = before_run(args);

	if(status != 0)
	{
      	    return status;
	}

	// Run the ORB.
	TheORB.run();

	System.err.println("The ApplicationServer shutdowns.");

	try
	{
	    // After running the ORB.
	    after_run();
	}
	catch(Exception exc)
	{
	}
    
	return 0;
    }

    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public abstract int
    before_run(String[] args);

    /**
     **  After running the CORBA::ORB.
     **/
    public abstract void
    after_run();
}
