/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class encapsulates a CosNaming::NamingContext reference.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public class NamingContext
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To store the CosNaming::NamingContext reference.
     **/
    private org.omg.CosNaming.NamingContext namingContext_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param namingContext The naming context to encapsulate.
     **/
    public
    NamingContext(org.omg.CosNaming.NamingContext namingContext)
    {
	namingContext_ = namingContext;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain the CosNaming::NamingContext reference.
     **
     **  @return The CosNaming::NamingContext reference.
     **/
    public org.omg.CosNaming.NamingContext
    getNamingContext()
    {
        return namingContext_;
    }

    /**
     **  Set the CosNaming::NamingContext reference.
     **
     **  @param namingContext The CosNaming::NamingContext reference.
     **/
    public void
    setNamingContext(org.omg.CosNaming.NamingContext namingContext)
    {
        namingContext_ = namingContext;
    }

    /**
     **  Convert a string path to a CosNaming::Name.
     **
     **  @param path The string path.
     **
     **  @return The associated CosNaming::Name.
     **/
    public static org.omg.CosNaming.NameComponent[]
    toName(String path)
    {
	// TODO: Must be completed to support path and not just name.

        org.omg.CosNaming.NameComponent[] result =
	    new org.omg.CosNaming.NameComponent[1];
	result[0] = new org.omg.CosNaming.NameComponent(path, "");
	return result;
    }

    /**
     **  Rebind a path name to an object reference.
     **
     **  @param path The path name.
     **  @param objRef The object reference.
     **/
    public void
    rebind(String path,
	   org.omg.CORBA.Object objRef)
    {
	try
        {
	    namingContext_.rebind(toName(path), objRef);
	}
	catch(org.omg.CosNaming.NamingContextPackage.InvalidName exc)
	{
	    // Wrap the CosNaming::NamingContext::InvalidName exception.
	    throw new UserExceptionWrapper(exc);
	}
	catch(org.omg.CosNaming.NamingContextPackage.NotFound exc)
	{
	    // Wrap the CosNaming::NamingContext::NotFound exception.
	    throw new UserExceptionWrapper(exc);
	}
	catch(org.omg.CosNaming.NamingContextPackage.CannotProceed exc)
	{
	    // Wrap the CosNaming::NamingContext::CannotProceed exception.
	    throw new UserExceptionWrapper(exc);
	}
    }

    /**
     **  Unbind a path name.
     **
     **  @param path The path name.
     **/
    public void
    unbind(String path)
    {
	try
        {
	    namingContext_.unbind(toName(path));
	}
	catch(org.omg.CosNaming.NamingContextPackage.InvalidName exc)
	{
	    // Wrap the CosNaming::NamingContext::InvalidName exception.
	    throw new UserExceptionWrapper(exc);
	}
	catch(org.omg.CosNaming.NamingContextPackage.NotFound exc)
	{
	    // Wrap the CosNaming::NamingContext::NotFound exception.
	    throw new UserExceptionWrapper(exc);
	}
	catch(org.omg.CosNaming.NamingContextPackage.CannotProceed exc)
	{
	    // Wrap the CosNaming::NamingContext::CannotProceed exception.
	    throw new UserExceptionWrapper(exc);
	}
    }
}
