/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class encapsulates the DynamicAny::DynAnyFactory singleton object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public abstract class TheDynamicAnyFactory
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To store the DynamicAny::DynAnyFactory singleton object.
     **/
    private static org.omg.DynamicAny.DynAnyFactory factory_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    TheDynamicAnyFactory()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain the DynamicAny::DynAnyFactory singleton object.
     **
     **  @return The DynamicAny::DynAnyFactory singleton object.
     **/
    public static org.omg.DynamicAny.DynAnyFactory
    getFactory()
    {
        if(factory_ == null)
	{
           factory_ = org.omg.DynamicAny.DynAnyFactoryHelper.narrow(
               TheORB.resolve_initial_reference("DynAnyFactory"));
        }
        return factory_;
    }

    /**
     **  Set the DynamicAny::DynAnyFactory singleton object.
     **
     **  @param factory The DynamicAny::DynAnyFactory singleton object.
     **/
    public static void
    setFactory(org.omg.DynamicAny.DynAnyFactory factory)
    {
        factory_ = factory;
    }

    /**
     **  Create a dynany form a CORBA::TypeCode.
     **
     **  @param tc The CORBA::TypeCode.
     **
     **  @return The created dyn any.
     **/
    public static org.omg.DynamicAny.DynAny
    create_dyn_any_from_type_code(org.omg.CORBA.TypeCode tc)
    {
	try
 	{
            return getFactory().create_dyn_any_from_type_code(tc);
	}
        catch(org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode exc)
        {
	    // Wrap the DynamicAny::DynAnyFactory::InconsistentTypeCode exception.
            throw new UserExceptionWrapper(exc);
	}
    }

    /**
     **  Create a DynamicAny::DynEnum from a CORBA::Any.
     **
     **  @param any The CORBA::Any source.
     **
     **  @return The created DynamicAny::DynEnum.
     **/
    public static org.omg.DynamicAny.DynEnum
    create_dyn_enum(org.omg.CORBA.Any any)
    {
        try
        {
            return org.omg.DynamicAny.DynEnumHelper.narrow(
                       getFactory().create_dyn_any(any));
        }
        catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode exc)
        {
	    // Wrap the DynamicAny::DynAnyFactory::InconsistentTypeCode exception.
            throw new UserExceptionWrapper(exc);
        }
    }

    /**
     **  Create a DynamicAny::DynFixed from a CORBA::Any.
     **
     **  @param any The CORBA::Any source.
     **
     **  @return The created DynamicAny::DynFixed.
     **/
    public static org.omg.DynamicAny.DynFixed
    create_dyn_fixed(org.omg.CORBA.Any any)
    {
        try
        {
            return org.omg.DynamicAny.DynFixedHelper.narrow(
                       getFactory().create_dyn_any(any));
        }
        catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode exc)
        {
	    // Wrap the DynamicAny::DynAnyFactory::InconsistentTypeCode exception.
            throw new UserExceptionWrapper(exc);
        }
    }
}
