/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class provides CORBA::TypeCode util functions.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public abstract class TypeCodeUtils
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The singleton CORBA::TypeCode for the OMG IDL void type.
     **/
    static private org.omg.CORBA.TypeCode tc_void_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    TypeCodeUtils()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain the CORBA::TypeCode for the OMG IDL void type.
     **
     **  @return The CORBA::TypeCode for the OMG IDL void type.
     **/
    public static org.omg.CORBA.TypeCode
    getTC_void()
    {
        if(tc_void_ == null)
	{
            tc_void_ = TheORB.getORB().get_primitive_tc(
                           org.omg.CORBA.TCKind.tk_void);
        }
	return tc_void_;
    }

    /**
     **  Obtain the orignal CORBA::TypeCode.
     **
     **  @param tc The aliased CORBA::TypeCode.
     **
     **  @return The unaliased CORBA::TypeCode.
     **/
    public static org.omg.CORBA.TypeCode
    getOrigType(org.omg.CORBA.TypeCode tc)
    {
        try
	{
	    while(tc.kind() == org.omg.CORBA.TCKind.tk_alias)
	    {
		tc = tc.content_type();
	    }
	    return tc;
	}
	catch(org.omg.CORBA.TypeCodePackage.BadKind exc)
	{
	    // Wrap the CORBA::TypeCode::BadKind exception.
            throw new UserExceptionWrapper(exc);
	}
    }
}
