/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This class is a basic implementation of the CCMHome interface.
 * It also implements all generic operations for CCM homes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class CCMHomeImpl
extends CCMHomeBase
implements org.omg.Components.KeylessCCMHomeOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    protected
    CCMHomeImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected final java.lang.String
    _component_uid_()
    {
        return _the_home_executor()._the_component_uid();
    }

    /**
     **
     **/
    protected final java.lang.String
    _home_uid_()
    {
        return _the_home_executor()._the_home_uid();
    }

    /**
     **
     **/
    protected final org.objectweb.ccm.Containers.HomeServant
    _home_servant_()
    {
        return _the_home_executor()._home_servant();
    }

    /**
     **  Create a CCM component reference.
     **
     **  @return The CCM component reference.
     **/
    protected final org.omg.Components.CCMObject
    _create()
    throws org.omg.Components.CreateFailure
    {
        org.omg.Components.CCMObject ref = null;

        // create the reference
        ref = _home_servant_().the_PCA().create_component_reference(_home_servant_());
				listener_.on_create(ref);

        return ref;
    }   

    // ==================================================================
    //
    // Internal methods that must be implemented by subclasses.
    //
    // ==================================================================

    /**
     **  To obtain the home executor.
     **/
    protected abstract org.objectweb.ccm.Containers.HomeExecutor
    _the_home_executor();

    // ==================================================================
    //
    // Methods for the Components::CCMHome interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/CCMHome/remove_component:1.0
    //
    /**
     **  Removes a component.
     **
     **  @param comp The reference to the component.
     **/
    public final void
    remove_component(org.omg.Components.CCMObject comp)
    throws org.omg.Components.RemoveFailure
    {
        // TO DO
        // MUST BE COMPLETED FOR PK HOMES !!!!
        // i.e. Remove the associated component from the list

        listener_.on_remove(comp);

      /*
        //
        //  TODO : other things such as destroying the reference, deactivating the component,...
        //
	      // TODO: Must be implemented!
	         throw new org.omg.Components.RemoveFailure();
      */
    }

    // ==================================================================
    //
    // Methods for the Components::KeylessCCMHome interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/KeylessCCMHome/create_component:1.0
    //
    /**
     **  Creates a keyless component.
     **
     **  @return The reference to the created component.
     **
     **/
    public final org.omg.Components.CCMObject
    create_component()
    throws org.omg.Components.CreateFailure
    {
        return _create();
    }
}


