/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This class is a basic implementation of the CCMHome interface.
 * It also implements all generic operations for CCM homes..
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class CCMHomeWithPKImpl
                extends CCMHomeBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    protected
    CCMHomeWithPKImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected final java.lang.String
    _component_uid_()
    {
        return _the_home_executor()._the_component_uid();
    }

    /**
     **
     **/
    protected final java.lang.String
    _home_uid_()
    {
        return _the_home_executor()._the_home_uid();
    }

    /**
     **
     **/
    protected final org.objectweb.ccm.Containers.HomeServant
    _home_servant_()
    {
        return _the_home_executor()._home_servant();
    }

    /**
     **  Create a CCM component reference with a primary key.
     **
     **  @param key The associated primary key.
     **
     **  @return The CCM component reference.
     **
     **  @exception org.omg.Components.DuplicateKeyValue
     **             The primary key is already used.
     **  @exception org.omg.Components.InvalidKey
     **             The primary key is invalid.
     **/
    protected final org.omg.Components.CCMObject
    _create(org.omg.Components.PrimaryKeyBase key)
    throws org.omg.Components.DuplicateKeyValue,
           org.omg.Components.InvalidKey,
           org.omg.Components.CreateFailure
    {
        org.omg.Components.CCMObject ref = null;

        // create the reference
        ref = _home_servant_().the_PCA().create_registered_component_reference(_home_servant_(), key);
				listener_.on_create(ref);
        return ref;
    }

    /**
     **
     **/
    protected final org.omg.Components.CCMObject
    _find_by_primary_key(org.omg.Components.PrimaryKeyBase key)
    throws org.omg.Components.UnknownKeyValue,
           org.omg.Components.InvalidKey,
           org.omg.Components.FinderFailure
    {
        //
        //  TODO : check for the others exceptions
        //  TODO : retrieve the reference to the returned ::Components::EnterpriseComponent
        //
        throw new org.omg.Components.FinderFailure();
/*
        try
        {
            return _the_home_executor()._find_by_primary_key_(key);
        }
        catch(org.omg.Components.CCMException ex)
        {
            throw new org.omg.Components.FinderFailure();
        }
 */
    }

    /**
     **
     **/
    protected final void
    _remove(org.omg.Components.PrimaryKeyBase key)
    throws org.omg.Components.UnknownKeyValue,
           org.omg.Components.InvalidKey,
           org.omg.Components.RemoveFailure
    {
        //
        //  TODO : check for the others exceptions
        //  TODO : other things such as destroying the reference, deactivating the component,...
        //
        throw new org.omg.Components.RemoveFailure();
/*
        try
        {
            _the_home_executor()._remove_(key);
        }
        catch(org.omg.Components.CCMException ex)
        {
            throw new org.omg.Components.RemoveFailure();
        }
 */
    }

    /**
     **
     **/
    protected final org.omg.Components.PrimaryKeyBase
    _get_primary_key(org.omg.Components.CCMObject comp)
    {
        try
        {
            return comp.get_primary_key();
        }
        catch(org.omg.Components.NoKeyAvailable ex)
        {
            // should never happen.
        }
        return null;
    }

    // ==================================================================
    //
    // Methods for the Components::CCMHome interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/CCMHome/remove_component:1.0
    //
    /**
     **  Removes a component.
     **
     **  @param comp The reference to the component.
     **/
    public final void
    remove_component(org.omg.Components.CCMObject comp)
    throws org.omg.Components.RemoveFailure
    {
				listener_.on_remove(comp);
        try
        {
            _remove(_get_primary_key(comp));
        }
        catch(org.omg.Components.UnknownKeyValue ex)
        {
            // should not happen.
        }
        catch(org.omg.Components.InvalidKey ex)
        {
            // should not happen.
        }
    }

    // ==================================================================
    //
    // Internal methods that must be implemented by subclasses.
    //
    // ==================================================================

    /**
     **  To obtain the home executor.
     **/
    protected abstract org.objectweb.ccm.Containers.HomeExecutorWithPK
    _the_home_executor();
}






