/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This class implements the Components::Cookie valuetype.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class CookieImpl
       extends org.omg.Components.Cookie
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     ** TODO
     **/
    private String random_ = "RANDOM NOT COMPUTED";

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    CookieImpl()
    {
    }

    /**
     **  The constructor.
     **
     **  @param random The generated random.
     **/
    public
    CookieImpl(String random)
    {
        random_ = random;
	// Set the cookieValue.
        cookieValue = random.getBytes();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Init after read from CORBA stream.
     **/
    protected void
    initAfterRead()
    {
	random_ = new String(cookieValue);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Generate a cookie.
     **
     **  @return The generated cookie.
     **/
    public static CookieImpl
    generate()
    {
	// Compute a random.
        String random = java.lang.Double.toString(java.lang.Math.random());

	// Create a cookie.
        CookieImpl result = new CookieImpl(random);

	// Return the generated cookie.
        return result;
    }

    // ==================================================================
    //
    // Methods for the java.lang.Object class.
    //
    // ==================================================================
 
    /**
     **  Obtain a string representation.
     **
     **  @return A string representation.
     **/
    public String
    toString()
    {
        return random_;
    }    

    /**
     **  Compute a hash code.
     **
     **  @return The hash code for the random.
     **/
    public int 
    hashCode()
    {
        return random_.hashCode();
    }

    /**
     **  Test equality with another object.
     **/
    public boolean 
    equals(Object o)
    {
	// Compare random attributes.
        return random_.equals(((CookieImpl)o).random_);
    }
}
