/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): _______________________________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * Base class for all generated wrapper classes
 * implementing the Monolithic Executor Strategy.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class MonolithicWrapperBase
              extends org.objectweb.corba.util.LocalObjectBase
           implements org.omg.Components.ExecutorLocator
{
    // ==================================================================
    //
    // Internal State.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    MonolithicWrapperBase()
    {
        // Nothing to do.
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::EnterpriseComponent
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/EnterpriseComponent/configuration_complete:1.0
    //
    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        getLogger().error(this,
                          "This method should never be called by containers");

        throw new org.omg.Components.InvalidConfiguration();
    }

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::ExecutorLocator
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
    //
    /**
     * Obtain an executor.
     *
     * @param name The executor name.
     *
     * @return The executor reference.
     *
     * @exception org.omg.Components.CCMException Thrown if any error.
     */
    public org.omg.CORBA.Object
    obtain_executor(java.lang.String name)
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this, "Obtaining executor " + name);

        getLogger().error(this,
                          "This method must be redefined by subclasses");

        throw new org.omg.Components.CCMException();
    }

    //
    //  IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
    //
    /**
     * Release an executor.
     *
     * @param executor The executor reference.
     *
     * @exception org.omg.Components.CCMException Thrown if any error.
     */
    public void
    release_executor(org.omg.CORBA.Object executor)
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this, "Releasing executor " + executor);

        getLogger().error(this, "This method must be redefined by subclasses");

        throw new org.omg.Components.CCMException();
    }
}
