/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This global class provides utility functions for the OpenCCM
 * Abstract Model.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class Runtime
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To avoid to init the runtime multiple times.
     **/
    protected static boolean already_initialized_ = false;

    /**
     **  The HomeManager (Finder and Register) reference.
     **/
    protected static HomeManager home_manager_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    Runtime()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Register the required Components valuetype factories.
     **
     **/
    protected static void
    register_valuetype_factories()
    {
        register_value_factory(org.omg.Components.ComponentPortDescriptionHelper.id(),
                               new ComponentPortDescriptionFactory());

        register_value_factory(org.omg.Components.ConfigValueHelper.id(),
                               new ConfigValueFactory());

        register_value_factory(org.omg.Components.ConnectionDescriptionHelper.id(),
                               new ConnectionDescriptionFactory());

        register_value_factory(org.omg.Components.ConsumerDescriptionHelper.id(),
                               new ConsumerDescriptionFactory());

        register_value_factory(org.omg.Components.CookieHelper.id(),
                               new CookieFactory());

        register_value_factory(org.omg.Components.EmitterDescriptionHelper.id(),
                               new EmitterDescriptionFactory());

        register_value_factory(org.omg.Components.FacetDescriptionHelper.id(),
                               new FacetDescriptionFactory());

        register_value_factory(org.omg.Components.FacetDescriptionHelper.id(),
                               new FacetDescriptionFactory());

        register_value_factory(org.omg.Components.PublisherDescriptionHelper.id(),
                               new PublisherDescriptionFactory());

        register_value_factory(org.omg.Components.ReceptacleDescriptionHelper.id(),
                               new ReceptacleDescriptionFactory());

        register_value_factory(org.omg.Components.SubscriberDescriptionHelper.id(),
                               new SubscriberDescriptionFactory());
    }

    /**
     **  Create the OpenCCM::HomeManager.
     **/
    protected static void
    create_home_manager()
    {
	// TODO: Must be reviewed to be configurable!!!
	//       e.g. Local or Remote or Trader or ...
        HomeManagerImpl manager = new HomeManagerImpl();

	// TODO: Must be changed!!!
        org.omg.CORBA.Object ref = manager._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
	HomeManager managerRef = HomeManagerHelper.narrow(ref);

	// Set the HomeManager singleton reference.
	home_manager_ = managerRef;

	// Register the ComponentHomeFinder initial service.
	org.objectweb.ccm.CORBA.TheORB.register_initial_reference(
            "ComponentHomeFinder", managerRef);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Initialize the OpenCCM Abstract Model runtime, i.e.
     **  register valuetype factories,
     **  create the OpenCCM::HomeManager (HomeFinder and HomeRegister).
     **/
    public static void
    init()
    {
	if(already_initialized_ == false)
	{
	    // Register valuetype factories.
	    register_valuetype_factories();

	    // Create the HomeManager.
	    create_home_manager();

	    already_initialized_ = true;
	}
    }

    /**
     **  Initialize the OpenCCM Abstract Model runtime, i.e.
     **  register valuetype factories,
     **  create the Components::HomeFinder and HomeRegister.
     **
     **  @param orb The ORB reference.
     **/
    public static void
    init(org.omg.CORBA.ORB orb)
    {
	// Set the ORB singleton reference.
	org.objectweb.ccm.CORBA.TheORB.setORB(orb);

	// Init the runtime.
	init();
    }

    /**
     **  Initialize the OpenCCM Abstract Model runtime, i.e.
     **  initialize the ORB,
     **  register valuetype factories,
     **  create the Components::HomeFinder and HomeRegister.
     **
     **  @param args The command line arguments.
     **
     **  @return The ORB reference.
     **/
    public static org.omg.CORBA.ORB
    init(String[] args)
    {
	// Init the ORB.
	org.objectweb.ccm.CORBA.TheORB.initialize(args);

	// Init the runtime.
	init();

	// return the ORB.
	return org.objectweb.ccm.CORBA.TheORB.getORB();
    }

    /**
     **  Register a valuetype factory.
     **
     **  @param id The valuetype repository Id.
     **  @param factory The valuetype factory.
     **/ 
    public static void
    register_value_factory(String id,
                           org.omg.CORBA.portable.ValueFactory factory)
    {
        org.objectweb.ccm.CORBA.TheORB.register_value_factory(id, factory);
    }

    /**
     **
     **/
    public static HomeManager
    get_home_manager()
    {
        return home_manager_;
    }
}
