/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc__________________________________.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * This interface allows the OpenCCM ComponentServer to deal with
 * instantiated homes.
 *
 * TODO: This interface must be stabilized as soon as possible!
 *       Then it can change during next releases.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface HomeExecutorBase
         extends org.omg.Components.HomeExecutorBase
{
    /**
     **  To know if the home has a primary key.
     **/
    public boolean
    _has_primary_key();

    /**
     **  To obtain the home ::CORBA::RepositoryId.
     **/
    public java.lang.String
    _the_home_uid();

    /**
     **  To obtain the component ::CORBA::RepositoryId.
     **/
    public java.lang.String
    _the_component_uid();

    /**
     **  To obtain the component ports ::CORBA::RepositoryId.
     **/
    public java.lang.String[]
    _the_port_uids();

    /**
     **  To obtain the ::CORBA::RepositoryId of facet or receptacle interface type or a
     **  event sink or source event type.
     **/
    public java.lang.String
    _get_port_type_uid(java.lang.String port_uid);

    /**
     **  To obtain a new interceptor instance for a specific port, the component or the home.
     **/
    public Interceptor
    _get_interceptor(java.lang.String uid);

    /**
     **  To obtain a new instance of the component executor.
     **/
    public ComponentExecutor
    _get_component_executor();

    /**
     **  To obtain a new instance of the executor locator wrapper for a
     **  monolithic executor instance.
     **/
    public org.omg.Components.ExecutorLocator
    _get_executor_locator(org.omg.Components.EnterpriseComponent comp);

    /**
     **  To obtain a new instance of a executor wrapper for a port executor instance.
     **  For the moment, only event consumer executors have a wrapper.
     **/
    public java.lang.Object
    _get_executor_wrapper(org.omg.CORBA.Object exe,
                          java.lang.String port_uid);

    /**
     **  To obtain the home servant.
     **/
    public HomeServant
    _home_servant();

    /**
     **  To set the home servant.
     **/
    public void
    _home_servant(HomeServant home);

    /**
     **  To set the delegate object.
     **/
    public void
    _delegate(org.omg.Components.HomeExecutorBase delegate);

    /**
     **  To obtain the delegate object.
     **/
    public org.omg.Components.HomeExecutorBase
    _delegate();
}
