/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * The RootPCA is the component equivalent of the CORBA RootPOA.
 * It's used to activate homes.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class RootPCAImpl
       extends PCAImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected SystemHomeFinder home_finder_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    RootPCAImpl(SystemHomeFinder home_finder)
    {
        super("RootPCA", null);
        home_finder_ = home_finder;
        init();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    init()
    {
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[7];
        org.omg.PortableServer.POA rootPOA = org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA();

        // create CORBA policies for the RootPCA
        policies[0] = rootPOA.create_thread_policy(org.omg.PortableServer.ThreadPolicyValue.ORB_CTRL_MODEL);
        policies[1] = rootPOA.create_lifespan_policy(org.omg.PortableServer.LifespanPolicyValue.TRANSIENT);
        policies[2] = rootPOA.create_id_uniqueness_policy(org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
        policies[3] = rootPOA.create_id_assignment_policy(org.omg.PortableServer.IdAssignmentPolicyValue.SYSTEM_ID);
        policies[4] = rootPOA.create_implicit_activation_policy(
                          org.omg.PortableServer.ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
        policies[5] = rootPOA.create_servant_retention_policy(org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);
        policies[6] = rootPOA.create_request_processing_policy(
                          org.omg.PortableServer.RequestProcessingPolicyValue.USE_ACTIVE_OBJECT_MAP_ONLY);

        try
        {
            // create the RootPCA ::PortableServer::POA and activate it
            String name = java.lang.Long.toString(java.lang.System.currentTimeMillis()); 
            the_poa_ = rootPOA.create_POA(name, null, policies);
            the_poa_.the_POAManager().activate();
        }
        catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
        {
            // should never happen !
            throw new Error("Problem with CORBA policies !!!");
        }
        catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
        {
            // should never happen !
            throw new Error("RootPCA already exists !!!");
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
        {
            // should never happen !
            throw new Error("RootPCA already exists !!!");
        }
    }

    // ==================================================================
    //
    // Public methods for the PCA interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/activate_home:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMHome
    activate_home(HomeServant servant)
    {
        org.omg.Components.CCMHome ref = null;
        try
        {
            PCA comp_pca = null;
            HomeExecutorBase home = servant.the_home_executor();

            // compute the home name
            java.lang.String home_name = home._the_home_uid().replace('/', 'x');

            // create the component dedicated PCA with the configured policies.
            PCAPolicy[] policies = null;

            if(servant.the_component_configuration() == null) {
                policies = new PCAPolicy[0];
            } else {
                policies = servant.the_component_configuration().create_PCAPolicies(
                                     home._the_component_uid(),
                                     home_finder_);
            }
            comp_pca = create_PCA(home_name, policies);

            // set the component dedicated PCA
            servant.the_PCA(comp_pca);

            // create the interceptor
            java.lang.String home_uid = home._the_home_uid();
            Interceptor inter = home._get_interceptor(home_uid);
            
            // create the call context for the interceptor and set the needed infos
            CallContext context = new HomeCallContext(servant, home_uid);
            inter._call_context(context);
            inter._delegate((java.lang.Object)home);

            // create the reference.
            byte[] oid = the_poa_.activate_object((org.omg.PortableServer.Servant)inter);
            ref = org.omg.Components.CCMHomeHelper.narrow(the_poa_.id_to_reference(oid));
            servant.the_home_ref(ref);

            // registers the home in the home manager.
/*
            org.omg.Components.Cookie cookie =
                org.objectweb.ccm.Components.Runtime.get_home_manager().register_home(
                    ref, home._the_home_uid(), home._the_component_uid(), home_name);
            servant.the_registration_info(cookie);
 */
        }
        catch(CreationFailed ex)
        {
            // should never happen
            throw new java.lang.Error("Error while activating home");
        }
        catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
        {
            // should never happen
            throw new java.lang.Error("Error while activating home");
        }
        catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
        {
            // should never happen
            throw new java.lang.Error("Error while activating home");
        }
        catch(org.omg.PortableServer.POAPackage.ObjectNotActive ex)
        {
            // should never happen
            throw new java.lang.Error("Error while activating home");
        }

        return ref;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/deactivate_home:1.0
    //
    /**
     **
     **/
    public void
    deactivate_home(HomeServant servant)
    {
        try {
            HomeExecutorBase home = servant.the_home_executor();

            // get the interceptor
            java.lang.String home_uid = home._the_home_uid();
            Interceptor inter = home._get_interceptor(home_uid);

            // Free the ref
            the_poa_.deactivate_object(((org.omg.PortableServer.Servant)inter)._object_id()); 
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy ex) {
            // should never happen
            throw new java.lang.Error("Error while deactivating home");
        } catch (org.omg.PortableServer.POAPackage.ObjectNotActive ex) {
            // should never happen
            throw new java.lang.Error("Error while deactivating home");
        }
    }


}
