/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Philippe Merle, Romain Rouvoy, Sylvain Leblanc.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * Customized implementation of a ServantLocator to manage component servants.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ServantLocatorImpl
       extends org.objectweb.ccm.CORBA.ServantLocatorBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected org.omg.PortableServer.POA poa_;

    /**
     **
     **/
    protected java.util.Random random_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.WeakTable oid2cs_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.WeakTable oid2uid_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    ServantLocatorImpl()
    {
        poa_ = null;
        oid2cs_ = new org.objectweb.ccm.util.WeakTable();
        oid2uid_ = new org.objectweb.ccm.util.WeakTable();
        random_ = new java.util.Random(java.lang.System.currentTimeMillis());
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected org.omg.CORBA.Object
    createReference(HomeServant home_servant,
                    org.omg.Components.EnterpriseComponent instance,
                    org.omg.Components.PrimaryKeyBase key)
    {
        HomeExecutorBase home = home_servant.the_home_executor();
        java.lang.String comp_uid = home._the_component_uid();
        java.lang.String oid = null;
        org.omg.CORBA.Object result = null;

        //
        //  TODO : is 128 bits enough for an oid ????
        //
        java.math.BigInteger big = new java.math.BigInteger(128, random_);

        // create reference for the component base facet.
        oid = new java.lang.String(big.toByteArray());

	try
        {
           result = poa_.create_reference_with_id(big.toByteArray(), comp_uid);

	   // For org.omg.PortableServer.POAPackage.WrongPolicy on VisiBroker 5
	} catch(Exception ex) { }

        ComponentServantImpl comp_servant = new ComponentServantImpl(this, home_servant, instance, key);

        oid2uid_.put(oid, comp_uid);
        oid2cs_.put(oid, comp_servant);

        // must be done after the hashtable update
        // because the narrow implies a preinvoke call to check _is_a
        // one consequence is that the component will be activated but we don't want that
        // so early (before a real business call)
        //
        // TODO : think about another solution to avoid this problem
        //        such as trapping _is_a call in preinvoke
        //
        comp_servant.setComponentRef(org.omg.Components.CCMObjectHelper.narrow(result));

        return result;
    }

    // ==================================================================
    //
    // Public methods for the ServantLocatorOperations interface.
    //
    // ==================================================================

    //
    //  IDL:omg.org/PortableServer/ServantLocator/preinvoke:1.0
    //
    /**
     **
     **/
    public org.omg.PortableServer.Servant
    preinvoke(byte[] oid,
              org.omg.PortableServer.POA adapter,
              java.lang.String operation,
              org.omg.PortableServer.ServantLocatorPackage.CookieHolder the_cookie)
    throws org.omg.PortableServer.ForwardRequest
    {
        java.lang.String soid = new java.lang.String(oid);
        java.lang.String uid = (java.lang.String)oid2uid_.get(soid);
        ComponentServant servant = (ComponentServant)oid2cs_.get(soid);

        if (servant == null) return null;

        if (servant.activation_context().state()==ActivationContext.DEACTIVATED)
            servant.activation_context().state(ActivationContext.ACTIVATED);

        return (org.omg.PortableServer.Servant)servant.get_native_interface(uid);
    }

    //
    //  IDL:omg.org/PortableServer/ServantLocator/postinvoke:1.0
    //
    /**
     **
     **/
    public void
    postinvoke(byte[] oid,
              org.omg.PortableServer.POA adapter,
              java.lang.String operation,
              java.lang.Object the_cookie,
              org.omg.PortableServer.Servant the_servant)
    {
            java.lang.String soid = new java.lang.String(oid);
            java.lang.String uid = (java.lang.String)oid2uid_.get(soid);
            ComponentServant servant = (ComponentServant)oid2cs_.get(soid);
            
        if (servant != null) {

            servant.release_native_interface((Interceptor)the_servant);
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setPOA(org.omg.PortableServer.POA poa)
    {
        poa_ = poa;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    createPortReference(ComponentServant servant,
                        java.lang.String port_uid)
    {
        HomeExecutorBase home = servant.the_home_servant().the_home_executor();
        org.omg.CORBA.Object result = null;

        //
        //  TODO : is 128 bits enough for an oid ????
        //
        java.math.BigInteger big = new java.math.BigInteger(128, random_);
        java.lang.String oid = "";

        // create reference for the port.
        big = new java.math.BigInteger(128, random_);
        oid = new java.lang.String(big.toByteArray());
        oid2uid_.put(oid, port_uid);
        oid2cs_.put(oid, servant);

	try
        {
          result = poa_.create_reference_with_id(big.toByteArray(), home._get_port_type_uid(port_uid));

	   // For org.omg.PortableServer.POAPackage.WrongPolicy on VisiBroker 5
	} catch(Exception ex) { }

        return result;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    createReference(HomeServant home_servant)
    {
        return createReference(home_servant, null, null);
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    createReference(HomeServant home_servant,
                    org.omg.Components.EnterpriseComponent instance)
    {
        return createReference(home_servant, instance, null);
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    findReference(HomeServant home_servant,
                  org.omg.Components.EnterpriseComponent instance)
    {
        ComponentServantImpl[] cs = null;
        cs = (ComponentServantImpl[])oid2cs_.values().toArray(new ComponentServantImpl[0]);
        org.omg.CORBA.Object result = null;
        for (int i=0;i<cs.length;i++)
        {
            result = cs[i].getReferenceWithInstance(instance);
            if (result!=null)
                break;
        }

        if (result==null)
            result = createReference(home_servant, instance);

        return result;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    createReference(HomeServant home_servant,
                    org.omg.Components.PrimaryKeyBase key)
    {
        return createReference(home_servant, null, key);
    }

    /**
     ** Remove the reference associated with the CORBA Object.
     **/
    public void 
    removeReference(org.omg.CORBA.Object obj)
        throws org.omg.Components.RemoveFailure
    {
        try {
            String oid = new String(poa_.reference_to_id(obj));
            oid2uid_.remove(oid);
            oid2cs_.remove(oid);
        } catch (org.omg.PortableServer.POAPackage.WrongAdapter ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy ex) {
            throw new org.omg.Components.RemoveFailure();
        }
    }
}
