/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Raphael Marvie, Philippe Merle.
Contributor(s): Mathieu Vadet______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the OMG IDL
 * Components::Deployment::ComponentInstallation interface.
 *
 * @author <a href="mailto:Raphael.Marvie@lifl.fr">Raphael Marvie</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentInstallationImpl 
       extends org.omg.Components.Deployment.ComponentInstallationPOA
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To store the loaded archives.
     **/
    protected org.objectweb.ccm.util.Table implementations_;

    /**
     **  The path where the archives are stored.
     **/
    protected java.io.File path_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param path The path where the archives are stored.
     **  @param classLoader The class loader used to load home classes.
     **/
    public
    ComponentInstallationImpl(String path)
    {
				implementations_ = new org.objectweb.ccm.util.Table();
				path_ = new java.io.File(path);
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the Components::Deployment::ComponentInstallation interface.
    //
    // ==================================================================
 
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/install:1.0
    //
    /**
     **  Install a component archive.
     **
     **  @param implUUID The implementation identifier.
     **  @param component_loc The location of the component archive.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.InvalidLocation
     **             Thrown if the location is invalid.
     **/
    public void
    install(String implUUID,
            String component_loc)
		throws org.omg.Components.Deployment.InvalidLocation,
				   org.omg.Components.Deployment.InstallationFailure
    {
				// TODO: Must be fully rewritten!!!!
				
				//
				if(implementations_.contains(implUUID))
						return ;
				
				java.io.InputStream  bis = null;
				java.io.OutputStream bos = null;
				java.net.URL url = null;
				
				// extraire le nom du fichier ou de l'archive pour le nom du fichier
				// de sortie.
				
				
				String filename = (new java.io.File (component_loc)).getName ();
				java.io.File output = new java.io.File (path_, filename);
				try
				{
						bos = new java.io.FileOutputStream (output);
				}
				catch (java.io.FileNotFoundException e)
				{
						throw new org.omg.Components.Deployment.InvalidLocation();
				}
				catch (java.io.IOException e1)
				{
            //
            //  TODO : which value to set ??
            //
						throw new org.omg.Components.Deployment.InstallationFailure();
				}

				try 
				{
            if (!component_loc.startsWith("http"))
						{
                bis = new java.io.FileInputStream(component_loc);
            }
            else
						{
      	        url = new java.net.URL (component_loc);
                bis = (java.io.InputStream) url.getContent();
            }
				} 
        catch (java.net.MalformedURLException e)
        {
						throw new org.omg.Components.Deployment.InvalidLocation ();
				}
        catch (java.io.FileNotFoundException e)
				{
            throw new org.omg.Components.Deployment.InvalidLocation();
        }
        catch (java.io.IOException e)
        {
            //
            //  TODO : which value to set ??
            //
						throw new org.omg.Components.Deployment.InstallationFailure();
				}
				
				// fetching the file
				try 
        {
            int length;
            byte b[] = new byte[2048];
            while (true)
						{
                length = bis.read(b, 0, 2048);
                if (length == -1)
                    break;
                bos.write(b, 0, length);
            }
						bis.close();
						bos.close();
				}
				catch (java.io.IOException e)
				{
            //
            //  TODO : which value to set ??
            //
						throw new org.omg.Components.Deployment.InstallationFailure();
				}

				implementations_.put(implUUID, filename);
				try
				{
						TheURLClassLoader.getURLClassLoader().addURL(output.toURL());
				}
				catch (java.net.MalformedURLException e)
				{
            throw new org.omg.Components.Deployment.InstallationFailure();
				}
    }
 
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/replace:1.0
    //
    /**
     **  Replace a component archive.
     **
     **  @param implUUID The implementation identifier.
     **  @param component_loc The location of the component archive.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.InvalidLocation
     **             Thrown if the location is invalid.
     **/
    public void
    replace(String implUUID,
            String component_loc)
    throws org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.Deployment.InstallationFailure
    {	
        install(implUUID, component_loc);
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/remove:1.0
    //
    /**
     **  Remove a component archive.
     **
     **  @param implUUID The implementation identifier.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.UnknownImplId
     **             Thrown if the implementation identifier is unknown.
     **/
    public void
    remove(String implUUID)
		throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.RemoveFailure
    {
        //
        Object o = implementations_.remove(implUUID);
				
				//
				if(o == null)
				{
						throw new org.omg.Components.Deployment.UnknownImplId();
				}

				// TODO: remove the file from the disk
    }
		
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/get_implementation:1.0
    //
    /**
     **  Get the implementation location on the local disk.
     **
     **  @param implUUID The implementation identifier.
     **
     **  @return The location of the implementation.
     **
     **  @exception org.omg.Components.Deployment.UnknownImplId
     **             Thrown if the implementation identifier is unknown.
     **/
    public String
    get_implementation(String implUUID)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.Deployment.InstallationFailure
    {
        //
				String impl = (String)implementations_.get(implUUID);
				
				//
				if(impl == null)
				{
						throw new org.omg.Components.Deployment.UnknownImplId();
				}

				//
				return impl;
    }
}



