/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the OMG IDL
 * ::Components::Deployment::ComponentServer interface.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */

public class ComponentServerBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector containers_;

    /**
     **
     **/
    protected org.omg.Components.ConfigValue[] configuration_;

    /**
     **
     **/
    protected ServerImpl the_server_;

    /**
     ** Listener for additionnal plugins
     **/
    protected org.objectweb.ccm.Deployment.listener.ComponentServerListener listener_; 
    

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param classLoader The class loader used to load home classes.
     **/
    public
    ComponentServerBase(ServerImpl server)
    {
        the_server_ = server;
        containers_ = new org.objectweb.ccm.util.Vector();
        configuration_ = new org.omg.Components.ConfigValue[0];
				listener_  = null;	  
	  
				org.objectweb.ccm.Deployment.listener.TheComponentServerConfigurator.get_instance().configure_component_server(new ComponentServerLocalImpl(this));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
		/** 
		 ** Add a new listener for the deployment
		 **
		 ** @param listener the listener to add 
		 **/
    public void
		add_listener (org.objectweb.ccm.Deployment.listener.ComponentServerListener listener)
    {
				if (listener_ == null)
						listener_ = listener;
				else
						listener_.set_delegate(listener);
    }
    /**
     **
     **/
    public org.omg.Components.Deployment.ComponentServer
    asComponentServer()
    {
        return the_server_.provide_component_server();
    }

    /**
     ** Register the reference into the internal list
		 **
		 ** @param cont_ref the reference to the container
		 **
     **/
    public void
		add_container (org.omg.Components.Deployment.Container cont_ref)
    {	
        containers_.add(cont_ref);
				listener_.on_add (cont_ref); 
    }
    
    // ==================================================================
    //
    // Methods for the ::Components::Deployment::ComponentServer interface.
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
    //
    /**
     **  To obtain the component server configuration.
     **
     **/
    public org.omg.Components.ConfigValue[]
    configuration()
    {
        return configuration_;
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
    //
    /**
     **  To set the component server configuration.
     **  Note : configuration is a readonly attribute, but this allow us to
     **  set the configuration later in the instanciation process
     **  which can be useful as we don't have server activator yet.
     **
     **/
    public void
    configuration(org.omg.Components.ConfigValue[] config)
    {
        configuration_ = config;
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/get_server_activator:1.0
    //
    /**
     **  To obtain the server activator that has created this component server.
     **
     **/
    public org.omg.Components.Deployment.ServerActivator
    get_server_activator()
    {
        //
        // TODO : implement, but needs to have the server activator too
        //
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/create_container:1.0
    //
    /**
     **  To create a container.
     **
     **/
    public org.omg.Components.Deployment.Container
    create_container(org.omg.Components.ConfigValue[] config)
    throws org.omg.Components.CreateFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        // create the object.
        ContainerImpl cont = new ContainerImpl(config, this);

        // activate the object.
        org.omg.CORBA.Object ref = cont._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
        org.omg.Components.Deployment.Container cont_ref = 
						org.omg.Components.Deployment.ContainerHelper.narrow(ref);
        cont.the_container_ref(cont_ref);
				listener_.on_create (cont_ref); 

        return cont_ref;
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/remove_container:1.0
    //
    /**
     **  To remove a container.
     **
     **/
    public void
    remove_container(org.omg.Components.Deployment.Container ref)
    throws org.omg.Components.RemoveFailure
    {
				listener_.on_remove (ref); 
        boolean b = containers_.remove(ref);
        if (!b)
            throw new org.omg.Components.RemoveFailure();
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/get_containers:1.0
    //
    /**
     **  To obtain all created containers references.
     **
     **/
    public org.omg.Components.Deployment.Container[]
    get_containers()
    {
        org.omg.Components.Deployment.Container[] result =
            new org.omg.Components.Deployment.Container[containers_.size()];

        for (int i=0;i<result.length;i++)
            result[i] = (org.omg.Components.Deployment.Container)containers_.get(i);

        return result;
    }

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/remove:1.0
    //
    /**
     **  To remove this component server.
     **
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        listener_.on_destroy();
        try {
            org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA().deactivate_object(org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA().reference_to_id(asComponentServer()));
        } catch (org.omg.PortableServer.POAPackage.WrongAdapter ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.ObjectNotActive ex) {
            throw new org.omg.Components.RemoveFailure();
        }
    }
} 

