/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc, Romain Rouvoy_________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the OMG IDL
 * OpenCCM::ComponentServer::Container interface.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */

public class ContainerBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected ComponentServerBase the_component_server_;

    /**
     **
     **/
    protected org.omg.Components.ConfigValue[] configuration_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector homes_;

    /**
     **
     **/
    protected fr.lifl.cim.jidlscript.JOSInterpreter josh_;

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.PCA rootPCA_;

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.SystemConfiguration container_configuration_;

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.SystemConfiguration home_configuration_;

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.SystemConfiguration component_configuration_;

    /**
     **
     **/
    protected SystemHomeManagerImpl home_manager_;
    
    /**
     **
     **/
    protected org.objectweb.ccm.Deployment.listener.ContainerListener listener_;

    /**
     **
     **/
    protected org.omg.Components.Deployment.Container the_container_ref_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param classLoader The class loader used to load home classes.
     **  @param script The base configuration of the container.
     **
     **  @throws BadConfiguration If the base configuration script is not valid.
     **/
    public
    ContainerBase(org.omg.Components.ConfigValue[] config,
                  ComponentServerBase component_server)
    throws org.omg.Components.CreateFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        the_component_server_ = component_server;
        configuration_ = config;
        homes_ = new org.objectweb.ccm.util.Vector();
        home_manager_ = new SystemHomeManagerImpl();
				listener_  = null;
				the_container_ref_ = null;
				org.objectweb.ccm.Deployment.listener.TheContainerConfigurator.get_instance().configure_container(new ContainerLocalImpl(this));
	

        container_configuration_ = null;
        home_configuration_ = null;
        component_configuration_ = null;
        fr.lifl.cim.jidlscript.lang.JISScope scope = null;

        // initialize the container.
        try
        {
            // create the interpreter
            josh_ = new fr.lifl.cim.jidlscript.JOSInterpreter();

            // create the root PCA.
            rootPCA_ = new org.objectweb.ccm.Containers.RootPCAImpl(home_manager_);
            scope = new fr.lifl.cim.jidlscript.lang.JISScope("CONTAINER");
            fr.lifl.cim.jidlscript.lang.JISRootScope.root().insertScoped(scope);
            create_alias(scope, "org.objectweb.ccm.Deployment.ContainerBase", this, "container");
            //
            //  TODO : other alias needed ???
            //
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            throw new org.omg.Components.CreateFailure();
        }

        // retrieve the script from the config values.
        java.lang.String script = "";
        for (int i=0;i<config.length;i++)
        {
            if (config[i].name.equals("container_script"))
            {
                script = config[i].value.extract_string();
                break;
            }
        }

        // executes the specific configuration script.
        try
        {
            josh_.eval(script);
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            //
            //  TODO : is it the good exception ???
            //
            //            throw new org.omg.Components.Deployment.InvalidConfiguration(-1, "container_script");
            throw new java.lang.Error("### script failure : \n"+ex.exception.toString());
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    create_alias(fr.lifl.cim.jidlscript.lang.JISScope scope,
                 java.lang.String clazz,
                 java.lang.Object instance,
                 java.lang.String alias)
    {
        try
        {
            fr.lifl.cim.jidlscript.java.JISJavaClass jc = 
                fr.lifl.cim.jidlscript.java.JISJavaClass.importClass(new fr.lifl.cim.jidlscript.lang.JISString(clazz));
            fr.lifl.cim.jidlscript.java.JISJavaInstance ji = new fr.lifl.cim.jidlscript.java.JISJavaInstance(jc, instance);
            scope.insertScoped(new fr.lifl.cim.jidlscript.lang.JISAlias(alias, ji));
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            //
            //  TODO : something else ???
            //
        }
    }

    /**
     **  Creates a CCMHome.
     **
     **  @param entrypt The entry point in the archive.
     **  @param name The name of the created home.
     **
     **  @return The created CCMHome.
     **
     **  @exception CreationFailed
     **             Thrown if the creation fails.
     **/
    protected org.omg.Components.HomeExecutorBase
    create(java.lang.String uuid,
           java.lang.String entrypt)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.Deployment.ImplEntryPointNotFound
    {
				try
				{
            // check that the implementation has been installed.
            TheComponentInstallation.getComponentInstallation().get_implementation(uuid);
						
						// Call the static 'create' method of the 'entrypt' class.
						java.lang.Object result =
                TheURLClassLoader.getURLClassLoader().callStaticClassMethod(entrypt, new Object[0]);

						// Check that the result is a CCMHome executor.
						return (org.omg.Components.HomeExecutorBase)result;
				}
        catch(org.omg.Components.Deployment.InstallationFailure ex)
				{
						throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
				}
        catch(java.lang.ClassNotFoundException ex)
        {
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.NoSuchMethodException ex)
        {
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.reflect.InvocationTargetException ex)
        {
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.IllegalAccessException ex)
        {
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
				catch(java.lang.ClassCastException ex)
        {
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
				}
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

		/**
		 ** Add a new plugin to the deployment process
		 **
		 ** @param listener the deployment listener to add
		 **/
    public void
    add_listener (org.objectweb.ccm.Deployment.listener.ContainerListener listener)
    {
				if (listener_ == null)
						listener_ = listener;
				else
						listener_.set_delegate(listener);
    }

    /**
     **
     **/
    public void 
    the_container_ref(org.omg.Components.Deployment.Container container)
    {
        the_container_ref_ = container;
    }

    /**
     **
     **/
    public org.omg.Components.Deployment.Container
    the_container_ref()
    {
        return the_container_ref_;
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.PCA 
    get_rootPCA()
    {
        return rootPCA_;
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemHome
    create_system_home(java.lang.String implUUID,
                       java.lang.String alias,
                       java.lang.String entrypt)
    throws org.objectweb.ccm.Containers.CreationFailed
    {
        return home_manager_.create_system_home(implUUID, alias, entrypt);
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemHome
    find_system_home(java.lang.String alias)
    throws org.objectweb.ccm.Containers.HomeNotFound
    {
        return home_manager_.find_system_home(alias);
    }

    /**
     **
     **/
    public void
    set_container_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        container_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_container_configuration()
    {
        return container_configuration_;
    }

    /**
     **
     **/
    public void
    set_home_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        home_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_home_configuration()
    {
        return home_configuration_;
    }

    /**
     **
     **/
    public void
    set_component_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        component_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_component_configuration()
    {
        return component_configuration_;
    }

		/**
		 ** Add the home to the container inner list
		 **
		 ** @param ref the reference of the home to add
		 **/
    public void
    add_home(org.omg.Components.CCMHome ref)
    {
        homes_.add(ref);
				listener_.on_add(ref); 
    }
    
    // ==================================================================
    //
    // Methods for the Components::Deployment::Container interface.
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
    //
    /**
     **  To obtain the container configuration.
     **
     **/
    public org.omg.Components.ConfigValue[]
    configuration()
    {
        return configuration_;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_component_server:1.0
    //
    /**
     **  To obtain a reference on the component server that has created this container.
     **/
    public org.omg.Components.Deployment.ComponentServer
    get_component_server()
    {
        return the_component_server_.asComponentServer();
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/install_home:1.0
    //
    /**
     **  Create and install a home in the container.
     **  This can be done more than once.
     **
     **  @param entry_pt The entry point in the archive.
     **  @param name The name of the created home.
     **  @param script The configuration script specific to the home.
     **
     **  @return The created home.
     **
     **  @exception CreationFailed If the home can not be created.
     **  @exception BadConfiguration If the configuration script is not valid.
     **
     **/
    public org.omg.Components.CCMHome
    install_home(java.lang.String uuid,
                 java.lang.String entrypt,
                 org.omg.Components.ConfigValue[] config)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.Deployment.ImplEntryPointNotFound,
           org.omg.Components.Deployment.InstallationFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        // retrieve the script from the config values.
        java.lang.String script = "";
        for (int i=0;i<config.length;i++)
        {
            if (config[i].name.equals("home_script"))
            {
                script = config[i].value.extract_string();
                break;
            }
        }

        // executes the specific configuration script.
        try
        {
            josh_.eval(script);
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            //
            //  TODO : is it the good exception ???
            //
            //            throw new org.omg.Components.Deployment.InvalidConfiguration(-1, "home_script");
            throw new java.lang.Error("### script failure : \n"+ex.exception.toString());
        }

        org.omg.Components.CCMHome ref = null;
        org.omg.Components.HomeExecutorBase home = null;

        // create the home executor.
        home = create(uuid, entrypt);


        //  for home of type H1 entrypt should be implemented 
        //  in the local implementation of H1.
        Class[] itfs = home.getClass().getInterfaces();
        String clazz_name = null;

        // find the CCM home executor implemented interface.
        for (int i=0 ; i<=itfs.length ; i++)
        {
            String tmp;
            int idx2 = itfs[i].getName().lastIndexOf(".");
            if (idx2 == -1) tmp = itfs[i].getName();
            else tmp = itfs[i].getName().substring(idx2+1);
            if (tmp.startsWith("CCM_")) 
            {
                clazz_name = itfs[i].getName().substring(0, idx2+1) + 
                    (new org.objectweb.ccm.visitorIDL3.java.JavaTranslator()).checkKeywords(tmp.substring(4)) + "CCM";
                break;
            }
        }

        // Instantiate the executor.
        org.objectweb.ccm.Containers.HomeExecutorBase home_exec;
        if (clazz_name != null) 
        {
             home_exec = (org.objectweb.ccm.Containers.HomeExecutorBase)create(uuid, clazz_name + "._create_home_executor");
        } else {
            throw new org.omg.Components.Deployment.InstallationFailure();
        }

        home_exec._delegate(home);

        // create the home servant and set infos.
        org.objectweb.ccm.Containers.HomeServantImpl servant = 
            new org.objectweb.ccm.Containers.HomeServantImpl();
        servant.the_container_base(this);
        home_exec._home_servant(servant);
        servant.the_home_executor(home_exec);
        servant.the_home_configuration(get_home_configuration());
        servant.the_component_configuration(get_component_configuration());
        servant.the_home_finder(home_manager_);


        // create and set the home reference.
        ref = rootPCA_.activate_home(servant);

				listener_.on_create(ref);
        return ref;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/remove_home:1.0
    //
    /**
     **
     **/
    public void
    remove_home(org.omg.Components.CCMHome ref)
    throws org.omg.Components.RemoveFailure
    {
        listener_.on_remove(ref);
        boolean b = homes_.remove(ref);
        if (!b)
            throw new org.omg.Components.RemoveFailure();
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_homes:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMHome[]
    get_homes()
    {
        org.omg.Components.CCMHome[] result =
            new org.omg.Components.CCMHome[homes_.size()];

        for (int i=0;i<result.length;i++)
            result[i] = (org.omg.Components.CCMHome)homes_.get(i);

        return result;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/remove:1.0
    //
    /**
     **
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        listener_.on_destroy();
        the_component_server_.remove_container(the_container_ref());
        try {
            org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA().deactivate_object(org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA().reference_to_id(the_container_ref()));
        } catch (org.omg.PortableServer.POAPackage.WrongAdapter ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.ObjectNotActive ex) {
            throw new org.omg.Components.RemoveFailure();
        }
    }
} 
