/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Romain Rouvoy_________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the OMG IDL
 * OpenCCM::ComponentServer::Container interface.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */

public class ContainerImpl
extends org.omg.Components.Deployment.ContainerPOA
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected ContainerBase the_container_;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param classLoader The class loader used to load home classes.
     **  @param script The base configuration of the container.
     **
     **  @throws BadConfiguration If the base configuration script is not valid.
     **/
    public
    ContainerImpl(org.omg.Components.ConfigValue[] config,
                  ComponentServerBase component_server)
		throws org.omg.Components.CreateFailure,
				   org.omg.Components.Deployment.InvalidConfiguration
    {
        the_container_ = new ContainerBase (config, component_server) ; 
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    public org.objectweb.ccm.Deployment.ContainerBase
		_delegate()
    {
				return the_container_;
    }


    public void
    add_listener (org.objectweb.ccm.Deployment.listener.ContainerListener listener)
    {
        the_container_.add_listener (listener); 
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemHome
    create_system_home(java.lang.String implUUID,
                       java.lang.String alias,
                       java.lang.String entrypt)
		throws org.objectweb.ccm.Containers.CreationFailed
    {
        return the_container_.create_system_home(implUUID, alias, entrypt);
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemHome
    find_system_home(java.lang.String alias)
		throws org.objectweb.ccm.Containers.HomeNotFound
    {
        return the_container_.find_system_home(alias);
    }

    /**
     **
     **/
    public void
    set_container_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        the_container_.set_container_configuration (config); 
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_container_configuration()
    {
        return the_container_.get_container_configuration () ;
    }

    /**
     **
     **/
    public void
    set_home_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        the_container_.set_home_configuration (config); 
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_home_configuration()
    {
        return the_container_.get_home_configuration () ;
    }

    /**
     **
     **/
    public void
    set_component_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        the_container_.set_component_configuration (config); 
    }

    /**
     **
     **/
    public org.objectweb.ccm.Containers.SystemConfiguration
    get_component_configuration()
    {
        return the_container_.get_component_configuration () ;
    }

     // ==================================================================
    //
    // Methods for the Components::Deployment::Container interface.
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
    //
    /**
     **  To obtain the container configuration.
     **
     **/
    public org.omg.Components.ConfigValue[]
    configuration()
    {
        return the_container_.configuration () ;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_component_server:1.0
    //
    /**
     **  To obtain a reference on the component server that has created this container.
     **/
    public org.omg.Components.Deployment.ComponentServer
    get_component_server()
    {
        return the_container_.get_component_server();
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/install_home:1.0
    //
    /**
     **  Create and install a home in the container.
     **  This can be done more than once.
     **
     **  @param entry_pt The entry point in the archive.
     **  @param name The name of the created home.
     **  @param script The configuration script specific to the home.
     **
     **  @return The created home.
     **
     **  @exception CreationFailed If the home can not be created.
     **  @exception BadConfiguration If the configuration script is not valid.
     **
     **/
    public org.omg.Components.CCMHome
    install_home(java.lang.String uuid,
                 java.lang.String entrypt,
                 org.omg.Components.ConfigValue[] config)
		throws org.omg.Components.Deployment.UnknownImplId,
				   org.omg.Components.Deployment.ImplEntryPointNotFound,
           org.omg.Components.Deployment.InstallationFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        return the_container_.install_home (uuid, entrypt, config); 
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/remove_home:1.0
    //
    /**
     **
     **/
    public void
    remove_home(org.omg.Components.CCMHome ref)
		throws org.omg.Components.RemoveFailure
    {
        the_container_.remove_home (ref); 
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_homes:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMHome[]
    get_homes()
    {
        return the_container_.get_homes (); 
    }

    /**
     * Set the corba reference for the relative container
     */
    public void
	  the_container_ref(org.omg.Components.Deployment.Container container)
    {
				the_container_.the_container_ref(container);
    }


    //
    //  IDL:omg.org/Components/Deployment/Container/remove:1.0
    //
    /**
     **
     **/
    public void
    remove()
		throws org.omg.Components.RemoveFailure
    {
        the_container_.remove (); 
    }
} 
