/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Raphael Marvie, Philippe Merle.
Contributor(s): Mathieu Vadet______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the OMG IDL
 * OpenCCM::ComponentServer::Server interface.
 *
 * @author <a href="mailto:Raphael.Marvie@lifl.fr">Raphael Marvie</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ServerImpl 
       extends ServerPOA
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The class loader used by the two facets.
     **/
    protected org.objectweb.ccm.util.URLClassLoader class_loader_;

    /**
     **  The ComponentServer facet.
     **/
    protected ComponentServerImpl component_server_facet_;

    /**
     **  The ComponentServer reference.
     **/
    protected org.omg.Components.Deployment.ComponentServer component_server_ref_;

    /**
     **  The ComponentInstallation facet.
     **/
    protected ComponentInstallationImpl install_facet_;

    /**
     **  The ComponentInstallation reference.
     **/
    protected org.omg.Components.Deployment.ComponentInstallation install_ref_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **/
    public
    ServerImpl()
    {
	// Create the ComponentServer facet.
        component_server_facet_ = new ComponentServerImpl(this);

	// Activate the ComponentServer facet.
	// TODO: Must be changed!!!
        org.omg.CORBA.Object ref = component_server_facet_._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
	component_server_ref_ = org.omg.Components.Deployment.ComponentServerHelper.narrow(ref);

	// Obtain the path where archives are stored.
	String path = System.getProperties().getProperty ("OpenCCM.ComponentServer.archive_cache", ".");

	// Create the install facet.
	install_facet_ = new ComponentInstallationImpl(path);

	// Activate the install facet.
        // TODO: Must be changed!!!
        ref = install_facet_._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
	install_ref_ =  org.omg.Components.Deployment.ComponentInstallationHelper.narrow(ref);
        TheComponentInstallation.setComponentInstallation(install_ref_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the OpenCCM::Shutdownable interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/Shutdownable/shutdown:1.0
    //
    /**
     **  Shutdown an OpenCCM server.
     **/
    public void
    shutdown()
    {
	org.objectweb.ccm.CORBA.TheORB.shutdown(false);
    }

    // ==================================================================
    //
    // Methods for the OpenCCM::ComponentServer::Server interface.
    //
    // ==================================================================
 
    //
    // IDL:goal.lifl.fr/OpenCCM/ComponentServer/Server/provide_component_server:1.0
    //
    /**
     **  Obtains the component server reference.
     **
     **  @return The reference to the ComponentServer object.
     **/
    public org.omg.Components.Deployment.ComponentServer
    provide_component_server()
    {
	return component_server_ref_;
    }

    //
    // IDL:goal.lifl.fr/OpenCCM/ComponentServer/Server/provide_install:1.0
    //
    /**
     **  Obtains the component installation reference.
     **
     **  @return The reference to the component installation.
     **/
    public org.omg.Components.Deployment.ComponentInstallation
    provide_install()
    {
	return install_ref_;
    }
}
