/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Raphael Marvie, Philippe Merle.
Contributor(s): Mathieu Vadet______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

/**
 * This class implements the Component Server main.
 *
 * @author <a href="mailto:Raphael.Marvie@lifl.fr">Raphael Marvie</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ServerMain
       extends org.objectweb.ccm.CORBA.ApplicationServer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     ** To store the component server name.
     **/
    protected String name_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **/
    public
    ServerMain()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  The bootstrap main method.
     **
     **  @param args The command-line arguments.
     **/
    public static void
    main(String args[])
    {
	ServerMain server = new ServerMain();
	int status = server.start(args);
	System.exit(status);
    }

    // ==================================================================
    //
    // Methods for the org.objectweb.ccm.CORBA.ApplicationServer.
    //
    // ==================================================================

    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    before_run(String[] args)
    {
	// Init the OpenCCM Components runtime.
	org.objectweb.ccm.Components.Runtime.init();

	// Check command-line arguments.
	if(args.length < 1)
	{
	    System.err.println("Usage: java " +
			       getClass().getName() +
			       " <serverName>");
	    return -1;
	} 

	// Obtain the name of this server.
	name_ = args[0];
	String fileName = args[1];

	// Create the server servant.
	ServerImpl server = new ServerImpl();

	// Activate it.
	// TODO: Must be changed!!!
        org.omg.CORBA.Object ref =
            server._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
	Server serverRef = ServerHelper.narrow(ref);

	// Obtain the Name Service.
	org.objectweb.ccm.CORBA.NamingContext ns =
	    org.objectweb.ccm.CORBA.TheNameService.getNamingContext();

	// Bind the server object in the name service.
	ns.rebind(name_, serverRef);
	
        // Output the IOR of the ComponentServer.
        org.objectweb.ccm.CORBA.TheORB.save_IOR(serverRef, fileName);

	// Return OK.
	System.out.println("The OpenCCM Java Component Server is ready.");
	return 0;
    }

    /**
     **  After running the CORBA::ORB.
     **/
    public void
    after_run()
    {
	// Obtain the Name Service.
	org.objectweb.ccm.CORBA.NamingContext ns =
	    org.objectweb.ccm.CORBA.TheNameService.getNamingContext();

	// Unbind the server object from the name service.
	ns.unbind(name_);	
    }
}
