/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment.listener; 


/**
 * Component Server Configurator
 * <br> Defines which deployment plugins needs to be plugged
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class ComponentServerConfigurator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

		/**
		 ** Reference to the plugin chain
		 **
		 **/
		protected org.objectweb.ccm.Deployment.listener.ComponentServerListener listener_;
		
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

		/**
		 ** Default Constructor
		 **
		 **/
    public
		ComponentServerConfigurator()
    {
				listener_ = null;
				init();
    }


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

		/**
		 ** Initialize the deployment with transaction or not 
		 ** <br> Depending on the TRANSACTIONAL System property
		 **
		 ** @deprecated waiting for assembly tool
		 **/
		protected void
		init()
		{
				String _transaction = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no");
				if (_transaction.equalsIgnoreCase("yes"))
						add_listener(new org.objectweb.ccm.plugins.transaction.deployment.ComponentServerListenerTransaction());
				else
						add_listener(new org.objectweb.ccm.Deployment.listener.simple.ComponentServerListenerSimple());
		}


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

		/**
		 ** Add a new listener to the end of the plugin chain
		 **
		 ** @param listener new listener to add
		 **/
    public void
		add_listener(org.objectweb.ccm.Deployment.listener.ComponentServerListener listener)
    {
				if (listener_ == null)
						listener_ = listener;
				else
						listener_.set_delegate(listener);
				// Create the corresponding chain into the Container plugin chain
				TheContainerConfigurator.get_instance().add_listener(listener.get_sub_listener());
    }

		/**
		 ** ComponentServer CallBack 
		 ** <br> Configure the component server 
		 **
		 ** @param component_server the component server to configure
		 **/
    public void
		configure_component_server(org.objectweb.ccm.Deployment.ComponentServerLocal component_server)
    {
				if (listener_ != null)
						listener_.configure_component_server(component_server);
				component_server.add_listener(new org.objectweb.ccm.Deployment.listener.ComponentServerListenerDefault(component_server));
    }

		/**
		 ** Reset the ComponentServer plugin chain
		 **
		 **/
		public void
		reset()
		{
				listener_ = null ;
				TheContainerConfigurator.get_instance().reset();
		}
}










