/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages an abstract interface declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AbstractInterfaceDeclImpl
       extends InterfaceDeclImpl
       implements AbstractInterfaceRef, AbstractInterfaceDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 AbstractInterfaceDef.
     **/
    private org.omg.CORBA.ExtAbstractInterfaceDef ext_abstract_interface_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    AbstractInterfaceDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the InterfaceDefImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_abstract_interface_def_ = null;
        the_declaration_kind_ = DeclarationKind._dk_abstract_interface;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_abstract_interface_def_ = org.omg.CORBA.ExtAbstractInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_abstract_interface_def_ = org.omg.CORBA.ExtAbstractInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the abstract interface
     **           declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_abstract_interface_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_abstract_interface;
    }

    // ==================================================================
    //
    // Methods for the InterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the abstract
     **          interface declaration.
     **/
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the abstract
     **          interface declaration.
     **/
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the InterfaceDecl interface.
    //
    // ==================================================================

    /**
     **  Add an interface.
     **
     **  @param itf The interface to add.
     **/
/*
 * the cast implies a NullPointerException when an abstract interface
 * inherits from a interface
 * 
    public void
    addInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            inherited_interfaces_.get(
                 ((AbstractInterfaceDeclImpl)itf));
        }
    }
 */

    // ==================================================================
    //
    // Methods for the AbstractInterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its AbstractInterfaceDef reference.
     **
     **  @return The AbstractInterfaceDef object associated with
     **          the abstract interface declaration.
     **/
    public org.omg.CORBA.ExtAbstractInterfaceDef
    getExtAbstractInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    /**
     **  Obtain its AbstractInterfaceDef reference.
     **
     **  @return The AbstractInterfaceDef object associated with
     **          the abstract interface declaration.
     **/
    public org.omg.CORBA.AbstractInterfaceDef
    getAbstractInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the abstract
     **          interface declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the abstract
     **          interface declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        org.omg.CORBA.AbstractInterfaceDef[] itfs =
         new org.omg.CORBA.AbstractInterfaceDef[inherited_interfaces_.size()];

        for(int i=0; i<itfs.length; i++)
            itfs[i] = ((AbstractInterfaceDeclImpl)(inherited_interfaces_.get(i))).getExtAbstractInterfaceDef();

        ext_abstract_interface_def_ = the_parent_.getContainer().create_ext_abstract_interface(getId(), getName(),
                                                                                               getVersion(), itfs);
    }
}
