/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class implements the typedef declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AliasDeclImpl
       extends DeclarationImpl
       implements TypeRef, AliasDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 AliasDef.
     **/
    private org.omg.CORBA.AliasDef alias_def_;

    /**
     **  The original type of the alias.
     **/
    private TypeRef original_type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    AliasDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        alias_def_ = null;
        original_type_ = null;
        the_declaration_kind_ = DeclarationKind._dk_alias;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 AliasDef.
     **
     **  @param aliasDef The AliasDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        alias_def_ = org.omg.CORBA.AliasDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(alias_def_.original_type_def()));
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 AliasDef.
     **
     **  @param aliasDef The AliasDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        alias_def_ = org.omg.CORBA.AliasDefHelper.narrow(contained);
        setType(getRepository().getAsMappedTypeRef(alias_def_.original_type_def()));
        super.load(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the constant declaration into the IFR.
     **/
    public void
    create()
    {
        alias_def_ = the_parent_.getContainer().create_alias(getId(), getName(), getVersion(),
                                                             original_type_.getIDLType());

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector alias_depend = new org.objectweb.ccm.util.Vector();

        // content
        if (getType().isDeclaration())
            alias_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                alias_depend.add(depend[i]);
        }


        dependencies_ = (Declaration[])alias_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the typedef declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return alias_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_alias;
    }

    // ==================================================================
    //
    // Methods for the AliasDecl interface.
    //
    // ==================================================================

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return original_type_;
    }

    /**
     **  Set the original type.
     **
     **  @param type To original type of the typedef declaration.
     **/
    public void
    setType(TypeRef type)
    {
        if(type != null)
        {
            original_type_ = type;
            original_type_.addRef();
        }
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (original_type_!=null)
            original_type_.removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the typedef declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return alias_def_;
    }
}
