/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages the declaration with an AnyValue.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AnyValueDeclImpl
       extends DeclarationImpl
       implements AnyValueDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The value of the constant.
     **/
    private AnyValue value_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    AnyValueDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // init internal state.
        value_ = null;
    }

    // ==================================================================
    //
    //  Methods for the AnyValueDecl interface.
    //
    // ==================================================================

    /**
     **  Obtain the any value.
     **
     **  @return The AnyValue object associated with the any declaration.
     **/
    public AnyValue
    getAnyValue()
    {
        return value_;
    }

    /**
     **  Set the any value.
     **
     **  @param value The AnyValue to set.
     **/
    public void
    setAnyValue(AnyValue value)
    {
        value_ = value;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the any declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }
}
