/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages the Attribute declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface AttributeDecl
       extends Declaration
{
    /** MUST BE REMOVED  **/
    public static int ATTR_NORMAL = org.omg.CORBA.AttributeMode._ATTR_NORMAL;

    /** MUST BE REMOVED  **/
    public static int ATTR_READONLY = org.omg.CORBA.AttributeMode._ATTR_READONLY;

    /**
     * Set mode as normal.
     */
    public void
    setNormal();

    /**
     * Set mode as readonly.
     */
    public void
    setReadonly();

    /**
     * Set the attribute type.
     *
     * @param type The TypeRef of the attribute declaration.
     */
    public void
    setType(TypeRef type);

    /**
     * Obtain the get exceptions list.
     *
     * @return The getRaises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared.
     */
    public ExceptionList
    getGetExceptionList();

    /**
     * Obtain the put exceptions list.
     *
     * @return The setRaises (or raises for readonly attributes)
     *         ExceptionList of the attribute declaration 
     *         or null if no exceptions were declared.
     */
    public ExceptionList
    getSetExceptionList();

    /**
     * Obtain the get exceptions list.
     *
     * @return The getRaises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared.
     */
    public ExceptionDecl[]
    getGetExceptions();

    /**
     * Obtain the put exceptions list.
     *
     * @return The setRaises (or raises for readonly attributes)
     *         ExceptionList of  the attribute declaration or
     *         null if no exceptions were declared.
     */
    public ExceptionDecl[]
    getSetExceptions();

    /**
     *
     */
    public ExceptionDecl[]
    getExceptions();

    /**
     * Is a readonly attribute
     *
     * @return True if readonly, else false.
     */
    public boolean
    isReadonly();

    /**
     * Obtain the attribute type.
     *
     * @return The attribute type.
     */
    public TypeRef
    getType();
}
