/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages a component declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface ComponentDecl
       extends InterfaceDecl
{
    /**
     * Set the base component.
     *
     * @param base_component The base component of the component.
     *        Note that if more then one base component 
     *        is set, only the first one is considered.
     */
    public void
    setBaseComponent(ComponentRef base_component);

    /**
     * Add an interface.
     *
     *  @param itf The supported interface to add.
     */
    public void
    addSupportedInterface(InterfaceRef itf);

    /**
     * Obtain the base component.
     *
     * @return The base component.
     */
    public ComponentDecl
    getBaseComponent();

    /**
     * Obtain the supported interface declarations.
     *
     * @return The supported interface declarations.
     */
    public InterfaceDecl[]
    getSupportedInterfaces();

    /**
     * Obtain the client interface mapping.
     *
     * @return The client interface mapping.
     */
    public InterfaceDecl
    getClientMapping();

    /**
     * Obtain the local context interface mapping.
     *
     * @return The local context interface mapping.
     */
    public InterfaceDecl
    getLocalContextMapping();

    /**
     * Obtain the local monolithic interface mapping.
     *
     * @return The local monolithic interface mapping.
     */
    public InterfaceDecl
    getLocalMonolithicMapping();

    /**
     * Obtain the local main interface mapping.
     *
     * @return The local main interface mapping.
     */
    public InterfaceDecl
    getLocalMainMapping();
}
