/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages consumes port declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ConsumesDeclImpl
       extends EventsDeclImpl
       implements ConsumesDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ConsumesDef.
     **/
    private org.omg.CORBA.ComponentIR.ConsumesDef consumes_def_;

    /**
     **
     **/
    private OperationDeclImpl[] client_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    ConsumesDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the EventsDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        consumes_def_ = null;
        client_mapping_ = null;
        the_declaration_kind_ = DeclarationKind._dk_consumes;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ConsumesDef.
     **
     **  @param consumes_def The ConsumesDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        consumes_def_ = org.omg.CORBA.ComponentIR.ConsumesDefHelper.narrow(contained);
        super.load(contained);
    }

    // ==================================================================
    //
    //   Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the consumes declaration into the IFR.
     **/
    public void
    create()
    {
        consumes_def_ = the_parent_.getComponentDef().create_consumes(getId(),
                                                                      getName(),
                                                                      getVersion(),
                                                                      event_.getEventDef());

        super.create();
    }

    // ==================================================================
    //
    //   Methods for the EventsDecl interface.
    //
    // ==================================================================

    /**
     **
     **/
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind._dk_null);

        client_mapping_ = new OperationDeclImpl[1];
        client_mapping_[0] = (OperationDeclImpl)client.find("get_consumer_"+getName());

        return client_mapping_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the consumes declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return consumes_def_;
    }
}
