/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * Base interface for IDL3 declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface Declaration
{
    //
    //  Parser view of a declaration.
    //
    //

    /**
     * Set the prefix.
     *
     * @param prefix The prefix to set. Note that setting the prefix should 
     *               only be done when a new prefix (ie different from the 
     *               declared-in scope prefix) was explicitly declared.
     */
    public void
    setPrefix(String prefix);

    /**
     * Set the name.
     *
     * @param name The name to set.
     */
    public void
    setName(String name);

    /**
     * Set the version.
     *
     * @param version The version to set.
     */
    public void
    setVersion(String version);

    /**
     * Set the repository ID.
     *
     * @param id The RepositoryId to set.
     */
    public void
    setId(String id);

    /**
     * Create it into the IR3.
     */
    public void
    create();

    /**
     * Destroy Declaration in the IR3.
     */
    public void
    destroy();

    //
    //  Visitor view of a declaration.
    //
    //

    /**
     * Obtain the name.
     *
     * @return The name of the declaration.
     */
    public String
    getName();

    /**
     * Obtain the absolute name.
     *
     * @return The absolute name of the declaration.
     */
    public String
    getAbsoluteName();

    /**
     * Obtain the prefix.
     *
     * @return The prefix of the declaration or null if there's no prefix.
     */
    public String
    getPrefix();

    /**
     * Obtain the version.
     *
     * @return The version of the declaration.
     *         Note that version "1.0" is assumed as default.
     */
    public String
    getVersion();

    /**
     * Obtain the repository ID.
     *
     * @return The RepositoryId of the declaration.
     */
    public String
    getId();

    /**
     * Returns the kind of the Declaration object.
     *
     * @return The DeclarationKind of the object.
     */
    public int
    getDeclKind();

    /**
     * Obtain the parent scope.
     *
     * @return The parent scope of the declaration
     *         or null if the declaration has no parent.
     */
    public Scope
    getParent();

    /**
     * Obtain the declaration external dependencies.
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies();

    /**
     * Obtain the declaration external dependencies.
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies(boolean recursion);
}
