/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class defines kinds of Declaration objects.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class DeclarationKind
{
    // ==================================================================
    //
    // Public constant fields.
    //
    // ==================================================================

    /**
     **  Int constant for null Declaration object kind.
     **/
    public static final int _dk_null = 1<<0;

    /**
     **  Int constant for all object kinds.
     **/
    public static final int _dk_all = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6)|
                                      (1<<7)|(1<<8)|(1<<9)|(1<<10)|(1<<11)|(1<<12)|(1<<13)|
                                      (1<<14)|(1<<15)|(1<<16)|(1<<17)|(1<<18)|(1<<19)|(1<<20)|
                                      (1<<21)|(1<<22)|(1<<23)|(1<<24)|(1<<25)|(1<<26);

    /**
     **  Int constant for ModuleDecl object kind.
     **/
    public static final int _dk_module = 1<<1;

    /**
     **  Int constant for AbstractInterfaceDecl object kind.
     **/
    public static final int _dk_abstract_interface = 1<<2;

    /**
     **  Int constant for LocalInterfaceDecl object kind.
     **/
    public static final int _dk_local_interface = 1<<3;

    /**
     **  Int constant for InterfaceDecl object kind.
     **/
    public static final int _dk_interface = 1<<4;

    /**
     **  Int constant for AliasDecl object kind.
     **/
    public static final int _dk_alias = 1<<5;

    /**
     **  Int constant for AttributeDecl object kind.
     **/
    public static final int _dk_attribute = 1<<6;

    /**
     **  Int constant for ConstantDecl object kind.
     **/
    public static final int _dk_constant = 1<<7;

    /**
     **  Int constant for EnumDecl object kind.
     **/
    public static final int _dk_enum = 1<<8;

    /**
     **  Int constant for ExceptionDecl object kind.
     **/
    public static final int _dk_exception = 1<<9;

    /**
     **  Int constant for NativeDecl object kind.
     **/
    public static final int _dk_native = 1<<10;

    /**
     **  Int constant for OperationDecl object kind.
     **/
    public static final int _dk_operation = 1<<11;

    /**
     **  Int constant for StructDecl object kind.
     **/
    public static final int _dk_struct = 1<<12;

    /**
     **  Int constant for UnionDecl object kind.
     **/
    public static final int _dk_union = 1<<13;

    /**
     **  Int constant for ValueBoxDecl object kind.
     **/
    public static final int _dk_value_box = 1<<14;

    /**
     **  Int constant for ValueDecl object kind.
     **/
    public static final int _dk_value = 1<<15;

    /**
     **  Int constant for ValueMemberDecl object kind.
     **/
    public static final int _dk_value_member = 1<<16;

    /**
     **  Int constant for ComponentDecl object kind.
     **/
    public static final int _dk_component = 1<<17;

    /**
     **  Int constant for ConsumesDecl object kind.
     **/
    public static final int _dk_consumes = 1<<18;

    /**
     **  Int constant for EmitsDecl object kind.
     **/
    public static final int _dk_emits = 1<<19;

    /**
     **  Int constant for PublishesDecl object kind.
     **/
    public static final int _dk_publishes = 1<<20;

    /**
     **  Int constant for ProvidesDecl object kind.
     **/
    public static final int _dk_provides = 1<<21;

    /**
     **  Int constant for UsesDecl object kind.
     **/
    public static final int _dk_uses = 1<<22;

    /**
     **  Int constant for HomeDecl object kind.
     **/
    public static final int _dk_home = 1<<23;

    /**
     **  Int constant for FactoryDecl object kind.
     **/
    public static final int _dk_factory = 1<<24;

    /**
     **  Int constant for FinderDecl object kind.
     **/
    public static final int _dk_finder = 1<<25;

    /**
     **  Int constant for EventDecl object kind.
     **/
    public static final int _dk_event = 1<<26;

    /**
     **  Int constant for Repository object kind.
     **/
    public static final int _dk_repository = 1<<27;

    /**
     **  Int constant for FileScope object kind.
     **/
    public static final int _dk_filescope = 1<<28;


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public static String
    toString(int kind)
    {
        switch(kind)
        {
        case _dk_null :
            return "null";
        case _dk_module :
            return "module";
        case _dk_abstract_interface :
            return "abstract_interface";
        case _dk_local_interface :
            return "local_interface";
        case _dk_interface :
            return "interface";
        case _dk_alias :
            return "alias";
        case _dk_attribute :
            return "attribute";
        case _dk_constant :
            return "constant";
        case _dk_enum :
            return "enum";
        case _dk_exception :
            return "exception";
        case _dk_native :
            return "native";
        case _dk_operation :
            return "operation";
        case _dk_struct :
            return "struct";
        case _dk_union :
            return "union";
        case _dk_value_box :
            return "value_box";
        case _dk_value :
            return "value";
        case _dk_value_member :
            return "value_member";
        case _dk_component :
            return "component";
        case _dk_consumes :
            return "consumes";
        case _dk_emits :
            return "emits";
        case _dk_publishes :
            return "publishes";
        case _dk_provides :
            return "provides";
        case _dk_uses :
            return "uses";
        case _dk_home :
            return "home";
        case _dk_factory :
            return "factory";
        case _dk_finder :
            return "finder";
        case _dk_event :
            return "event";
        case _dk_repository :
            return "repository";
        case _dk_filescope :
            return "filescope";
        default :
            return "null";
        }
    }
}
