/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages list of declarators.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class DeclaratorList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  List of declarator names.
     **/
    private java.util.Vector names_;

    /**
     **  List of declarator types.
     **/
    private java.util.Vector types_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public DeclaratorList()
    {
        // Init internal state.
        names_ = new java.util.Vector();
        types_ = new java.util.Vector();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Add a declarator.
     **
     **  @param name The name of the declarator.
     **  @param type The TypeRef of the declarator.
     **/
    public void
    addDeclarator(String name,
                  TypeRef type)
    {
        names_.addElement(name);
        types_.addElement(type);
    }

    /**
     **  Obtain the number of declarators.
     **
     **  @return The number of declarators.
     **/
    public int
    size()
    {
        return names_.size();
    }

    /**
     **  Obtain the name of the index declarator.
     **
     **  @param index The index of the searched declarator.
     **
     **  @return The name of the declarator at the specified index.
     **/
    public String
    nameAt(int index)
    {
        return (String)names_.elementAt(index);
    }

    /**
     **  Obtain the type of the index declarator.
     **
     **  @param index The index of the searched declarator.
     **
     **  @return The TypeRef of the declarator at the specified index.
     **/
    public TypeRef
    typeAt(int index)
    {
        return (TypeRef)types_.elementAt(index);
    }
}
