/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages event type declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EventDeclImpl
       extends ValueDeclImpl
       implements EventRef, EventDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ValueDef.
     **/
    private org.omg.CORBA.ComponentIR.EventDef event_def_;

    /**
     **
     **/
    private InterfaceDeclImpl client_mapping_;

    /**
     **
     **/
    private InterfaceDeclImpl local_consumer_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the value declaration.
     **/
    public
    EventDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the EventDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        event_def_ = null;
        client_mapping_ = null;
        local_consumer_mapping_ = null;
        the_declaration_kind_ = DeclarationKind._dk_event;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ValueDef.
     **
     **  @param valueDef The ValueDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        event_def_ = org.omg.CORBA.ComponentIR.EventDefHelper.narrow(contained);

        // load it's mapped declarations
        getClientMapping();
        getLocalConsumerMapping();

        super.load(contained);
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the value declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return event_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_event;
    }

    // ==================================================================
    //
    // Methods for the EventRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its EventDef reference.
     **
     **  @return The ValueDef associated with the value declaration.
     **/
    public org.omg.CORBA.ComponentIR.EventDef
    getEventDef()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Methods for the ValueRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its ValueDef reference.
     **
     **  @return The ValueDef associated with the value declaration.
     **/
    public org.omg.CORBA.ExtValueDef
    getExtValueDef()
    {
        return event_def_;
    }

    /**
     **  Obtain its ValueDef reference.
     **
     **  @return The ValueDef associated with the value declaration.
     **/
    public org.omg.CORBA.ValueDef
    getValueDef()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Methods for the EventDecl interface.
    //
    // ==================================================================

    /**
     **
     **/
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // load the context mapping for this component.
        String base_id = getId();
        int idx = base_id.lastIndexOf(':');
        base_id = base_id.substring(0, idx);
        String id = base_id + "Consumer:"+getVersion();

        client_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return client_mapping_;
    }

    /**
     **
     **/
    public InterfaceDecl
    getLocalConsumerMapping()
    {
        if (local_consumer_mapping_!=null)
            return local_consumer_mapping_;

        // load the local consumer mapping for this event.
        String parent_base_id = the_parent_.getId();
        int idx = parent_base_id.lastIndexOf(':');
        parent_base_id = parent_base_id.substring(0, idx);
        String id = parent_base_id + "/CCM_"+getName()+"Consumer:"+getVersion();

        local_consumer_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return local_consumer_mapping_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the value declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return event_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the value declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        org.omg.CORBA.ValueDef[] abstract_base_values =
	    new org.omg.CORBA.ValueDef[abstract_base_values_.size()];
        for(int i=0; i<abstract_base_values.length; i++)
        {
            abstract_base_values[i] = 
                ((ValueRef)(abstract_base_values_.get(i))).getExtValueDef();
        }

        org.omg.CORBA.InterfaceDef[] supported_interfaces =
	    new org.omg.CORBA.InterfaceDef[supported_interfaces_.size()];
        for(int i=0; i<supported_interfaces.length; i++)
        {
            supported_interfaces[i] = 
                ((InterfaceRef)(supported_interfaces_.get(i))).getExtInterfaceDef();
        }

        // Create a EventDef into the IFR.
        event_def_ = the_parent_.getComponentContainer().create_event(
                                       getId(), getName(), getVersion(),
                                       is_custom_, is_abstract_,
                                       getBaseValueDef(),
                                       is_truncatable_,
                                       abstract_base_values,
                                       supported_interfaces,
                                       new org.omg.CORBA.ExtInitializer[0]);
    }
}
