/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages events port declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class EventsDeclImpl
                extends DeclarationImpl
                implements EventsDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The port value type.
     **/
    protected EventDeclImpl event_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the event declaration.
     **/
    protected
    EventsDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        event_ = null;
    }

    // ==================================================================
    //
    // Public methods for the import statement.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 EventDef.
     **
     **  @param eventDef The EventDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        org.omg.CORBA.ComponentIR.EventPortDef event_port =
            org.omg.CORBA.ComponentIR.EventPortDefHelper.narrow(contained);
        setEvent((EventRef)getRepository().lookupId(event_port.event().id()));
        super.load(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector events_depend = new org.objectweb.ccm.util.Vector();

        // content
        if (getType().isDeclaration())
            events_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                events_depend.add(depend[i]);
        }

        dependencies_ = (Declaration[])events_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    //  Methods for the EventsDecl interface.
    //
    // ==================================================================

    /**
     **  Set the event value type.
     **
     **  @param value The EventRef of the event.
     **/
    public void
    setEvent(EventRef event)
    {
        if (event != null)
        {
            event_ = (EventDeclImpl)event;
            event_.addRef();
        }
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return event_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (event_!=null)
            event_.removeRef();
    }
}
