/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages exception lists.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExceptionListImpl
       implements ExceptionList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  List of exceptions.
     **/
    protected org.objectweb.ccm.util.Vector list_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public ExceptionListImpl()
    {
        // Init internal state.
        list_ = new org.objectweb.ccm.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    destroy()
    {
        for (int i=0;i<list_.size();i++)
        {
            ((ExceptionRef)list_.get(i)).removeRef();
        }
    }

    // ==================================================================
    //
    // Methods for the ExceptionList interface.
    //
    // ==================================================================

    /**
     **  Add an exception.
     **
     **  @param exception The exception to add to the list.
     **/
    public void
    add(ExceptionRef exception)
    {
        if (exception != null)
        {
            exception.addRef();
            list_.add(exception);
        }
    }

    /**
     **
     **/
    public ExceptionDecl[]
    getExceptions()
    {
        ExceptionDecl[] result = new ExceptionDecl[list_.size()];

        for(int i=0; i<result.length; i++)
            result[i] = (ExceptionDecl)(list_.get(i));

        return result;
    }

    /**
     **
     **/
    public org.omg.CORBA.ExceptionDef[]
    getExceptionDefs()
    {
        org.omg.CORBA.ExceptionDef[] result = new org.omg.CORBA.ExceptionDef[list_.size()];

        for(int i=0; i<result.length; i++)
            result[i] = ((ExceptionRef)list_.get(i)).getExceptionDef();

        return result;
    }

    /**
     **
     **/
    public org.omg.CORBA.ExceptionDescription[]
    getExceptionDescriptions()
    {
        org.omg.CORBA.ExceptionDescription[] result = new org.omg.CORBA.ExceptionDescription[list_.size()];

        ExceptionDeclImpl exc = null;
        for(int i=0; i<result.length; i++)
        {
            exc = (ExceptionDeclImpl)list_.get(i);
            result[i] = new org.omg.CORBA.ExceptionDescription(exc.getName(),
                                                               exc.getId(),
                                                               exc.the_parent_.getId(),
                                                               exc.getVersion(),
                                                               exc.getExceptionDef().type());
        }

        return result;
    }
}
