/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages a FileScope object.
 * It's used to represent declarations within an OMG IDL file.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FileScope
       extends ScopeImpl
{
    // ==================================================================
    // 
    //  Internal state.
    // 
    // ==================================================================

    /** To store included files names. */
    protected org.objectweb.ccm.util.Vector included_files_;

    // ==================================================================
    // 
    //  Constructor.
    // 
    // ==================================================================

    /**
     **  The constructor with the parent scope and declaration tables.
     **
     **  @param parent The parent scope of the declaration.
     **  @param decl The hashtable of contained declarations.
     **  @param odecl The vector of contained declarations.
     **/
    public
    FileScope(Repository rep,
              ScopeImpl parent, 
              org.objectweb.ccm.util.Vector decls)
    {
        super(rep, parent);

	// Init internal state.
        contained_decls_ = decls;
        included_files_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_filescope;
    }

    // ==================================================================
    // 
    //  Internal methods.
    // 
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the
     **          FileScope declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return the_parent_.getContainer();
    }

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
       return the_parent_.getComponentContainer();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return null because no Contained object is associated
     **          with the FileScope.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods for included files.
    //
    // ==================================================================

    /**
     **  Add an included file name to the current list.
     **
     **  @param fs The parent FileScope.
     **/
    public void
    addIncluded(FileScope fs)
    {
        included_files_.add(fs);
    }

    /**
     **  Obtain the included file names in order.
     **
     **  @return An array containing the included file names.
     **/
    public String[]
    getIncluded()
    {
        String[] filenames = new String[included_files_.size()];
        for (int i=0;i<filenames.length;i++)
            filenames[i] = ((Declaration)included_files_.get(i)).getName();

        return filenames;
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    /**
     **  Destroy all the contained declarations of the file scope in the IR3.
     **/
    public void
    destroy()
    {
        // if this FileScope is the root FileScope then destroy it as a Scope.
        if (getName().equals(""))
            super.destroy();
    }
}
