/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages finder operation declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FinderDeclImpl
       extends OperationDeclImpl
       implements FinderDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 FinderDef.
     **/
    private org.omg.CORBA.ComponentIR.FinderDef finder_def_;

    /**
     **
     **/
    private OperationDeclImpl[] client_mapping_;

    /**
     **
     **/
    private OperationDeclImpl[] local_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the finder declaration.
     **/
    protected
    FinderDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the OperationDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        finder_def_ = null;
        client_mapping_ = null;
        local_mapping_ = null;
        the_declaration_kind_ = DeclarationKind._dk_finder;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 FinderDef.
     **
     **  @param finder_def The FinderDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        finder_def_ = org.omg.CORBA.ComponentIR.FinderDefHelper.narrow(contained);
        super.load(contained);
    }

    // ==================================================================
    //
    //  For Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the finder declaration into the IFR.
     **/
    public void
    create()
    {
        finder_def_ = the_parent_.getHomeDef().create_finder(getId(),
                                                             getName(),
                                                             getVersion(),
                                                             parameters_.getParameterDescs(),
                                                             exceptions_.getExceptionDefs());
    }

    // ==================================================================
    //
    //  Methods for the FinderDecl interface.
    //
    // ==================================================================

    /**
     **
     **/
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((HomeDecl)the_parent_).getClientExplicitMapping();
        client.getContents(true, DeclarationKind._dk_null);

        client_mapping_ = new OperationDeclImpl[1];
        client_mapping_[0] = (OperationDeclImpl)client.find(getName());

        return client_mapping_;
    }

    /**
     **
     **/
    public OperationDecl[]
    getLocalMapping()
    {
        if (local_mapping_!=null)
            return local_mapping_;

        // retrieve the local mapping interface.
        InterfaceDecl local = ((HomeDecl)the_parent_).getLocalMapping();
        local.getContents(true, DeclarationKind._dk_null);

        local_mapping_ = new OperationDeclImpl[1];
        local_mapping_[0] = (OperationDeclImpl)local.find(getName());

        return local_mapping_;
    }

    // ==================================================================
    //
    //  For DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the finder declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return finder_def_;
    }
}
