/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages scopes with forward declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class ForwardScopeImpl
                extends ScopeImpl
                implements ForwardScope
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Is it forward?
     **/
    private boolean forward_;

    /**
     **  Is it completed?
     **/
    private boolean completed_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the forward declaration.
     **/
    protected
    ForwardScopeImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        forward_ = false;
        completed_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    abstract protected void
    createContainer();

    /**
     **  Complete the container object.
     **/
    abstract protected void
    completeContainer();

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the interface declaration into the IFR.
     **/
    public void
    create()
    {
        if (getContainer() == null)
        {
            createContainer();
            if (the_parent_!=null)
                the_parent_.addDecl(this);

            completed_ = !forward_;
        }
        else
        {
            if(forward_)
                 return;

            if(completed_)
            {
                throw new org.omg.CORBA.BAD_PARAM(
                         org.objectweb.ccm.IR3.SystemExceptionMinorValues.NameAlreadyUsed,
                         org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            else
            {
                completeContainer();
                completed_ = true;

                // move it in the same container to have the good order
                getContained().move(the_parent_.getContainer(), getName(), getVersion());
            }
        }
    }

    // ==================================================================
    //
    // Methods for the ForwardScope interface.
    //
    // ==================================================================

    /**
     **  Is it a forward declaration?
     **
     **  @return True if it's a forward declaration, false otherwise.
     **/
    public boolean
    isForward()
    {
        return forward_;
    }

    /**
     **  Set as forward.
     **
     **  @param f True to set the declaration as a forward declaration,
     **           false otherwise.
     **/
    public void
    setForward(boolean f)
    {
        forward_ = f;
    }

    /**
     **  Is it a completed declaration?
     **
     **  @return True if the declaration is complete, false otherwise.
     **/
    public boolean
    isCompleted()
    {
        return completed_;
    }
}
