/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages home declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface HomeDecl
       extends InterfaceDecl
{
    /**
     * Set the base home.
     *
     * @param base_home The base home.
     */
    public void
    setBaseHome(HomeRef base_home);

    /**
     * Set the managed component.
     *
     * @param managed_component The managed component.
     */
    public void
    setManagedComponent(ComponentRef managed_component);

    /**
     * Set the primary key.
     *
     * @param primary_key The primary key.
     */
    public void
    setPrimaryKey(ValueRef primary_key);

    /**
     * Add an interface.
     *
     * @param itf The supported interface to add.
     */
    public void
    addSupportedInterface(InterfaceRef itf);

    /**
     * Obtain the base home.
     *
     * @return The base home.
     */
    public HomeDecl
    getBaseHome();

    /**
     * Obtain the managed component.
     *
     * @return The managed component.
     */
    public ComponentDecl
    getManagedComponent();

    /**
     * Obtain the primary key.
     *
     * @return The primary key.
     */
    public ValueDecl
    getPrimaryKey();

    /**
     * Obtain the supported interface declarations.
     *
     * @return The supported interface declarations.
     */
    public InterfaceDecl[]
    getSupportedInterfaces();

    /**
     * Obtain the client explicit interface mapping.
     *
     * @return The client explicit interface mapping.
     */
    public InterfaceDecl
    getClientExplicitMapping();

    /**
     * Obtain the client implicit interface mapping.
     *
     * @return The client implicit interface mapping.
     */
    public InterfaceDecl
    getClientImplicitMapping();

    /**
     * Obtain the client interface mapping.
     *
     * @return The client interface mapping.
     */
    public InterfaceDecl
    getClientMapping();

    /**
     * Obtain the local interface mapping.
     *
     * @return The local interface mapping.
     */
    public InterfaceDecl
    getLocalMapping();
}
