/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages value initializer declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class InitializerImpl
       extends ExceptionListImpl
       implements Initializer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    private String name_;

    /**
     **  List of parameters.
     **/
    private ParameterListImpl parameters_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the initializer declaration.
     **/
    protected
    InitializerImpl()
    {
        // Init internal state.
        name_ = "";
        parameters_ = new ParameterListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its Initializer.
     **
     **  @return A new created Initializer object.
     **/
    public org.omg.CORBA.ExtInitializer
    getExtInitializer()
    {
        String[] names = parameters_.getParamNames();
        TypeRef[] types = parameters_.getParamTypes();
        org.omg.CORBA.StructMember[] result =
            new org.omg.CORBA.StructMember[names.length];
        for(int i=0; i<result.length; i++)
        {
            result[i] = new org.omg.CORBA.StructMember(
                            names[i],
                            org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                            types[i].getIDLType());
        }

        return new org.omg.CORBA.ExtInitializer(result,
                                                getExceptionDefs(),
                                                getExceptionDescriptions(),
                                                name_);
    }

    /**
     **
     **/
    protected void
    destroy()
    {
        parameters_.destroy();
    }

    // ==================================================================
    //
    // Methods for the InitializerDecl interface.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setName(String name)
    {
        name_ = name;
    }

    /**
     **
     **/
    public String
    getName()
    {
        return name_;
    }

    /**
     **
     **/
    public ParameterList
    getParameters()
    {
        return parameters_;
    }

    // ==================================================================
    //
    // Methods for the WithInParameters interface.
    //
    // ==================================================================

    /**
     **  Add a parameter.
     **
     **  @param name The name of the parameter.
     **  @param type The TypeRef of the parameter.
     **/
    public void
    addInParam(String name,
               TypeRef type)
    {
        parameters_.addParam(name, type, OperationDecl.PARAM_IN);
    }
}
