/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL local interface declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class LocalInterfaceDeclImpl
       extends InterfaceDeclImpl
       implements LocalInterfaceRef, LocalInterfaceDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 LocalInterfaceDef.
     **/
    private org.omg.CORBA.ExtLocalInterfaceDef ext_local_interface_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the local interface declaration.
     **/
    protected
    LocalInterfaceDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the InterfaceDefImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_local_interface_def_ = null;
        the_declaration_kind_ = DeclarationKind._dk_local_interface;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_local_interface_def_ = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_local_interface_def_ = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(contained);
        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the local interface declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_local_interface_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_local_interface;
    }

    // ==================================================================
    //
    // Methods for the InterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef associated with
     **          the local interface declaration.
     **/
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef associated with
     **          the local interface declaration.
     **/
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the InterfaceDecl interface.
    //
    // ==================================================================

    /**
     **  Add an interface.
     **
     **  @param itf The base interface to add.
     **/
/*
 * same pb as with abstract interface
 *
    public void
    addInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            inherited_interfaces_.addElement(
                 ((LocalInterfaceRef)itf)
            );
    }
 */

    // ==================================================================
    //
    // Methods for the LocalInterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its LocalInterfaceDef reference.
     **
     **  @return The LocalInterfaceDef associated with the local
     **          interface declaration.
     **/
    public org.omg.CORBA.ExtLocalInterfaceDef
    getExtLocalInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    /**
     **  Obtain its LocalInterfaceDef reference.
     **
     **  @return The LocalInterfaceDef associated with the local
     **          interface declaration.
     **/
    public org.omg.CORBA.LocalInterfaceDef
    getLocalInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the local
     **          interface declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the local
     **          interface declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        org.omg.CORBA.InterfaceDef[] itfs =
            new org.omg.CORBA.InterfaceDef[inherited_interfaces_.size()];

        for(int i=0; i<itfs.length; i++)
            itfs[i] = ((InterfaceDeclImpl)(inherited_interfaces_.get(i))).getExtInterfaceDef();

        ext_local_interface_def_ = the_parent_.getContainer().create_ext_local_interface(getId(), getName(),
                                                                                         getVersion(), itfs);
    }
}
