/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL module declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ModuleDeclImpl
       extends ForwardScopeImpl
       implements ModuleDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ModuleDef.
     **/
    private org.omg.CORBA.ComponentIR.ModuleDef component_module_def_;

    /**
     **
     **/
    private boolean completed_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the module declaration.
     **/
    protected
    ModuleDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        component_module_def_ = null;
        the_declaration_kind_ = DeclarationKind._dk_module;
        completed_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ModuleDef.
     **
     **  @param moduleDef The ModuleDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        component_module_def_ = org.omg.CORBA.ComponentIR.ModuleDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 ModuleDef.
     **
     **  @param moduleDef The ModuleDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        component_module_def_ = org.omg.CORBA.ComponentIR.ModuleDefHelper.narrow(contained);
        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the interface declaration into the IFR.
     **/
    public void
    create()
    {
        if (getContainer() == null)
	{
            createContainer();
            if (the_parent_!=null)
                the_parent_.addDecl(this);
        }
        else if (!completed_)
            completeContainer();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        Declaration[] decls = getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);
        Declaration[] depend = null;
        org.objectweb.ccm.util.Vector module_depend = new org.objectweb.ccm.util.Vector();

        // contents
        for (int i=0;i<decls.length;i++)
        {
            depend = decls[i].getDependencies(recursion);
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (module_depend.indexOf(depend[j])==-1))
                    module_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])module_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the module declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return component_module_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        // Create it only if it was not previously created.
        if (component_module_def_==null)
        {
           // Create a ModuleDef into the IFR.
           component_module_def_ =
               the_parent_.getComponentContainer().create_component_module(getId(), getName(), getVersion());
        }
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        completed_ = true;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return component_module_def_;
    }

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
       return component_module_def_;
    }
}
