/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL native declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class NativeDeclImpl
       extends DeclarationImpl
       implements TypeRef, NativeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 NativeDef.
     **/
    private org.omg.CORBA.NativeDef native_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the native declaration.
     **/
    protected
    NativeDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        native_def_ = null;
        the_declaration_kind_ = DeclarationKind._dk_native;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 NativeDef.
     **
     **  @param native_def The NativeDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        native_def_ = org.omg.CORBA.NativeDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 NativeDef.
     **
     **  @param native_def The NativeDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        native_def_ = org.omg.CORBA.NativeDefHelper.narrow(contained);
        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the native declaration into the IFR.
     **/
    public void
    create()
    {
        // Create a NativeDef into the IFR.
        native_def_ = the_parent_.getContainer().create_native(getId(), getName(), getVersion());

        super.create();
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the native declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return native_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_native;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the native declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return native_def_;
    }
}
