/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages IDL operation declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface OperationDecl
       extends Declaration, WithInParameters
{
    /** TODO: COMMENT */
    public static int OP_NORMAL = org.omg.CORBA.OperationMode._OP_NORMAL;

    /** TODO: COMMENT */
    public static int OP_ONEWAY = org.omg.CORBA.OperationMode._OP_ONEWAY;

    /** TODO: COMMENT */
    public static int PARAM_IN = org.omg.CORBA.ParameterMode._PARAM_IN;

    /** TODO: COMMENT */
    public static int PARAM_OUT = org.omg.CORBA.ParameterMode._PARAM_OUT;

    /** TODO: COMMENT */
    public static int PARAM_INOUT = org.omg.CORBA.ParameterMode._PARAM_INOUT;

    /**
     * Set mode as normal.
     */
    public void
    setNormal();

    /**
     * Set mode as oneway.
     */
    public void
    setOneway();

    /**
     * Set the result type.
     *
     * @param type The TypeRef of the returned object.
     */
    public void
    setResultType(TypeRef type);

    /**
     * Add an out parameter.
     *
     * @param name The name of the parameter.
     * @param type The TypeRef of the parameter.
     */
    public void
    addOutParam(String name, TypeRef type);

    /**
     * Add an inout parameter.
     *
     * @param name The name of the parameter.
     * @param type The TypeRef of the parameter.
     */
    public void
    addInOutParam(String name, TypeRef type);

    /**
     * Add a context.
     *
     * @param name The name of the context.
     */
    public void
    addContext(String name);

    /**
     * Is it oneway?
     *
     * @return True if oneway operation, else false.
     */
    public boolean
    isOneway();

    /**
     * Obtain the result type.
     *
     * @return The result type.
     */
    public TypeRef
    getType();

    /**
     * Obtain the exception list.
     *
     * @return The exception list.
     */
    public ExceptionList
    getExceptionList();

    /**
     **  Obtain the exceptions list.
     **
     **  @return The list of exceptions that can be raised.
     **/
    public ExceptionDecl[]
    getExceptions();

    /**
     * Obtain the parameter list.
     *
     * @return The parameter list.
     */
    public ParameterList
    getParameters();

    /**
     * Obtain the contexts.
     *
     * @return The contexts.
     */
    public String[]
    getContexts();
}
