/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL parameter lists.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ParameterListImpl
       implements ParameterList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  List of members names.
     **/
    protected org.objectweb.ccm.util.Vector param_names_;

    /**
     **  List of members TypeRef.
     **/
    protected org.objectweb.ccm.util.Vector param_types_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector param_modes_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public ParameterListImpl()
    {
        // Init internal state.
        param_names_ = new org.objectweb.ccm.util.Vector();
        param_types_ = new org.objectweb.ccm.util.Vector();
        param_modes_ = new org.objectweb.ccm.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its ParameterDescription[].
     **
     **  @return An array containing the parameters description.
     **/
    protected org.omg.CORBA.ParameterDescription[]
    getParameterDescs()
    {
        org.omg.CORBA.ParameterDescription[] result =
            new org.omg.CORBA.ParameterDescription[param_names_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = new org.omg.CORBA.ParameterDescription(
                (String)param_names_.get(i),
                org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                ((TypeRef)param_types_.get(i)).getIDLType(),
                org.omg.CORBA.ParameterMode.from_int(((Integer)param_modes_.get(i)).intValue()));

        }
        return result;
    }

    /**
     **
     **/
    protected void
    destroy()
    {
        for (int i=0;i<param_types_.size();i++)
        {
            ((TypeRef)param_types_.get(i)).removeRef();
        }
    }

    // ==================================================================
    //
    // Methods for the ParameterList interface.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    addParam(String name,
             TypeRef type,
             int mode)
    {
        if (type!=null)
        {
            param_names_.add(name);
            type.addRef();
            param_types_.add(type);
            param_modes_.add(new Integer(mode));
        }
    }

    /**
     **
     **/
    public String[]
    getParamNames()
    {
        String[] result = new String[param_names_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (String)param_names_.get(i);
        }
        return result;
    }

    /**
     **
     **/
    public int[]
    getParamModes()
    {
        int[] result = new int[param_modes_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = ((Integer)param_modes_.get(i)).intValue();
        }
        return result;
    }

    /**
     **
     **/
    public TypeRef[]
    getParamTypes()
    {
        TypeRef[] result = new TypeRef[param_types_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (TypeRef)param_types_.get(i);
        }
        return result;
    }
}
