/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface is the factory for primitive TypeRefs.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface PrimitiveTypeFactory
{
    /**
     * Obtain the null type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_null();

    /**
     * Obtain the void type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_void();

    /**
     * Obtain the short type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_short();

    /**
     * Obtain the long type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_long();

    /**
     * Obtain the unsigned short type.
     *
     *  @return The new created TypeRef.
     */
    public TypeRef
    type_unsigned_short();

    /**
     * Obtain the unsigned long type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_unsigned_long();

    /**
     * Obtain the float type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_float();

    /**
     * Obtain the double type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_double();

    /**
     * Obtain the boolean type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_boolean();

    /**
     * Obtain the char type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_char();

    /**
     * Obtain the octet type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_octet();

    /**
     * Obtain the any type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_any();

    /**
     * Obtain the TypeCode type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_TypeCode();

    /**
     * Obtain the Principal type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_Principal();

    /**
     * Obtain the string type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_string();

    /**
     * Obtain the Object type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_Object();

    /**
     * Obtain the long long type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_long_long();

    /**
     * Obtain the unsigned long long type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_unsigned_long_long();

    /**
     * Obtain the long double type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_long_double();

    /**
     * Obtain the wchar type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_wchar();

    /**
     * Obtain the wstring type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_wstring();

    /**
     * Obtain the value base type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    type_value_base();

    /**
     * Create an anonymous string<bound> type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createString(int bound);

    /**
     * Create an anonymous wstring<bound> type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createWstring(int bound);

    /**
     * Create an anonymous sequence<bound,element_type> type.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createSequence(int bound,
                   TypeRef element_type);

    /**
     * Create an anonymous array type, i.e. element_type[length].
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createArray(int length,
                TypeRef element_type);

    /**
     * Create a fixed type, i.e. fixed<digits,scale>.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createFixed(short digits,
                short scale);
}
