/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages the Repository scope.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class Repository
       extends ScopeImpl
       implements PrimitiveTypeFactory   
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the PrimitiveDef for the null type.
     **/
    private TypeRefImpl null_ = null;

    /**
     **  Reference to the PrimitiveDef for the void type.
     **/
    private TypeRefImpl void_ = null;

    /**
     **  Reference to the PrimitiveDef for the short type.
     **/
    private TypeRefImpl short_ = null;

    /**
     **  Reference to the PrimitiveDef for the long type.
     **/
    private TypeRefImpl long_ = null;

    /**
     **  Reference to the PrimitiveDef for the unsigned short type.
     **/
    private TypeRefImpl unsigned_short_ = null;

    /**
     **  Reference to the PrimitiveDef for the unsigned long type.
     **/
    private TypeRefImpl unsigned_long_ = null;

    /**
     **  Reference to the PrimitiveDef for the float type.
     **/
    private TypeRefImpl float_ = null;

    /**
     **  Reference to the PrimitiveDef for the double type.
     **/
    private TypeRefImpl double_ = null;

    /**
     **  Reference to the PrimitiveDef for the boolean type.
     **/
    private TypeRefImpl boolean_ = null;

    /**
     **  Reference to the PrimitiveDef for the char type.
     **/
    private TypeRefImpl char_ = null;

    /**
     **  Reference to the PrimitiveDef for the octet type.
     **/
    private TypeRefImpl octet_ = null;

    /**
     **  Reference to the PrimitiveDef for the any type.
     **/
    private TypeRefImpl any_ = null;

    /**
     **  Reference to the PrimitiveDef for the TypeCode type.
     **/
    private TypeRefImpl TypeCode_ = null;

    /**
     **  Reference to the PrimitiveDef for the Principal type.
     **/
    private TypeRefImpl Principal_ = null;

    /**
     **  Reference to the PrimitiveDef for the string type.
     **/
    private TypeRefImpl string_ = null;

    /**
     **  Reference to the PrimitiveDef for the Object type.
     **/
    private TypeRefImpl object_ = null;

    /**
     **  Reference to the PrimitiveDef for the long long type.
     **/
    private TypeRefImpl long_long_ = null;

    /**
     **  Reference to the PrimitiveDef for the unsigned long long type.
     **/
    private TypeRefImpl unsigned_long_long_ = null;

    /**
     **  Reference to the PrimitiveDef for the long double type.
     **/
    private TypeRefImpl long_double_ = null;

    /**
     **  Reference to the PrimitiveDef for the wchar type.
     **/
    private TypeRefImpl wchar_ = null;

    /**
     **  Reference to the PrimitiveDef for the wstring type.
     **/
    private TypeRefImpl wstring_ = null;

    /**
     **  Reference to the PrimitiveDef for the value base type.
     **/
    private TypeRefImpl value_base_ = null;

    /**
     **
     **/
    private org.objectweb.ccm.util.Table declarations_;

    /**
     **
     **/
    private org.objectweb.ccm.util.Table mapped_declarations_;

    /**
     **  Reference to the CORBA 3.0 Repository.
     **/
    private org.omg.CORBA.Repository repository_;

    /**
     **
     **/
    private org.objectweb.ccm.ComponentRepository openccm_repository_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param repository The CORBA Repository that should be used.
     **/
    public 
    Repository(org.omg.CORBA.Repository repository)
    {
        // Call the ScopeImpl constructor.
        super(null, null);

        // Init internal state.
        the_declaration_kind_ = DeclarationKind._dk_repository;
        repository_ = repository;
        openccm_repository_ = org.objectweb.ccm.ComponentRepositoryHelper.narrow(repository);
        declarations_ = new org.objectweb.ccm.util.Table();
        mapped_declarations_ = new org.objectweb.ccm.util.Table();
        the_repository_ = this;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    addDeclInRep(String rep_id, Declaration decl)
    {
        declarations_.put(rep_id, decl);
    }

    /**
     **
     **/
    protected void
    removeDeclFromRep(String rep_id)
    {
        declarations_.remove(rep_id);
    }

    /**
     **
     **/
    protected void
    addMappedDeclInRep(String rep_id, Declaration decl)
    {
        mapped_declarations_.put(rep_id, decl);
    }

    /**
     **
     **/
    protected void
    removeMappedDeclFromRep(String rep_id)
    {
        mapped_declarations_.remove(rep_id);
    }

    /**
     **  Obtain a newly created Declaration of a IR3 Contained.
     **  Note: only the following Contained can be loaded :
     **    dk_Module, dk_Interface, dk_LocalInterface, dk_AbstractInterface,
     **    dk_Value, dk_Struct, dk_Union, dk_Exception, dk_Component, dk_Home,
     **    dk_Event.
     **
     **  @param contained The Contained to use.
     **  @param scope The Scope in which the Contained will be loaded.
     **  @return The associated Declaration or null if it not exists or if 
     **          the DefinitionKind is not valid.
     **/
    protected Declaration
    loadContained(Scope scope, org.omg.CORBA.Contained contained)
    {
        ScopeImpl si = (ScopeImpl)scope;
        int kind = contained.def_kind().value();

        if ((kind!=org.omg.CORBA.DefinitionKind._dk_Module) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Interface) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_LocalInterface) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_AbstractInterface) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Value) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Struct) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Union) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Exception) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Component) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Home) &&
            (kind!=org.omg.CORBA.DefinitionKind._dk_Event))
            return null;

        return si.loadContained(contained);
    }

    /**
     **
     **/
    protected TypeRef
    getAsTypeRef(org.omg.CORBA.IDLType type,
                 boolean is_mapped)
    {
        org.omg.CORBA.TypeCode tc = type.type();
        try
        {
            switch(tc.kind().value())
            {
            case org.omg.CORBA.TCKind._tk_null:
                return type_null();
            case org.omg.CORBA.TCKind._tk_void:
                return type_void();
            case org.omg.CORBA.TCKind._tk_short:
                return type_short();
            case org.omg.CORBA.TCKind._tk_long:
                return type_long();
            case org.omg.CORBA.TCKind._tk_ushort:
                return type_unsigned_short();
            case org.omg.CORBA.TCKind._tk_ulong:
                return type_unsigned_long();
            case org.omg.CORBA.TCKind._tk_float:
                return type_float();
            case org.omg.CORBA.TCKind._tk_double:
                return type_double();
            case org.omg.CORBA.TCKind._tk_boolean:
                return type_boolean();
            case org.omg.CORBA.TCKind._tk_char:
                return type_char();
            case org.omg.CORBA.TCKind._tk_octet:
                return type_octet();
            case org.omg.CORBA.TCKind._tk_any:
                return type_any();
            case org.omg.CORBA.TCKind._tk_TypeCode:
                return type_TypeCode();
            case org.omg.CORBA.TCKind._tk_Principal:
                return type_Principal();
            case org.omg.CORBA.TCKind._tk_longlong:
                return type_long_long();
            case org.omg.CORBA.TCKind._tk_ulonglong:
                return type_unsigned_long_long();
            case org.omg.CORBA.TCKind._tk_longdouble:
                return type_long_double();
            case org.omg.CORBA.TCKind._tk_wchar:
                return type_wchar();

            case org.omg.CORBA.TCKind._tk_string:
                TypeRefImpl tr = new TypeRefImpl(type, TypeKind._tk_string);
                tr.setLength(tc.length());
                return tr;
            case org.omg.CORBA.TCKind._tk_sequence:
                tr = new TypeRefImpl(type, TypeKind._tk_sequence);
                tr.setLength(tc.length());
                org.omg.CORBA.SequenceDef seq = org.omg.CORBA.SequenceDefHelper.narrow(type);
                tr.setContentType(getAsTypeRef(seq.element_type_def()));
                return tr;
            case org.omg.CORBA.TCKind._tk_array:
                tr = new TypeRefImpl(type, TypeKind._tk_array);
                tr.setLength(tc.length());
                org.omg.CORBA.ArrayDef arr = org.omg.CORBA.ArrayDefHelper.narrow(type);
                tr.setContentType(getAsTypeRef(arr.element_type_def()));
                return tr;
            case org.omg.CORBA.TCKind._tk_wstring:
                tr = new TypeRefImpl(type, TypeKind._tk_wstring);
                tr.setLength(tc.length());
                return tr;
            case org.omg.CORBA.TCKind._tk_fixed:
                tr = new TypeRefImpl(type, TypeKind._tk_fixed);
                tr.setDigits(tc.fixed_digits());
                tr.setScale(tc.fixed_scale());
                return tr;

            case org.omg.CORBA.TCKind._tk_objref:
              if (tc.id().equals("IDL:omg.org/CORBA/Object:1.0"))
                return type_Object();

            case org.omg.CORBA.TCKind._tk_struct:
            case org.omg.CORBA.TCKind._tk_union:
            case org.omg.CORBA.TCKind._tk_enum:
            case org.omg.CORBA.TCKind._tk_alias:
            case org.omg.CORBA.TCKind._tk_value:
            case org.omg.CORBA.TCKind._tk_value_box:
            case org.omg.CORBA.TCKind._tk_native:
            case org.omg.CORBA.TCKind._tk_abstract_interface:
/*
 * not defined !
            case org.omg.CORBA.TCKind._tk_local_interface:
            case org.omg.CORBA.TCKind_.tk_event:
            case org.omg.CORBA.TCKind._tk_home:
            case org.omg.CORBA.TCKind._tk_component:
 */
                if (is_mapped)
                    return (TypeRef)lookupMappedId(tc.id());
                else
                    return (TypeRef)lookupId(tc.id());

            //
            // PB : an exception is not an IDLType
            // so it can't have an associated TypeRef
            //
            case org.omg.CORBA.TCKind._tk_except:
            default:
                return type_null();
            }
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            throw new Error("Should not have happened !");
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    useIDL3Repository()
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_!=null)
            openccm_repository_.as_IDL3_repository();
    }

    /**
     **
     **/
    public void
    useIDL2Repository()
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_!=null)
            openccm_repository_.as_IDL2_repository();
    }

    // ==================================================================
    //
    // Public methods for IRObject loading (for import statement).
    //
    // ==================================================================

    /**
     **  Loads a IR3 Contained in a IDL3 Scope.
     **
     **  @param scope The Scope in which the Contained will be loaded.
     **  @param name The name of the Contained.
     **
     **  @return The newly created Declaration associated with the Contained.
     **/
    public Declaration
    load(Scope scope, String name)
    {
        org.omg.CORBA.Contained decl = repository_.lookup(name);
        if (decl==null)
            return null;

        return loadContained(scope, decl);
    }

    /**
     **  Loads a IR3 Contained in a IDL3 Scope.
     **
     **  @param scope The Scope in which the Contained will be loaded.
     **  @param id The RepositoryId of the Contained.
     **
     **  @return The newly created Declaration associated with the Contained.
     **/
    public Declaration
    loadId(Scope scope, String id)
    {
        org.omg.CORBA.Contained decl = repository_.lookup_id(id);
        if (decl==null)
            return null;

        return loadContained(scope, decl);
    }

    /**
     **
     **/
    public Declaration
    loadMapping(Scope scope, String rep_id)
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_==null)
            return null;

        // use the OpenCCM repository as an IDL2 repository to extract the mapping !!!
        openccm_repository_.as_IDL2_repository();

        // check if it was not already loaded.
        Declaration result = (Declaration)mapped_declarations_.get(rep_id);
        if (result!=null)
            return result;

        org.omg.CORBA.Contained decl = openccm_repository_.lookup_mapped_id(rep_id);
        if (decl==null)
            return null;

        // load the contained.
        ScopeImpl si = (ScopeImpl)scope;
        result = si.loadContainedAsMapping(decl);

        // get back to an IDL3 repository.
        openccm_repository_.as_IDL3_repository();

        return result;
    }

    // ==================================================================
    //
    // Public methods for IR3 infos extraction.
    //
    // ==================================================================

    /**
     **  To obtain the Declaration object with it's absolute name.
     **
     **  @param abs_name The absolute name of the searched Declaration object.
     **
     **  @return The Declaration object or null if the absolute
     **          name was not found.
     **/
    public Declaration
    lookup(String scoped_name)
    {
        if (scoped_name.startsWith("::"))
            return super.lookup(scoped_name.substring(2));
        else
            return super.lookup(scoped_name);
    }

    /**
     **
     **/
    public Declaration
    lookupMappedId(String rep_id)
    {
        Object obj = mapped_declarations_.get(rep_id);
        if (obj!=null)
            return (Declaration)obj;

        // if not found then load it from the IR3
        org.omg.CORBA.Contained decl = openccm_repository_.lookup_mapped_id(rep_id);

        // if not found, it's a "normal" id
        if (decl==null)
            return lookupId(rep_id);

        // else load it as a mapped id
        org.omg.CORBA.Contained decl2 = null;
        String abs_name = decl.absolute_name();
        String name = "";
        if (abs_name.startsWith("::"))
            abs_name = abs_name.substring(2, abs_name.length());

        int idx = 0;
        ScopeImpl scope = null;
        obj = this;
        while (true)
        {
            scope = (ScopeImpl)obj;
            idx = abs_name.indexOf(':');
            if (idx==-1)
                return scope.loadContainedAsMapping(decl);
            name = abs_name.substring(0, idx);
            abs_name = abs_name.substring(idx+2, abs_name.length());
            obj = scope.findInScope(name);
            if (obj==null)
            {
                decl2 = scope.getContainer().lookup(name);
                obj = scope.loadContainedAsMapping(decl2);
            }
        }
    }

    /**
     **
     **/
    public Declaration
    lookupId(String rep_id)
    {
        Object obj = declarations_.get(rep_id);

        if (obj!=null)
            return (Declaration)obj;

        // if not found then load it from the IR3
        org.omg.CORBA.Contained decl = repository_.lookup_id(rep_id);
        org.omg.CORBA.Contained decl2 = null;
        String abs_name = decl.absolute_name();
        String name = "";
        if (abs_name.startsWith("::"))
            abs_name = abs_name.substring(2, abs_name.length());

        int idx = 0;
        ScopeImpl scope = null;
        obj = this;
        while (true)
        {
            scope = (ScopeImpl)obj;
            idx = abs_name.indexOf(':');
            if (idx==-1)
                return scope.loadContained(decl);
            name = abs_name.substring(0, idx);
            abs_name = abs_name.substring(idx+2, abs_name.length());
            obj = scope.findInScope(name);
            if (obj==null)
            {
                decl2 = scope.getContainer().lookup(name);
                obj = scope.loadContained(decl2);
            }
        }
    }

    // ==================================================================
    //
    // Methods for the PrimitiveTypeFactory interface.
    //
    // ==================================================================

    /**
     **
     **/
    public TypeRef
    getAsMappedTypeRef(org.omg.CORBA.IDLType type)
    {
        return getAsTypeRef(type, true);
    }

    /**
     **
     **/
    public TypeRef
    getAsTypeRef(org.omg.CORBA.IDLType type)
    {
        return getAsTypeRef(type, false);
    }

    /**
     **  Obtain the null type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_null()
    {
       if (null_ == null)
       {
           null_ = new TypeRefImpl(
              repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_null),
              TypeKind._tk_null);
       }
       return null_;
    }

    /**
     **  Obtain the void type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_void()
    {
       if (void_ == null)
       {
	   void_ = new TypeRefImpl(
               repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_void),
               TypeKind._tk_void);
       }
       return void_;
    }

    /**
     **  Obtain the short type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_short()
    {
       if (short_ == null)
       {
	   short_ = new TypeRefImpl(
              repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_short),
              TypeKind._tk_short);
       }
       return short_;
    }

    /**
     **  Obtain the long type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_long()
    {
       if (long_ == null)
       {
	   long_ = new TypeRefImpl(
               repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_long),
               TypeKind._tk_long);
       }
       return long_;
    }

    /**
     **  Obtain the unsigned short type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_unsigned_short()
    {
       if (unsigned_short_ == null)
       {
	   unsigned_short_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_ushort),
             TypeKind._tk_ushort);
       }
       return unsigned_short_;
    }

    /**
     **  Obtain the unsigned long type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_unsigned_long()
    {
       if (unsigned_long_ == null)
       {
	   unsigned_long_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_ulong),
             TypeKind._tk_ulong);
       }
       return unsigned_long_;
    }

    /**
     **  Obtain the float type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_float()
    {
       if (float_ == null)
       {
	   float_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_float),
             TypeKind._tk_float);
       }
       return float_;
    }

    /**
     **  Obtain the double type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_double()
    {
       if (double_ == null)
       {
	   double_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_double),
             TypeKind._tk_double);
       }
       return double_;
    }

    /**
     **  Obtain the boolean type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_boolean()
    {
       if (boolean_ == null)
       {
	   boolean_ = new TypeRefImpl(
           repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_boolean),
           TypeKind._tk_boolean);
       }
       return boolean_;
    }

    /**
     **  Obtain the char type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_char()
    {
       if (char_ == null)
       {
	   char_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_char),
             TypeKind._tk_char);
       }
       return char_;
    }

    /**
     **  Obtain the octet type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_octet()
    {
       if (octet_ == null)
       {
	   octet_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_octet),
             TypeKind._tk_octet);
       }
       return octet_;
    }

    /**
     **  Obtain the any type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_any()
    {
       if (any_ == null)
       {
	   any_ = new TypeRefImpl(
	       repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_any),
           TypeKind._tk_any);
       }
       return any_;
    }

    /**
     **  Obtain the TypeCode type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_TypeCode()
    {
       if (TypeCode_ == null)
       {
	   TypeCode_ =new TypeRefImpl(
           repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_TypeCode),
           TypeKind._tk_TypeCode);
       }
       return TypeCode_;
    }

    /**
     **  Obtain the Principal type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_Principal()
    {
       if (Principal_ == null)
       {
	   Principal_ =new TypeRefImpl(
	   repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_Principal),
       TypeKind._tk_Principal);
       }
       return Principal_;
    }

    /**
     **  Obtain the string type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_string()
    {
       if (string_ == null)
       {
	   string_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_string),
             TypeKind._tk_string);
       }
       return string_;
    }

    /**
     **  Obtain the Object type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_Object()
    {
       if (object_ == null)
       {
	   object_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_objref),
             TypeKind._tk_objref);
       }
       return object_;
    }

    /**
     **  Obtain the long long type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_long_long()
    {
       if (long_long_ == null)
       {
	   long_long_ = new TypeRefImpl(
               repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_longlong),
               TypeKind._tk_longlong);
       }
       return long_long_;
    }

    /**
     **  Obtain the unsigned long long type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_unsigned_long_long()
    {
       if (unsigned_long_long_ == null)
       {
	   unsigned_long_long_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_ulonglong),
             TypeKind._tk_ulonglong);
       }
       return unsigned_long_long_;
    }

    /**
     **  Obtain the long double type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_long_double()
    {
       if (long_double_ == null)
       {
	   long_double_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_longdouble),
             TypeKind._tk_longdouble);
       }
       return long_double_;
    }

    /**
     **  Obtain the wchar type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_wchar()
    {
       if (wchar_ == null)
       {
	   wchar_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_wchar),
             TypeKind._tk_wchar);
       }
       return wchar_;
    }

    /**
     **  Obtain the wstring type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_wstring()
    {
       if (wstring_ == null)
       {
	   wstring_ = new TypeRefImpl(
            repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_wstring),
            TypeKind._tk_wstring);
       }
       return wstring_;
    }

    /**
     **  Obtain the value base type.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    type_value_base()
    {
       if (value_base_ == null)
       {
	   value_base_ = new TypeRefImpl(
             repository_.get_primitive(org.omg.CORBA.PrimitiveKind.pk_value_base),
             //
             // PB : no TypeCode is associated with value base
             //
             TypeKind._tk_null);
       }
       return value_base_;
    }

    /**
     **  Create an anonymous string<bound> type.
     **
     **  @param bound The length of the string.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    createString(int bound)
    {
        // Create a StringDef into the IFR.
        org.omg.CORBA.StringDef stringDef = repository_.create_string(bound);
        TypeRefImpl tr = new TypeRefImpl(stringDef, TypeKind._tk_string);
        tr.setLength(bound);
        return tr;
    }

    /**
     **  Create an anonymous wstring<bound> type.
     **
     **  @param bound The length of the wstring.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    createWstring(int bound)
    {
        // Create a WstringDef into the IFR.
        org.omg.CORBA.WstringDef wstringDef = repository_.create_wstring(bound);
        TypeRefImpl tr = new TypeRefImpl(wstringDef, TypeKind._tk_wstring);
        tr.setLength(bound);
        return tr;
    }

    /**
     **  Create an anonymous sequence<bound,element_type> type.
     **
     **  @param bound The length of the sequence.
     **  @param element_type The TypeRef of the elements.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    createSequence(int bound,
                   TypeRef element_type)
    {
        if(element_type == null)
            return null;

        // Create a SequenceDef into the IFR.
        org.omg.CORBA.SequenceDef sequenceDef =
            repository_.create_sequence(bound, element_type.getIDLType());
        TypeRefImpl tr =  new TypeRefImpl(sequenceDef, TypeKind._tk_sequence);
        tr.setLength(bound);
        tr.setContentType(element_type);
        return tr;
    }

    /**
     **  Create an anonymous array type, i.e. element_type[length].
     **
     **  @param length The length of the array.
     **  @param element_type The TypeRef of the elements.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    createArray(int length,
                TypeRef element_type)
    {
        if(element_type == null)
            return null;

        // Create an ArrayDef into the IFR.
        org.omg.CORBA.ArrayDef arrayDef =
             repository_.create_array(length, element_type.getIDLType());
        TypeRefImpl tr =  new TypeRefImpl(arrayDef, TypeKind._tk_array);
        tr.setLength(length);
        tr.setContentType(element_type);
        return tr;
    }

    /**
     **  Create a fixed type, i.e. fixed<digits,scale>.
     **
     **  @param digits The number of decimal digits.
     **  @param scale The position of the decimal point.
     **
     **  @return The new created TypeRef.
     **/
    public TypeRef
    createFixed(short digits,
                short scale)
    {
        // Create a FixedDef into the IFR.
        org.omg.CORBA.FixedDef fixedDef =
            repository_.create_fixed (digits,scale);
        TypeRefImpl tr = new TypeRefImpl(fixedDef, TypeKind._tk_fixed);
        tr.setDigits(digits);
        tr.setScale(scale);
        return tr;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return null because a Repository does not support the
     **          Contained interface.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Repository.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return repository_;
    }

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
        return openccm_repository_;
    }
}
