/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL struct members.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructMembersImpl
       implements StructMembers
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  List of members names.
     **/
    private org.objectweb.ccm.util.Vector member_names_;

    /**
     **  List of members TypeRef.
     **/
    private org.objectweb.ccm.util.Vector member_types_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    StructMembersImpl()
    {
        // Init internal state.
        member_names_ = new org.objectweb.ccm.util.Vector();
        member_types_ = new org.objectweb.ccm.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its StructMember[].
     **
     **  @return An array containing the members of the struct.
     **/
    protected org.omg.CORBA.StructMember[]
    getMemberDefs()
    {
        org.omg.CORBA.StructMember[] result =
            new org.omg.CORBA.StructMember[member_names_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = new org.omg.CORBA.StructMember(
                            (String)member_names_.get(i),
                            org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                            ((TypeRef)member_types_.get(i)).getIDLType());
        }

        return result;
    }

    /**
     **  Return the index name.
     **
     **  @param index The index of the member that should be returned.
     **
     **  @return The name of the member at the specified index.
     **/
    protected String
    nameAt(int index)
    {
        return (String)member_names_.get(index);
    }

    /**
     **
     **/
    protected void
    destroy()
    {
        for (int i=0;i<member_types_.size();i++)
        {
            ((TypeRef)member_types_.get(i)).removeRef();
        }
    }

    // ==================================================================
    //
    // Methods for the StructMembers interface.
    //
    // ==================================================================

    /**
     **  Add a member.
     **
     **  @param name The name of the member.
     **  @param type The TypeRef of the member.
     **/
    public void
    addMember(String name,
              TypeRef type)
    {
        if(type != null)
        {
            member_names_.add(name);
            type.addRef();
            member_types_.add(type);
        }
    }

    /**
     **
     **/
    public String[]
    getMemberNames()
    {
        String[] result = new String[member_names_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (String)member_names_.get(i);
        }
        return result;
    }

    /**
     **
     **/
    public TypeRef[]
    getMemberTypes()
    {
        TypeRef[] result = new TypeRef[member_types_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (TypeRef)member_types_.get(i);
        }
        return result;
    }
}
