/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class implements IDL type declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class TypeDeclImpl
       extends DeclarationImpl
       implements TypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 IDLType.
     **/
    private org.omg.CORBA.IDLType type_;

    /**
     **
     **/
    private int kind_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **  @param type The IDLType of the type declaration.
     **/
    protected
    TypeDeclImpl(Repository rep,
                 ScopeImpl parent,
                 org.omg.CORBA.IDLType type,
                 int kind)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        type_ = type;
        kind_ = kind;
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the constant declaration into the IFR.
     **/
    public void
    create()
    {
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the type declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return type_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return kind_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the type declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }
}
