/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface returns the reference to the IR3 IDLType object.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface TypeRef
       extends Reference
{
    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType();

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public int
    getTypeKind();

    /**
     * Obtain the type external dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies();

    /**
     * Obtain the type external dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies(boolean recursion);

    /**
     * To know if a type reference is in fact a declaration reference.
     *
     * @return True if the type is a declaration, false otherwise.
     */
    public boolean
    isDeclaration();
}
