/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class contains the reference to the IR3 IDLType object.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class TypeRefImpl
       implements TypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 IDLType.
     **/
    private org.omg.CORBA.IDLType type_;
 
    /**
     **
     **/
    private int kind_;

    /**
     **
     **/
    private int length_;
    private int scale_;
    private TypeRef content_;

    /**
     **
     **/
    private Declaration[] dependencies_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param type The IDLType that should be referenced.
     **/
    protected
    TypeRefImpl(org.omg.CORBA.IDLType type,
                int kind)
    {
        // Init internal state.
        type_ = type;
        kind_ = kind;
        length_ = 0;
        scale_ = 0;
        content_ = null;
        dependencies_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the type reference.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return type_;
    }

    /**
     **
     **/
    public void
    addRef()
    {
        // nothing to do
    }

    /**
     **
     **/
    public void
    removeRef()
    {
        // nothing to do
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return kind_;
    }

    /**
     **  Obtain the type external dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the type external dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        if ((kind_!=TypeKind._tk_sequence) &&
            (kind_!=TypeKind._tk_array))
            dependencies_ = new Declaration[0];
        else if (content_.isDeclaration())
        {
            Declaration[] depend = content_.getDependencies(recursion);
            Declaration[] depend2 = new Declaration[1+depend.length];
            for (int i=0;i<depend.length;i++)
                depend2[i] = depend[i];

            depend2[depend.length] = (Declaration)content_;
            dependencies_ = depend2;
        }
        else
            dependencies_ = content_.getDependencies(recursion);

        return dependencies_;
    }

    /**
     **
     **/
    public boolean
    isDeclaration()
    {
        return false;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setLength(int length)
    {
        length_ = length;
    }

    /**
     **
     **/
    public void
    setDigits(int digits)
    {
        length_ = digits;
    }

    /**
     **
     **/
    public void
    setScale(int scale)
    {
        scale_ = scale;
    }

    /**
     **
     **/
    public void
    setContentType(TypeRef content)
    {
        content_ = content;
    }

    /**
     **
     **/
    public int
    getLength()
    {
        if ((kind_==TypeKind._tk_string) || 
            (kind_==TypeKind._tk_sequence) ||
            (kind_==TypeKind._tk_array) ||
            (kind_==TypeKind._tk_wstring))
            return length_;
        else
            throw new Error("Bad TypeKind !");
    }

    /**
     **
     **/
    public int
    getDigits()
    {
        if (kind_==TypeKind._tk_fixed)
            return length_;
        else
            throw new Error("Bad TypeKind !");
    }

    /**
     **
     **/
    public int
    getScale()
    {
        if (kind_==TypeKind._tk_fixed)
            return scale_;
        else
            throw new Error("Bad TypeKind !");
    }

    /**
     **
     **/
    public TypeRef
    getContentType()
    {
        if ((kind_==TypeKind._tk_sequence) ||
            (kind_==TypeKind._tk_array))
            return content_;
        else
            throw new Error("Bad TypeKind !");
    }

    /**
     **
     **/
    public String
    getId()
    {
        if (kind_==TypeKind._tk_objref)
            return "IDL:omg.org/CORBA/Object:1.0";

        throw new Error("Bad TypeKind !");
    }
}
