/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages IDL union members.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface UnionMembers
{
    /**
     * Add a member.
     *
     * @param name The name of the member.
     * @param type The TypeRef of the member.
     */
    public void
    addMember(String name,
              TypeRef type,
              AnyValue value);

    /**
     * Set the discriminator type.
     *
     * @param type The discriminator type.
     */
    public void
    setDiscriminator(TypeRef type);

    /**
     * Obtain the discriminator type.
     *
     * @return The discriminator type.
     */
    public TypeRef
    getDiscriminator();

    /**
     * Obtain the member names.
     *
     * @return The member names.
     */
    public String[]
    getMemberNames();

    /**
     * Obtain the member types.
     *
     * @return The member types.
     */
    public TypeRef[]
    getMemberTypes();

    /**
     * Obtain the member values.
     *
     * @return The member values.
     */
    public AnyValue[]
    getMemberValues();
}
