/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages IDL uses port declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface UsesDecl
       extends Declaration
{
    /**
     * Set as multiple.
     */
    public void
    setMultiple();

    /**
     * Set the type.
     *
     * @param itf The provided interface reference for the uses declaration.
     */
    public void
    setInterface(InterfaceRef itf);

    /**
     * Obtain the associated interface.
     *
     * @return The associated interface.
     */
    public TypeRef
    getType();

    /**
     * Is it multiple.
     *
     * @return True if multiple, else false.
     */
    public boolean
    isMultiple();

    /**
     * Obtain the client operation mapping.
     *
     * @return The client operation mapping.
     */
    public OperationDecl[]
    getClientMapping();

    /**
     * Obtain the local context operation mapping.
     *
     * @return The local context operation mapping.
     */
    public OperationDecl[]
    getLocalContextMapping();
}
