/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages IDL value type declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface ValueDecl
       extends ForwardScope
{
    /**
     * Set as custom.
     */
    public void
    setCustom();

    /**
     * Set as abstract.
     */
    public void
    setAbstract();

    /**
     * Set as truncatable.
     */
    public void
    setTruncatable();

    /**
     * Set the base value.
     *
     * @param value The base value.
     */
    public void
    setBaseValue(ValueRef value);

    /**
     * Add an abstract base value.
     *
     * @param value The abstract base value.
     */
    public void
    addAbstractValue(ValueRef value);

    /**
     * Add a supported interface.
     *
     * @param itf The supported interface.
     */
    public void
    addInterface(InterfaceRef itf);

    /**
     * Is it custom?
     *
     * @return True if custom, else false.
     */
    public boolean
    isCustom();

    /**
     * Is it abstract?
     *
     * @return True if abstract, else false.
     */
    public boolean
    isAbstract();

    /**
     * Is it truncatable?
     *
     * @return True if truncatable, else false.
     */
    public boolean
    isTruncatable();

    /**
     * Is it primary key?
     *
     * @return True if primary key, else false.
     */
    public boolean
    isPrimaryKey();

    /**
     * Obtain the base value.
     *
     * @return The base value.
     */
    public ValueDecl
    getBaseValue();

    /**
     * Obtain the abstract values.
     *
     * @return The abstract values.
     */
    public ValueDecl[]
    getAbstractValues();

    /**
     * Obtain the supported interfaces.
     *
     * @return The supported interfaces.
     */
    public InterfaceDecl[]
    getSupportedInterfaces();

    /**
     * Obtain the initializers.
     *
     * @return The initializers.
     */
    public Initializer[]
    getInitializers();
}
