/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL valuetype declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueDeclImpl
       extends ForwardScopeImpl
       implements ValueRef, ValueDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ValueDef.
     **/
    protected org.omg.CORBA.ExtValueDef ext_value_def_;

    /**
     **  Is it custom?
     **/
    protected boolean is_custom_;

    /**
     **  Is it abstract?
     **/
    protected boolean is_abstract_;

    /**
     **  The base value.
     **/
    protected ValueDeclImpl base_value_;

    /**
     **  Is it truncatable?
     **/
    protected boolean is_truncatable_;

    /**
     **  The abstract base values. 
     **/
    protected org.objectweb.ccm.util.Vector abstract_base_values_;

    /**
     **  The supported interfaces. 
     **/
    protected org.objectweb.ccm.util.Vector supported_interfaces_;

    /**
     **  The initializers. 
     **/
    protected org.objectweb.ccm.util.Vector initializers_;
 
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the value declaration.
     **/
    public
    ValueDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ForwardScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_value_def_ = null;
        is_custom_ = false;
        is_abstract_ = false;
        base_value_ = null;
        is_truncatable_ = false;
        abstract_base_values_ = new org.objectweb.ccm.util.Vector();
        supported_interfaces_ = new org.objectweb.ccm.util.Vector();
        initializers_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_value;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain the base value.
     **
     **  @return The ValueDef of the base value or null if it not exists.
     **/
    protected org.omg.CORBA.ExtValueDef
    getBaseValueDef()
    {
        return (base_value_ != null)
	     ? (base_value_.getExtValueDef()) : null;
    }

    /**
     **  Loads infos of the CORBA 3.0 ValueDef.
     **
     **  @param valueDef The ValueDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_value_def_ = org.omg.CORBA.ExtValueDefHelper.narrow(contained);
        is_custom_ = ext_value_def_.is_custom();
        is_abstract_ = ext_value_def_.is_abstract();
        is_truncatable_ = ext_value_def_.is_truncatable();
        if (ext_value_def_.base_value()!=null)
            setBaseValue((ValueRef)getRepository().lookupId(ext_value_def_.base_value().id()));

        org.omg.CORBA.ValueDef[] abstract_base_values = ext_value_def_.abstract_base_values();
        org.omg.CORBA.InterfaceDef[] supported_interfaces = ext_value_def_.supported_interfaces();
        org.omg.CORBA.ExtInitializer[] initializers = ext_value_def_.ext_initializers();

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.load(contained);

        for (int i=0;i<abstract_base_values.length;i++)
            addAbstractValue(
                (ValueRef)getRepository().lookupId(abstract_base_values[i].id()));

        for (int i=0;i<supported_interfaces.length;i++)
            addInterface(
                (InterfaceRef)getRepository().lookupId(supported_interfaces[i].id()));

        Initializer initializer;
        for (int i=0;i<initializers.length;i++)
        {
            initializer = startInitializer(initializers[i].name);
            initializers_.add(initializer);
            org.omg.CORBA.StructMember[] params = initializers[i].members;
            for (int j=0;j<params.length;j++)
            {
                initializer.addInParam(params[j].name,
                                       getRepository().getAsTypeRef(params[j].type_def));
            }
        }
    }

    /**
     **  Loads infos of the CORBA 3.0 ValueDef.
     **
     **  @param valueDef The ValueDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_value_def_ = org.omg.CORBA.ExtValueDefHelper.narrow(contained);
        is_custom_ = ext_value_def_.is_custom();
        is_abstract_ = ext_value_def_.is_abstract();
        is_truncatable_ = ext_value_def_.is_truncatable();
        if (ext_value_def_.base_value()!=null)
            setBaseValue((ValueRef)getRepository().lookupMappedId(ext_value_def_.base_value().id()));

        org.omg.CORBA.ValueDef[] abstract_base_values = ext_value_def_.abstract_base_values();
        org.omg.CORBA.InterfaceDef[] supported_interfaces = ext_value_def_.supported_interfaces();
        org.omg.CORBA.ExtInitializer[] initializers = ext_value_def_.ext_initializers();

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.loadAsMapping(contained);

        for (int i=0;i<abstract_base_values.length;i++)
            addAbstractValue(
                (ValueRef)getRepository().lookupMappedId(abstract_base_values[i].id()));

        for (int i=0;i<supported_interfaces.length;i++)
            addInterface(
                (InterfaceRef)getRepository().lookupMappedId(supported_interfaces[i].id()));

        Initializer initializer;
        for (int i=0;i<initializers.length;i++)
        {
            initializer = startInitializer(initializers[i].name);
            initializers_.add(initializer);
            org.omg.CORBA.StructMember[] params = initializers[i].members;
            for (int j=0;j<params.length;j++)
            {
                initializer.addInParam(params[j].name,
                                       getRepository().getAsMappedTypeRef(params[j].type_def));
            }
        }
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Find a declaration.
     **
     **  Note that the declaration is also recursively searched in the
     **  parent scope.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    public Declaration
    find(String name)
    {
        // Finds in the current scope.
        Declaration decl = super.find(name);
        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in base value.           
        if(base_value_ != null)
        {
            decl = base_value_.find(name);
            // If found then returns it.
            if(decl != null) return decl;
        }

        // If not found then finds in abstract values.
        for(int i=0; i<abstract_base_values_.size(); i++)
        {
            decl = ((ValueDeclImpl)(abstract_base_values_.get(i))).find(name);
            // If found then returns it.
            if(decl != null) return decl;
        }

        // If not found then finds in supported interfaces.
        for(int i=0; i<supported_interfaces_.size(); i++)
        {
            decl = ((InterfaceDeclImpl)(supported_interfaces_.get(i))).find(name);
            // If found then returns it.
            if(decl != null) return decl;
        }

        // Else not found.
        return null;
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        org.objectweb.ccm.util.Vector value_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // base value
        if (getBaseValue()!=null)
        {
            value_depend.add(getBaseValue());
            if (recursion) {
                depend = getBaseValue().getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (depend[j]!=this)
                        value_depend.add(depend[j]);
                }
            }
        }

        // abstract base values
        ValueDecl[] abs = getAbstractValues();
        for (int i=0;i<abs.length;i++)
        {
            value_depend.add(abs[i]);
            if (recursion) {
                depend = abs[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((depend[j]!=this) &&
                        (value_depend.indexOf(depend[j])==-1))
                        value_depend.add(depend[j]);
                }
            }
        }

        // supported interfaces
        InterfaceDecl[] supp = getSupportedInterfaces();
        for (int i=0;i<supp.length;i++)
        {
            value_depend.add(supp[i]);
            if (recursion) {
                depend = supp[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((depend[j]!=this) &&
                        (value_depend.indexOf(depend[j])==-1))
                        value_depend.add(depend[j]);
                }
            }
        }

        // initializers
        Initializer[] inits = getInitializers();
        for (int i=0;i<inits.length;i++)
        {
            TypeRef[] param_types = inits[i].getParameters().getParamTypes();
            for (int j=0;j<param_types.length;j++)
            {
                depend = param_types[j].getDependencies(recursion);
                for (int k=0;k<depend.length;k++)
                {
                    if ((!containsDecl(depend[k])) &&
                        (depend[k]!=this) &&
                        (value_depend.indexOf(depend[k])==-1))
                        value_depend.add(depend[k]);
                }
            }
        }

        // contents
        Declaration[] decls = getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);
        for (int i=0;i<decls.length;i++)
        {
            depend = decls[i].getDependencies(recursion);
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (depend[j]!=this) &&
                    (value_depend.indexOf(depend[j])==-1))
                    value_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])value_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the value declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_value_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_value;
    }

    // ==================================================================
    //
    // Methods for the ValueRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its ValueDef reference.
     **
     **  @return The ValueDef associated with the value declaration.
     **/
    public org.omg.CORBA.ExtValueDef
    getExtValueDef()
    {
        return ext_value_def_;
    }

    /**
     **  Obtain its ValueDef reference.
     **
     **  @return The ValueDef associated with the value declaration.
     **/
    public org.omg.CORBA.ValueDef
    getValueDef()
    {
        return ext_value_def_;
    }

    // ==================================================================
    //
    // Methods for the ValueDecl interface (parser view).
    //
    // ==================================================================

    /**
     **  Set as custom.
     **/
    public void
    setCustom()
    {
        is_custom_ = true;
    }

    /**
     **  Set as abstract.
     **/
    public void
    setAbstract()
    {
        is_abstract_ = true;
    }

    /**
     **  Set as truncatable.
     **/
    public void
    setTruncatable()
    {
        is_truncatable_ = true;
    }

    /**
     **  Set the base value.
     **
     **  @param value The base value.
     **/
    public void
    setBaseValue(ValueRef value)
    {
        if(value != null)
        {
            base_value_ = (ValueDeclImpl)value;
            base_value_.addRef();
        }
    }

    /**
     **  Add a initializer (factory declaration).
     **
     **  @param init The initializer.
     **/
    public void
    addInitializer(Initializer init)
    {
        initializers_.add(init);
    }

    /**
     **  Add an abstract base value.
     **
     **  @param value The abstract base value.
     **/
    public void
    addAbstractValue(ValueRef value)
    {
        if(value != null)
        {
            value.addRef();
            abstract_base_values_.add(value);
        }
    }

    /**
     **  Add a supported interface.
     **
     **  @param itf The supported interface.
     **/
    public void
    addInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            itf.addRef();
            supported_interfaces_.add(itf);
        }
    }

    // ==================================================================
    //
    // Methods for the ValueDecl interface (visitor view).
    //
    // ==================================================================

    /**
     **
     **/
    public boolean
    isCustom()
    {
        return is_custom_;
    }

    /**
     **
     **/
    public boolean
    isAbstract()
    {
        return is_abstract_;
    }

    /**
     **
     **/
    public boolean
    isTruncatable()
    {
        return is_truncatable_;
    }

    /**
     **
     **/
    public boolean
    isPrimaryKey()
    {
        boolean is_pk = false;
        ValueDecl[] abs = getAbstractValues();
        for (int i=0;i<abs.length;i++)
        {
            if (abs[i].getAbsoluteName().equals("::Components::PrimaryKeyBase"))
                return true;

            is_pk = abs[i].isPrimaryKey();
            if (is_pk)
                return true;
        }

        if (getBaseValue()!=null)
            is_pk = getBaseValue().isPrimaryKey();

        return is_pk;
    }

    /**
     **
     **/
    public ValueDecl
    getBaseValue()
    {
        return base_value_;
    }

    /**
     **
     **/
    public ValueDecl[]
    getAbstractValues()
    {
        ValueDecl[] result = new ValueDecl[abstract_base_values_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (ValueDecl)abstract_base_values_.get(i);
        }
        return result;
    }

    /**
     **
     **/
    public InterfaceDecl[]
    getSupportedInterfaces()
    {
        InterfaceDecl[] result = new InterfaceDecl[supported_interfaces_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (InterfaceDecl)supported_interfaces_.get(i);
        }
        return result;
    }

    /**
     **
     **/
    public Initializer[]
    getInitializers()
    {
        Initializer[] result = new Initializer[initializers_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (Initializer)initializers_.get(i);
        }
        return result;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (base_value_!=null)
            base_value_.removeRef();

        for (int i=0;i<abstract_base_values_.size();i++)
        {
            ((ValueRef)abstract_base_values_.get(i)).removeRef();
        }
        for (int i=0;i<supported_interfaces_.size();i++)
        {
            ((InterfaceRef)supported_interfaces_.get(i)).removeRef();
        }
        for (int i=0;i<initializers_.size();i++)
        {
            ((InitializerImpl)initializers_.get(i)).destroy();
        }
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the value declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_value_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the value declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return ext_value_def_;
    }

    /**
     **  Create an attribute definition.
     **
     **  @param attribute The attribute declaration.
     **  @param type The IDLType of the attribute.
     **  @param mode The mode of the attribute.
     **  @param get_exceptions An array containing the exceptions that
     **                        the accessor method can raise.
     **  @param put_exceptions An array containing the exceptions that
     **                        the mutator method can raise.
     **
     **  @return The new created AttributeDef.
     **/
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
        // Create an AttributeDef into the IFR!;
        org.omg.CORBA.ExtAttributeDef ext_attribute_def =
            getExtValueDef().create_ext_attribute(attribute.getId(),
                                                  attribute.getName(),
                                                  attribute.getVersion(),
                                                  type, mode,
                                                  get_exceptions,
                                                  set_exceptions);
        return ext_attribute_def;
    }

    /**
     **  Create an operation
     **
     **  @param operation The operation declaration.
     **  @param type The IDLType of the object that the operations returns.
     **  @param mode The mode of the operation ie normal or oneway.
     **  @param params An array containing the parameters description.
     **  @param exceptions An array containing the exceptions that
     **                    the operation can raise.
     **  @param contexts An array containing the possible context values.
     **
     **  @return The new created OperationDef.
     **/
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        // Create an OperationDef into the IFR.
        org.omg.CORBA.OperationDef operation_def =
            getExtValueDef().create_operation(operation.getId(),
                                              operation.getName(),
                                              operation.getVersion(),
                                              type, mode,
                                              params,
                                              exceptions,
                                              contexts);
        return operation_def;
    }

    /**
     **  To obtain all the contained Declaration objects.
     **
     **  @param exclude_inherited If false return also objects contained in inherited scopes.
     **  @param limited_types A logical combination of DeclarationKind.
     **
     **  @return An array of Declaration objects.
     **/
    public Declaration[]
    getContents(boolean exclude_inherited, int limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = getSupportedInterfaces();
        for (int i=0;i<itfs.length;i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        ValueDecl[] vals = getAbstractValues();
        for (int i=0;i<vals.length;i++)
        {
            Declaration[] tmp1 = vals[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        if (base_value_==null)
            return res;

        Declaration[] tmp1 = base_value_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        org.omg.CORBA.ValueDef[] abstract_base_values =
	    new org.omg.CORBA.ValueDef[abstract_base_values_.size()];
        for(int i=0; i<abstract_base_values.length; i++)
        {
            abstract_base_values[i] = ((ValueDeclImpl)(abstract_base_values_.get(i))).getExtValueDef();
        }

        org.omg.CORBA.InterfaceDef[] supported_interfaces =
	    new org.omg.CORBA.InterfaceDef[supported_interfaces_.size()];
        for(int i=0; i<supported_interfaces.length; i++)
        {
            supported_interfaces[i] = 
                ((InterfaceDeclImpl)(supported_interfaces_.get(i))).getExtInterfaceDef();
        }

        // Create a ValueDef into the IFR.
        ext_value_def_ = the_parent_.getContainer().create_ext_value(getId(), getName(), getVersion(),
                                                                     is_custom_, is_abstract_,
                                                                     getBaseValueDef(),
                                                                     is_truncatable_,
                                                                     abstract_base_values,
                                                                     supported_interfaces,
                                                                     new org.omg.CORBA.ExtInitializer[0]);
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        org.omg.CORBA.ValueDef base_value = getBaseValueDef();
        if(base_value != null)
            getExtValueDef().base_value(base_value);

        org.omg.CORBA.ValueDef[] abstract_base_values =
	    new org.omg.CORBA.ValueDef[abstract_base_values_.size()];
        for(int i=0; i<abstract_base_values.length; i++)
        {
            abstract_base_values[i] = 
                ((ValueDeclImpl)(abstract_base_values_.get(i))).getExtValueDef();
        }

        if(abstract_base_values.length != 0)
            getExtValueDef().abstract_base_values(abstract_base_values);

        org.omg.CORBA.InterfaceDef[] supported_interfaces =
	    new org.omg.CORBA.InterfaceDef[supported_interfaces_.size()];
        for(int i=0; i<supported_interfaces.length; i++)
        {
            supported_interfaces[i] = 
                ((InterfaceDeclImpl)(supported_interfaces_.get(i))).getExtInterfaceDef();
        }

        if(supported_interfaces.length != 0)
            getExtValueDef().supported_interfaces(supported_interfaces);

        org.omg.CORBA.ExtInitializer[] initializers = 
	    new org.omg.CORBA.ExtInitializer[initializers_.size()];
        for(int i=0; i<initializers.length; i++)
        {
            initializers[i] = 
                ((InitializerImpl)(initializers_.get(i))).getExtInitializer();
        }

        if(initializers.length != 0)
            getExtValueDef().ext_initializers(initializers);
    }
}
