/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ArrayDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ArrayDef_impl
       extends IRObject_impl
       implements ArrayDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ArrayDef_impl(IFR ifr)
    {
        // call the IRObject_impl contructor.
        super(ifr);

        // set the TIE servant object.
        setServant(new ArrayDefPOATie(this));

        // set the length to zero.
        length_ = 0;

        // set the element_type_def to null.
        element_type_def_ = new IDLType_ref();
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The array length.
     **/
    protected int length_;

    /**
        The array element type.
     **/
    protected IDLType_ref element_type_def_;

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        TypeCode tc = null;
        try
        {
            Contained_impl cont = (Contained_impl)element_type_def_.getBaseImpl();
            int idx = idSeq.indexOf(cont.id());
            if (idx!=-1)
            {
                // check if the contained can be a recursive type
                // only valuetypes can be recursive in an array
                // also unwound alias to original type to do the check
                int dk = cont.def_kind().value();
                if (dk==DefinitionKind._dk_Alias)
                {
                    AliasDef_impl alias = (AliasDef_impl)cont;
                    IDLType type = alias.original_type_def();
                    dk = type.def_kind().value();
                    AliasDef alias_def = null;
                    while (dk==DefinitionKind._dk_Alias)
                    {
                        alias_def = AliasDefHelper.narrow(type);
                        type = alias_def.original_type_def();
                        dk = type.def_kind().value();
                    }
                }

                if (dk!=DefinitionKind._dk_Value)
                    throw exceptionIllegalRecursion("array", cont.id());

                tc = getIFR().getTCF().create_recursive_tc(cont.id());
            }
        }
        catch (java.lang.ClassCastException ex)
        {
        }

        if (tc==null)
            tc = element_type_def_.recursiveType(idSeq);

        // delegation to the TypeCodeFactory.
        return getIFR().getTCF().create_array_tc(length(), tc);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ArrayDef object reference.
     **/
    public ArrayDef
    asArrayDef()
    {
        return ArrayDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Array;
    }

    /**
        IDL:omg.org/IR/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency.
        element_type_def_.cutDependency();

        // calls the IRObject_impl::destroy method.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================


    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::ArrayDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ArrayDef/length:1.0
     **/
    public int
    length()
    {
        return length_;
    }

    public void
    length(int val)
    {
        length_ = val;
    }

    /**
        IDL:omg.org/CORBA/ArrayDef/element_type:1.0
     **/
    public TypeCode
    element_type()
    {
        // delegate to the IDLType reference.
        return element_type_def_.recursiveType();
    }

    /**
        IDL:omg.org/CORBA/ArrayDef/element_type_def:1.0
    **/
    public IDLType
    element_type_def()
    {
        // delegate to the IDLType reference.
        return element_type_def_.asIDLType();
    }

    public void
    element_type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        element_type_def_.setDependency( castToLocal(val) );
    }
}
