/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::AttributeDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AttributeDef_impl
       extends FeatureContained
       implements AttributeDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    AttributeDef_impl(IFR ifr,
                      Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new AttributeDefPOATie(this));

        // create the type_def reference.
        type_def_ = new IDLType_ref();

        // init internal state.
        mode_ = AttributeMode.ATTR_NORMAL;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The type of the attribute.
     **/
    protected IDLType_ref type_def_;

    /**
        The mode of the attribute.
     **/
    protected AttributeMode mode_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::AttributeDef object reference.
     **/
    public AttributeDef
    asAttributeDef()
    {
        return AttributeDefHelper.narrow(asObject());
    }

    /**
        Obtain its CORBA::AttributeDescription.
     **/
    public AttributeDescription
    getAttributeDescription()
    {
        // create the CORBA::AttributeDescription.
        AttributeDescription description = new AttributeDescription();

        // set its fields.
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = type();
        description.mode = mode();

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // insert its CORBA::AttributeDescription into the any.
        AttributeDescriptionHelper.insert(any, getAttributeDescription());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Attribute;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
    **/
    public void
    destroy()
    {
        // cut the dependency.
        type_def_.cutDependency();

        mode_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::AttributeDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/AttributeDef/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegate to the IDLType reference.
        return type_def_.recursiveType();
    }

    /**
        IDL:omg.org/CORBA/AttributeDef/type_def:1.0
     **/
    public IDLType
    type_def()
    {
        // delegate to the IDLType reference.
        return type_def_.asIDLType();
    }

    public void
    type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        type_def_.setDependency( castToLocal(val) );
    }

    /**
        IDL:omg.org/CORBA/AttributeDef/mode:1.0
     **/
    public AttributeMode
    mode()
    {
        return mode_;
    }

    public void
    mode(AttributeMode val)
    {
        mode_ = val;
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // check the container kind
        if ((container_.def_kind().value()!=org.omg.CORBA.DefinitionKind._dk_Home) &&
            (container_.def_kind().value()!=org.omg.CORBA.DefinitionKind._dk_Component))
            return ;

        // else create the mapping according to the CCM spec

        // for homes
        if (container_.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Home)
        {
            //
            //  copy the attribute in the home explicit interface
            //
            ExtInterfaceDef_impl explicit = (ExtInterfaceDef_impl)((HomeDef_impl)container_).base_interfaces_[0].getImpl();
            String explicit_base_id = explicit.getBaseID();
            String id = explicit_base_id + name() + ":" + version();

            container_.mapping_started_ = true;
            explicit.create_ext_attribute(id, name(), version(), type_def(), mode(),
                                          new org.omg.CORBA.ExceptionDef[0],
                                          new org.omg.CORBA.ExceptionDef[0]);
            container_.mapping_started_ = false;

        //
        // CIDL mapping
        //
            //
            // copy the attribute in the home explicit executor interface
            //
            String cont_base_id = container_.getBaseContainerID();
            String explicit_exe_id = cont_base_id + "CCM_" + container_.name() + "Explicit:" + container_.version();
            org.omg.CORBA.ExtLocalInterfaceDef explicit_exe = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(explicit_exe_id));

            id = cont_base_id + "CCM_" + container_.name() + "Explicit/" + name() + ":" + version();
            explicit_exe.create_ext_attribute(id, name(), version(), type_def(), mode(),
                                              new org.omg.CORBA.ExceptionDef[0],
                                              new org.omg.CORBA.ExceptionDef[0]);

        }
        // for components
        else if (container_.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Component)
        {
            //
            //  copy the attribute in the component main local interface executor
            //
            String cont_base_id = container_.getBaseContainerID();
            // for main executor
            String litf_id = cont_base_id + "CCM_" + container_.name() + "_Executor:" + container_.version();
            org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
                 getIFR().getRepository().lookup_mapped_id(litf_id));

            String id = cont_base_id + "CCM_" + container_.name() + "_Executor/" + name() + ":" + version();
            litf.create_ext_attribute(id,
                                      name(),
                                      version(),
                                      type_def(),
                                      mode(),
                                      new org.omg.CORBA.ExceptionDef[0],
                                      new org.omg.CORBA.ExceptionDef[0]);
        }
    }
}
