/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet, Sylvain Leblanc.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ComponentIR::ComponentDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentDef_impl
       extends ExtInterfaceDef_impl
       implements org.omg.CORBA.ComponentIR.ComponentDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The supported interfaces.
     **/
    protected InterfaceDef_ref[] supported_interfaces_;

    /**
     **  The base component type.
     **/
    protected ComponentDef_ref base_component_;

    /**
     **  To know if the component must be considered as an interface during contained objects
     **  type checking.
     **/
    protected boolean as_interface_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    ComponentDef_impl(IFR ifr,
                      Container_impl container)
    {
        // call the InterfaceDef_impl constructor.
        super(ifr, container, false);

        // set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.ComponentDefPOATie(this));

        // init internal state.
        supported_interfaces_ = null;
        base_component_ = new ComponentDef_ref();
        as_interface_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  To obtain the list of base interfaces.
     **/
    protected InterfaceDef_ref[]
    getBaseInterfaces()
    {
        if (!getIFR().getRepository().isIDL2Repository())
            return new InterfaceDef_ref[0];
        else
            return base_interfaces_;
    }

    /**
     **  Add a contained object to the container.
     **/
    protected void
    addContained(Contained_impl contained)
    {
        // check if the kind of this contained object is accepted.
        if (!checkDefinitionKind(contained.def_kind()))
            throw exceptionInvalidContainer();

        if (!mapping_started_)
            contained_elements_.add(contained);

        if (mapping_started_)
            contained_with_mappings_.add(contained);

        if (contained.def_kind() == org.omg.CORBA.DefinitionKind.dk_Attribute)
            contained_with_mappings_.add(contained);
    }

    /**
     **  Obtain the CORBA::ComponentIR::ProvidesDescriptionSeq of the component.
     **/
    protected org.omg.CORBA.ComponentIR.ProvidesDescription[]
    getProvidesDescriptionSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, org.omg.CORBA.DefinitionKind.dk_Provides, true);

        org.omg.CORBA.ComponentIR.ProvidesDescription[] result = 
            new org.omg.CORBA.ComponentIR.ProvidesDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((ProvidesDef_impl)selection.elementAt(i))
                        .describe_provides();

        return result;
    }

    /**
     **  Obtain the CORBA::ComponentIR::UsesDescriptionSeq of the component.
     **/
    protected org.omg.CORBA.ComponentIR.UsesDescription[]
    getUsesDescriptionSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, org.omg.CORBA.DefinitionKind.dk_Uses, true);

        org.omg.CORBA.ComponentIR.UsesDescription[] result = 
            new org.omg.CORBA.ComponentIR.UsesDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((UsesDef_impl)selection.elementAt(i))
                        .describe_uses();

        return result;
    }

    /**
     **  Obtain the CORBA::ComponentIR::EventPortDescriptionSeq of the component
     **  for a specific DefinitionKind.
     **/
    protected org.omg.CORBA.ComponentIR.EventPortDescription[]
    getEventPortDescriptionSeq(org.omg.CORBA.DefinitionKind kind)
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, kind, true);

        org.omg.CORBA.ComponentIR.EventPortDescription[] result = 
            new org.omg.CORBA.ComponentIR.EventPortDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((EventPortDef_impl)selection.elementAt(i))
                        .describe_event_port();

        return result;
    }

    /**
     **  Obtain the CORBA::ComponentIR::EventPortDescriptionSeq of the component
     **  for the emits declarations.
     **/
    protected org.omg.CORBA.ComponentIR.EventPortDescription[]
    getEmitsDescriptionSeq()
    {
        return getEventPortDescriptionSeq(org.omg.CORBA.DefinitionKind.dk_Emits);
    }

    /**
     **  Obtain the CORBA::ComponentIR::EventPortDescriptionSeq of the component
     **  for the publishes declarations.
     **/
    protected org.omg.CORBA.ComponentIR.EventPortDescription[]
    getPublishesDescriptionSeq()
    {
        return getEventPortDescriptionSeq(org.omg.CORBA.DefinitionKind.dk_Publishes);
    }

    /**
     **  Obtain the CORBA::ComponentIR::EventPortDescriptionSeq of the component
     **  for the consumes declarations.
     **/
    protected org.omg.CORBA.ComponentIR.EventPortDescription[]
    getConsumesDescriptionSeq()
    {
        return getEventPortDescriptionSeq(org.omg.CORBA.DefinitionKind.dk_Consumes);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::ComponentDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.ComponentDef
    asComponentDef()
    {
        return org.omg.CORBA.ComponentIR.ComponentDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
     **  To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(org.omg.CORBA.Any any)
    {
        // create the CORBA::ComponentIR::ComponentDef::ComponentDescription.
        org.omg.CORBA.ComponentIR.ComponentDescription description =
            new org.omg.CORBA.ComponentIR.ComponentDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.base_component = base_component_.getId();
        description.supported_interfaces = InterfaceDef_ref.
                         computeRepositoryIdSeq(supported_interfaces_);
        description.provided_interfaces = getProvidesDescriptionSeq();
        description.used_interfaces = getUsesDescriptionSeq();
        description.emits_events = getEmitsDescriptionSeq();
        description.publishes_events = getPublishesDescriptionSeq();
        description.consumes_events = getConsumesDescriptionSeq();
        description.attributes = getExtAttrDescriptionSeq();
        //        description.is_basic = is_basic();
        description.type = type();

        // insert it into the any.
        org.omg.CORBA.ComponentIR.ComponentDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For Container_impl class.
    //
    // ==================================================================

    /**
     **  Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(org.omg.CORBA.DefinitionKind dk)
    {
        if ((as_interface_) || (getIFR().getRepository().isIDL2Repository()))
            // check kind as for an interface
            // necessary for IDL2 mapping creation
            return super.checkDefinitionKind(dk);

        return (dk == org.omg.CORBA.DefinitionKind.dk_Attribute)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Provides)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Uses)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Emits)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Publishes)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Consumes);
    }

    /**
     **  Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        // calls the InterfaceDef_impl::checkName operations.
        super.checkName(name, exclude_inherited, in_inherited);

        if(!exclude_inherited)
        {
            // TODO;
        }
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        if (getIFR().getRepository().isIDL2Repository())
            return org.omg.CORBA.DefinitionKind.dk_Interface;

        return org.omg.CORBA.DefinitionKind.dk_Component;
    }

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to supported interfaces.
        InterfaceDef_ref.cutDependencies(supported_interfaces_);
        supported_interfaces_ = null;

        // cut the dependency to the base component.
        base_component_.cutDependency();
        base_component_ = null;

        // call the InterfaceDef_impl::destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public org.omg.CORBA.TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.

        //
        // TODO: need to change ORBacus
        //
        // return getIFR().getTCF().create_component_tc(id(), name());

        return getIFR().getTCF().create_interface_tc(id(), name());
    }

    // ==================================================================
    //
    // For CORBA::Contained interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/Contained/move:1.0
     **/
    public void
    move(org.omg.CORBA.Container new_container,
         String new_name,
         String new_version)
    {
        // move the component first
        super.move(new_container, new_name, new_version);

        org.omg.CORBA.LocalInterfaceDef litf = null;
        String cont_base_id = getBaseContainerID();

        container_.mapping_started_ = true;

        // move the main executor
        java.lang.String name = "CCM_"+name()+"_Executor";
        java.lang.String id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        // move the monolithic executor
        name = "CCM_"+name();
        id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        // move the context
        name = "CCM_"+name()+"_Context";
        id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        container_.mapping_started_ = false;

        // make the mappings completion
        whenMoved();
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public org.omg.CORBA.Contained
    lookup(String search_name)
    {
        // calls the InterfaceDef_impl::lookup operation.
        org.omg.CORBA.Contained contained = super.lookup(search_name);

        if (contained == null)
        {
            // TODO
        }

        return contained;
    }

    /**
     **  IDL:omg.org/CORBA/Container/contents:1.0
     **/
    public org.omg.CORBA.Contained[]
    contents(org.omg.CORBA.DefinitionKind limit_type,
             boolean exclude_inherited)
    {
        // calls the InterfaceDef_impl::contents operation.
        org.omg.CORBA.Contained[] result = super.contents(limit_type, exclude_inherited);

        if(!exclude_inherited && (base_component() != null))
        { 
            org.omg.CORBA.Contained[] result2 = base_component().contents(limit_type, exclude_inherited);
            org.omg.CORBA.Contained[] result3 = result;
            result = new org.omg.CORBA.Contained[result3.length + result2.length];
            System.arraycopy(result2, 0, result, 0, result2.length);
            System.arraycopy(result3, 0, result, result2.length, result3.length);
        }

        return result;
    }

    /**
     **  IDL:omg.org/CORBA/Container/lookup_name:1.0
     **/
    public org.omg.CORBA.Contained[]
    lookup_name(String search_name,
                int levels_to_search,
                org.omg.CORBA.DefinitionKind limit_type,
                boolean exclude_inherited)
    {
        // calls the InterfaceDef_impl::contents operation.
        org.omg.CORBA.Contained[] result =
            super.lookup_name(search_name, levels_to_search,
                              limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            // TODO
        }
 
        return result;
    }

    // ==================================================================
    //
    // For CORBA::ComponentIR::ComponentDef interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/supported_interfaces:1.0
     **/
    public org.omg.CORBA.InterfaceDef[]
    supported_interfaces()
    {
        return InterfaceDef_ref.computeInterfaceDefSeq(supported_interfaces_);
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/supported_interfaces:1.0
     **/
    public void
    supported_interfaces(org.omg.CORBA.InterfaceDef[] val)
    {
        InterfaceDef_impl[] itfs = new InterfaceDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            itfs[i] = castToLocal(val[i]);

        // TODO: check inheritance

        InterfaceDef_ref[] refs = new InterfaceDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new InterfaceDef_ref( itfs[i] );

        // cut the previous dependency to base interfaces.
        InterfaceDef_ref.cutDependencies(supported_interfaces_);

        supported_interfaces_ = refs;
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/base_component:1.0
     **/
    public org.omg.CORBA.ComponentIR.ComponentDef
    base_component()
    {
        // delegate to the ComponentDef reference.
        return base_component_.asComponentDef();
    }

    // Warning: base_component is a readonly attribute!
    //
    // Here, this allows us to implement the CORBA::Repository::
    // create_component() operation in the same way that other
    // create_* operations.
    // 
    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/base_component:1.0
     **/
    public void
    base_component(org.omg.CORBA.ComponentIR.ComponentDef val)
    {
        if (val == null)
            base_component_.setDependency( null );
        else
        {
            // try to cast val to a local ComponentDef object
            // and delegate to the ComponentDef reference.
            base_component_.setDependency(castToLocal(val));
        }
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/create_provides:1.0
     **/
    public org.omg.CORBA.ComponentIR.ProvidesDef
    create_provides(String id,
                    String name,
                    String version,
                    org.omg.CORBA.InterfaceDef interface_type)
    {
        ProvidesDef_impl pr = new ProvidesDef_impl(getIFR(), this);

        try
        {
            addContained(pr, id, name, version);
            pr.interface_type(interface_type);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            pr.destroy();
            throw exc;
        }

        as_interface_ = true;
        pr.whenCreated();
        as_interface_ = false;
        return pr.asProvidesDef();
    }

    /**
     **   IDL:omg.org/CORBA/ComponentIR/ComponentDef/create_uses:1.0
     **/
    public org.omg.CORBA.ComponentIR.UsesDef
    create_uses(String id,
                String name,
                String version,
                org.omg.CORBA.InterfaceDef interface_type,
                boolean is_multiple)
    {
        UsesDef_impl uses = new UsesDef_impl(getIFR(), this);

        try
        {
            addContained(uses, id, name, version);
            uses.interface_type(interface_type);
            uses.is_multiple(is_multiple);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            uses.destroy();
            throw exc;
        }

        as_interface_ = true;
        uses.whenCreated();
        as_interface_ = false;
        return uses.asUsesDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/create_emits:1.0
     **/
    public org.omg.CORBA.ComponentIR.EmitsDef
    create_emits(String id,
                 String name,
                 String version,
                 org.omg.CORBA.ComponentIR.EventDef value)
    {
        EmitsDef_impl e = new EmitsDef_impl(getIFR(), this);

        try
        {
            addContained(e, id, name, version);
            e.event(value);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            e.destroy();
            throw exc;
        }

        as_interface_ = true;
        e.whenCreated();
        as_interface_ = false;
        return e.asEmitsDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/create_publishes:1.0
     **/
    public org.omg.CORBA.ComponentIR.PublishesDef
    create_publishes(String id,
                     String name,
                     String version,
                     org.omg.CORBA.ComponentIR.EventDef value)
    {
        PublishesDef_impl p = new PublishesDef_impl(getIFR(), this);

        try
        {
            addContained(p, id, name, version);
            p.event(value);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            p.destroy();
            throw exc;
        }

        as_interface_ = true;
        p.whenCreated();
        as_interface_ = false;
        return p.asPublishesDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/create_consumes:1.0
     **/
    public org.omg.CORBA.ComponentIR.ConsumesDef
    create_consumes(String id,
                    String name,
                    String version,
                    org.omg.CORBA.ComponentIR.EventDef value)
    {
        ConsumesDef_impl c = new ConsumesDef_impl(getIFR(), this);

        try
        {
            addContained(c, id, name, version);
            c.event(value);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            c.destroy();
            throw exc;
        }

        as_interface_ = true;
        c.whenCreated();
        as_interface_ = false;
        return c.asConsumesDef();
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        // CIDL mapping
        //
        // Component Main Executor Interface
        //
        //  local CCM_<component_name>_Executor : 
        //    ( CCM_<base_component_name>_Executor | Components::EnterpriseComponent )
        //    [ <supported_interface_names> ]
        //  {
        //      <component_attributes>
        //  };
        //
        String cont_base_id = getBaseContainerID();
        String name = "CCM_"+name()+"_Executor";
        String version = version();
        String id = cont_base_id + name + ":" + version;
        container_.mapping_started_ = true;
        org.omg.CORBA.ExtLocalInterfaceDef litf =
            container_.create_ext_local_interface(id, name, version, new org.omg.CORBA.InterfaceDef[0]);
        container_.mapping_started_ = false;


        //
        // Component Monolithic Executor Interface
        //
        //  local CCM_<component_name> : CCM_<component_name>_Executor
        //  {
        //      <base_monolithic_equivalent_operations>
        //      <facets_operations>
        //      <event_sinks_operations>
        //  };
        //
        name = "CCM_"+name();
        id = cont_base_id + name + ":" + version;

        container_.mapping_started_ = true;
        org.omg.CORBA.ExtLocalInterfaceDef litf2 =
            container_.create_ext_local_interface(id, name, version, new org.omg.CORBA.InterfaceDef[0]);
        container_.mapping_started_ = false;


        //
        // Component Context Interface
        //
        //  local interface CCM_<component_name>_Context :
        //    ( CCM_<base_component_name>_Context | Components::CCMContext )
        //  {
        //      <facets_operations>
        //      <event_sinks_operations>
        //  };
        //
        name = "CCM_"+name()+"_Context";
        id = cont_base_id + name + ":" + version;
        container_.mapping_started_ = true;
        container_.create_ext_local_interface(id, name, version, new org.omg.CORBA.InterfaceDef[0]);
        container_.mapping_started_ = false;
    }

    /**
     **
     **/
    protected void
    whenMoved()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else complete the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        //
        //
        //  add inheritance for ::ComponentsEventBase or <base_component>
        //
        org.omg.CORBA.InterfaceDef[] itfs =
            new org.omg.CORBA.InterfaceDef[1+supported_interfaces_.length];
        if (base_component_.getImpl()==null)
        {
            itfs[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CCMObject"));
        }
        else
        {
            itfs[0] = base_component_.asComponentDef();
        }
        //
        //  add inheritance for the supported interfaces
        //
        for (int i=0;i<supported_interfaces_.length;i++)
        {
            itfs[1+i] = supported_interfaces_[i].asInterfaceDef();
        }

        base_interfaces(itfs);
        container_.mapping_started_ = true;
        getIFR().getRepository().addRID(id(), this);
        container_.mapping_started_ = false;

        //
        // CIDL mapping
        //
        // Component Main Executor Interface
        //
        //  local CCM_<component_name>_Executor : 
        //    ( CCM_<base_component_name>_Executor | Components::EnterpriseComponent )
        //    [ <supported_interface_names> ]
        //  {
        //      <component_attributes>
        //  };
        //

        // Retrieve the interface declaration
        String cont_base_id = getBaseContainerID();
        String name = "CCM_"+name()+"_Executor";
        String version = version();
        String id = cont_base_id + name + ":" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(getIFR().getRepository().lookup_mapped_id(id));

        // complete this declaration
        org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[supported_interfaces_.length+1];
        if (base_component_.getImpl()==null)
        {
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::EnterpriseComponent"));
        }
        else
        {
            String base_cont_base_id = base_component_.getImpl().getBaseContainerID();
            String base_component_exe_id = base_cont_base_id + "CCM_" + 
                                           base_component_.getImpl().name() + "_Executor:" + 
                                           base_component_.getImpl().version();
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(base_component_exe_id));
        }

        for (int i=0;i<supported_interfaces_.length;i++)
        {
            bases[1+i] = supported_interfaces_[i].asInterfaceDef();
        }

        container_.mapping_started_ = true;
        litf.base_interfaces(bases);
        container_.mapping_started_ = false;


        //
        // Component Monolithic Executor Interface
        //
        //  local CCM_<component_name> : CCM_<component_name>_Executor
        //  {
        //      <base_monolithic_equivalent_operations>
        //      <facets_operations>
        //      <event_sinks_operations>
        //  };
        //
        bases = new org.omg.CORBA.InterfaceDef[1];
        bases[0] = litf;
        name = "CCM_"+name();
        id = cont_base_id + name + ":" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf2 = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(getIFR().getRepository().lookup_mapped_id(id));

        container_.mapping_started_ = true;
        litf2.base_interfaces(bases);
        container_.mapping_started_ = false;

        // copy the contents of base monolithic executor.
        if (base_component_.getImpl()!=null)
        {
            String base_cont_base_id = base_component_.getImpl().getBaseContainerID();
            String base_component_exe_id = base_cont_base_id + "CCM_" + 
                                           base_component_.getImpl().name() + ":" + 
                                           base_component_.getImpl().version();
            org.omg.CORBA.LocalInterfaceDef base_monolithic = 
                org.omg.CORBA.LocalInterfaceDefHelper.narrow(
                    getIFR().getRepository().lookup_mapped_id(base_component_exe_id));

            org.omg.CORBA.Contained[] contents =
                base_monolithic.contents(org.omg.CORBA.DefinitionKind.dk_Operation,
                                         true);

            org.omg.CORBA.OperationDef op = null;
            String op_base_id = cont_base_id + name + "/";


            for (int i=0;i<contents.length;i++)
            {

                op = org.omg.CORBA.OperationDefHelper.narrow(contents[i]);
                litf2.create_operation(op_base_id+op.name()+":"+op.version(),
                                       op.name(),
                                       op.version(),
                                       op.result_def(),
                                       op.mode(),
                                       op.params(),
                                       op.exceptions(),
                                       op.contexts());
            }
        }



        //
        // Component Context Interface
        //
        //  local interface CCM_<component_name>_Context :
        //    ( CCM_<base_component_name>_Context | Components::CCMContext )
        //  {
        //      <facets_operations>
        //      <event_sinks_operations>
        //  };
        //
        name = "CCM_"+name()+"_Context";
        id = cont_base_id + name + ":" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf3 = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(getIFR().getRepository().lookup_mapped_id(id));

        bases = new org.omg.CORBA.InterfaceDef[1];
        if (base_component_.getImpl()==null)
        {
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CCMContext"));
        }
        else
        {
            String base_cont_base_id = base_component_.getImpl().getBaseContainerID();
            String base_component_exe_id = base_cont_base_id + "CCM_" + 
                                           base_component().name() + "_Context:" + 
                                           base_component().version();
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(base_component_exe_id));
        }

        container_.mapping_started_ = true;
        litf3.base_interfaces(bases);
        container_.mapping_started_ = false;
    }

}
