/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::Repository interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentRepository_impl
             extends Repository_impl
             implements org.objectweb.ccm.ComponentRepositoryOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ComponentRepository_impl(IFR ifr)
    {
        // call the Contained_impl contructor.
        this(ifr, false);
    }

    public
    ComponentRepository_impl(IFR ifr,
                             boolean enable_mappings)
    {
        // call the Contained_impl contructor.
        super(ifr, enable_mappings);

        // set the tie servant object.
        setServant(new org.objectweb.ccm.ComponentRepositoryPOATie(this));

        // Init internal state.
        isIDL2Repository_ = false;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** Obtain its CORBA::ComponentIR::Repository object reference.
     **/
    public org.omg.CORBA.ComponentIR.Repository
    asComponentRepository()
    {
        super.isIDL2Repository_ = false;
        return org.omg.CORBA.ComponentIR.RepositoryHelper.narrow(asObject());
    }

    // ==================================================================
    //
    //  For CORBA::ComponentIR::Container interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/ComponentIR/Container/create_component_module:1.0
     **/
    public org.omg.CORBA.ComponentIR.ModuleDef
    create_component_module(String id,
                            String name,
                            String version)
    {
        ComponentModuleDef_impl module = new ComponentModuleDef_impl(getIFR(), this);

        try
        {
            addContained(module, id, name, version);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            module.destroy();
            throw exc;
        }

        return module.asComponentModuleDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/Container/create_component:1.0
     **/
    public org.omg.CORBA.ComponentIR.ComponentDef
    create_component(String id,
                     String name,
                     String version,
                     org.omg.CORBA.ComponentIR.ComponentDef base_component,
                     org.omg.CORBA.InterfaceDef[] supported_interfaces)
    {
        ComponentDef_impl c = new ComponentDef_impl(getIFR(), this);
        try
        {
            addContained(c, id, name, version);
            c.base_component(base_component);
            c.supported_interfaces(supported_interfaces);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            c.destroy();
            throw exc;
        }

        c.whenCreated();
        return c.asComponentDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/Container/create_home:1.0
     **/
    public org.omg.CORBA.ComponentIR.HomeDef
    create_home(String id,
                String name,
                String version,
                org.omg.CORBA.ComponentIR.HomeDef base_home,
                org.omg.CORBA.ComponentIR.ComponentDef managed_component,
                org.omg.CORBA.InterfaceDef[] supported_interfaces,
                org.omg.CORBA.ValueDef primary_key)
    {
        HomeDef_impl h = new HomeDef_impl(getIFR(), this);

        try
        {
            addContained(h, id, name, version);
            h.base_home(base_home);
            h.managed_component(managed_component);
            h.supported_interfaces(supported_interfaces);
            h.primary_key(primary_key);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            h.destroy();
            throw exc;
        }

        h.whenCreated();
        return h.asHomeDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/Container/create_value:1.0
     **/
    public org.omg.CORBA.ComponentIR.EventDef
    create_event(String id,
                 String name,
                 String version,
                 boolean is_custom,
                 boolean is_abstract,
                 org.omg.CORBA.ValueDef base_value,
                 boolean is_truncatable,
                 org.omg.CORBA.ValueDef[] abstract_base_values,
                 org.omg.CORBA.InterfaceDef[] supported_interfaces,
                 org.omg.CORBA.ExtInitializer[] initializers)
    {
        EventDef_impl event = new EventDef_impl(getIFR(), this);

        try
        {
            addContained(event, id, name, version);
            event.is_custom(is_custom);
            event.is_abstract(is_abstract);
            event.base_value(base_value);
            event.is_truncatable(is_truncatable);
            event.abstract_base_values(abstract_base_values);
            event.supported_interfaces(supported_interfaces);
            event.ext_initializers(initializers);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            event.destroy();
            throw exc;
        }

        event.whenCreated();
        return event.asEventDef();
   }

    // ==================================================================
    //
    // Methods for the OpenCCM::ComponentRepository interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/ComponentRepository/as_IDL2_repository:1.0
    //
    /**
     **  To view the repository as an IDL2 repository.
     **  Note: the mappings of IDL3 declarations will be accessible.
     **/
    public org.omg.CORBA.Repository
    as_IDL2_repository()
    {
        super.isIDL2Repository_ = true;
        return super.asRepository();
    }

    //
    // IDL:goal.lifl.fr/OpenCCM/ComponentRepository/as_IDL3_repository:1.0
    //
    /**
     **  To view the repository as an IDL3 repository.
     **  Note: the mappings of IDL3 declarations will NOT be accessible.
     **/
    public org.omg.CORBA.ComponentIR.Repository
    as_IDL3_repository()
    {
        super.isIDL2Repository_ = false;
        return asComponentRepository();
    }

    //
    // IDL:goal.lifl.fr/OpenCCM/ComponentRepository/lookup_mapped_id:1.0
    //
    /**
     **  To obtain a Contained that comes from a mapping.
     **/
    public org.omg.CORBA.Contained
    lookup_mapped_id(String search_id)
    {
        Contained_impl contained = getMappedRID(search_id);

        // If not found then returns a nil contained reference.
        if (contained == null) return null;
        
        return contained.asContained();
    }


    // ==================================================================
    //
    // Methods for the OpenCCM::Shutdownable interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/Shutdownable/shutdown:1.0
    //
    /**
     **  Shutdown an OpenCCM server.
     **/
    public void
    shutdown()
    {
	org.objectweb.ccm.CORBA.TheORB.shutdown(false);
    }
}
