/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ConstantDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ConstantDef_impl
       extends Contained_impl
       implements ConstantDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ConstantDef_impl(IFR ifr,
                     Container_impl container)
    {
        // call the Contained_impl contructor.
        super(ifr, container);

	// set the tie servant object.
        setServant(new ConstantDefPOATie(this));

        // create the type_def reference.
        type_def_ = new IDLType_ref();

        // null value.
        value_ = null;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The type of the constant.
     **/
    protected IDLType_ref type_def_;

    /**
        The value of the constant.
     **/
    protected Any value_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ConstantDef object reference.
     **/
    public ConstantDef
    asConstantDef()
    {
        return ConstantDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // create the CORBA::ConstantDescription.
        ConstantDescription description = new ConstantDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = type();
        description.value = value();

        // insert it into the any.
        ConstantDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Constant;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency.
        type_def_.cutDependency();

        value_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ConstantDef/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegate to the IDLType reference.
        return type_def_.recursiveType();
    }

    /**
        IDL:omg.org/CORBA/ConstantDef/type_def:1.0
     **/
    public IDLType
    type_def()
    {
        // delegate to the IDLType reference.
        return type_def_.asIDLType();
    }

    public void
    type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        type_def_.setDependency( castToLocal(val) );
    }

    /**
        IDL:omg.org/CORBA/ConstantDef/value:1.0
     **/
    public Any
    value()
    {
        if (value_ == null)
            throw exceptionBadInvOrder("IDL:omg.org/CORBA/ConstantDef/value:1.0");

        return value_;
    }

    public void
    value(Any val)
    {
	// The following line does not work on OpenORB!!!
	//       if (!val.type().equal(type()))
        if (!val.type().equivalent(type()))
	//
            throw exceptionBadParam("IDL:omg.org/CORBA/ConstantDef/value:1.0");

        value_ = val;
    }
}
