/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::EmitsDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EmitsDef_impl
       extends EventPortDef_impl
       implements org.omg.CORBA.ComponentIR.EmitsDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    EmitsDef_impl(IFR ifr,
                  Container_impl container)
    {
        // call the EventDef_impl contructor.
        super(ifr, container);

	// set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.EmitsDefPOATie(this));
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::EmitsDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.EmitsDef
    asEmitsDef()
    {
        return org.omg.CORBA.ComponentIR.EmitsDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        return org.omg.CORBA.DefinitionKind.dk_Emits;
    }

    // ==================================================================
    //
    // For CORBA::ComponentIR::EventPortDef interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For CORBA::ComponentIR::EmitsDef interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        // compute the container base repository id
        String cont_base_id = getBaseContainerID();

        // emits mapping :
        //
        //  void connect_<source_name>(in <event_type>Consumer consumer)
        //  raises(::Components::AlreadyConnected);
        //
        String version = version();
        String name = "connect_" + name();
        String id = cont_base_id + name + ":" + version;
        InterfaceDef_impl container = (InterfaceDef_impl)container_;
        String cons_id = computeIDWithoutVersion(event_.asEventDef().id());
        cons_id = cons_id + "Consumer:"+ event_.asEventDef().version();

        org.omg.CORBA.IDLType consumer_itf = org.omg.CORBA.IDLTypeHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(cons_id));
        org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
        params[0] = new org.omg.CORBA.ParameterDescription();
        params[0].name = "consumer";
        params[0].type_def = consumer_itf;
        params[0].type = params[0].type_def.type();
        params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
        org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[1];
        excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
            getIFR().getRepository().lookup("::Components::AlreadyConnected"));

        container_.mapping_started_ = true;
        container.create_operation(id, name, version,
                                   getIFR().getRepository().get_primitive(
                                       org.omg.CORBA.PrimitiveKind.pk_void),
                                   org.omg.CORBA.OperationMode.OP_NORMAL,
                                   params, excs, new java.lang.String[0]);
        container_.mapping_started_ = false;

        //
        //  <event_type>Consumer disconnect_<source_name>()
        //  raises(::Components::NoConnection);
        //
        name = "disconnect_" + name();
        id = cont_base_id + name + ":" + version;
        excs = new org.omg.CORBA.ExceptionDef[1];
        excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
            getIFR().getRepository().lookup("::Components::NoConnection"));

        container_.mapping_started_ = true;
        container.create_operation(id, name, version,
                                   consumer_itf,
                                   org.omg.CORBA.OperationMode.OP_NORMAL,
                                   new org.omg.CORBA.ParameterDescription[0],
                                   excs, new java.lang.String[0]);
        container_.mapping_started_ = false;

        //
        //  CIDL mapping
        //
        //  local interface CCM_<component_name>_Context : ...
        //  {
        //      void push_<name>(in <event_name> event);
        //  };
        //
        cont_base_id = container_.getBaseContainerID();
        version = container_.version();

        String litf_id = cont_base_id + "CCM_" + container_.name() + "_Context:" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
             getIFR().getRepository().lookup_mapped_id(litf_id));

        cont_base_id = computeBaseID(litf_id);
        version = litf.version();
        name = "push_" + name();
        id = cont_base_id + name + ":" + version;

        params = new org.omg.CORBA.ParameterDescription[1];
        params[0] = new org.omg.CORBA.ParameterDescription();
        params[0].name = "event";
        params[0].type_def = event_.getImpl().asIDLType();
        params[0].type = params[0].type_def.type();
        params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
        litf.create_operation(id, name, version,
                              getIFR().getRepository().get_primitive(
                                           org.omg.CORBA.PrimitiveKind.pk_void),
                              org.omg.CORBA.OperationMode.OP_NORMAL,
                              params,
                              new org.omg.CORBA.ExceptionDef[0],
                              new java.lang.String[0]);
    }
}
