/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::EnumDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EnumDef_impl
       extends Contained_impl
       implements EnumDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    EnumDef_impl(IFR ifr,
                 Container_impl container)
    {
        // call the Contained_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new EnumDefPOATie(this));

        // init internal state.
        members_ = new String[0];
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The members of the enum.
     **/
    protected String[] members_;

    // ==================================================================
    //
    // Internal methods for the inherited IRObject_impl class.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::EnumDef object reference.
     **/
    public EnumDef
    asEnumDef()
    {
        return EnumDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Enum;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        members_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_enum_tc(id(), name(), members());
    }

    // ==================================================================
    //
    // For CORBA::EnumDef interface.
    //
    // ==================================================================
 
    /**
        IDL:omg.org/CORBA/EnumDef/members:1.0
     **/
    public String[]
    members()
    {
        return members_;
    }

    public void
    members(String[] names)
    {
        // check member names. 
        for(int i=0; i<names.length; i++)
	{
            String name = names[i];

            for(int j=i+1; j<names.length; j++)
                if(name.equalsIgnoreCase(names[j]))
                    throw exceptionDuplicateDeclarator(name, j);

            container_.checkName(name, false, false);

            if (name.equalsIgnoreCase(name()))
               throw exceptionNameAlreadyUsedByImmediateScope(name);
        }

        members_ = names;
    }
}
