/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::EventDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EventDef_impl
       extends ExtValueDef_impl
       implements org.omg.CORBA.ComponentIR.EventDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The ::Components::EventBase value.
     **/
    protected ValueDef_ref event_base_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **
     **/
    public
    EventDef_impl(IFR ifr,
                  Container_impl container)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.EventDefPOATie(this));

        // Init internal state.
        event_base_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  To obtain the list of abstract base values.
     **/
    protected ValueDef_ref[]
    getAbstractBaseValues()
    {
        if ((!getIFR().getRepository().isIDL2Repository()) ||
            (event_base_==null))
            return abstract_base_values_;

        ValueDef_ref[] values = new ValueDef_ref[abstract_base_values_.length+1];
        values[0] = event_base_;
        for (int i=0;i<abstract_base_values_.length;i++)
            values[i+1] = abstract_base_values_[i];

        return values;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::EventDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.EventDef
    asEventDef()
    {
        return org.omg.CORBA.ComponentIR.EventDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        if (getIFR().getRepository().isIDL2Repository())
            return org.omg.CORBA.DefinitionKind.dk_Value;

        return org.omg.CORBA.DefinitionKind.dk_Event;
    }

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to the eventbase value.
        if (event_base_!=null)
        {
            event_base_.cutDependency();
            event_base_ = null;
        }

        // call the ValueDef_impl::destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::Contained interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/Contained/move:1.0
     **/
    public void
    move(org.omg.CORBA.Container new_container,
         String new_name,
         String new_version)
    {
        // move the event first
        super.move(new_container, new_name, new_version);

        org.omg.CORBA.InterfaceDef litf = null;
        String cont_base_id = getBaseContainerID();

        container_.mapping_started_ = true;

        // move the consumer
        java.lang.String name = name()+"Consumer";
        java.lang.String id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.InterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        // move the local consumer
        name = "CCM_"+name()+"Consumer";
        id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.InterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        container_.mapping_started_ = false;
    }


    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        //
        //  IDL2 mapping
        //
        //
        //  add inheritance from ::Components::EventBase
        //
        if (getBaseValue().getImpl()==null)
        {
            ValueDef_impl val_impl = castToLocal(org.omg.CORBA.ValueDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::EventBase")));
            event_base_ = new ValueDef_ref(val_impl);
        }

        container_.mapping_started_ = true;
        getIFR().getRepository().addRID(id(), this);
        container_.mapping_started_ = false;


        //
        // create the <event_type>Consumer interface
        //
        String base_id = computeIDWithoutVersion(id());
        String name = name() + "Consumer";
        String version = version();
        String id = base_id + "Consumer:" + version;
        org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[1];
        if (getBaseValue().getImpl()==null)
        {
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::EventConsumerBase"));
        }
        else
        {
            String base_consumer_id  = computeIDWithoutVersion(getBaseValue().getId());
            base_consumer_id = base_consumer_id +
                               "Consumer:" + getBaseValue().getImpl().version();
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(base_consumer_id));
        }

        container_.mapping_started_ = true;
        org.omg.CORBA.InterfaceDef consumer = container_.create_ext_interface(id, name, version, bases);
        container_.mapping_started_ = false;

        //
        // add the push operation
        //
        id = base_id + "Consumer/push_" + name() + ":" + version;
        name = "push_" + name();
        org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
        params[0] = new org.omg.CORBA.ParameterDescription();
        params[0].name = "the_" + name().toLowerCase();
        params[0].type_def = asValueDef();
        params[0].type = params[0].type_def.type();
        params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;

        consumer.create_operation(id, name, version,
            getIFR().getRepository().get_primitive(org.omg.CORBA.PrimitiveKind.pk_void),
            org.omg.CORBA.OperationMode.OP_NORMAL,
            params,
            new org.omg.CORBA.ExceptionDef[0],
            new java.lang.String[0]);

        //
        // CIDL mapping
        //
        //
        //  local interface CCM_<eventtype_name>Consumer { 
        //      void push(in <eventtype_name> event);
        //  };
        //
        String cont_base_id = getBaseContainerID();
        name = "CCM_"+name()+"Consumer";
        id = cont_base_id + name + ":" + version;

        container_.mapping_started_ = true;
        org.omg.CORBA.LocalInterfaceDef litf =
            container_.create_ext_local_interface(id, name, version,
                                                  new org.omg.CORBA.InterfaceDef[0]);
        container_.mapping_started_ = false;
        
        id = cont_base_id + name + "/push" + ":" + version;
        name = "push";
        params = new org.omg.CORBA.ParameterDescription[1];
        params[0] = new org.omg.CORBA.ParameterDescription();
        params[0].name = "event";
        params[0].type_def = asValueDef();
        params[0].type = params[0].type_def.type();
        params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
        litf.create_operation(id, name, version,
            getIFR().getRepository().get_primitive(org.omg.CORBA.PrimitiveKind.pk_void),
            org.omg.CORBA.OperationMode.OP_NORMAL,
            params,
            new org.omg.CORBA.ExceptionDef[0],
            new java.lang.String[0]);
    }
}
