/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::EventPortDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class EventPortDef_impl
                extends FeatureContained
                implements org.omg.CORBA.ComponentIR.EventPortDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The event value type.
     **/
    protected EventDef_ref event_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    EventPortDef_impl(IFR ifr,
                      Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

        // init internal state.
        event_ = new EventDef_ref();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain the CORBA::ComponentIR::EventPortDescription of the declaration.
     **/
    public org.omg.CORBA.ComponentIR.EventPortDescription
    describe_event_port()
    {
        // create the CORBA::EventDescription.
        org.omg.CORBA.ComponentIR.EventPortDescription description = new org.omg.CORBA.ComponentIR.EventPortDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.event = event().id();

        return description;
    }


    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
     **  To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(org.omg.CORBA.Any any)
    {
        // insert it into the any.
        org.omg.CORBA.ComponentIR.EventPortDescriptionHelper.insert(any, describe_event_port());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to the event type.
        event_.cutDependency();
        event_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::ComponentIR::EventPortDef interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/ComponentIR/EventPortDef/is_a:1.0
     **/
    public boolean
    is_a(String event_id)
    {
        // delegate to the ValueDef event.
        return event_.getImpl().is_a(event_id);
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/EventPortDef/event:1.0
     **/
    public org.omg.CORBA.ComponentIR.EventDef
    event()
    {
        // delegate to the ValueDef reference.
        return event_.asEventDef();
    }

    // Warning: event is a readonly attribute!
    //
    // Here, this allows us to implement the CORBA::Component::create_provides()
    // operation in the same way that other create_* operations.
    // 
    /**
     **  IDL:omg.org/CORBA/ComponentIR/EventPortDef/event:1.0
     **/
    public void
    event(org.omg.CORBA.ComponentIR.EventDef val)
    {
        // try to cast val to a local ValueDef object
        // and delegate to the IDLType reference.
        event_.setDependency(castToLocal(val));
    }
}
