/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ExtAbstractInterfaceDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExtAbstractInterfaceDef_impl
       extends ExtInterfaceDef_impl
       implements ExtAbstractInterfaceDefOperations
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     **
     **/
    public
    ExtAbstractInterfaceDef_impl(IFR ifr,
                                 Container_impl container)
    {
        // call the InterfaceDef_impl constructor.
        super(ifr, container, false);

        // set the tie servant object.
        setServant(new ExtAbstractInterfaceDefPOATie(this));
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ExtAbstractInterfaceDef object reference.
     **/
    public ExtAbstractInterfaceDef
    asExtAbstractInterfaceDef()
    {
        return ExtAbstractInterfaceDefHelper.narrow(asObject());
    }

    /**
     **
     **/
    public void
    base_interfaces(AbstractInterfaceDef[] val)
    {
        InterfaceDef[] tmp = new InterfaceDef[val.length];
        for(int i=0; i<tmp.length; i++)
            tmp[i] = val[i];

        super.base_interfaces(tmp);
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_AbstractInterface;
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_abstract_interface_tc(id(), name());
    }

    // ==================================================================
    //
    // For CORBA::InterfaceDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/InterfaceDef/base_interfaces:1.0
     **/
    public void
    base_interfaces(InterfaceDef[] itfs)
    {
        // Check that all itfs are AbstractInterfaceDef.
        for(int i=0; i<itfs.length; i++)
            if(itfs[i].def_kind() != DefinitionKind.dk_AbstractInterface)
                throw exceptionInvalidAbstractInterfaceInheritance(
                                                  itfs[i].absolute_name());

        super.base_interfaces(itfs);
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/is_a:1.0
     **/
    public boolean
    is_a(String interface_id)
    {
        if(interface_id.equals(id()))
            return true;

        if(interface_id.equals("IDL:omg.org/CORBA/AbstractBase:1.0"))
            return true;

        for(int i=0; i<base_interfaces_.length; i++)
            if (base_interfaces_[i].getImpl().is_a(interface_id))
                return true;

        return false;
    }
}
