/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ExtAttributeDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExtAttributeDef_impl
       extends AttributeDef_impl
       implements ExtAttributeDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ExtAttributeDef_impl(IFR ifr,
                      Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

	// set the tie servant object.
        setServant(new ExtAttributeDefPOATie(this));

        // init internal state.
        get_exceptions_ = null;
        set_exceptions_ = null;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The exceptions when getting the attribute.
     **/
    protected ExceptionDef_ref[] get_exceptions_;

    /**
        The exceptions when setting the attribute.
     **/
    protected ExceptionDef_ref[] set_exceptions_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ExtAttributeDef object reference.
     **/
    public ExtAttributeDef
    asExtAttributeDef()
    {
        return ExtAttributeDefHelper.narrow(asObject());
    }

    /**
        Obtain its CORBA::ExtAttributeDescription.
     **/
    public ExtAttributeDescription
    getExtAttributeDescription()
    {
        // create the CORBA::ExtAttributeDescription.
        ExtAttributeDescription description = new ExtAttributeDescription();

        // set its fields.
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = type();
        description.mode = mode();
        description.get_exceptions =
            ExceptionDef_impl.compute_ExcDescriptionSeq(get_exceptions_);
        description.set_exceptions =
            ExceptionDef_impl.compute_ExcDescriptionSeq(set_exceptions_);

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // insert its CORBA::ExtAttributeDescription into the any.
        ExtAttributeDescriptionHelper.insert(any, getExtAttributeDescription());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
    **/
    public void
    destroy()
    {
        // cut the dependency.
        type_def_.cutDependency();

        mode_ = null;

        // cut the previous dependency to get exceptions.
        ExceptionDef_ref.cutDependencies(get_exceptions_);
        get_exceptions_ = null;

        // cut the previous dependency to put exceptions.
        ExceptionDef_ref.cutDependencies(set_exceptions_);
        set_exceptions_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::ExtAttributeDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ExtAttributeDef/get_exceptions:1.0
     **/
    public ExceptionDef[]
    get_exceptions()
    {
        ExceptionDef[] result = new ExceptionDef[get_exceptions_.length];

        for(int i=0; i<get_exceptions_.length; i++)
             result[i] = get_exceptions_[i].asExceptionDef();

        return result;
    }

    public void
    get_exceptions(ExceptionDef[] val)
    {
        ExceptionDef_impl[] exceptions = new ExceptionDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            exceptions[i] = castToLocal(val[i]);

        // TODO: check unicity

        ExceptionDef_ref[] refs = new ExceptionDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new ExceptionDef_ref( exceptions[i] );

        // cut the previous dependency to get exceptions.
        ExceptionDef_ref.cutDependencies(get_exceptions_);

        get_exceptions_ = refs;
    }

    /**
        IDL:omg.org/CORBA/ExtAttributeDef/set_exceptions:1.0
     **/
    public ExceptionDef[]
    set_exceptions()
    {
        ExceptionDef[] result = new ExceptionDef[set_exceptions_.length];

        for(int i=0; i<set_exceptions_.length; i++)
             result[i] = set_exceptions_[i].asExceptionDef();

        return result;
    }

    public void
    set_exceptions(ExceptionDef[] val)
    {
        ExceptionDef_impl[] exceptions = new ExceptionDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            exceptions[i] = castToLocal(val[i]);

        // TODO: check unicity

        ExceptionDef_ref[] refs = new ExceptionDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new ExceptionDef_ref( exceptions[i] );

        // cut the previous dependency to put exceptions.
        ExceptionDef_ref.cutDependencies(set_exceptions_);

        set_exceptions_ = refs;
    }

    /**
        IDL:omg.org/CORBA/ExtAttributeDef/describe:1.0
     **/
    public org.omg.CORBA.ExtAttributeDescription
    describe_attribute()
    {
        return getExtAttributeDescription();
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // check the container kind
        if ((container_.def_kind().value()!=org.omg.CORBA.DefinitionKind._dk_Home) &&
            (container_.def_kind().value()!=org.omg.CORBA.DefinitionKind._dk_Component))
            return ;

        // else create the mapping according to the CCM spec

        // for homes
        if (container_.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Home)
        {
            //
            //  copy the attribute in the home explicit interface
            //
            ExtInterfaceDef_impl explicit = (ExtInterfaceDef_impl)((HomeDef_impl)container_).base_interfaces_[0].getImpl();
            String explicit_base_id = explicit.getBaseContainerID();
            String id = explicit_base_id + name() + ":" + version();

            container_.mapping_started_ = true;
            explicit.create_ext_attribute(id, name(), version(), type_def(), mode(),
                                          get_exceptions(), set_exceptions());
            container_.mapping_started_ = false;

        //
        // CIDL mapping
        //
            //
            // copy the attribute in the home explicit executor local interface
            //
            String cont_base_id = container_.getBaseContainerID();
            String explicit_exe_id = cont_base_id + "CCM_" + container_.name() + "Explicit:" + container_.version();
            org.omg.CORBA.ExtLocalInterfaceDef explicit_exe = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(explicit_exe_id));

            id = cont_base_id + "CCM_" + container_.name() + "Explicit/" + name() + ":" + version();
            explicit_exe.create_ext_attribute(id, name(), version(), type_def(), mode(),
                                              get_exceptions(), set_exceptions());

        }
        // for components
        else if (container_.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Component)
        {
            //
            //  copy the attribute in the component main local interface executor
            //
            String cont_base_id = container_.getBaseContainerID();
            // for main executor
            String litf_id = cont_base_id + "CCM_" + container_.name() + "_Executor:" + container_.version();
            org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
                 getIFR().getRepository().lookup_mapped_id(litf_id));

            String id = cont_base_id + "CCM_" + container_.name() + "_Executor/" + name() + ":" + version();
            litf.create_ext_attribute(id,
                                      name(),
                                      version(),
                                      type_def(),
                                      mode(),
                                      get_exceptions(),
                                      set_exceptions());
        }
    }
}
