/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ExtInitializer helper class.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExtInitializer_impl
       extends Initializer_impl
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ExtInitializer_impl(ValueDef_impl owner)
    {
        super(owner);

        // Init internal state.
        exceptions_ = null;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected ExceptionDef_ref[] exceptions_;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain as CORBA::Initializer.
     **/
    public Initializer
    getInitializer()
    {
        return new Initializer(getStructMemberSeq(), name_);
    }

    /**
        Set as CORBA::Initializer.
     **/
    public void
    setInitializer(Initializer val)
    {
        setStructMemberSeq(val.members);
        name_ = val.name;
        exceptions_ = new ExceptionDef_ref[0];
    }

    /**
        Obtain as CORBA::Initializer.
     **/
    public ExtInitializer
    getExtInitializer()
    {
        ExceptionDef[] exceptions = new ExceptionDef[exceptions_.length];
        for (int i=0;i<exceptions.length;i++)
            exceptions[i] = exceptions_[i].asExceptionDef();

        return new ExtInitializer(getStructMemberSeq(),
                                  exceptions,
                                  ExceptionDef_impl.compute_ExcDescriptionSeq(exceptions_),
                                  name_);
    }

    /**
        Set as CORBA::Initializer.
     **/
    public void
    setExtInitializer(ExtInitializer val)
    {
        setStructMemberSeq(val.members);
        name_ = val.name;

        ExceptionDef_impl[] exceptions = new ExceptionDef_impl[val.exceptions_def.length];
        for (int i=0; i<val.exceptions_def.length; i++)
            exceptions[i] = contained_.castToLocal(val.exceptions_def[i]);

        ExceptionDef_ref[] refs = new ExceptionDef_ref[exceptions.length];
        for (int i=0; i<exceptions.length; i++)
            refs[i] = new ExceptionDef_ref(exceptions[i]);

        // cut the previous dependency to exceptions.
        ExceptionDef_ref.cutDependencies(exceptions_);

        exceptions_ = refs;
    }
}
