/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;
import org.omg.CORBA.ExtInterfaceDefPackage.ExtFullInterfaceDescription;

/**
 * Implementation of the CORBA::ExtInterfaceDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExtInterfaceDef_impl
       extends InterfaceDef_impl
       implements ExtInterfaceDefOperations
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public
    ExtInterfaceDef_impl(IFR ifr,
                         Container_impl container)
    {
        this(ifr, container, false);

        // set the tie servant object.
        setServant(new ExtInterfaceDefPOATie(this));
    }

    public
    ExtInterfaceDef_impl(IFR ifr,
                         Container_impl container,
                         boolean dummy)
    {
        // call the Container_impl contructor.
        super(ifr, container, dummy);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ExtInterfaceDef object reference.
     **/
    public ExtInterfaceDef
    asExtInterfaceDef()
    {
        return ExtInterfaceDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::InterfaceDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/InterfaceDef/create_attribute:1.0
     **/
    public AttributeDef
    create_attribute(String id,
                     String name,
                     String version,
                     IDLType type,
                     AttributeMode mode)
    {
        ExtAttributeDef_impl attr = new ExtAttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
            attr.get_exceptions(new ExceptionDef[0]);
            attr.set_exceptions(new ExceptionDef[0]);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        attr.whenCreated();
        return attr.asAttributeDef();
    }

    // ==================================================================
    //
    // For CORBA::ExtInterfaceDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ExtInterfaceDef/create_attribute:1.0
     **/
    public ExtAttributeDef
    create_ext_attribute(String id,
                         String name,
                         String version,
                         IDLType type,
                         AttributeMode mode,
                         ExceptionDef[] get_exceptions,
                         ExceptionDef[] set_exceptions)
    {
        ExtAttributeDef_impl attr = new ExtAttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
            attr.get_exceptions(get_exceptions);
            attr.set_exceptions(set_exceptions);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        attr.whenCreated();
        return attr.asExtAttributeDef();
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/describe_ext_interface:1.0
     **/
    public ExtFullInterfaceDescription
    describe_ext_interface()
    {
        ExtFullInterfaceDescription description = new ExtFullInterfaceDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.operations = getOpDescriptionSeq();
        description.attributes = getExtAttrDescriptionSeq();
        description.base_interfaces =
                InterfaceDef_ref.computeRepositoryIdSeq(base_interfaces_);
        description.type = type();
        return description;
    }
}
