/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;
import org.omg.CORBA.ExtValueDefPackage.ExtFullValueDescription;

/**
 * Implementation of the CORBA::ExtValueDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExtValueDef_impl
       extends ValueDef_impl
       implements ExtValueDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected ExtInitializer_impl[] ext_initializers_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ExtValueDef_impl(IFR ifr,
                     Container_impl container)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        ext_initializers_ = new ExtInitializer_impl[0];

        // set the tie servant object.
        setServant(new ExtValueDefPOATie(this));
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ExtValueDef object reference.
     **/
    public ExtValueDef
    asExtValueDef()
    {
        return ExtValueDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::ExtValueDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ValueDef/create_attribute:1.0
     **/
    public AttributeDef
    create_attribute(String id,
                     String name,
                     String version,
                     IDLType type,
                     AttributeMode mode)
    {
        ExtAttributeDef_impl attr = new ExtAttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
            attr.get_exceptions(new ExceptionDef[0]);
            attr.set_exceptions(new ExceptionDef[0]);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        return attr.asAttributeDef();
    }

    /**
        IDL:omg.org/CORBA/ValueDef/initializers:1.0
     **/
    public Initializer[]
    initializers()
    {
        Initializer[] result = new Initializer[initializers_.length];

        for(int i=0; i<result.length; i++)
            result[i] = ext_initializers_[i].getInitializer();

        return result;
    }

    public void
    initializers(Initializer[] val)
    {
        ExtInitializer_impl[] tmp = new ExtInitializer_impl[val.length];
        for(int i=0; i<tmp.length; i++)
	{
            tmp[i] = new ExtInitializer_impl(this);
            tmp[i].setInitializer(val[i]);
        }
        ext_initializers_ = tmp;
    }

    // ==================================================================
    //
    // For CORBA::ExtValueDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ExtValueDef/create_ext_attribute:1.0
     **/
    public ExtAttributeDef
    create_ext_attribute(String id,
                         String name,
                         String version,
                         IDLType type,
                         AttributeMode mode,
                         ExceptionDef[] get_exceptions,
                         ExceptionDef[] set_exceptions)
    {
        ExtAttributeDef_impl attr = new ExtAttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
            attr.get_exceptions(get_exceptions);
            attr.set_exceptions(set_exceptions);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        return attr.asExtAttributeDef();
    }

    /**
        IDL:omg.org/CORBA/ExtValueDef/ext_initializers:1.0
     **/
    public ExtInitializer[]
    ext_initializers()
    {
        ExtInitializer[] result = new ExtInitializer[ext_initializers_.length];

        for(int i=0; i<result.length; i++)
            result[i] = ext_initializers_[i].getExtInitializer();

        return result;
    }

    /**
        IDL:omg.org/CORBA/ExtValueDef/ext_initializers:1.0
     **/
    public void
    ext_initializers(ExtInitializer[] val)
    {
        ExtInitializer_impl[] tmp = new ExtInitializer_impl[val.length];
        for(int i=0; i<tmp.length; i++)
	{
            tmp[i] = new ExtInitializer_impl(this);
            tmp[i].setExtInitializer(val[i]);
        }
        ext_initializers_ = tmp;
    }

    /**
        IDL:omg.org/CORBA/ExtValueDef/describe_ext_value:1.0
     **/
    public ExtFullValueDescription
    describe_ext_value()
    {
        ExtFullValueDescription description = new ExtFullValueDescription();
        description.name = name();
        description.id = id();
        description.is_abstract = is_abstract();
        description.is_custom = is_custom();
        description.defined_in = getContainerID();
        description.version = version();
        description.operations = getOpDescriptionSeq();
        description.attributes = getExtAttrDescriptionSeq();
        description.members = getValueMemberSeq();
        description.initializers = ext_initializers();
        description.supported_interfaces = InterfaceDef_ref.
                         computeRepositoryIdSeq(supported_interfaces_);
        description.abstract_base_values = ValueDef_ref.
                         computeRepositoryIdSeq(abstract_base_values_);
        description.is_truncatable = is_truncatable();
        description.base_value = base_value_.getId();
        description.type = type();
        return description;
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **  TODO : change code
     **/
    protected void
    whenCreated()
    {
        if (abstract_base_values_.length==0)
            return ;

        for (int i=0;i<abstract_base_values_.length;i++)
        {
            if (abstract_base_values_[i].getImpl().absolute_name().equals("::Components::EventBase"))
            {
                //
                // create the <event_type>Consumer interface
                //
                String cont_base_id = getBaseContainerID();
                String name = name() + "Consumer";
                String version = version();
                String id = cont_base_id + name + ":" + version;
                org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[1];
                bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                    getIFR().getRepository().lookup("::Components::EventConsumerBase"));
                org.omg.CORBA.InterfaceDef consumer = container_.create_interface(id, name, version, bases);

                //
                // add the push operation
                //
                id = cont_base_id + name + "/push:" + version;
                name = "push";
                org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
                params[0] = new org.omg.CORBA.ParameterDescription();
                params[0].name = "event";
                params[0].type_def = asValueDef();
                params[0].type = params[0].type_def.type();
                params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
                consumer.create_operation(id, name, version,
                                          getIFR().getRepository().get_primitive(
                                              org.omg.CORBA.PrimitiveKind.pk_void),
                                          org.omg.CORBA.OperationMode.OP_NORMAL,
                                          params,
                                          new org.omg.CORBA.ExceptionDef[0],
                                          new java.lang.String[0]);
                break ;
            }
        }
    }
}
