/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::FinderDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FinderDef_impl
       extends HomeOperationBase
       implements org.omg.CORBA.ComponentIR.FinderDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    FinderDef_impl(IFR ifr,
                   HomeDef_impl home)
    {
        // call the HomeOperationBase constructor.
        super(ifr, home);

        // set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.FinderDefPOATie(this));
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::FinderDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.FinderDef
    asFinderDef()
    {
        return org.omg.CORBA.ComponentIR.FinderDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        return org.omg.CORBA.DefinitionKind.dk_Finder;
    }

    // ==================================================================
    //
    // For CORBA::OperationDef interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For CORBA::ComponentIR::FinderDef interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        //
        //
        //  add finder as an operation in the explicit interface.
        //
        InterfaceDef_impl explicit = ((HomeDef_impl)container_).base_interfaces_[0].getImpl();
        String explicit_base_id = explicit.getBaseID();
        String id = explicit_base_id + name() + ":" + version();
        org.omg.CORBA.ExceptionDef[] excs1 = exceptions();
        org.omg.CORBA.ExceptionDef[] excs2 = new org.omg.CORBA.ExceptionDef[excs1.length+1];
        excs2[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::FinderFailure"));
        for (int i=0;i<excs1.length;i++)
            excs2[i+1] = excs1[i];

        explicit.create_operation(id, name(), version(), result_def(), mode(), params(),
                                  excs2, contexts());

        //
        // CIDL mapping
        //
        //
        //  add factory as an operation in the explicit executor interface
        //
        String cont_base_id = container_.getBaseContainerID();

        String explicit_exe_id = cont_base_id + "CCM_" + container_.name() + "Explicit:" + container_.version();
        org.omg.CORBA.ExtLocalInterfaceDef explicit_exe = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
             getIFR().getRepository().lookup_mapped_id(explicit_exe_id));
        org.omg.CORBA.InterfaceDef enterprise_comp = org.omg.CORBA.InterfaceDefHelper.narrow(
             getIFR().getRepository().lookup("::Components::EnterpriseComponent"));

        id = cont_base_id + "CCM_" + container_.name() + "Explicit/" + name() + ":" + version();

        explicit_exe.create_operation(id, name(), version(),
                                      enterprise_comp,
                                      mode(),
                                      params(),
                                      exceptions(),
                                      contexts());
    }
}
