/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::FixedDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FixedDef_impl
       extends AnonymousIDLType
       implements FixedDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    FixedDef_impl(IFR ifr)
    {
        // call the AnonymousIDLType contructor.
        super(ifr);

        // set the TIE servant object.
        setServant(new FixedDefPOATie(this));

        // sets the digits and scale to zero.
        digits_ = (short)0;
        scale_ = (short)0;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The fixed digits.
     **/
    protected short digits_;

    /**
        The fixed scale.
     **/
    protected short scale_;

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::FixedDef object reference.
     **/  
    public FixedDef
    asFixedDef()
    {
        return FixedDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Fixed;
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegates to the TypeCodeFactory.
        return getIFR().getTCF().create_fixed_tc(digits(), scale());
    }

    // ==================================================================
    //
    // For CORBA::FixedDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/FixedDef/digits:1.0
     **/
    public short
    digits()
    {
        return digits_;
    }

    public void
    digits(short d)
    {
        // must be set to a value between 1 and 31.
        if(d < 1 || d > 31)
            throw exceptionBadParam("digits must be between 1 and 31");

        digits_ = d;
    }

    /**
        IDL:omg.org/CORBA/FixedDef/scale:1.0
     **/
    public short
    scale()
    {
        return scale_;
    }

    public void
    scale(short s)
    {
        scale_ = s;
    }
}
