/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::HomeDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class HomeDef_impl
       extends ExtInterfaceDef_impl
       implements org.omg.CORBA.ComponentIR.HomeDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The supported interfaces.
     **/
    protected InterfaceDef_ref[] supported_interfaces_;

    /**
     **  The base home.
     **/
    protected HomeDef_ref base_home_;
    
    /**
     **  The managed component.
     **/
    protected ComponentDef_ref managed_component_;

    /**
     **  The primary key.
     **/
    protected ValueDef_ref primary_key_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    HomeDef_impl(IFR ifr,
                 Container_impl container)
    {
        // call the InterfaceDef_impl constructor.
        super(ifr, container, false);

        // set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.HomeDefPOATie(this));

        // inits internal state.
        supported_interfaces_ = null;
        base_home_ = new HomeDef_ref();
        managed_component_ = new ComponentDef_ref();
        primary_key_ = new ValueDef_ref();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Add a contained object to the container.
     **/
    protected void
    addContained(Contained_impl contained)
    {
        // check if the kind of this contained object is accepted.
        if (!checkDefinitionKind(contained.def_kind()))
            throw exceptionInvalidContainer();

        if (!mapping_started_)
            contained_elements_.add(contained);

        if ((contained.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Finder) ||
            (contained.def_kind().value()==org.omg.CORBA.DefinitionKind._dk_Factory))
            contained_with_mappings_.add(contained);
    }

    /**
     **  To obtain the list of base interfaces.
     **/
    protected InterfaceDef_ref[]
    getBaseInterfaces()
    {
        if (!getIFR().getRepository().isIDL2Repository())
            return new InterfaceDef_ref[0];
        else
            return base_interfaces_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::HomeDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.HomeDef
    asHomeDef()
    {
        return org.omg.CORBA.ComponentIR.HomeDefHelper.narrow(asObject());
    }

    /**
     **  Obtain the CORBA::ValueDescription of the primary key.
     **/
    public org.omg.CORBA.ValueDescription
    getPrimaryKeyDescription()
    {
        return primary_key_.getImpl().getValueDescription();
    }

    /**
     **  Obtain the CORBA::OperationDescription of a specific DefinitionKind.
     **/
    public org.omg.CORBA.OperationDescription[]
    getOpDescriptionSeq(org.omg.CORBA.DefinitionKind kind)
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, kind, true);

        org.omg.CORBA.OperationDescription[] result =
            new org.omg.CORBA.OperationDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((OperationDef_impl)selection.elementAt(i))
                        .getOperationDescription();

        return result;
    }

    /**
     **  Obtain the CORBA::OperationDescription of the factories.
     **/
    public org.omg.CORBA.OperationDescription[]
    getFactoryDescriptionSeq()
    {
        return getOpDescriptionSeq(org.omg.CORBA.DefinitionKind.dk_Factory);
    }

    /**
     **  Obtain the CORBA::OperationDescription of the finders.
     **/
    public org.omg.CORBA.OperationDescription[]
    getFinderDescriptionSeq()
    {
        return getOpDescriptionSeq(org.omg.CORBA.DefinitionKind.dk_Factory);
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
     **  To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(org.omg.CORBA.Any any)
    {
        // create the CORBA::HomeDescription.
        org.omg.CORBA.ComponentIR.HomeDescription description =
            new org.omg.CORBA.ComponentIR.HomeDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.base_home = base_home_.getId();
        description.managed_component = managed_component_.getId();
        description.primary_key = getPrimaryKeyDescription();
        description.factories = getFactoryDescriptionSeq();
        description.finders = getFinderDescriptionSeq();
        description.operations = getOpDescriptionSeq();
        description.attributes = getExtAttrDescriptionSeq();
        //        description.is_basic = is_basic();
        description.type = type();

        // insert it into the any.
        org.omg.CORBA.ComponentIR.HomeDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For Container_impl class.
    //
    // ==================================================================

    /**
     **  Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(org.omg.CORBA.DefinitionKind dk)
    {
        return super.checkDefinitionKind(dk)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Factory)
            || (dk == org.omg.CORBA.DefinitionKind.dk_Finder);
    }

    /**
     **  Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        // calls the InterfaceDef_impl::checkName operation.
        super.checkName(name, exclude_inherited, in_inherited);

        if(!exclude_inherited)
        {
            // TODO;
        }
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        if (getIFR().getRepository().isIDL2Repository())
            return org.omg.CORBA.DefinitionKind.dk_Interface;

        return org.omg.CORBA.DefinitionKind.dk_Home;
    }

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to the base home.
        base_home_.cutDependency();
        base_home_ = null;

        // cut the dependency to the managed component.
        managed_component_.cutDependency();
        managed_component_ = null;

        // cut the dependency to the primary key.
        primary_key_.cutDependency();
        primary_key_ = null;

        // call the InterfaceDef_impl::destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public org.omg.CORBA.TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.

        //
        // TODO: need to change ORBacus
        //
        //  return getIFR().getTCF().create_home_tc(id(), name());

        return getIFR().getTCF().create_interface_tc(id(), name());
    }

    // ==================================================================
    //
    // For CORBA::Contained interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/Contained/move:1.0
     **/
    public void
    move(org.omg.CORBA.Container new_container,
         String new_name,
         String new_version)
    {
        // move the home first
        super.move(new_container, new_name, new_version);

        org.omg.CORBA.LocalInterfaceDef litf = null;
        String cont_base_id = getBaseContainerID();

        container_.mapping_started_ = true;
 
        // move the local explicit home
        java.lang.String name = "CCM_"+name()+"Explicit";
        java.lang.String id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        // move the local implicit home
        name = "CCM_"+name()+"Implicit";
        id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        // move the local home
        name = "CCM_"+name();
        id = cont_base_id + name + ":" + new_version;
        litf = org.omg.CORBA.LocalInterfaceDefHelper.narrow(
            getIFR().getRepository().lookup_mapped_id(id));
        litf.move(new_container, name, new_version);

        container_.mapping_started_ = false;
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public org.omg.CORBA.Contained
    lookup(String search_name)
    {
        // calls the InterfaceDef_impl::lookup operation.
        org.omg.CORBA.Contained contained = super.lookup(search_name);

        if (contained == null)
        {
            // TODO
        }

        return contained;
    }

    /**
     **  IDL:omg.org/CORBA/Container/contents:1.0
     **/
    public org.omg.CORBA.Contained[]
    contents(org.omg.CORBA.DefinitionKind limit_type,
             boolean exclude_inherited)
    {
        // calls the InterfaceDef_impl::contents operation.
        org.omg.CORBA.Contained[] result = super.contents(limit_type, exclude_inherited);

        if(!exclude_inherited && (base_home() != null))
        { 
            org.omg.CORBA.Contained[] result2 = base_home().contents(limit_type, exclude_inherited);
            org.omg.CORBA.Contained[] result3 = result;
            result = new org.omg.CORBA.Contained[result3.length + result2.length];
            System.arraycopy(result2, 0, result, 0, result2.length);
            System.arraycopy(result3, 0, result, result2.length, result3.length);
        }
 
        return result;
    }

    /**
     **  IDL:omg.org/CORBA/Container/lookup_name:1.0
     **/
    public org.omg.CORBA.Contained[]
    lookup_name(String search_name,
                int levels_to_search,
                org.omg.CORBA.DefinitionKind limit_type,
                boolean exclude_inherited)
    {
        // calls the InterfaceDef_impl::contents operation.
        org.omg.CORBA.Contained[] result =
            super.lookup_name(search_name, levels_to_search,
                              limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            // TODO
        }
 
        return result;
    }

    // ==================================================================
    //
    // For CORBA::ExtInterfaceDef interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For CORBA::ComponentIR::HomeDef interface.
    //
    // ==================================================================

    /**
     **   IDL:omg.org/CORBA/ComponentIR/HomeDef/base_home:1.0
     **/
    public org.omg.CORBA.ComponentIR.HomeDef
    base_home()
    {
        // delegate to the HomeDef reference.
        return base_home_.asHomeDef();
    }

    /**
     **   IDL:omg.org/CORBA/ComponentIR/HomeDef/base_home:1.0
     **/
    public void
    base_home(org.omg.CORBA.ComponentIR.HomeDef val)
    {
        if(val == null)
            base_home_.setDependency(null);
        else
        {
            // try to cast val to a local object
            // and delegate to the HomeDef reference.
            base_home_.setDependency(castToLocal(val));
        }
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/managed_component:1.0
     **/
    public org.omg.CORBA.ComponentIR.ComponentDef
    managed_component()
    {
        // delegate to the ComponentDef reference.
        return managed_component_.asComponentDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/managed_component:1.0
     **/
    public void
    managed_component(org.omg.CORBA.ComponentIR.ComponentDef val)
    {
        // try to cast val to a local object
        // and delegate to the ComponentDef reference.
        managed_component_.setDependency(castToLocal(val));
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/primary_key:1.0
     **/
    public org.omg.CORBA.ValueDef
    primary_key()
    {
        // delegate to the ValueDef reference.
        return primary_key_.asValueDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/primary_key:1.0
     **/
    public void
    primary_key(org.omg.CORBA.ValueDef val)
    {
        // TODO: check CCM specification!
        if (val == null)
            primary_key_.setDependency(null);
        else
        {
            // try to cast val to a local object
            // and delegate to the ValueDef reference.
            primary_key_.setDependency( castToLocal(val) );
        }
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/supported_interfaces:1.0
     **/
    public org.omg.CORBA.InterfaceDef[]
    supported_interfaces()
    {
        return InterfaceDef_ref.computeInterfaceDefSeq(supported_interfaces_);
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ComponentDef/supported_interfaces:1.0
     **/
    public void
    supported_interfaces(org.omg.CORBA.InterfaceDef[] val)
    {
        InterfaceDef_impl[] itfs = new InterfaceDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            itfs[i] = castToLocal(val[i]);

        // TODO: check inheritance

        InterfaceDef_ref[] refs = new InterfaceDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new InterfaceDef_ref( itfs[i] );

        // cut the previous dependency to base interfaces.
        InterfaceDef_ref.cutDependencies(supported_interfaces_);

        supported_interfaces_ = refs;
    }

    /**
     **  IDL:omg.org/CORBA/HomeDef/is_basic:1.0
     **/
/*
 * not in the spec
 *
    public boolean
    is_basic()
    {
        // TODO: implement
        return false;
    }
 */


    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/create_factory:1.0
     **/
    public org.omg.CORBA.ComponentIR.FactoryDef
    create_factory(String id,
                   String name,
                   String version,
                   org.omg.CORBA.ParameterDescription[] params,
                   org.omg.CORBA.ExceptionDef[] exceptions)
    {
        FactoryDef_impl f = new FactoryDef_impl(getIFR(), this);
        try
        {
            addContained(f, id, name, version);
            f.params(params);
            f.exceptions(exceptions);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            f.destroy();
            throw exc;
        }

        f.whenCreated();
        return f.asFactoryDef();
    }

    /**
     **  IDL:omg.org/CORBA/ComponentIR/HomeDef/create_finder:1.0
     **/
    public org.omg.CORBA.ComponentIR.FinderDef
    create_finder(String id,
                  String name,
                  String version,
                  org.omg.CORBA.ParameterDescription[] params,
                  org.omg.CORBA.ExceptionDef[] exceptions)
    {
        FinderDef_impl f = new FinderDef_impl(getIFR(), this);

        try
        {
            addContained(f, id, name, version);
            f.params(params);
            f.exceptions(exceptions);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            f.destroy();
            throw exc;
        }

        f.whenCreated();
        return f.asFinderDef();
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        //
        // creation of the explicit interface
        //
        String cont_base_id = getBaseContainerID();
        String name = name() + "Explicit";
        String version = version();
        String id = cont_base_id + name + ":" + version;
        org.omg.CORBA.InterfaceDef[] supported = supported_interfaces();
        org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[1+supported.length];
        //
        //  add inheritance for supported interfaces
        //
        for (int i=0;i<supported.length;i++)
            bases[1+i] = supported[i];

        //
        //  add inheritance for ::Components::CCMHome or the base home explicit interface
        //
        if (base_home_.getImpl()==null)
        {
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CCMHome"));
        }
        else
        {
            bases[0] = base_home_.getImpl().base_interfaces_[0].asInterfaceDef();
        }

        container_.mapping_started_ = true;
        org.omg.CORBA.InterfaceDef explicit = container_.create_ext_interface(id, name, version, bases);
        container_.mapping_started_ = false;

        //
        // creation of the implicit interface
        //
        name = name() + "Implicit";
        String implicit_base_id = cont_base_id + name;
        id = implicit_base_id + ":" + version;
        //
        //  add inheritance for ::Components::KeylessCCMHome or null
        //
        if (primary_key_.getImpl()==null)
        {
            bases = new org.omg.CORBA.InterfaceDef[1];
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::KeylessCCMHome"));
        }
        else
        {
            bases = new org.omg.CORBA.InterfaceDef[0];
        }

        container_.mapping_started_ = true;
        org.omg.CORBA.InterfaceDef implicit = container_.create_ext_interface(id, name, version, bases);
        container_.mapping_started_ = false;

        if (primary_key_.getImpl()==null)
        {
            //
            //  <component_type> create()
            //  raises(::Components::CreateFailure);
            //
            name = "create";
            version = implicit.version();
            id = implicit_base_id + name + ":" + version;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CreateFailure"));

            implicit.create_operation(id, name, version,
                                      managed_component_.asComponentDef(),
                                      org.omg.CORBA.OperationMode.OP_NORMAL,
                                      new org.omg.CORBA.ParameterDescription[0],
                                      excs,
                                      new java.lang.String[0]);
        }
        else
        {
            //
            //  <component_type> create(in <key_type> key)
            //  raises(::Components::DuplicateKeyValue,
            //         ::Components::InvalidKey,
            //         ::Components::CreateFailure);
            //
            name = "create";
            version = implicit.version();
            id = implicit_base_id + name + ":" + version;
            org.omg.CORBA.IDLType key = primary_key_.asValueDef();
            org.omg.CORBA.IDLType component = managed_component_.asComponentDef();
            org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[3];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::DuplicateKeyValue"));
            excs[1] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidKey"));
            excs[2] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CreateFailure"));

            implicit.create_operation(id, name, version,
                                      component,
                                      org.omg.CORBA.OperationMode.OP_NORMAL,
                                      params,
                                      excs,
                                      new java.lang.String[0]);

            //
            //  <component_type> find_by_primary_key(in <key_type> key)
            //  raises(::Components::UnknownKeyValue,
            //         ::Components::InvalidKey,
            //         ::Components::FinderFailure);
            //
            name = "find_by_primary_key";
            id = implicit_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            excs = new org.omg.CORBA.ExceptionDef[3];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::UnknownKeyValue"));
            excs[1] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidKey"));
            excs[2] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::FinderFailure"));

            implicit.create_operation(id, name, version,
                                      component,
                                      org.omg.CORBA.OperationMode.OP_NORMAL,
                                      params,
                                      excs,
                                      new java.lang.String[0]);

            //
            //  void remove(in <key_type> key)
            //  raises(::Components::UnknownKeyValue,
            //         ::Components::InvalidKey,
            //         ::Components::RemoveFailure);
            //
            name = "remove";
            id = implicit_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            excs = new org.omg.CORBA.ExceptionDef[3];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::UnknownKeyValue"));
            excs[1] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidKey"));
            excs[2] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::RemoveFailure"));

            implicit.create_operation(id, name, version,
                                      getIFR().getRepository().get_primitive(
                                         org.omg.CORBA.PrimitiveKind.pk_void),
                                      org.omg.CORBA.OperationMode.OP_NORMAL,
                                      params,
                                      excs,
                                      new java.lang.String[0]);

            //
            //  <key_type> get_primary_key(in <component_type> comp);
            //
            name = "get_primary_key";
            id = implicit_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "comp";
            params[0].type_def = component;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            implicit.create_operation(id, name, version,
                                      key,
                                      org.omg.CORBA.OperationMode.OP_NORMAL,
                                      params,
                                      new org.omg.CORBA.ExceptionDef[0],
                                      new java.lang.String[0]);
        }

        //
        // Set explicit and implicit as base interfaces
        //
        bases = new org.omg.CORBA.InterfaceDef[2];
        bases[0] = explicit;
        bases[1] = implicit;
        base_interfaces(bases);
        container_.mapping_started_ = true;
        getIFR().getRepository().addRID(id(), this);
        container_.mapping_started_ = false;


        //
        // CIDL mapping
        //
        // Home Explicit Executor Interface
        //
        //  local interface CCM_<home_name>Explicit :
        //    ( ::Components::HomeExecutorBase | CCM_<base_home_name>Explicit )
        //    [ <supported_interfaces> ]
        //  {
        //      <attributes>
        //      <operations>
        //      <finders>
        //      <factories>
        //  };
        //
        cont_base_id = getBaseContainerID();
        name = "CCM_"+name()+"Explicit";
        version = version();
        id = cont_base_id + name + ":" + version;
        bases = new org.omg.CORBA.InterfaceDef[supported_interfaces_.length+1];
        if (base_home_.getImpl()==null)
        {
            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::HomeExecutorBase"));
        }
        else
        {
            String base_cont_base_id = base_home_.getImpl().getBaseContainerID();
            String base_home_exe_id = base_cont_base_id + "CCM_" + 
                                      base_home_.getImpl().name() + "Explicit:" + 
                                      base_home_.getImpl().version();

            bases[0] = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup_mapped_id(base_home_exe_id));
        }

        for (int i=0;i<supported_interfaces_.length;i++)
        {
            bases[1+i] = supported_interfaces_[i].asInterfaceDef();
        }

        container_.mapping_started_ = true;
        org.omg.CORBA.ExtLocalInterfaceDef explicit_exe =
            container_.create_ext_local_interface(id, name, version, bases);
        container_.mapping_started_ = false;

        //
        // Home Implicit Executor Interface
        //
        //  local interface CCM_<home_name>Implicit
        //  {
        //  };
        //
        name = "CCM_"+name()+"Implicit";
        id = cont_base_id + name + ":" + version;
        container_.mapping_started_ = true;
        org.omg.CORBA.ExtLocalInterfaceDef implicit_exe =
            container_.create_ext_local_interface(id, name, version,
                                                  new org.omg.CORBA.InterfaceDef[0]);
        container_.mapping_started_ = false;
        String implicit_exe_base_id = cont_base_id + name;

        if (primary_key_.getImpl()==null)
        {
            //
            // Home Implicit Executor Interface without key
            //
            //  ::Components::EnterpriseComponent create()
            //   raises(::Components::CCMException);
            //
            name = "create";
            version = implicit_exe.version();
            id = implicit_exe_base_id + name + ":" + version;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CCMException"));
            org.omg.CORBA.InterfaceDef enterprise_comp = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::EnterpriseComponent"));

            implicit_exe.create_operation(id, name, version,
                                          enterprise_comp,
                                          org.omg.CORBA.OperationMode.OP_NORMAL,
                                          new org.omg.CORBA.ParameterDescription[0],
                                          excs,
                                          new java.lang.String[0]);
        }
        else
        {
            org.omg.CORBA.ExceptionDef ccm_exception = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::CCMException"));
            org.omg.CORBA.InterfaceDef enterprise_comp = org.omg.CORBA.InterfaceDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::EnterpriseComponent"));
            org.omg.CORBA.IDLType key = primary_key_.asValueDef();

            //
            // Home Implicit Executor Interface with key
            //
            //  ::Components::EnterpriseComponent create(in <key_type> key)
            //  raises(::Components::CCMException);
            //
            name = "create";
            version = implicit_exe.version();
            id = implicit_exe_base_id + name + ":" + version;
            org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = ccm_exception;

            implicit_exe.create_operation(id, name, version,
                                          enterprise_comp,
                                          org.omg.CORBA.OperationMode.OP_NORMAL,
                                          params,
                                          excs,
                                          new java.lang.String[0]);

            //
            //  ::Components::EnterpriseComponent find_by_primary_key(in <key_type> key)
            //  raises(::Components::CCMException);
            //
            name = "find_by_primary_key";
            id = implicit_exe_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = ccm_exception;

            implicit_exe.create_operation(id, name, version,
                                          enterprise_comp,
                                          org.omg.CORBA.OperationMode.OP_NORMAL,
                                          params,
                                          excs,
                                          new java.lang.String[0]);

            //
            //  void remove(in <key_type> key)
            //  raises(::Components::CCMException);
            //
            name = "remove";
            id = implicit_exe_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "key";
            params[0].type_def = key;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = ccm_exception;

            implicit_exe.create_operation(id, name, version,
                                          getIFR().getRepository().get_primitive(
                                              org.omg.CORBA.PrimitiveKind.pk_void),
                                          org.omg.CORBA.OperationMode.OP_NORMAL,
                                          params,
                                          excs,
                                          new java.lang.String[0]);
        }

        //
        // Home Main Executor Interface
        //
        //  local interface CCM_<home_name> : CCM_<home_name>Explicit,
        //                                    CCM_<home_name>Implicit
        //  {
        //  };
        //
        name = "CCM_" + name();
        id = cont_base_id + name + ":" + version;
        bases = new org.omg.CORBA.InterfaceDef[2];
        bases[0] = explicit_exe;
        bases[1] = implicit_exe;

        container_.mapping_started_ = true;
        container_.create_ext_local_interface(id, name, version, bases);
        container_.mapping_started_ = false;

    }
}
