/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Base class for FactoryDef_impl and FinderDef_impl classes.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class HomeOperationBase
       extends OperationDef_impl
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    HomeOperationBase(IFR ifr,
                      HomeDef_impl home)
    {
        // call the OperationDef_impl constructor.
        super(ifr, home, false);

        // force the operation mode.
        mode_ = OperationMode.OP_NORMAL;

        // force the operation contexts.
        contexts_ = new String[0];
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
        Obtain the component type managed by the home.
     **/
    protected ComponentDef_ref
    getComponentManagedByHome()
    {
        // delegate to the HomeDef implementation.
        return ((HomeDef_impl)container_).managed_component_;
    }

    // ==================================================================
    //
    // For CORBA::OperationDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/OperationDef/result:1.0
     **/
    public TypeCode
    result()
    {
        // delegate to the ComponentDef implementation.
        return getComponentManagedByHome().recursiveType();
    }

    /**
        IDL:omg.org/CORBA/OperationDef/result_def:1.0
     **/
    public IDLType
    result_def()
    {
        // delegate to the ComponentDef reference.
        return getComponentManagedByHome().asComponentDef();
    }

    public void
    result_def(IDLType val)
    {
        // Do nothing!;
    }

    /**
        IDL:omg.org/CORBA/OperationDef/mode:1.0
     **/
    public void
    mode(OperationMode val)
    {
        // Do nothing!
    }

    /**
        IDL:omg.org/CORBA/OperationDef/contexts:1.0
     **/
    public void
    contexts(String[] val)
    {
         // Do nothing!
    }
}
