/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * This IFR class implements some common features for IFR objects:
 * global object storing and error managements.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IFR
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public
    IFR()
    {
        // create conversion table.
        tie2impl_ = new java.util.Hashtable();

        rootPOA_ = org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA();

        // create the singleton Repository object
        // with mappings enabled.
        repository_ = new ComponentRepository_impl(this, true);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        Conversion table between TIE Servant <---> IFRObject.
     **/
    protected java.util.Hashtable tie2impl_;

    /**
        Reference to the RootPOA singleton.
     **/
    protected org.omg.PortableServer.POA rootPOA_;

    /**
        Reference to the CORBA::ComponentIR::Repository singleton.
     **/
    protected ComponentRepository_impl repository_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain the Repository object.
     **/
    public ComponentRepository_impl
    getRepository()
    {
        return repository_;
    }

    /**
        Obtain the TypeCodeFactory object.
     **/
    public org.omg.CORBA.ORB
    getTCF()
    {
        return org.objectweb.ccm.CORBA.TheORB.getORB();
    }

    /**
        Activate TIE servant.
     **/
    public void
    activate(org.omg.PortableServer.Servant servant,
             IFRObject impl)
    {
        log(100,"activate " + servant.toString() + " for " + impl.toString());
        activateServant(servant);
        tie2impl_.put(servant, impl);
    }

    /**
        Deactivate TIE servant.
     **/
    public void
    deactivate(org.omg.PortableServer.Servant servant)
    {
        log(100,"deactivate " + servant.toString());
        deactivateServant(servant);
        tie2impl_.remove(servant);
    }

    /**
        Obtain the IFRObject implementing the object reference.
     **/
    public IFRObject
    toImpl(org.omg.CORBA.Object object)
    {
       org.omg.PortableServer.Servant servant = toServant(object);
       if (servant == null)
           return null;

       IFRObject impl = (IFRObject)tie2impl_.get(servant);
       return impl;
    }

    /**
        Activate a Servant.
     **/
    public void
    activateServant(org.omg.PortableServer.Servant servant)
    {
       try
       {
           byte[] oid = rootPOA_.activate_object(servant);
       }
       catch(Exception ex)
       {
           ex.printStackTrace();
       }
    }

    /**
        Deactivate a Servant.
     **/
    public void
    deactivateServant(org.omg.PortableServer.Servant servant)
    {
       try
       {
           byte[] oid = rootPOA_.servant_to_id(servant);
           rootPOA_.deactivate_object(oid);
       }
       catch(Exception ex)
       {
           ex.printStackTrace();
       }
    }

    /**
        Obtain the Servant implementing the object reference.
     **/
    public org.omg.PortableServer.Servant
    toServant(org.omg.CORBA.Object object)
    {
       try
       {
           return rootPOA_.reference_to_servant(object);
       }
       catch(Exception ex)
       {
       }
       return null;
    }

    /**
        Log a message.
     **/
    public void
    log(int level, String message)
    {
        if(level < 10)
           System.out.println("(Level=" + level + "): " + message);
    }
}
