/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Base class for IFR object implementation classes:
 * storing the IFR and POA Tie servant references.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class IFRObject
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
         Create the IFRObject with an IFR reference.
     **/
    public
    IFRObject(IFR ifr)
    {
        // init internal state.
        ifr_ = ifr;
        servant_ = null;
        ref_counter_ = 0;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        Reference to the IFR.
     **/
    private IFR ifr_;

    /**
        Reference to the POA tie Servant.
     **/
    private org.omg.PortableServer.Servant servant_;

    /**
        Store the number of references to this IRObject.
     **/
    private int ref_counter_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain the IFR object.
     **/
    public IFR
    getIFR()
    {
        return ifr_;
    }

    /**
        Obtain the POA tie servant.
     **/
    public org.omg.PortableServer.Servant
    getServant()
    {
        return servant_;
    }

    /**
        Obtain the Servant object reference.
     **/
    public org.omg.CORBA.Object
    asObject()
    {
        return servant_._this_object();
    }

    /**
        Set the POA tie servant.
     **/
    public void
    setServant(org.omg.PortableServer.Servant servant)
    {
        // deactivate the previous servant.
        if (servant_ != null)
            ifr_.deactivate(servant_);

        servant_ = servant;
 
        // activate the new servant.
        if (servant != null)
            ifr_.activate(servant, this);
    }

    /**
        Obtain the reference counter.
     **/
    public int
    getRefCounter()
    {
        return ref_counter_;
    }

    /**
        Increment the reference counter.
     **/
    public void
    add_ref()
    {
        ref_counter_ ++;
    }

    /**
        Decrement the reference counter.
     **/
    public void
    remove_ref()
    {
        ref_counter_ --;
    }
}
