/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::IRObject reference.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IRObject_ref
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public
    IRObject_ref()
    {
        impl_ = null;
    }

    public
    IRObject_ref(IRObject_impl object)
    {
        // create the new dependency.
        object.add_ref();
        impl_ = object;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The reference.
     **/
    protected IRObject_impl impl_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its IRObject_impl.
     **/
    public IRObject_impl
    getBaseImpl()
    {
        // TODO: check if the reference is initialized!
        return impl_;
    }

    /**
         Set the dependency.
     **/
    public void
    setDependency(IRObject_impl object)
    {
        // do nothing if there is no change.
        if (object == impl_) return;

        // cut the previous dependency.
        if (impl_ != null)
            impl_.remove_ref();

        // create the new dependency.
        object.add_ref();
        impl_ = object;
    }

    /**
         Cut the dependency.
     **/
    public void
    cutDependency()
    {
        if (impl_ != null)
        {
            // cut the previous dependency.
            impl_.remove_ref();
            impl_ = null;
        }
    }
}
