/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;
import org.omg.CORBA.InterfaceDefPackage.FullInterfaceDescription;

/**
 * Implementation of the CORBA::InterfaceDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class InterfaceDef_impl
       extends Container_impl
       implements InterfaceDefOperations
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public
    InterfaceDef_impl(IFR ifr,
                      Container_impl container)
    {
        this(ifr, container, false);

        // set the tie servant object.
        setServant(new InterfaceDefPOATie(this));
    }

    public
    InterfaceDef_impl(IFR ifr,
                      Container_impl container,
                      boolean dummy)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        // init internal state.
        base_interfaces_ = new InterfaceDef_ref[0];
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The base interfaces.
     **/
    protected InterfaceDef_ref[] base_interfaces_;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  To obtain the list of base interfaces.
     **/
    protected InterfaceDef_ref[]
    getBaseInterfaces()
    {
        return base_interfaces_;
    }

    // ==================================================================
    //
    // Internal methods for the inherited IRObject_impl class.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::InterfaceDef object reference.
     **/
    public InterfaceDef
    asInterfaceDef()
    {
        return InterfaceDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // create the CORBA::InterfaceDescription.
        InterfaceDescription description = new InterfaceDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.base_interfaces =
                InterfaceDef_ref.computeRepositoryIdSeq(getBaseInterfaces());

        // insert it into the any.
        InterfaceDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For Container_impl class.
    //
    // ==================================================================

    /**
         Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(DefinitionKind dk)
    {
        return (dk == DefinitionKind.dk_Attribute)
            || (dk == DefinitionKind.dk_Constant)
            || (dk == DefinitionKind.dk_Exception)
            || (dk == DefinitionKind.dk_Operation)
            || (dk == DefinitionKind.dk_Alias)
            || (dk == DefinitionKind.dk_Struct)
            || (dk == DefinitionKind.dk_Union)
            || (dk == DefinitionKind.dk_Enum)
            || (dk == DefinitionKind.dk_Native);
    }

    /**
        Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        // calls the Container_impl::validateName operations.
        super.checkName(name, exclude_inherited, in_inherited);

        if(!exclude_inherited)
        {
            InterfaceDef_ref[] bases = getBaseInterfaces();
            for(int i=0; i<bases.length; i++)
                bases[i].getImpl().checkName(name, false, true);
        }
    }

    /**
        Select named contained objects.
     **/
    public java.util.Vector
    selectNamedContained(java.util.Vector selection,
                         String search_name,
                         int levels_to_search,
                         DefinitionKind limit_type,
                         boolean exclude_inherited)
    {
        // call the Container_impl::selectContained operation.
        super.selectNamedContained(selection, search_name, levels_to_search,
                                   limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            InterfaceDef_ref[] bases = getBaseInterfaces();
            for(int i=0; i<bases.length; i++)
            {
                bases[i].getImpl().selectNamedContained(selection,
                                    search_name, levels_to_search,
                                    limit_type, false);
            }
        }

        return selection;
    }

    /**
        Select contained objects.
     **/
    public java.util.Vector
    selectContained(java.util.Vector selection,
                    DefinitionKind limit_type,
                    boolean exclude_inherited)
    {
        // call the Container_impl::selectContained operation.
        super.selectContained(selection, limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            InterfaceDef_ref[] bases = getBaseInterfaces();
            for(int i=0; i<bases.length; i++)
            {
                bases[i].getImpl().selectContained(
                                           selection, limit_type, false);
            }
        }

        return selection;
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Interface;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to base interfaces.
        InterfaceDef_ref.cutDependencies(base_interfaces_);
        base_interfaces_ = null;

        // call the Container_impl::destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_interface_tc(id(), name());
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public Contained
    lookup(String search_name)
    {
        // call the Container_impl::lookup operation.
        Contained contained1 = super.lookup(search_name);

        if (contained1 == null)
        {
            InterfaceDef_ref[] bases = getBaseInterfaces();
            for(int i=0; i<bases.length; i++)
            {
                Contained contained2 = bases[i].getImpl().lookup(search_name);
                if(contained2 != null)
                {
                    if(contained1 != null)
                    {
                        String id1 = contained1.id();
                        String id2 = contained2.id();
                        if(!id1.equals(id2))
                            throw exceptionAmbiguousLookup(id1, id2);
                    }

                    contained1 = contained2;
                }
            }    
        }

        return contained1;
    }

    // ==================================================================
    //
    // For CORBA::InterfaceDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/InterfaceDef/base_interfaces:1.0
     **/
    public InterfaceDef[]
    base_interfaces()
    {
        return InterfaceDef_ref.computeInterfaceDefSeq(getBaseInterfaces());
    }

    public void
    base_interfaces(InterfaceDef[] val)
    {
        InterfaceDef_impl[] itfs = new InterfaceDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            itfs[i] = castToLocal(val[i]);

        // TODO: check inheritance

        InterfaceDef_ref[] refs = new InterfaceDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new InterfaceDef_ref( itfs[i] );

        // cut the previous dependency to base interfaces.
        InterfaceDef_ref.cutDependencies(getBaseInterfaces());

        base_interfaces_ = refs;
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/is_a:1.0
     **/
    public boolean
    is_a(String interface_id)
    {
        if(interface_id.equals(id()))
            return true;

        if(interface_id.equals("IDL:omg.org/CORBA/Object:1.0"))
            return true;

        InterfaceDef_ref[] bases = getBaseInterfaces();
        for(int i=0; i<bases.length; i++)
            if (bases[i].getImpl().is_a(interface_id))
                return true;

        return false;
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/describe_interface:1.0
     **/
    public FullInterfaceDescription
    describe_interface()
    {
        FullInterfaceDescription description = new FullInterfaceDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.operations = getOpDescriptionSeq();
        description.attributes = getAttrDescriptionSeq();
        description.base_interfaces =
                InterfaceDef_ref.computeRepositoryIdSeq(getBaseInterfaces());
        description.type = type();
        return description;
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/create_attribute:1.0
     **/
    public AttributeDef
    create_attribute(String id,
                     String name,
                     String version,
                     IDLType type,
                     AttributeMode mode)
    {
        AttributeDef_impl attr = new AttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        attr.whenCreated();
        return attr.asAttributeDef();
    }

    /**
        IDL:omg.org/CORBA/InterfaceDef/create_operation:1.0
     **/
    public OperationDef
    create_operation(String id,
                     String name,
                     String version,
                     IDLType result,
                     OperationMode mode,
                     ParameterDescription[] params,
                     ExceptionDef[] exceptions,
                     String[] contexts)
    {
        OperationDef_impl op = new OperationDef_impl(getIFR(), this);

        try
        {
            addContained(op, id, name, version);
            op.result_def(result);
            op.mode(mode);
            op.params(params);
            op.exceptions(exceptions);
            op.contexts(contexts);
        }
        catch(SystemException exc)
        {
            op.destroy();
            throw exc;
        }

        op.whenCreated();
        return op.asOperationDef();
    }
}
