/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.InterfaceDef;

/**
 * Implementation of the CORBA::InterfaceDef reference.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class InterfaceDef_ref
       extends IDLType_ref
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public
    InterfaceDef_ref()
    {
        super();
    }

    public
    InterfaceDef_ref(InterfaceDef_impl object)
    {
        super(object);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
        Cut the dependency to a set of interfaces.
     **/
    static public void
    cutDependencies(InterfaceDef_ref[] itfs)
    {
        if (itfs != null)
        {
            for(int i=0; i<itfs.length; i++)
                itfs[i].cutDependency();
        }
    }

    /**
        Compute the CORBA::RepositoryIdSeq for an InterfaceDef_ref[].
     **/
    static public String[]
    computeRepositoryIdSeq(InterfaceDef_ref[] itfs)
    {
        String[] result = new String[itfs.length];

        for(int i=0; i<itfs.length; i++)
            result[i] = itfs[i].getId();

        return result;
    }

    /**
        Compute the CORBA::InterfaceDefSeq for an InterfaceDef_ref[].
     **/
    static public InterfaceDef[]
    computeInterfaceDefSeq(InterfaceDef_ref[] itfs)
    {
        InterfaceDef[] result = new InterfaceDef[itfs.length];

        for(int i=0; i<itfs.length; i++)
             result[i] = itfs[i].asInterfaceDef();

        return result;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its implementation.
     **/
    public InterfaceDef_impl
    getImpl()
    {
        return (InterfaceDef_impl)(super.getBaseImpl());
    }

    /**
        Obtain its CORBA::InterfaceDef reference.
     **/
    public InterfaceDef
    asInterfaceDef()
    {
        if (impl_ == null) return null;
        return getImpl().asInterfaceDef();
    }

    /**
        Obtain its repository ID.
     **/
    public String
    getId()
    {
        if (impl_ == null) return "";
        return getImpl().id();
    }
}
