/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Base class for storing member names and types.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class Members
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The contained object.
     **/
    protected Contained_impl contained_;

    /**
        The container object.
     **/
    protected Container_impl container_;

    /**
        The members names.
     **/
    protected String[] names_;

    /**
        The members types.
    **/
    protected IDLType_ref[] types_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    Members(Contained_impl contained, Container_impl container)
    {
        // init internal state.
        contained_ = contained;
        container_ = container;
        names_ = new String[0];
        types_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
         Check if an array of names.
     **/
    protected void
    checkNames(String[] names)
    {
        for(int i=0; i<names.length; i++)
	{
            String name = names[i];

            for(int j=i+1; j<names.length; j++)
                if(name.equalsIgnoreCase(names[j]))
                    throw contained_.exceptionDuplicateDeclarator(name, j);

            if (container_ != null)
	    {
                container_.checkName(name, false, false);

                if (name.equalsIgnoreCase(contained_.name()))
                   throw contained_.exceptionNameAlreadyUsedByImmediateScope(name);
	    }
	}
    }

    /**
        Set names and types.
     **/
    protected void
    setNamesTypes(String[] names,
                  IDLType_ref[] types)
    {
        // cut the previous dependency to members types.
        cutDependencies();

        names_ = names;
        types_ = types;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Returns member names.
     **/
    public String[]
    getNames()
    {
        return names_;
    }

    /**
         Check if a name is also a member name.
     **/
    public void
    checkName(String name)
    {
        for(int i=0; i<names_.length; i++)
            if(name.equalsIgnoreCase(names_[i]))
                throw contained_.exceptionNameAlreadyUsed(name);
    }

    /**
        Cut members types dependencies.
     **/
    public void
    cutDependencies()
    {
        if (types_ != null)
        {
            for(int i=0; i<types_.length; i++)
                types_[i].cutDependency();
        }
    }
}
