/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ModuleDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ModuleDef_impl
       extends Container_impl
       implements ModuleDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ModuleDef_impl(IFR ifr,
                   Container_impl container)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new ModuleDefPOATie(this));
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ModuleDef object reference.
     **/
    public ModuleDef
    asModuleDef()
    {
        return ModuleDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // create the CORBA::ModuleDescription.
        ModuleDescription description = new ModuleDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();

        // insert it into the any.
        ModuleDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Module;
    }

    // ==================================================================
    //
    // For CORBA::ModuleDef interface.
    //
    // ==================================================================

}
