/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::PrimitiveDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class PrimitiveDef_impl
       extends IRObject_impl
       implements PrimitiveDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The TypeCode.
     **/
    protected TypeCode tc_;

    /**
        The primitive kind.
     **/
    protected PrimitiveKind primitive_kind_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    PrimitiveDef_impl(IFR ifr,
                      PrimitiveKind pk)
    {
        // call the IRObject_impl contructor.
        super(ifr);

        // create the tie servant object.
        setServant(new PrimitiveDefPOATie(this));

        // obtain the TypeCodeFactory.
        ORB tcf = ifr.getTCF();

        //
        // create the appropriate TypeCode.
        //
        switch(pk.value())
        {
        case PrimitiveKind._pk_null:
            tc_ = tcf.get_primitive_tc(TCKind.tk_null);
            break;

        case PrimitiveKind._pk_void:
            tc_ = tcf.get_primitive_tc(TCKind.tk_void);
            break;

        case PrimitiveKind._pk_short:
            tc_ = tcf.get_primitive_tc(TCKind.tk_short);
            break;

        case PrimitiveKind._pk_long:
            tc_ = tcf.get_primitive_tc(TCKind.tk_long);
            break;

        case PrimitiveKind._pk_ushort:
            tc_ = tcf.get_primitive_tc(TCKind.tk_ushort);
            break;

        case PrimitiveKind._pk_ulong:
            tc_ = tcf.get_primitive_tc(TCKind.tk_ulong);
            break;

        case PrimitiveKind._pk_float:
            tc_ = tcf.get_primitive_tc(TCKind.tk_float);
            break;

        case PrimitiveKind._pk_double:
            tc_ = tcf.get_primitive_tc(TCKind.tk_double);
            break;

        case PrimitiveKind._pk_boolean:
            tc_ = tcf.get_primitive_tc(TCKind.tk_boolean);
            break;

        case PrimitiveKind._pk_char:
            tc_ = tcf.get_primitive_tc(TCKind.tk_char);
            break;

        case PrimitiveKind._pk_octet:
            tc_ = tcf.get_primitive_tc(TCKind.tk_octet);
            break;

        case PrimitiveKind._pk_any:
            tc_ = tcf.get_primitive_tc(TCKind.tk_any);
            break;

        case PrimitiveKind._pk_TypeCode:
            tc_ = tcf.get_primitive_tc(TCKind.tk_TypeCode);
            break;

        case PrimitiveKind._pk_Principal:
            tc_ = tcf.get_primitive_tc(TCKind.tk_Principal);
            break;

        case PrimitiveKind._pk_string:
            tc_ = tcf.create_string_tc(0);
            break;

        case PrimitiveKind._pk_objref:
            tc_ = tcf.create_interface_tc("IDL:omg.org/CORBA/Object:1.0",
                                          "Object");
            break;

        case PrimitiveKind._pk_longlong:
            tc_ = tcf.get_primitive_tc(TCKind.tk_longlong);
            break;

        case PrimitiveKind._pk_ulonglong:
            tc_ = tcf.get_primitive_tc(TCKind.tk_ulonglong);
            break;

        case PrimitiveKind._pk_longdouble:
            tc_ = tcf.get_primitive_tc(TCKind.tk_longdouble);
            break;

        case PrimitiveKind._pk_wchar:
            tc_ = tcf.get_primitive_tc(TCKind.tk_wchar);
            break;

        case PrimitiveKind._pk_wstring:
            tc_ = tcf.create_wstring_tc(0);
            break;

        case PrimitiveKind._pk_value_base:
	    tc_ = org.objectweb.ccm.CORBA.TheORB.create_valuebase_tc();
	    break;
        }

        // set the PrimitiveKind.
        primitive_kind_ = pk;
    }

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::PrimitiveDef object reference.
     **/
    public PrimitiveDef
    asPrimitiveDef()
    {
        return PrimitiveDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Primitive;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // this is an indestructible IFR object!
        throw exceptionCanNotBeDestroyed();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        return tc_;
    }

    // ==================================================================
    //
    // For CORBA::PrimitiveDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/PrimitiveDef/kind:1.0
     **/
    public PrimitiveKind
    kind()
    {
        return primitive_kind_;
    }
}
