/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::ProvidesDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ProvidesDef_impl
       extends FeatureContained
       implements org.omg.CORBA.ComponentIR.ProvidesDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The provided interface.
     **/
    protected InterfaceDef_ref interface_type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    ProvidesDef_impl(IFR ifr,
                     Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

	// set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.ProvidesDefPOATie(this));

        // inits internal state.
        interface_type_ = new InterfaceDef_ref();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::ProvidesDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.ProvidesDef
    asProvidesDef()
    {
        return org.omg.CORBA.ComponentIR.ProvidesDefHelper.narrow(asObject());
    }

    /**
     **  Obtain the CORBA::ComponentIR::ProvidesDescription of the declaration.
     **/
    public org.omg.CORBA.ComponentIR.ProvidesDescription
    describe_provides()
    {
        // create the CORBA::ProvidesDescription.
        org.omg.CORBA.ComponentIR.ProvidesDescription description =
            new org.omg.CORBA.ComponentIR.ProvidesDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.interface_type = interface_type().id();

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
     **  To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(org.omg.CORBA.Any any)
    {
        // insert it into the any.
        org.omg.CORBA.ComponentIR.ProvidesDescriptionHelper.insert(any, describe_provides());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        return org.omg.CORBA.DefinitionKind.dk_Provides;
    }

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to the provided interface type.
        interface_type_.cutDependency();
        interface_type_ = null;

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::ComponentIR::ProvidesDef interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/ComponentIR/ProvidesDef/interface_type:1.0
     **/
    public org.omg.CORBA.InterfaceDef
    interface_type()
    {
        // delegate to the InterfaceDef reference.
        return interface_type_.asInterfaceDef();
    }

    // Warning: interface_type is a readonly attribute!
    //
    // Here, this allows us to implement the CORBA::Component::create_provides()
    // operation in the same way that other create_* operations.
    // 
    /**
     **  IDL:omg.org/CORBA/ComponentIR/ProvidesDef/interface_type:1.0
     **/
    public void
    interface_type(org.omg.CORBA.InterfaceDef val)
    {
        // try to cast val to a local InterfaceDef object
        // and delegate to the InterfaceDef reference.
        interface_type_.setDependency(castToLocal(val));
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        // compute the container base repository id
        String cont_base_id = getBaseContainerID();

        // provides mappping :
        //
        //  <interface_type> provide_<name>();
        //
        String name = "provide_" + name();
        String version = version();
        String id = cont_base_id + name + ":" + version;
        InterfaceDef_impl container = (InterfaceDef_impl)container_;
        container_.mapping_started_ = true;
        container.create_operation(id, name, version,
                                   interface_type_.asIDLType(),
                                   org.omg.CORBA.OperationMode.OP_NORMAL,
                                   new org.omg.CORBA.ParameterDescription[0],
                                   new org.omg.CORBA.ExceptionDef[0],
                                   new java.lang.String[0]);
        container_.mapping_started_ = false;

        //
        //  CIDL mapping
        //
        // check if the CCM_<interface_name> already exists.
        cont_base_id = interface_type_.getImpl().getBaseContainerID();
        version = interface_type_.getImpl().version();

        name = "CCM_"+interface_type_.getImpl().name();
        id = cont_base_id + name + ":" + version;
        org.omg.CORBA.Contained contained = getIFR().getRepository().lookup_mapped_id(id);
        org.omg.CORBA.InterfaceDef facet_executor = null;
        if (contained==null)
        {
            // 
            //  local interface CCM_<interface_name> : <interface_name> {};
            //
            org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[1];
            bases[0] = interface_type();
            interface_type_.getImpl().container_.mapping_started_ = true;
            facet_executor = interface_type_.getImpl().container_.create_ext_local_interface(id, name, version, bases);
            interface_type_.getImpl().container_.mapping_started_ = false;
        }
        else
        {
            facet_executor = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(contained);
        }

        //  local interface CCM_<component_name> : ...
        //  {
        //      CCM_<interface_name> get_<name>();
        //  };
        //
        cont_base_id = container_.getBaseContainerID();
        version = container_.version();

        String litf_id = cont_base_id + "CCM_" + container_.name() + ":" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
             getIFR().getRepository().lookup_mapped_id(litf_id));

        cont_base_id = computeBaseID(litf_id);
        version = litf.version();
        name = "get_" + name();
        id = cont_base_id + name + ":" + version;

        litf.create_operation(id, name, version,
                              facet_executor,
                              org.omg.CORBA.OperationMode.OP_NORMAL,
                              new org.omg.CORBA.ParameterDescription[0],
                              new org.omg.CORBA.ExceptionDef[0],
                              new java.lang.String[0]);
    }
}
