/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::Repository interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class Repository_impl
             extends Container_impl
             implements RepositoryOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    Repository_impl(IFR ifr)
    {
        // call the Contained_impl contructor.
        this(ifr, false);
    }

    public
    Repository_impl(IFR ifr,
                    boolean enable_mappings)
    {
        // call the Contained_impl contructor.
        super(ifr, null);

        // set the tie servant object.
        setServant(new RepositoryPOATie(this));

        //
        // create the PrimitiveDef objects.
        //
        primDef_null_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_null);
        primDef_void_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_void);
        primDef_short_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_short);
        primDef_long_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_long);
        primDef_ushort_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_ushort);
        primDef_ulong_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_ulong);
        primDef_float_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_float);
        primDef_double_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_double);
        primDef_boolean_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_boolean);
        primDef_char_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_char);
        primDef_octet_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_octet);
        primDef_any_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_any);
        primDef_TypeCode_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_TypeCode);
        primDef_Principal_ = new PrimitiveDef_impl(ifr,
                                                   PrimitiveKind.pk_Principal);
        primDef_string_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_string);
        primDef_objref_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_objref);
        primDef_longlong_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_longlong);
        primDef_ulonglong_ = new PrimitiveDef_impl(ifr,
                                                   PrimitiveKind.pk_ulonglong);
        primDef_longdouble_ = new PrimitiveDef_impl(ifr,
                                                    PrimitiveKind.pk_longdouble);
        primDef_wchar_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_wchar);
        primDef_wstring_ = new PrimitiveDef_impl(ifr, PrimitiveKind.pk_wstring);
        primDef_value_base_ = new PrimitiveDef_impl(ifr,
                                                    PrimitiveKind.pk_value_base);

        // creates the repository ID list.
        repID_elements_ = new java.util.Hashtable();
        mapped_elements_ = new java.util.Hashtable();

        // Init internal state.
        isIDL2Repository_ = true;
        enable_mappings_ = enable_mappings;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected boolean isIDL2Repository_;

    /**
     **
     **/
    protected boolean enable_mappings_;

    /**
        All the PrimitiveDef objects.
     **/
    protected PrimitiveDef_impl primDef_null_;
    protected PrimitiveDef_impl primDef_void_;
    protected PrimitiveDef_impl primDef_short_;
    protected PrimitiveDef_impl primDef_long_;
    protected PrimitiveDef_impl primDef_ushort_;
    protected PrimitiveDef_impl primDef_ulong_;
    protected PrimitiveDef_impl primDef_float_;
    protected PrimitiveDef_impl primDef_double_;
    protected PrimitiveDef_impl primDef_boolean_;
    protected PrimitiveDef_impl primDef_char_;
    protected PrimitiveDef_impl primDef_octet_;
    protected PrimitiveDef_impl primDef_any_;
    protected PrimitiveDef_impl primDef_TypeCode_;
    protected PrimitiveDef_impl primDef_Principal_;
    protected PrimitiveDef_impl primDef_string_;
    protected PrimitiveDef_impl primDef_objref_;
    protected PrimitiveDef_impl primDef_longlong_;
    protected PrimitiveDef_impl primDef_ulonglong_;
    protected PrimitiveDef_impl primDef_longdouble_;
    protected PrimitiveDef_impl primDef_wchar_;
    protected PrimitiveDef_impl primDef_wstring_;
    protected PrimitiveDef_impl primDef_value_base_;

    /**
     **  All the Contained objects.
     **/
    protected java.util.Hashtable repID_elements_;

    /**
     **  All the Contained objects that comes from a mapping.
     **/
    protected java.util.Hashtable mapped_elements_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  To know if the repository is viewed as an IDL2 or IDL3 repository.
     **/
    public boolean
    isIDL2Repository()
    {
        return isIDL2Repository_;
    }

    /**
        Obtain its CORBA::Repository object reference.
     **/
    public Repository
    asRepository()
    {
        return RepositoryHelper.narrow(asObject());
    }

    /**
        Add a Contained_impl object.
     **/
    public void
    addRID(String id, Contained_impl contained)
    {
        if (contained.container_.mapping_started_)
            mapped_elements_.put(id, contained);
        else
            repID_elements_.put(id, contained);
    }

    /**
        Remove a Contained_impl object.
     **/
    public void
    removeRID(String id)
    {
         repID_elements_.remove(id);
    }

    /**
        Obtain a Contained_impl object by its repository ID.
     **/
    public Contained_impl
    getRID(String id)
    {
        if (!isIDL2Repository())
            return (Contained_impl)repID_elements_.get(id);

        java.lang.Object obj = mapped_elements_.get(id);
        if (obj!=null)
            return (Contained_impl)obj;

        return (Contained_impl)repID_elements_.get(id);
    }

    /**
        Obtain a Contained_impl object by its repository ID.
     **/
    public Contained_impl
    getMappedRID(String id)
    {
        return (Contained_impl)mapped_elements_.get(id);
    }

    /**
        Obtain the PrimitiveDef for the void type.
     **/
    public PrimitiveDef_impl
    getPrimDefVoid()
    {
        return primDef_void_;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.

        Do nothing  because this method is here due to the lack of
        multiple inheritance in Java!
    **/
    public void
    setDescriptionValue(Any any)
    {
       // Nothing to do!
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Repository;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // this is an indestructible IFR object!
        throw exceptionCanNotBeDestroyed();
    }

    // ==================================================================
    //
    // For CORBA::Contained interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public Contained
    lookup(String search_name)
    {
        // Remove the :: at the begining of the search_name.
        while(search_name.startsWith("::"))
            search_name = search_name.substring(2);

        // Call the Container_impl::lookup() operation.
        return super.lookup(search_name);
    }

    // ==================================================================
    //
    // For CORBA::Repository interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/Repository/lookup_id:1.0
     **/
    public Contained
    lookup_id(String search_id)
    {
        Contained_impl contained = getRID(search_id);

        // If not found then returns a nil contained reference.
        if (contained == null) return null;
        
        return contained.asContained();
    }

    /**
        IDL:omg.org/CORBA/Repository/get_canonical_typecode:1.0
     **/
    public TypeCode
    get_canonical_typecode(TypeCode tc)
    {
        // Something to do?
        // But this depends on the ORB!
        return tc;
    }

    /**
        IDL:omg.org/CORBA/Repository/get_primitive:1.0
     **/
    public PrimitiveDef
    get_primitive(PrimitiveKind kind)
    {
        switch(kind.value())
        {
        case PrimitiveKind._pk_null:
	    return primDef_null_.asPrimitiveDef();

        case PrimitiveKind._pk_void:
	    return primDef_void_.asPrimitiveDef();

        case PrimitiveKind._pk_short:
	    return primDef_short_.asPrimitiveDef();

        case PrimitiveKind._pk_long:
	    return primDef_long_.asPrimitiveDef();

        case PrimitiveKind._pk_ushort:
	    return primDef_ushort_.asPrimitiveDef();

        case PrimitiveKind._pk_ulong:
	    return primDef_ulong_.asPrimitiveDef();

        case PrimitiveKind._pk_float:
	    return primDef_float_.asPrimitiveDef();

        case PrimitiveKind._pk_double:
	    return primDef_double_.asPrimitiveDef();

        case PrimitiveKind._pk_boolean:
	    return primDef_boolean_.asPrimitiveDef();

        case PrimitiveKind._pk_char:
	    return primDef_char_.asPrimitiveDef();

        case PrimitiveKind._pk_octet:
	    return primDef_octet_.asPrimitiveDef();

        case PrimitiveKind._pk_any:
	    return primDef_any_.asPrimitiveDef();

        case PrimitiveKind._pk_TypeCode:
	    return primDef_TypeCode_.asPrimitiveDef();

        case PrimitiveKind._pk_Principal:
	    return primDef_Principal_.asPrimitiveDef();

        case PrimitiveKind._pk_string:
	    return primDef_string_.asPrimitiveDef();

        case PrimitiveKind._pk_objref:
	    return primDef_objref_.asPrimitiveDef();

        case PrimitiveKind._pk_longlong:
	    return primDef_longlong_.asPrimitiveDef();

        case PrimitiveKind._pk_ulonglong:
	    return primDef_ulonglong_.asPrimitiveDef();

        case PrimitiveKind._pk_longdouble:
	    return primDef_longdouble_.asPrimitiveDef();

        case PrimitiveKind._pk_wchar:
	    return primDef_wchar_.asPrimitiveDef();

        case PrimitiveKind._pk_wstring:
	    return primDef_wstring_.asPrimitiveDef();

        case PrimitiveKind._pk_value_base:
	    return primDef_value_base_.asPrimitiveDef();
        }

       return null;
    }

    /**
        IDL:omg.org/CORBA/Repository/create_string:1.0
     **/
    public StringDef
    create_string(int bound)
    {
        StringDef_impl s = new StringDef_impl(getIFR());

        try
        {
            s.bound(bound);
        }
        catch(SystemException exc)
        {
            s.destroy();
            throw exc;
        }

        return s.asStringDef();
    }

    /**
        IDL:omg.org/CORBA/Repository/create_wstring:1.0
     **/
    public WstringDef
    create_wstring(int bound)
    {
        WstringDef_impl s = new WstringDef_impl(getIFR());

        try
        {
            s.bound(bound);
        }
        catch(SystemException exc)
        {
            s.destroy();
            throw exc;
        }

        return s.asWstringDef();
    }

    /**
        IDL:omg.org/CORBA/Repository/create_sequence:1.0
     **/
    public SequenceDef
    create_sequence(int bound,
                    IDLType element_type)
    {
        SequenceDef_impl s = new SequenceDef_impl(getIFR());

        try {
            s.bound(bound);
            s.element_type_def(element_type);
        } catch(SystemException exc) {
            s.destroy();
            throw exc;
        }

        return s.asSequenceDef();
    }

    /**
        IDL:omg.org/CORBA/Repository/create_array:1.0
     **/
    public ArrayDef
    create_array(int length,
                 IDLType element_type)
    {
        ArrayDef_impl a = new ArrayDef_impl(getIFR());

        try
        {
            a.length(length);
            a.element_type_def(element_type);
        }
        catch(SystemException exc)
        {
            a.destroy();
            throw exc;
        }

        return a.asArrayDef();
    }

    /**
        IDL:omg.org/CORBA/Repository/create_fixed:1.0
     **/
    public FixedDef
    create_fixed(short digits,
                 short scale)
    {
        FixedDef_impl f = new FixedDef_impl(getIFR());

        try
        {
            f.digits(digits);
            f.scale(scale);
        }
        catch(SystemException exc)
        {
            f.destroy();
            throw exc;
        }

        return f.asFixedDef();
    }
}
