/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::SequenceDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class SequenceDef_impl
       extends AnonymousIDLType
       implements SequenceDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    SequenceDef_impl(IFR ifr)
    {
        // call the AnonymousIDLType contructor.
        super(ifr);

        // set the TIE servant object.
        setServant(new SequenceDefPOATie(this));

        // set the bound to zero.
        bound_ = 0;

        // create the element_type_def reference.
        element_type_def_ = new IDLType_ref();
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The sequence bound.
     **/
    protected int bound_;

    /**
        The sequence element type.
     **/
    protected IDLType_ref element_type_def_;

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        TypeCode tc = null;
        try
        {
            Contained_impl cont = (Contained_impl)element_type_def_.getBaseImpl();
            int idx = idSeq.indexOf(cont.id());
            if (idx!=-1)
            {
                // check if the contained can be a recursive type
                int dk = cont.def_kind().value();
                if ((dk!=DefinitionKind._dk_Struct) &&
                    (dk!=DefinitionKind._dk_Exception) &&
                    (dk!=DefinitionKind._dk_Union) &&
                    (dk!=DefinitionKind._dk_Value) &&
                    (dk!=DefinitionKind._dk_Alias))
                    throw exceptionIllegalRecursion("anonymous sequence", cont.id());

                tc = getIFR().getTCF().create_recursive_tc(cont.id());
            }
        }
        catch (java.lang.ClassCastException ex)
        {
        }

        if (tc==null)
            tc = element_type_def_.recursiveType(idSeq);

        return getIFR().getTCF().create_sequence_tc(bound(), tc);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::SequenceDef object reference.
     **/
    public SequenceDef
    asSequenceDef()
    {
        return SequenceDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Sequence;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency.
        element_type_def_.cutDependency();

        // call the IRObject_impl::destroy method.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::SequenceDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/SequenceDef/bound:1.0
     **/
    public int
    bound()
    {
        return bound_;
    }

    public void
    bound(int b)
    {
        bound_ = b;
    }

    /**
        IDL:omg.org/CORBA/SequenceDef/element_type:1.0
     **/
    public TypeCode
    element_type()
    {
        // delegate to the IDLType reference.
        return element_type_def_.recursiveType();
    }

    /**
        IDL:omg.org/CORBA/SequenceDef/element_type_def:1.0
     **/
    public IDLType
    element_type_def()
    {
        // delegate to the IDLType reference.
        return element_type_def_.asIDLType();
    }

    public void
    element_type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        element_type_def_.setDependency( castToLocal(val) );
    }
}
