/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::StructDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructDef_impl
       extends Container_impl
       implements StructDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    StructDef_impl(IFR ifr,
                   Container_impl container)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new StructDefPOATie(this));

        // init internal state.
        members_ = new StructMembers(this, this);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The struct members.
     **/
    protected StructMembers members_;

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        int idx = idSeq.indexOf(id());
        if (idx!=-1)
            return getIFR().getTCF().create_recursive_tc(id());

        idSeq.add(id());
        StructMember[] members = members_.getStructMemberSeq(idSeq);
        idSeq.remove(id());

        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_struct_tc(id(), name(), members);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::StructDef object reference.
     **/
    public StructDef
    asStructDef()
    {
        return StructDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For Container_impl class.
    //
    // ==================================================================

    /**
         Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(DefinitionKind dk)
    {
        // Only Structs, Unions and Enums are accepted!
        return (dk == DefinitionKind.dk_Struct)
            || (dk == DefinitionKind.dk_Union)
            || (dk == DefinitionKind.dk_Enum);
    }

    /**
        Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        // call the Container_impl::checkName operation.
        super.checkName(name, exclude_inherited, in_inherited);

        // check if the name is a member name.
        members_.checkName(name);
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Struct;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut dependencies to struct members types.
        members_.cutDependencies();

        // call the Container_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // For CORBA::StructDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/StructDef/members:1.0
     **/
    public StructMember[]
    members()
    {
        // delegate to the StructMembers.
        return members_.getStructMemberSeq();
    }

    public void
    members(StructMember[] val)
    {
        // delegate to the StructMembers.
        members_.setStructMemberSeq(val);
    }
}
