/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::StructMemberSeq helper class.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructMembers
       extends Members
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    StructMembers(Contained_impl contained, Container_impl container)
    {
        super(contained, container);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain as CORBA::StructMemberSeq
     **/
    public StructMember[]
    getStructMemberSeq(org.objectweb.ccm.util.Vector idSeq)
    {
        StructMember[] result = new StructMember[names_.length];

        for(int i=0; i<result.length; i++)
	{
            result[i] = new StructMember();
            result[i].name = names_[i];
            // recompute the typecode of each member.
            result[i].type = types_[i].recursiveType(idSeq);
            result[i].type_def = types_[i].asIDLType();
	}

        return result;
    }

    /**
        Obtain as CORBA::StructMemberSeq
     **/
    public StructMember[]
    getStructMemberSeq()
    {
        StructMember[] result = new StructMember[names_.length];
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();

        for(int i=0; i<result.length; i++)
	{
            idSeq.clear();
            result[i] = new StructMember();
            result[i].name = names_[i];
            // recompute the typecode of each member.
            result[i].type = types_[i].recursiveType(idSeq);
            result[i].type_def = types_[i].asIDLType();
	}

        return result;
    }

    /**
        Set as CORBA::StructMemberSeq
     **/
    public void
    setStructMemberSeq(StructMember[] val)
    {
        // duplicates members names.
        String[] names = new String[val.length];
        for (int i=0; i<val.length; i++)
            names[i] = val[i].name;

        checkNames(names);

        IRObject_impl[] types = new IRObject_impl[val.length];
        for (int i=0; i<val.length; i++)
            types[i] = contained_.castToLocal(val[i].type_def);

        IDLType_ref[] refs = new IDLType_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new IDLType_ref( types[i] );

        super.setNamesTypes(names, refs);
    }
}
