/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::UnionMemberSeq helper class.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UnionMembers
       extends Members
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    UnionMembers(UnionDef_impl owner)
    {
        super(owner, owner);

        // init internal state.
        labels_ = null;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

   /**
        The members labels.
    **/
    protected Any[] labels_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain as CORBA::UnionMemberSeq
     **/
    public UnionMember[]
    getUnionMemberSeq(org.objectweb.ccm.util.Vector idSeq)
    {
        UnionMember[] result = new UnionMember[names_.length];

        for(int i=0; i<result.length; i++)
	{
            result[i] = new UnionMember();
            result[i].name = names_[i];
            result[i].label = labels_[i];
            // recompute the typecode of each member.
            result[i].type = types_[i].recursiveType(idSeq);
            result[i].type_def = types_[i].asIDLType();
	}

        return result;
    }

    /**
        Obtain as CORBA::UnionMemberSeq
     **/
    public UnionMember[]
    getUnionMemberSeq()
    {
        UnionMember[] result = new UnionMember[names_.length];
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();

        for(int i=0; i<result.length; i++)
	{
            idSeq.clear();
            result[i] = new UnionMember();
            result[i].name = names_[i];
            result[i].label = labels_[i];
            // recompute the typecode of each member.
            result[i].type = types_[i].recursiveType(idSeq);
            result[i].type_def = types_[i].asIDLType();
	}

        return result;
    }

    /**
        Set as CORBA::UnionMemberSeq
     **/
    public void
    setUnionMemberSeq(UnionMember[] val)
    {
        // TODO checks members type and label.

        // duplicates members names.
        String[] names = new String[val.length];
        for (int i=0; i<val.length; i++)
            names[i] = val[i].name;

        for(int i=0; i<names.length; i++)
	{
            String name = names[i];

            container_.checkName(name, false, false);

            if (name.equalsIgnoreCase(contained_.name()))
                contained_.exceptionNameAlreadyUsedByImmediateScope(name);
	}

        // duplicates members labels.
        Any[] labels = new Any[val.length];
        for (int i=0; i<val.length; i++)
            labels[i] = val[i].label;

        IRObject_impl[] types = new IRObject_impl[val.length];
        for (int i=0; i<val.length; i++)
            types[i] = contained_.castToLocal(val[i].type_def);

        IDLType_ref[] refs = new IDLType_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new IDLType_ref( types[i] );

        labels_ = labels;
        super.setNamesTypes(names, refs);
    }
}
