/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

/**
 * Implementation of the CORBA::ComponentIR::UsesDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UsesDef_impl
       extends FeatureContained
       implements org.omg.CORBA.ComponentIR.UsesDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The provided interface.
     **/
    protected InterfaceDef_ref interface_type_;

    /**
     **  Is multiple.
     **/
    protected boolean is_multiple_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default constructor.
     **/
    public
    UsesDef_impl(IFR ifr,
                 Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new org.omg.CORBA.ComponentIR.UsesDefPOATie(this));

        // inits internal state.
        interface_type_ = new InterfaceDef_ref();
        is_multiple_ = false;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA::ComponentIR::UsesDef object reference.
     **/
    public org.omg.CORBA.ComponentIR.UsesDef
    asUsesDef()
    {
        return org.omg.CORBA.ComponentIR.UsesDefHelper.narrow(asObject());
    }

    /**
     **  Obtain the CORBA::ComponentIR::UsesDescription of the declaration.
     **/
    public org.omg.CORBA.ComponentIR.UsesDescription
    describe_uses()
    {
        // create the CORBA::UsesDescription.
        org.omg.CORBA.ComponentIR.UsesDescription description = new org.omg.CORBA.ComponentIR.UsesDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.interface_type = interface_type().id();
        description.is_multiple = is_multiple();

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
     **  To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(org.omg.CORBA.Any any)
    {
        // insert it into the any.
        org.omg.CORBA.ComponentIR.UsesDescriptionHelper.insert(any, describe_uses());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public org.omg.CORBA.DefinitionKind
    def_kind()
    {
        return org.omg.CORBA.DefinitionKind.dk_Uses;
    }

    /**
     **  IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to the provided interface type.
        interface_type_.cutDependency();
        interface_type_ = null;

        // calls the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::UsesDef interface.
    //
    // ==================================================================

    /**
     **  IDL:omg.org/CORBA/UsesDef/interface_type:1.0
     **/
    public org.omg.CORBA.InterfaceDef
    interface_type()
    {
        // delegate to the InterfaceDef reference.
        return interface_type_.asInterfaceDef();
    }

    // Warning: interface_type is a readonly attribute!
    //
    // Here, this allows us to implement the CORBA::ComponentIR::Component::create_uses()
    // operation in the same way that other create_* operations.
    // 
    public void
    interface_type(org.omg.CORBA.InterfaceDef val)
    {
        // try to cast val to a local InterfaceDef object
        // and delegate to the IDLType reference.
        interface_type_.setDependency(castToLocal(val));
    }

    /**
     **  IDL:omg.org/CORBA/UsesDef/is_multiple:1.0
     **/
    public boolean
    is_multiple()
    {
        return is_multiple_;
    }

    // Warning: is_multiple is a readonly attribute!
    //
    // Here, this allows us to implement the CORBA::Component::create_uses()
    // operation in the same way that other create_* operations.
    // 
    public void
    is_multiple(boolean val)
    {
        is_multiple_ = val;
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        // compute the container base repository id
        String cont_base_id = getBaseContainerID();

        String version = version();
        String name = "";
        String id = "";
        InterfaceDef_impl container = (InterfaceDef_impl)container_;
        org.omg.CORBA.AliasDef connection_sequence = null;

        // uses multiple mapping :
        if (is_multiple_)
        {
            //
            //  struct <receptacle_name>Connection {
            //      <interface_type> objref;
            //      Components::Cookie ck;
            //  };
            //
            name = name() + "Connection";
            id = cont_base_id + name + ":" + version;
            org.omg.CORBA.StructMember[] members = new org.omg.CORBA.StructMember[2];
            members[0] = new org.omg.CORBA.StructMember();
            members[0].name = "objref";
            members[0].type_def = interface_type_.asIDLType();
            members[0].type = members[0].type_def.type();
            members[1] = new org.omg.CORBA.StructMember();
            members[1].name = "ck";
            members[1].type_def = org.omg.CORBA.IDLTypeHelper.narrow(
                getIFR().getRepository().lookup("::Components::Cookie"));
            members[1].type = members[1].type_def.type();

            container_.mapping_started_ = true;
            org.omg.CORBA.IDLType struct = container_.create_struct(id, name, version, members);
            container_.mapping_started_ = false;
    
            //
            //  sequence<<receptacle_name>Connection> <receptacle_name>Connections;
            //
            name = name() + "Connections";
            id = cont_base_id + name + ":" + version;
            org.omg.CORBA.IDLType type = getIFR().getRepository().create_sequence(0, struct);

            container_.mapping_started_ = true;
            connection_sequence = container_.create_alias(id, name, version, type);
            container_.mapping_started_ = false;

            //
            //  Components::Cookie connect_<receptacle_name>(in <interface_type> connection)
            //  raises(Components::ExceededConnectionLimit,
            //         Components::InvalidConnection);
            //
            name = "connect_" + name();
            id = cont_base_id + name + ":" + version;
            org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "connexion";
            params[0].type_def = interface_type_.asIDLType();
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[2];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::ExceededConnectionLimit"));
            excs[1] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidConnection"));

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       members[1].type_def,
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;
    
            //
            //  <interface_type> disconnect_<receptacle_name>(in Components::Cookie)
            //  raises(Components::InvalidConnection);
            //
            name = "disconnect_" + name();
            id = cont_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "ck";
            params[0].type_def = members[1].type_def;
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            org.omg.CORBA.ExceptionDef[] excs2 = new org.omg.CORBA.ExceptionDef[1];
            excs2[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidConnection"));

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       interface_type_.asIDLType(),
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;
    
            //
            //  <receptacle_name>Connections get_connections_<receptacle_name>();
            //
            name = "get_connections_" + name();
            id = cont_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[0];
            excs = new org.omg.CORBA.ExceptionDef[0];

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       connection_sequence,
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;
        }
        // uses mapping :
        else
        {
            //
            //  void connect_<receptacle_name>(in <interface_type> connection)
            //  raises(Components::AlreadyConnected,
            //         Components::InvalidConnection);
            //
            name = "connect_" + name();
            id = cont_base_id + name + ":" + version;
            org.omg.CORBA.ParameterDescription[] params = new org.omg.CORBA.ParameterDescription[1];
            params[0] = new org.omg.CORBA.ParameterDescription();
            params[0].name = "connexion";
            params[0].type_def = interface_type_.asIDLType();
            params[0].type = params[0].type_def.type();
            params[0].mode = org.omg.CORBA.ParameterMode.PARAM_IN;
            org.omg.CORBA.ExceptionDef[] excs = new org.omg.CORBA.ExceptionDef[2];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::AlreadyConnected"));
            excs[1] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::InvalidConnection"));

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       getIFR().getRepository().get_primitive(
                                           org.omg.CORBA.PrimitiveKind.pk_void),
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;

            //
            //  <interface_type> disconnect_<receptacle_name>()
            //  raises(Components::NoConnection);
            //
            name = "disconnect_" + name();
            id = cont_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[0];
            excs = new org.omg.CORBA.ExceptionDef[1];
            excs[0] = org.omg.CORBA.ExceptionDefHelper.narrow(
                getIFR().getRepository().lookup("::Components::NoConnection"));

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       interface_type_.asIDLType(),
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;
    
            //
            //  <receptacle_name>Connection get_connection_<receptacle_name>();
            //
            name = "get_connection_" + name();
            id = cont_base_id + name + ":" + version;
            params = new org.omg.CORBA.ParameterDescription[0];
            excs = new org.omg.CORBA.ExceptionDef[0];

            container_.mapping_started_ = true;
            container.create_operation(id, name, version,
                                       interface_type_.asIDLType(),
                                       org.omg.CORBA.OperationMode.OP_NORMAL,
                                       params, excs, new java.lang.String[0]);
            container_.mapping_started_ = false;
        }

        //
        //  CIDL mapping
        //
        // check if the CCM_<interface_name> already exists.
        cont_base_id = interface_type_.getImpl().getBaseContainerID();
        version = interface_type_.getImpl().version();

        name = "CCM_"+interface_type_.getImpl().name();
        id = cont_base_id + name + ":" + version;
        org.omg.CORBA.Contained contained = getIFR().getRepository().lookup_mapped_id(id);
        org.omg.CORBA.InterfaceDef facet_executor = null;
        if (contained==null)
        {
            // 
            //  local interface CCM_<interface_name> : <interface_name> {};
            //
            org.omg.CORBA.InterfaceDef[] bases = new org.omg.CORBA.InterfaceDef[1];
            bases[0] = interface_type();
            interface_type_.getImpl().container_.mapping_started_ = true;
            facet_executor = interface_type_.getImpl().container_.create_ext_local_interface(id, name, version, bases);
            interface_type_.getImpl().container_.mapping_started_ = false;
        }
        else
        {
            facet_executor = org.omg.CORBA.InterfaceDefHelper.narrow(contained);
        }

        //
        //  local interface CCM_<component_name>_Context : ...
        //  {
        //      <interface_name> get_connection_<name>();
        //  };
        //
        cont_base_id = container_.getBaseContainerID();
        version = container_.version();

        String litf_id = cont_base_id + "CCM_" + container_.name() + "_Context:" + version;
        org.omg.CORBA.ExtLocalInterfaceDef litf = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
             getIFR().getRepository().lookup_mapped_id(litf_id));

        cont_base_id = computeBaseID(litf_id);
        version = litf.version();
        // uses multiple mapping
        if (!is_multiple_)
        {
            name = "get_connection_" + name();
            id = cont_base_id + name + ":" + version;

            litf.create_operation(id, name, version,
                                  interface_type_.asIDLType(),
                                  org.omg.CORBA.OperationMode.OP_NORMAL,
                                  new org.omg.CORBA.ParameterDescription[0],
                                  new org.omg.CORBA.ExceptionDef[0],
                                  new java.lang.String[0]);
        }
        // uses mapping
        else
        {
            name = "get_connections_" + name();
            id = cont_base_id + name + ":" + version;

            litf.create_operation(id, name, version,
                                  connection_sequence,
                                  org.omg.CORBA.OperationMode.OP_NORMAL,
                                  new org.omg.CORBA.ParameterDescription[0],
                                  new org.omg.CORBA.ExceptionDef[0],
                                  new java.lang.String[0]);
        }
    }
}
