/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ValueBoxDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueBoxDef_impl
       extends Contained_impl
       implements ValueBoxDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ValueBoxDef_impl(IFR ifr,
                     Container_impl container)
    {
        // call the Contained_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new ValueBoxDefPOATie(this));

        // create the original_type_def reference.
        original_type_def_ = new IDLType_ref();
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The original type of the alias.
     **/
    protected IDLType_ref original_type_def_;

    // ==================================================================
    //
    // Internal methods for the inherited IRObject_impl class.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        try
        {
            Contained_impl cont = (Contained_impl)original_type_def_.getBaseImpl();
            int idx = idSeq.indexOf(cont.id());
            if (idx!=-1)
            {
                throw exceptionIllegalRecursion(id(), cont.id());
            }
        }
        catch (java.lang.ClassCastException ex)
        {
        }

        TypeCode tc = original_type_def_.recursiveType(idSeq);

        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_value_box_tc(id(), name(), tc);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ValueBoxDef object reference.
     **/
    public ValueBoxDef
    asValueBoxDef()
    {
        return ValueBoxDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_ValueBox;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency.
        original_type_def_.cutDependency();

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::ValueBoxDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ValueBoxDef/original_type_def:1.0
     **/
    public IDLType
    original_type_def()
    {
        // delegate to the IDLType reference.
        return original_type_def_.asIDLType();
    }

    public void
    original_type_def(IDLType val)
    {
        // Try to cast val to a local object.
        IRObject_impl type = castToLocal(val);

        // Obtain the unaliased TypeCode of the type.
        TypeCode tc = type.type();
        while (tc.kind() == TCKind.tk_alias)
        {
            try
            {
                tc = tc.content_type();
            }
            catch(org.omg.CORBA.TypeCodePackage.BadKind exc)
	    {
	    }
	}

        // Check the type code kind.
        TCKind tk = tc.kind();
        if(  tk == TCKind.tk_null
          || tk == TCKind.tk_void
          || tk == TCKind.tk_Principal
          || tk == TCKind.tk_except
          || tk == TCKind.tk_native
          || tk == TCKind.tk_abstract_interface
          || tk == TCKind.tk_value
          || tk == TCKind.tk_value_box)
            throw exceptionInvalidValueBoxType();

        // Delegate to the IDLType reference.
        original_type_def_.setDependency( type );
    }
}
