/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::ValueMemberDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueMemberDef_impl
       extends FeatureContained
       implements ValueMemberDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ValueMemberDef_impl(IFR ifr,
                        Container_impl container)
    {
        // call the FeatureContained contructor.
        super(ifr, container);

	// set the tie servant object.
        setServant(new ValueMemberDefPOATie(this));

        // create the type_def reference.
        type_def_ = new IDLType_ref();

        // inits access mode.
        access_ = (short)1;
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The type of the value member.
     **/
    protected IDLType_ref type_def_;

    /**
        The access mode of the value member.
     **/
    protected short access_;

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        try
        {
            Contained_impl cont = (Contained_impl)type_def_.getBaseImpl();
            int idx = idSeq.indexOf(cont.id());
            if (idx!=-1)
                return getIFR().getTCF().create_recursive_tc(cont.id());
        }
        catch(java.lang.ClassCastException ex)
        {
        }

        return type_def_.recursiveType(idSeq);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ValueMemberDef object reference.
     **/
    public ValueMemberDef
    asValueMemberDef()
    {
        return ValueMemberDefHelper.narrow(asObject());
    }

    /**
        Obtain its CORBA::ValueMember.
     **/
    public ValueMember
    getValueMember()
    {
        // create the CORBA::ValueMember.
        ValueMember description = new ValueMember();

        // set its fields.
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = type();
        description.type_def = type_def();
        description.access = access();

        return description;
    }

    /**
        Obtain its CORBA::ValueMember.
     **/
    public ValueMember
    getValueMember(org.objectweb.ccm.util.Vector idSeq)
    {
        // create the CORBA::ValueMember.
        ValueMember description = new ValueMember();

        // set its fields.
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = recursiveType(idSeq);
        description.type_def = type_def();
        description.access = access();

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // insert its CORBA::ValueMember into the any.
        ValueMemberHelper.insert(any, getValueMember());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_ValueMember;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency.
        type_def_.cutDependency();

        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::ValueMemberDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ValueMemberDef/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    /**
        IDL:omg.org/CORBA/ValueMemberDef/type_def:1.0
     **/
    public IDLType
    type_def()
    {
        // delegate to the IDLType reference.
        return type_def_.asIDLType();
    }

    public void
    type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        type_def_.setDependency( castToLocal(val) );
    }

    /**
        IDL:omg.org/CORBA/ValueMemberDef/access:1.0
     **/
    public short
    access()
    {
        return access_;
    }

    public void
    access(short val)
    {
        access_ = val;
    }
}
