/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::WstringDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class WstringDef_impl
       extends IRObject_impl
       implements WstringDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The string bound. */
    protected int bound_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    WstringDef_impl(IFR ifr)
    {
        // call the IRObject_impl contructor.
        super(ifr);

        // set the TIE servant object.
        setServant(new WstringDefPOATie(this));

        // set the bound to zero.
        bound_ = 0;
    }

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::WstringDef object reference.
     **/  
    public WstringDef
    asWstringDef()
    {
        return WstringDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Wstring;
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // delegates to the TypeCodeFactory.
        return getIFR().getTCF().create_wstring_tc(bound());
    }

    // ==================================================================
    //
    // For CORBA::WstringDef interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/CORBA/WstringDef/bound:1.0
    //
    public int
    bound()
    {
        return bound_;
    }

    public void
    bound(int b)
    {
        if(b == 0)
            throw exceptionBadParam("the wstring bound can't be set to zero");

        bound_ = b;
    }
}
