/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.controller;

/**
 * This class implements an transactionnal profile for the
 * OpenCCM::Containers::CallController interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public abstract class WithControlController
		extends WithCurrentController
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Reference to the current transaction
     *
     */
    protected org.omg.CosTransactions.Control control_;

    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    WithControlController()
    {
				control_ = null;
		}


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

		/**
		 ** Suspend the current transaction
		 **
		 ** @param ctx the operation call context
		 **/
		protected void 
		suspend(org.objectweb.ccm.Containers.CallContext ctx)
		{
				control_ = current_.suspend();
		}

		/**
		 ** Resume the suspended trnasaction
		 **
		 ** @param ctx the operation call context
		 **/
		protected void
		resume(org.objectweb.ccm.Containers.CallContext ctx)
		{
				if (control_ != null)
						{
								try
										{
												current_.resume(control_);
										} catch (org.omg.CosTransactions.InvalidControl ex) {
										}
								control_ = null;
						}				
		}
		
    // ==================================================================
    //
    // Public methods for the CallController interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for the CallProtocol interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallProtocol/preinvoke:1.0
    //
    /**
     **
     **/
    public abstract void
		preinvoke(org.objectweb.ccm.Containers.CallContext ctx);
		
    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallProtocol/postinvoke:1.0
    //
    /**
     **
     **/
    public abstract void
		postinvoke(org.objectweb.ccm.Containers.CallContext ctx);

    // ==================================================================
    //
    // Public methods for the SystemComponent interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for the Configuration interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/Configuration/configure:1.0
    //
    /**
     **
     **/
    public abstract void
	  configure(org.objectweb.ccm.Containers.PropertySet config)
				throws org.objectweb.ccm.Containers.ConfigurationFailed;
}
