/*====================================================================
 * 
 OpenCCM: The Open CORBA Component Model Platform
 Copyright (C) 2000-2002 USTL - LIFL - GOAL
 Contact: openccm-team@objectweb.org
 * 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 * 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 * 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 * 
 Initial developer(s): Romain Rouvoy.
 Contributor(s): ______________________________________.
 * 
 ====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment; 

import org.objectweb.ccm.Deployment.listener.simple.ComponentServerListenerSimple ;


/**
 * Transaction implementation for the
 * ::plugins::deployment::ComponentServerListener interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public class ComponentServerListenerTransaction
extends ComponentServerListenerSimple
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * List of resources non committed
   */ 
  protected org.objectweb.ccm.util.Vector resources_;
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   * Constructor with parameter
   *
   * @param source the event source
   */ 
  public
  ComponentServerListenerTransaction()
  {
    resources_ = new org.objectweb.ccm.util.Vector (); 
  }    
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  /**
   * Provide a new instance of the class
   *
   * @return new instance
   */
  protected org.objectweb.ccm.Deployment.listener.ComponentServerListener
  _new()
  {
    return new ComponentServerListenerTransaction();
  }
  
  /**
   * Provide the reference of the resource corresponding to the current transaction
   *
   * @return the current resource
   */
  protected ComponentServerResource
  get_resource()
  {
    try {
      org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("TransactionCurrent");
      org.omg.CosTransactions.Current current = org.omg.CosTransactions.CurrentHelper.narrow( obj );
      org.omg.CosTransactions.Control control = current.get_control();
      if (control == null) return null ;
      org.omg.CosTransactions.Coordinator coordinator = control.get_coordinator();
      ComponentServerResource resource = null;
      
      for ( int i=0; i<resources_.size(); i++ ) {
        resource = ( ComponentServerResource ) resources_.elementAt( i );
        
        if ( coordinator.is_same_transaction(resource.coordinator()) )
          return resource;
      }								
      return new ComponentServerResourceImpl(this,
                                             source_, 
                                             coordinator, 
                                             TheTransactionManager.get_instance().getCurrent());								
    } catch ( Exception ex ) {
      return null;
    }
  }  
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  public void
  add_resource(ComponentServerResource resource)
  {
    resources_.addElement( resource );
  }
  
  public void
  remove_resource(ComponentServerResource resource)
  {
    resources_.removeElement( resource );
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ComponentServerListener interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerListener/get_sub_listener:1.0
  //
  /**
   * Provide the equivalent Container listener for the current listener
   *
   * @return the equivalent Container listener
   */
  public org.objectweb.ccm.Deployment.listener.ContainerListener 
  get_sub_listener()
  {
    return new ContainerListenerTransaction();
  }    
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ComponentServerEvents interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_create:1.0
  //
  /**
   * Event sent after the create function is called on ComponentServer object
   *
   * @param ref the object created
   */
  public void
  on_create (org.omg.Components.Deployment.Container ref)
  {
    delegate_.on_create (ref); 
    ComponentServerResource resource = get_resource();
    if (resource == null) 
      super.on_create(ref);
    else
      resource.on_create(ref); 
  }
}






