/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment; 

import org.objectweb.ccm.Deployment.listener.simple.ContainerListenerSimple ;

/**
 * Transaction implementation for the
 * ::plugins::deployment::ContainerListener interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public class ContainerListenerTransaction
extends ContainerListenerSimple
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * List of resources non committed
   */ 
  protected org.objectweb.ccm.util.Vector resources_;
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   * Constructor with parameter
   *
   * @param source the event source
   */ 
  public
  ContainerListenerTransaction()
  {
    resources_ = new org.objectweb.ccm.util.Vector ();
  }
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  /**
   * Provide a new instance of the class
   *
   * @return new instance
   */
  protected org.objectweb.ccm.Deployment.listener.ContainerListener
  _new()
  {
    return new ContainerListenerTransaction();
  }
  
  /**
   * Provide the reference of the resource corresponding to the current transaction
   *
   * @return the current resource
   */
  protected ContainerResource
  get_resource()
  {
    try {
      org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheORB.getORB()
        .resolve_initial_references("TransactionCurrent");            
      org.omg.CosTransactions.Current current = org.omg.CosTransactions.CurrentHelper.narrow(obj);
      org.omg.CosTransactions.Control control = current.get_control();
      if (control == null) return null ;
      org.omg.CosTransactions.Coordinator coordinator = control.get_coordinator();
      ContainerResource resource = null;
      
      for ( int i=0; i<resources_.size(); i++ ) {
        resource = ( ContainerResource ) resources_.elementAt( i );
        
        if ( coordinator.is_same_transaction( resource.coordinator() ) )
          return resource;
      }            
      return new ContainerResourceImpl(this,
                                       source_, 
                                       coordinator, 
                                       TheTransactionManager.get_instance().getCurrent()); 
    } catch ( Exception ex ) {
      return null;
    }
  }   
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  public void
  add_resource(ContainerResource resource)
  {
    resources_.addElement( resource );
  }
  
  public void
  remove_resource(ContainerResource resource)
  {
    resources_.addElement( resource );
  }
  
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ContainerEvents interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/ContainerEvents/on_create:1.0
  //
  /**
   * Event sent after the create function is called on Container object
   *
   * @param ref the object created
   */ 
  public void
  on_create (org.omg.Components.CCMHome ref)
  {
    delegate_.on_create (ref);
    ContainerResource resource = get_resource();
    if (resource == null) 
      super.on_create(ref);
    else
      resource.on_create(ref); 
  }
  
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ContainerListener interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/ContainerListener/get_sub_listener:1.0
  //
  /**
   * Provide the equivalent CCMHome listener for the current listener
   *
   * @return the equivalent CCMHome listener
   */
  public org.objectweb.ccm.Deployment.listener.CCMHomeListener 
  get_sub_listener()
  {
    return new CCMHomeListenerTransaction();
  }
}


