/*================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment;

/**
 * Local interface for access to the Resource Manager.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public interface ResourceManagerLocal
{
    /**
     * Add a new Component Server Resource in the Resource Manager
     * 
     * @param  ref reference of the ComponentServerResource to register
     * 
     */
    public void 
    add_component_server(ComponentServerResource ref);

    /**
     * Add a new Container Resource in the Resource Manager
     * 
     * @param  ref reference of the ContainerResource to register
     * 
     */
    public void 
    add_container(ContainerResource ref);

    /**
     * Add a new Home Resource in the Resource Manager
     * 
     * @param  ref reference of the CCMHomeResource to register
     * 
     */
    public void 
    add_home(CCMHomeResource ref);
    
    /**
     * Remove a Component Server Resource from the Resource Manager
     * 
     * @param  ref reference of the ComponentServerResource to remove
     * 
     */
    public void 
    remove_component_server(ComponentServerResource ref)
				throws org.omg.Components.RemoveFailure;

    /**
     * Remove a Container Resource from the Resource Manager
     * 
     * @param  ref reference of the ContainerResource to remove
     * 
     */
    public void 
    remove_container(ContainerResource ref)
				throws org.omg.Components.RemoveFailure;
    
    /**
     * Remove a Home Resource from the Resource Manager
     * 
     * @param  ref reference of the HomeResource to remove
     * 
     */
    public void 
		remove_home(CCMHomeResource ref)
				throws org.omg.Components.RemoveFailure;
    
    /**
     * Provide the reference of the transaction coordinator
     *
     * @return reference on the coordinator
     */ 
    public org.omg.CosTransactions.Coordinator 
		coordinator();
}
