/*================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment ;


/**
 * This class implements
 * ::plugins::deployment::transaction::TransactionManagerLocal
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class TransactionManagerImpl
implements TransactionManagerLocal
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * List of the transactions started
   *
   */
  protected org.objectweb.ccm.util.Vector transactions_;
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   * Default Constructor
   *
   */
  public
  TransactionManagerImpl()
  {
    transactions_ = new org.objectweb.ccm.util.Vector (); 
  }
  
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  /**
   * Create a new resource manager
   */
  protected ResourceManagerLocal
  create_manager (org.omg.CosTransactions.Coordinator coordinator)
  {        
    try {
      ResourceManagerImpl _resource = new ResourceManagerImpl (coordinator, this);
      transactions_.add (_resource);
      ResourceManager _res_ref =  _resource._this (org.objectweb.ccm.CORBA.TheORB.getORB()); 
      coordinator.register_resource( _res_ref );
      
      return _resource; 
    } catch (org.omg.CosTransactions.Inactive ex) { }
    return null; 
  }
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Methods for the TransactionManagerLocal interface.
  //
  // ==================================================================
  //
  //  JAVA:TransactionManagerLocal/getCurrent
  //
  /**
   * Provide the current resource manager
   *
   * @return instance of the current resource manager
   */
  public ResourceManagerLocal
  getCurrent()
  {
    try {
      org.omg.CORBA.Object _obj = org.objectweb.ccm.CORBA.TheORB.getORB()
                                     .resolve_initial_references("TransactionCurrent");
      org.omg.CosTransactions.Current _current =
        org.omg.CosTransactions.CurrentHelper.narrow( _obj );
      org.omg.CosTransactions.Control _control =
        _current.get_control();
      org.omg.CosTransactions.Coordinator _coordinator =
        _control.get_coordinator();
      
      java.util.Iterator _iterator = transactions_.iterator();
      
      while (_iterator.hasNext()) {
        ResourceManagerLocal _resource = (ResourceManagerLocal) _iterator.next ();
        if ( _coordinator.is_same_transaction( _resource.coordinator() ) )
          return _resource;
      }
      return create_manager (_coordinator);  
    } catch (org.omg.CORBA.ORBPackage.InvalidName ex) { 
    } catch (org.omg.CosTransactions.Unavailable ex) { 
    }
    return null; 
  }
  
  //
  //  JAVA:TransactionManagerLocal/remove_manager
  //
  /**
   * Remove the resource manager from the transaction manager
   *
   * @param resource manager to remove
   *
   * @exception Components::RemoveFailure
   */
  public void
  remove_manager(ResourceManagerLocal ref)
    throws org.omg.Components.RemoveFailure
  {
    transactions_.remove (ref); 
  } 
}






