/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.scripts;

/**
 * A options manager implementation for OMG IDL compiler..
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IDL3OptionsManager
       extends org.objectweb.ccm.util.compiler.OptionsManager_impl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    protected String objectName_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    protected String commandName_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param indentation The indentation.
     **  @param commandName The command name.
     **/
    public
    IDL3OptionsManager(String indentation,
                       String commandName)
    {
        super(indentation);

        addOption(new org.objectweb.ccm.util.compiler.Option("-h", 0,
                  "-h                Display this help."));
        addOption(new org.objectweb.ccm.util.compiler.Option("-o", 1,
                  "-o <file>         Display the mapping on file <file>."));
        addOption(new org.objectweb.ccm.util.compiler.Option("-i", 1,
                  "-i <ir_object>    Add <ir_object> in an import statement."));
        addOption(new org.objectweb.ccm.util.compiler.Option("-f", 1,
                  "-f <prop.file>   Define the file to use when generating the IDL import/include instructions."));

        objectName_ = "";
        commandName_ = commandName;
    }

    /**
     **  The constructor.
     **
     **  @param indentation The indentation.
     **/
    public
    IDL3OptionsManager(String indentation)
    {
        super(indentation);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Display the usage of the OMG IDL compiler.
     **/
    public void
    usage()
    {
        System.err.println("usage:");
        System.err.println(commandName_+" [options] <ir_object>");
        System.err.println("");
        System.err.println("options:");
        optionsUsage("");
    }

    /**
     **  Analyse a command line.
     **
     **  @param commandLine The command line arguments.
     **
     **  @return -1 if an error occurs.
     **/
    public int
    analyse(String[] commandLine)
    {
        int length = commandLine.length;
        if (length < 1)
        {
            System.err.println("Missing <ir_object>");
            usage();
            return -1;
        }
        if (analyseOptions(commandLine, 1) == -1)
	{
            return -1;
        }
	if (isSet("-h") || commandLine[length-1].equals("-h"))
        {
            usage();
            return -1;
        }
        if (commandLine[length-1].startsWith("-"))
        {
            System.err.println("Invalid <ir_object>: " +
                               commandLine[length-1]);
            usage();
            return -1;
        }
        objectName_ = commandLine[length-1];
        return 0;
    }

    /**
     **  Obtain the source filename.
     **
     **  @return The source filename.
     **/
    public String
    getObjectName()
    {
        return objectName_;
    }
}
