/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.scripts;

/**
 * This class manages an OMG IDL3 compiler.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IR3FeedCompiler
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private IR3FeedOptionsManager manager_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private org.objectweb.ccm.IDL3.Repository repository_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private org.objectweb.ccm.IDL3.FileScope fileScope_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private org.objectweb.ccm.util.compiler.CompilerManager preprocessor_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private String version_;

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    private boolean printMsg_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param repository
     **  @param globalRep
     **/
    public
    IR3FeedCompiler(org.objectweb.ccm.IDL3.Repository repository,
                    org.omg.CORBA.Repository globalRep)
    {
        manager_ = new IR3FeedOptionsManager(Version.label);
        repository_ = repository;
        version_ = Version.label;
        printMsg_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    protected org.objectweb.ccm.IDL3.Parser
    create_parser(String inputfile)
    {
        org.objectweb.ccm.IDL3.Parser parser = null;
        message("Reading from file " + inputfile + "...");
        try
        {
            message("Preprocessing file " + inputfile + "...");
            preprocessor_ = new org.objectweb.ccm.util.compiler.
               CompilerManager_impl(inputfile, "cpp", manager_.getOthers());
            if (preprocessor_.start() != 0)
            {
                preprocessor_.displayStderr("");
                message("Preprocessing failed");
                System.exit(1);
            }
            message("File " + inputfile + " preprocessed");
            parser = new org.objectweb.ccm.IDL3.Parser(
                       new java.io.FileInputStream(preprocessor_.getOutputFile()),
                       repository_,
                       new org.objectweb.ccm.IDL3.ErrorManager(inputfile));
        }
        catch (java.io.FileNotFoundException e)
        {
            message("File " + inputfile + " not found.");
            System.exit(1);
        }
        return parser;
    }

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    protected void
    message(String text)
    {
        if (printMsg_)
            System.out.println(version_ + ": " + text);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Run the parser on the input file.
     **
     **  MUST BE COMMENTED with JavaDoc!
     **/
    public boolean
    run(String[] commandLine)
    {
        return run(commandLine, true);
    }

    /**
     **  Run the parser on the input file.
     **
     **  MUST BE COMMENTED with JavaDoc!
     **/
    public boolean
    run(String[] commandLine, boolean printMsg)
    {
        printMsg_ = printMsg;
        if (manager_.analyse(commandLine) == -1)
            return false;

        org.objectweb.ccm.IDL3.Parser parser =
            create_parser(manager_.getFileName());

        message("Feeding the Interface Repository ...");

        try
	{
            fileScope_ = parser.specification();
        }
        catch(Exception exc)
	{
            exc.printStackTrace();
            return false;
	}

        int nberrors = parser.getErrorManager().getNbErrors();
        int nbwarnings =  parser.getErrorManager().getNbWarnings();
        printMsg_ = true;
        if (nberrors > 0)
        {
            message("Compilation failed: " + nberrors + " errors, " +
                     nbwarnings + " warnings.");
            return false;
        }
        message("Compilation completed: " + nbwarnings + " warnings.");
        return true;
    }

    /**
     **  Obtain the option manager.
     **
     **  @return The option manager.
     **/
    IR3FeedOptionsManager
    getManager()
    {
        return manager_;
    }

    /**
     **  Obtain the FileScope of the parsed file.
     **
     **  @return The FileScope of the parsed file.
     **/
    public org.objectweb.ccm.IDL3.FileScope
    getFileScope()
    {
        return fileScope_;
    }

    /**
     **  Clear the temporory files.
     **/
    public void
    clear()
    {
        if (preprocessor_ != null)
            preprocessor_.clear();
    }
}
