/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.scripts;

/**
 * A options manager implementation for OMG IDL compiler.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IR3FeedOptionsManager
       extends org.objectweb.ccm.util.compiler.OptionsManager_impl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  MUST BE COMMENTED with JavaDoc!
     **/
    protected String fileName_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param indentation The indentation.
     **/
    public
    IR3FeedOptionsManager(String indentation)
    {
        super(indentation);
        addOption(new org.objectweb.ccm.util.compiler.Option("-h",0,
                  "-h                Display this help.")); 
        fileName_ = "";
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Display the usage of the OMG IDL compiler.
     **/
    public void
    usage()
    {
        System.err.println("usage:");
        System.err.println("ir3_feed [options] <idl_file>");
        System.err.println("");
        System.err.println("option:");
        optionsUsage("");
        System.err.println("");
        System.err.println("options for cpp preprocessor :");
        System.err.println("-DNAME            Define NAME as 1.");
        System.err.println("-DNAME=DEF        Define NAME as DEF.");
        System.err.println("-UNAME            Remove any definition for NAME.");
        System.err.println("-IDIR             Put DIR in the include file search path.");
    }

    /**
     **  Analyse a command line.
     **
     **  @return -1 if an error occurs.
     **/
    public int
    analyse(String[] commandLine)
    {
        int length = commandLine.length;
        if (length < 1)
        {
            System.err.println("Missing <idl_file>");
            usage();
            return -1;
        }
        if (analyseOptions(commandLine, 1) == -1)
	{
            return -1;
        }
	if (isSet("-h") || commandLine[length-1].equals("-h"))
        {
            usage();
            return -1;
        }
        if (commandLine[length-1].startsWith("-"))
        {
            System.err.println("Invalid <idl_file> : " +
                                commandLine[length-1]);
            usage();
            return -1;
        }
        fileName_ = commandLine[length-1];
        return 0;
    }

    /**
     **  Obtain the source filename.
     **
     **  @return The source filename.
     **/
    public String
    getFileName()
    {
        return fileName_;
    }
}
