/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.scripts;

/**
 * This class is launched by the ir3_idl3 script.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IR3toIDL3
       extends org.objectweb.ccm.CORBA.Application
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    IR3toIDL3()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** The main bootstrap method.
     **/
    public static void
    main(String[] args)
    {
        IR3toIDL3 ir3 = new IR3toIDL3();
        ir3.start(args);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    run(String[] args)
    {
        // Obtain the Interface Repository.
        org.objectweb.ccm.ComponentRepository repositoryRef =
            org.objectweb.ccm.ComponentRepositoryHelper.narrow(
                org.objectweb.ccm.CORBA.TheInterfaceRepository.getRepository());

	// Create an OMG IDL Compiler.
        IDL3Compiler compiler = new IDL3Compiler();

	// Run it.
        boolean ok = compiler.run(args);
        if (!ok)
	{
            return -1;
        }

        String filename = "stdout";

        IDL3OptionsManager manager = compiler.getManager();
        if (manager.isSet("-o"))
            filename = manager.get("-o")[0];

        String[] imports = new String[0];
        if (manager.isSet("-i"))
            imports = manager.get("-i");

        java.lang.String[] propfiles = new java.lang.String[0];
        if (manager.isSet("-f"))
            propfiles = manager.get("-f");

        // Start an IDL3 Repository
        org.objectweb.ccm.IDL3.Repository rep = 
            new org.objectweb.ccm.IDL3.Repository(repositoryRef.as_IDL3_repository());

        try
        {
            org.objectweb.ccm.visitorIDL3.idl.IDL3_IDL3 mi = 
                new org.objectweb.ccm.visitorIDL3.idl.IDL3_IDL3(rep);

            mi.generate(manager.getObjectName(),
                        filename,
                        imports,
                        propfiles);
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            System.err.println(ex.exception.toString());
            return 1;
        }

	return 0;
    }
}
