/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Raphael Marvie.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util;

/**
 * This class provides an extended URLClassLoader.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Raphael.Marvie@lifl.fr">Raphael Marvie</a>
 *
 * @version 0.3
 */

public class URLClassLoader
       extends java.net.URLClassLoader
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    URLClassLoader()
    {
	super(new java.net.URL[0]);
    }

    /**
     **  The constructor.
     **
     **  @param urls The list of initial URLs.
     **/
    public
    URLClassLoader(java.net.URL urls[])
    {
	super(urls);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Add an URL into the list of URLs of this URLClassLoader.
     **
     **  @param url The URL to add.
     **/
    public void
    addURL(java.net.URL url)
    {
	super.addURL(url);
    }

    /**
     **  Call a static class method.
     **
     **  @param entrypt The entry point i.e. 'class_name.method_name'.
     **  @param args The method arguments.
     **
     **  @return The result of the called static class method.
     **/
    public Object
    callStaticClassMethod(String entrypt,
			  Object[] args)
    throws java.lang.ClassNotFoundException,
           java.lang.NoSuchMethodException,
           java.lang.reflect.InvocationTargetException,
           java.lang.IllegalAccessException
    {
        // extract the class name and the method name.
        int idx = entrypt.lastIndexOf('.');
        java.lang.String class_name = entrypt.substring(0, idx);
        java.lang.String method_name = entrypt.substring(idx+1);

	// Search the class.
	java.lang.Class theClass = super.loadClass(class_name);

	// Search the static method.
	java.lang.reflect.Method theMethod = theClass.getMethod(method_name, new java.lang.Class[0]);

	// Call the static method.
        Object result = theMethod.invoke(null, args);

	// Return the result.
	return result;
    }
}
