/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.compiler;

/**
 * A compiler manager interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface CompilerManager
{

   /**
    **  Display the stderr of the compiler
    **  with a specific indentation.
    **/ 
   public void
   displayStderr(String indentation);

   /**
    ** Display the stdout of the compiler
    ** with a specific indentation
    **/
   public void
   displayStdout(String indentation);

   /**
    ** Start the compilation.
    **
    ** Returns 0 if success
    **/
   public int
   start();
    
   /**
    **  Set the input file
    **/
   public void 
   setInputFile(String input);

   /**
    ** Set the compiler to be used
    **/
   public void
   setCompiler(String compiler);

   /**
    ** Get the ouput file 
    **/
   public String
   getOutputFile();

   /**
    ** Clear the temporary objects
    **/
   public void
   clear();

   /**
    ** Add options to the compiler
    **/
   public void
   addOptions(String options);

}
