/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.compiler;

/**
 * A compiler manager implementation.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class CompilerManager_impl
             implements CompilerManager
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **   Constructor.
     **/
    public
    CompilerManager_impl(String inputFile,
                         String compiler,
                         String options)
    {
        inputFile_ = inputFile;
        process_ = null;
        compiler_ = compiler;
        indentation_ = "";
        options_ = new StringBuffer(options);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    private java.io.File tmpFile_;

    private java.lang.Process process_;

    private String inputFile_;     

    private StringBuffer options_;

    private String compiler_;

    private String indentation_;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Display a java.io.BufferedReader
     **/
    protected void
    display(java.io.BufferedReader buffer)
    {
         try
         {
             String line = buffer.readLine();
             while (line != null)
             {
                 System.out.println(indentation_ + line);
                 line = buffer.readLine();
             }
         }
         catch (java.io.IOException e)
         {
             System.err.println(indentation_ + e.getMessage());
         }
    }

    /**
     **  Display a java.io.InputStream
     **/
    protected void
    display(java.io.InputStream input)
    {
        java.io.BufferedReader buffer = 
                   new java.io.BufferedReader(
                           new java.io.InputStreamReader(input)
                   );
        display(buffer); 
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Display the stderr of the compiler.
     **/ 
    public void
    displayStderr(String indentation)
    {
        indentation_ = indentation;
        display(process_.getErrorStream());
    }

    /** 
     ** Display the stdout of the compiler.
     **/
    public void
    displayStdout(String indentation)
    {
        indentation_ = indentation;
        display(process_.getInputStream());
    }

    /**
     ** Starts the compilation.
     **
     ** Returns 0 if success
     **/
    public int
    start()
    {
        try
        {
            tmpFile_ = java.io.File.createTempFile("tmp",".tmp");
            process_ = Runtime.getRuntime().exec(
                         compiler_ + " " + options_.toString() +
                         " " + inputFile_ + " " + tmpFile_.toString());
            return process_.waitFor();
        }
        catch (Exception e) 
        { 
            System.err.println(e.getMessage());  
            return -1;
        }

    }
    
    /**
     **  Set the input file
     **/
    public void 
    setInputFile(String input)
    {
        inputFile_ = input;
    }

    /**
     ** Set the compiler to be used
     **/
    public void
    setCompiler(String compiler)
    {
        compiler_ = compiler;
    }

    /**
     ** Get the ouput file 
     **/
    public String
    getOutputFile()
    {
        try {
            java.io.BufferedReader br =
                new java.io.BufferedReader(
                    new java.io.FileReader(tmpFile_));

            StringBuffer sb = new StringBuffer();
            String s;
            while ((s = br.readLine()) != null) {
                sb.append(s).append("\n");
            }

            String regexPattern =
                "#\\s*1\\s*\".*\"\\s*" +
                "#\\s*1\\s*\"\\<.*\\>\"\\s*" +
                "#\\s*1\\s*\"\\<.*\\>\"\\s*" +
                "#\\s*1\\s*\".*\"\\s*";

            java.util.regex.Pattern p =
                java.util.regex.Pattern.compile(regexPattern);
            java.util.regex.Matcher m = p.matcher(sb.toString());

            java.io.FileWriter fw = new java.io.FileWriter(tmpFile_);
            fw.write(m.replaceFirst(""));
            fw.close();
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

        return tmpFile_.toString();

    }

    /**
     ** Clear the temporary objects
     **/
    public void
    clear()
    {
        try
        {
            tmpFile_.delete();
        }
        catch (Exception e) 
	{
            System.err.println(e.getMessage());
        }
    }

    /**
     ** Add options to the compiler
     **/
    public void
    addOptions(String options)
    {
        options_.append(" " + options);
    }

}
