/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.compiler;

/**
 * A options manager interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface OptionsManager
{
    /**
     ** Add the option opt in the possible options list.
     **/
    public void
    addOption(Option opt);

    /**
     ** Check if the option opt is set to true.
     **/
    public boolean
    isSet(String opt);

    /**
     ** Return the values associated with the option opt.
     **/
    public String[]
    get(String opt);

    /**
     ** Return all the options that are not recognized and their possible values.
     ** We intend it to be used for returning the options that will be passed to another compiler.
     **/
    public String
    getOthers();

    /**
     ** Analyse a command line.
     ** Return -1 if an error occurs.
     **/
    public int
    analyse(String[] commandline);

    /**
     ** Display the usage.
     **/
    public void
    usage();
}
