/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.compiler;

/**
 * A options manager implementation.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class OptionsManager_impl
                implements OptionsManager
{

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **   Constructor.
     **/
    public 
    OptionsManager_impl(String indentation)
    {
        indentation_ = indentation;
        options_ = new java.util.Hashtable();
        others_ = new StringBuffer("");
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected String indentation_;

    protected StringBuffer others_;

    protected java.util.Hashtable options_;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     ** Display the options usage for options beginning with prefix.
     **/
    protected void
    optionsUsage(String prefix)
    {
        java.util.Enumeration enume = options_.elements();
        Option opt = null;
        while (enume.hasMoreElements())
        {
            opt = (Option)enume.nextElement();
            if (opt.cmd_.startsWith(prefix))
                System.err.println(opt.usage_);
        }
    }

    /**
     ** Analyse options on a command line.
     ** Return -1 if an error occurs.
     **/
    protected int
    analyseOptions(String[] commandline,int n)
    {
        Option opt = null;
        for (int i=0;i<commandline.length-n;i++)
	{
            if (options_.containsKey(commandline[i]))
            {
                try
                {
                    opt = (Option)options_.get(commandline[i]);
                    for(int j=0;j<opt.paramNumber_;j++)
                        opt.value_.add(commandline[i+j+1]);
                    opt.isSet_ = true;
                    i += opt.paramNumber_;
                }
                catch (java.lang.ArrayIndexOutOfBoundsException e)
                {
                    System.err.println("Missing parameter for option " + opt.cmd_);
                    return -1;
                }
            }
            else
            {
                others_.append(" "+commandline[i]+" ");
            }
        }
        return 0;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** Add the option opt in the possible options list.
     **/
    public void
    addOption(Option opt)
    {
        options_.put(opt.cmd_,opt);
    }

    /**
     ** Check if the option opt is set to true.
     **/
    public boolean
    isSet(String opt)
    {
        if (!options_.containsKey(opt))
            return false;
        return ((Option)options_.get(opt)).isSet_;
    }

    /**
     ** Return the values associated with the option opt.
     **/
    public String[]
    get(String opt)
    {
        if (!options_.containsKey(opt))
            return null;
        return (String[])((Option)options_.get(opt)).value_.toArray(new String[0]);       
    }

    /**
     ** Return all the options that are not recognized and their possible values.
     ** We intend it to be used for returning the options that will be passed to another compiler.
     **/
    public String
    getOthers()
    {
        return others_.toString();
    }

    /**
     **  Display the usage of the OMG IDL compiler.
     **/
    public abstract void
    usage();

    /**
     ** Analyse a command line.
     ** Return -1 if an error occurs.
     **/
    public abstract int
    analyse(String[] commandline);
 
}
