/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.common;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class Printer
       extends fr.lifl.cim.jidlscript.JOSInterpreter
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    private org.objectweb.ccm.util.io.PrintStreamManager the_stream_manager_;

    /**
     **
     **/
    private org.objectweb.ccm.util.io.PrintStream stream_;

    /**
     **
     **/
    private org.objectweb.ccm.util.Table string_mappings_;

    /**
     **
     **/
    private org.objectweb.ccm.util.Vector file_mappings_;

    /**
     **
     **/
    private java.lang.String remaining_text_;

    /**
     **
     **/
    private int char_to_skip_;

    /**
     **
     **/
    private java.lang.String indentation_;

    /**
     **
     **/
    private char call_char_;

    /**
     **
     **/
    private char comment_char_;

    /**
     **
     **/
    private Generator generator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    Printer(Generator gen,
            char call_char,
            char comment_char,
            java.lang.String indentation)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(new java.lang.String[0]);

        // init internal state.
        the_stream_manager_ = new org.objectweb.ccm.util.io.PrintStreamManager();
        string_mappings_ = new org.objectweb.ccm.util.Table();
        file_mappings_ = new org.objectweb.ccm.util.Vector();
        remaining_text_ = "";
        char_to_skip_ = 0;
        stream_ = the_stream_manager_.create("stdout", indentation);
        setGenerator(gen);
        indentation_ = indentation;
        call_char_ = call_char;
        comment_char_ = comment_char;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    private void
    print(java.lang.String text)
    {
        if (stream_==null)
            return ;

        int idx = text.indexOf("\\n");
        if (idx==-1)
            remaining_text_ = remaining_text_ + text;
        else
        {
            stream_.print(remaining_text_ + text.substring(0, idx) + '\n');
            remaining_text_ = "";
            if (idx+2<text.length())
                print(text.substring(idx+2, text.length()));
        }
    }

    /**
     **
     **/
    private void
    analyze(java.lang.String id,
            java.lang.String mapping)
    {
        org.objectweb.ccm.util.Vector mappings = new org.objectweb.ccm.util.Vector();
        org.objectweb.ccm.util.Vector to_eval = new org.objectweb.ccm.util.Vector();
        MapInfo info = null;
        java.lang.String tmp = "";

        if (mapping.length()==0)
        {
            info = new MapInfo(mappings, to_eval);
            string_mappings_.put(id.toUpperCase(), info);
            return ;
        }

        int i = 0;
        int j = mapping.indexOf(call_char_, i);        
        while (j!=-1)
        {
            if (i!=j)
            {
                tmp = mapping.substring(i, j);
                mappings.add(tmp);
                to_eval.add(new java.lang.Boolean(false));
            }

            i = j+1;
            if (i>=mapping.length())
            {
                info = new MapInfo(mappings, to_eval);
                string_mappings_.put(id.toUpperCase(), info);
                return ;
            }
            j = mapping.indexOf(call_char_, i);
            if (j==-1)
            {
                java.lang.String msg =
                    "error in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                    "   operation : printFromString\n"+
                    "   message : Missing " + call_char_ + " character after : "+
                    mapping.substring(i)+" in : "+id;
                //                throw new Error(msg);
                System.err.println(msg);
            }

            if (i!=j)
            {
                tmp = mapping.substring(i, j);
                mappings.add(tmp);
                to_eval.add(new java.lang.Boolean(true));
            }

            i = j+1;
            if (i>=mapping.length())
            {
                info = new MapInfo(mappings, to_eval);
                string_mappings_.put(id.toUpperCase(), info);
                return ;
            }
            j = mapping.indexOf(call_char_, i);
        }

        tmp = mapping.substring(i, mapping.length());
        mappings.add(tmp);
        to_eval.add(new java.lang.Boolean(false));

        info = new MapInfo(mappings, to_eval);
        string_mappings_.put(id.toUpperCase(), info);
    }

    /**
     **
     **/
    private void
    evaluate(MapInfo info,
             java.lang.String[] aliases,
             java.lang.Object[] objs)
    {
        java.lang.String[] mappings =
            (java.lang.String[])info.mappings.toArray(new java.lang.String[0]);
        java.lang.Boolean[] to_eval =
            (java.lang.Boolean[])info.to_eval.toArray(new java.lang.Boolean[0]);

        for (int i=0;i<mappings.length;i++)
        {
            if (to_eval[i].booleanValue())
                print(joshEval(aliases, objs, mappings[i]));
            else if (char_to_skip_!=0)
            {
                print(mappings[i].substring(char_to_skip_));
                char_to_skip_ = 0;
            }
            else
                print(mappings[i]);
        }
    }

    /**
     **
     **/
    private java.lang.String
    joshEval(java.lang.String[] aliases,
              java.lang.Object[] objs,
              java.lang.String mapping)
    {
        fr.lifl.cim.jidlscript.lang.JISObject result = null;
        try
        {
            for (int i=0;i<aliases.length;i++)
                setAlias(aliases[i], objs[i]);

            result = super.eval(mapping);
            return ((fr.lifl.cim.jidlscript.java.JISJavaString)result).getValue();
        }
        catch(java.lang.ClassCastException ex)
        {
            java.lang.String msg =
                "error in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                "   operation : joshEval\n"+
                "   message : result of eval is not a string : "+ex.getMessage()+"\n"+
                "             while visiting the following template :\n"+
                "             "+mapping+"\n";
            //                throw new Error("");
            System.err.println(msg);
            return "";
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            java.lang.String msg =
                "error in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                "   operation : joshEval\n"+
                "   message : an error occured while evaluating the following template :\n"+
                "             "+mapping+"\n"+
                "   reason  : "+ex.exception.toString();
            //                throw new Error("");
            System.err.println(msg);
            return "";
        }
    }

    // ==================================================================
    //
    // Final public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public final char
    getCallChar()
    {
        return call_char_;
    }

    /**
     **
     **/
    public final char
    getCommentChar()
    {
        return comment_char_;
    }

    /**
     **
     **/
    public final java.lang.String
    getIndentation()
    {
        return indentation_;
    }

    /**
     **
     **/
    public final Generator
    getGenerator()
    {
        return generator_;
    }

    /**
     **
     **/
    public final void
    delete(int n)
    {
        if (remaining_text_.length()>n)
            remaining_text_ = remaining_text_.substring(0, remaining_text_.length()-n);
        else
            remaining_text_ = "";
    }

    /**
     **
     **/
    public final void
    skip(int n)
    {
        char_to_skip_ = n;
    }

    /**
     **
     **/
    public final void
    insert(int n)
    {
        int length = remaining_text_.length();
        java.lang.String tmp1 = remaining_text_.substring(0, length-n);
        java.lang.String tmp2 = remaining_text_.substring(length-n, length);
        remaining_text_ = tmp1 + "\n" + stream_.getTab() + tmp2;
    }

    /**
     **
     **/
    public final void
    loadFile(java.lang.String filename)
    {
        boolean already_parsed = file_mappings_.contains(filename);
        if (already_parsed)
            return ;

        java.lang.String mapping = "";
        java.lang.String tmp = "";
        java.lang.String curr_id = "";
        try
        {
            java.io.BufferedReader file = new java.io.BufferedReader(
                new java.io.FileReader(generator_.getTemplateLocation()+"/"+filename));
            tmp = file.readLine();
            while (tmp!=null)
            {
                if ((tmp.length()>0) && (tmp.charAt(0)==comment_char_))
                {
                    if ((tmp.length()>2) && (tmp.substring(1, 3).toLowerCase().equals("id")))
                    {
                        if (!mapping.equals(""))
                            analyze(curr_id, mapping);

                        mapping = "";
                        curr_id = tmp.substring(4);
                        curr_id = curr_id.trim();
                    }
                }
                else
                    mapping = mapping+modifyCurrentLine(tmp);

                tmp = file.readLine();
            }

            analyze(curr_id, mapping);
        }
        catch(java.io.FileNotFoundException ex)
        {
            java.lang.String msg =
                "exception in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                "   operation : mapFromFile\n"+
                "   message : "+ex.getMessage();
            //            throw new Error(msg);
            System.err.println(msg);
        }
        catch(java.io.IOException ex)
        {
            java.lang.String msg =
                "exception in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                "   operation : mapFromFile\n"+
                "   message : "+ex.getMessage();
            //            throw new Error(msg);
            System.err.println(msg);
        }

        file_mappings_.add(filename);
    }

    // ==================================================================
    //
    // Internal methods that can be overrided by subclasses.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setDirectory(java.lang.String dirname)
    {
        the_stream_manager_.setDirectory(dirname);
    }

    /**
     **
     **/
    public java.lang.String
    open(java.lang.String filename)
    {
        stream_ = the_stream_manager_.create(filename, indentation_);
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    close()
    {
        the_stream_manager_.close(stream_.getName());
        stream_ = null;
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    inc()
    {
        stream_.incDepth();
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    dec()
    {
        stream_.decDepth();
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    modifyCurrentLine(java.lang.String str)
    {
        return str;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setAlias(java.lang.String alias,
             java.lang.Object obj)
    {
        try
        {
            fr.lifl.cim.jidlscript.java.JISJavaClass jc = null;
            fr.lifl.cim.jidlscript.java.JISJavaInstance ji = null;
            java.lang.String itfname = obj.getClass().getName();

            if (itfname.equals("java.lang.String"))
                ji = new fr.lifl.cim.jidlscript.java.JISJavaString((java.lang.String)obj);
            else
                {
                    jc = fr.lifl.cim.jidlscript.java.JISJavaClass.importClass(
                        new fr.lifl.cim.jidlscript.lang.JISString(itfname));
                    ji = new fr.lifl.cim.jidlscript.java.JISJavaInstance(jc, obj);
                }

            fr.lifl.cim.jidlscript.lang.JISRootScope.root().insertScoped(
                new fr.lifl.cim.jidlscript.lang.JISAlias(alias, ji));
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            java.lang.String msg =
                "error in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                "   operation : setAlias\n"+
                "   message : "+ex.exception.toString();
            //                throw new Error(msg);
            System.err.println(msg);
        }
    }

    /**
     **
     **/
    public void
    setGenerator(Generator gen)
    {
        generator_ = gen;

        // set the "generator" alias in root scope on the Generator class instance.
        setAlias("generator", gen);
    }

    /**
     **
     **/
    public void
    map(java.lang.String template_id,
        java.lang.String[] aliases,
        java.lang.Object[] objs)
    {
        java.lang.Object obj = string_mappings_.get(template_id);
        if (obj==null)
        {
                java.lang.String msg =
                    "error in class org.objectweb.ccm.visitorIDL3.Printer\n"+
                    "   operation : map\n"+
                    "   message : Unknown mapping identifier : "+template_id+"\n";
                //                throw new Error(msg);
                System.err.println(msg);
        }

        evaluate((MapInfo)obj, aliases, objs);
    }

    /**
     **
     **/
    public void
    printImmediately(java.lang.String text)
    {
        print(text);
    }
}


/**
 **  Nested class MapInfo
 **/
class MapInfo
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    public org.objectweb.ccm.util.Vector mappings;

    /**
     **
     **/
    public org.objectweb.ccm.util.Vector to_eval;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    protected
    MapInfo(org.objectweb.ccm.util.Vector maps,
            org.objectweb.ccm.util.Vector bools)
    {
        mappings = maps;
        to_eval = bools;
    }
}
