/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.idl;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */


public class IDL3Printer
       extends Printer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected IDLTranslator translator_;

    /**
     **
     **/
    protected java.lang.String last_prefix_;

    protected org.objectweb.ccm.util.Table dependencies_tree_;

    protected org.objectweb.ccm.IDL3.Declaration target_decl_;

    protected boolean corba_used_;
    protected boolean typecode_used_;
    protected boolean components_used_;


    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Special constructor.
     **/
    public
    IDL3Printer(Generator gen,
                char call_char,
                char comment_char,
                java.lang.String indent)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, call_char, comment_char, indent);

        // init internal state.
        translator_ = new IDLTranslator();
        last_prefix_ = "";
        dependencies_tree_ = new org.objectweb.ccm.util.Table();
        target_decl_ = null;
        corba_used_ = false;
        typecode_used_ = false;
        components_used_ = false;
    }


    /**
     **  The default constructor.
     **/
    public
    IDL3Printer(Generator gen,
                char call_char,
                char comment_char,
                java.lang.String indent,
                java.lang.String filename)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, call_char, comment_char, indent);

        // init internal state.
        translator_ = new IDLTranslator();
        last_prefix_ = "";
        dependencies_tree_ = new org.objectweb.ccm.util.Table();
        target_decl_ = null;
        corba_used_ = false;
        typecode_used_ = false;
        components_used_ = false;

        // load the template file.
        super.loadFile(filename);

        // set the "self" alias in root scope on this object.
        super.setAlias("self", this);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected boolean
    isBusinessDeclaration(org.objectweb.ccm.IDL3.Declaration decl)
    {
        java.lang.String abs_name = decl.getAbsoluteName();
        if (abs_name.equals("::CORBA::TypeCode"))
        {
            typecode_used_ = true;
            return false;
        }
        else if (abs_name.startsWith("::CORBA"))
        {
            corba_used_ = true;
            return false;
        }
        else if (abs_name.startsWith("::Components"))
        {
            components_used_ = true;
            return false;
        }

        return true;
    }

    /**
     **
     **/
    protected boolean
    isBusinessType(org.objectweb.ccm.IDL3.TypeRef type)
    {
        if (!type.isDeclaration())
            return true;

        return isBusinessDeclaration((org.objectweb.ccm.IDL3.Declaration)type);
    }

    /**
     ** Find longest common prefix of two strings.
     **/
    protected String
    longestCommonPrefix(String s, String t) {
        return s.substring(0, longestCommonPrefixLength(s, t));
    }

    /**
     ** Find the length of the longest common prefix of two strings.
     **/
    protected int
    longestCommonPrefixLength(String s, String t) {
        int m = Math.min(s.length (), t.length());
        for (int k = 0; k < m; ++k)
            if (s.charAt(k) != t.charAt(k))
                return k;
        return m;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    visitContained(org.objectweb.ccm.IDL3.Declaration contained,
                   java.lang.String extension)
    {
        java.lang.String id =
            org.objectweb.ccm.IDL3.DeclarationKind.toString(contained.getDeclKind()).toUpperCase()+
            extension;

        java.lang.String[] aliases = { "obj" };
        java.lang.Object[] objs = { contained };
        super.map(id, aliases, objs);

        return "";
    }

    /**
     **
     **/
    public IDLTranslator
    getTranslator()
    {
        return translator_;
    }

    /**
     **
     **/
    public java.lang.String
    which()
    {
        return "ir3_idl3";
    }

    /**
     **
     **/
    public int
    forward_check()
    {
        return (org.objectweb.ccm.IDL3.DeclarationKind._dk_interface +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_local_interface +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_abstract_interface +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_component +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_value + 
                org.objectweb.ccm.IDL3.DeclarationKind._dk_struct +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_union +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_event);
    }

    /**
     **
     **/
    public int
    interface_check()
    {
        return (org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_constant +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_exception +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_operation +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_alias +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_struct +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_union +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_enum +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_native);
    }

    /**
     **
     **/
    public int
    component_check()
    {
        return (org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_uses +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_emits +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute);
    }

    /**
     **
     **/
    public int
    home_check()
    {
        return (org.objectweb.ccm.IDL3.DeclarationKind._dk_factory +
                org.objectweb.ccm.IDL3.DeclarationKind._dk_finder +
                interface_check());
    }

    /**
     **
     **/
    public int
    value_check()
    {
        return (org.objectweb.ccm.IDL3.DeclarationKind._dk_value_member +
                interface_check());
    }

    /**
     **
     **/
    public java.lang.String
    visit(org.objectweb.ccm.util.Vector vect)
    {
        org.objectweb.ccm.IDL3.Declaration[] decls = (org.objectweb.ccm.IDL3.Declaration[])
            vect.toArray(new org.objectweb.ccm.IDL3.Declaration[0]);

        for (int i=0;i<decls.length;i++)
        {
            if (decls[i]==target_decl_)
                visitContained(decls[i], "");
            else if (decls[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_module)
            {
                java.lang.String[] aliases = { "module", "vect" };
                java.lang.Object[] objs = { decls[i], dependencies_tree_.get(decls[i]) };
                super.map("MODULE_WITH_DEPENDENCIES", aliases, objs);
            }
            else
                visitContained(decls[i], "");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String[]
    getPropertyFilesList()
    {
        return (((IDL3_IDL3) getGenerator()).getPropertyFiles());
    }

    /**
     **
     **/
    public java.lang.String
    visit(org.objectweb.ccm.IDL3.Declaration obj)
    {
        // get the first non module scope that contains the target object
        org.objectweb.ccm.IDL3.Declaration nmodule = obj;
        org.objectweb.ccm.IDL3.Scope module = obj.getParent();
        while ((module.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_repository) &&
               (module.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_module))
        {
            nmodule = module;
            module = nmodule.getParent();
        }

        // set the first non module scope as target declaration
        //        System.err.println("### Generating for declaration : "+nmodule.getAbsoluteName());
        target_decl_ = nmodule;

        // get for dependencies.
        org.objectweb.ccm.IDL3.Declaration[] depend = nmodule.getDependencies(false);

        // sort dependencies
        java.util.Arrays.sort(depend, new org.objectweb.ccm.IDL3.DependencyComparator());

        // get a list of absolute name
        java.lang.String[] depend_abs_names = new java.lang.String[depend.length];
        for (int i = 0; i < depend.length; ++i) {
            depend_abs_names[i] = depend[i].getAbsoluteName();
        }

        // read properties from files
        java.lang.String[] propfiles = getPropertyFilesList();
        /*
        if (this instanceof IDL2Printer) {
            propfiles = ((IDL3_IDL2) getGenerator()).getPropertyFiles();
        } else {
            propfiles = ((IDL3_IDL3) getGenerator()).getPropertyFiles();
        }
        */
        java.util.Properties p = new java.util.Properties();
        for (int i = 0; i < propfiles.length; ++i) {
            try {
                p.load(new java.io.FileInputStream(propfiles[i]));
            } catch (java.io.FileNotFoundException e) {
                System.err.println("Warning : file " + propfiles[i] + " cannot be found.");
            } catch (java.io.IOException e) {
                System.err.println(e);
            }
        }
        
        // find the best match
        org.objectweb.ccm.util.Vector ordered_include_idl_filenames = new org.objectweb.ccm.util.Vector();
        java.lang.String best_property_match_value = "";
        int longest_match_len = 0;
        int match_len = 0;
        for (int i = 0; i < depend_abs_names.length; ++i) {
            java.util.Enumeration propertyNames = p.propertyNames();

            while (propertyNames.hasMoreElements()) {
                java.lang.String s = (java.lang.String) propertyNames.nextElement();
                match_len = longestCommonPrefixLength(depend_abs_names[i], s);
                java.lang.String ss = s.substring(0, match_len);

                if ((match_len > longest_match_len) && (ss.lastIndexOf(':') > 1)) {
                    longest_match_len = match_len;
                    best_property_match_value = p.getProperty(s);
                }
            }

            // we don't want duplicates
            if (!ordered_include_idl_filenames.contains(best_property_match_value)) {
                ordered_include_idl_filenames.add(best_property_match_value);
            }

            longest_match_len = 0;
            best_property_match_value = "";
        }

        // generate filenames
        super.printImmediately(imported(ordered_include_idl_filenames));

        // org.objectweb.ccm.IDL3.Declaration[] depend = new org.objectweb.ccm.IDL3.Declaration[0];
        //        System.err.println("### Found Dependencies : "+depend.length);
        //        for (int i=0;i<depend.length;i++)
        //            System.err.println("    "+depend[i].getAbsoluteName());

        // build dependencies list in correct order
        //
        //  TODO : inter module cycles are not taken into account
        //         intra module cycles will be resolved by forward declarations
        //
        org.objectweb.ccm.util.Vector ordered_depend = new org.objectweb.ccm.util.Vector();
        /*
        for (int i=0;i<depend.length;i++)
        {
            if (isBusinessDeclaration(depend[i]))
                ordered_depend.add(depend[i]);
        }
        */

        ordered_depend.add(nmodule);
        depend = (org.objectweb.ccm.IDL3.Declaration[])
            ordered_depend.toArray(new org.objectweb.ccm.IDL3.Declaration[0]);

        //        System.err.println("### Dependency list cleaned : "+depend.length);
        //        for (int i=0;i<depend.length;i++)
        //            System.err.println("    "+depend[i].getAbsoluteName());

        java.util.Arrays.sort(depend, new org.objectweb.ccm.IDL3.DependencyComparator());

        //        System.err.println("### Dependency list sorted : "+depend.length);
        //        for (int i=0;i<depend.length;i++)
        //            System.err.println("    "+depend[i].getAbsoluteName());

        //        System.err.println(" ");

        // build dependencies tree
        org.objectweb.ccm.IDL3.Scope parent = null;
        org.objectweb.ccm.util.Vector childs = null;
        org.objectweb.ccm.IDL3.Declaration obj1 = null;
        org.objectweb.ccm.util.Vector root_dependencies = new org.objectweb.ccm.util.Vector();
        for (int i=0;i<depend.length;i++)
        {
            obj1  = depend[i];
            parent = obj1.getParent();

            while (parent.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_repository)
            {
                if (dependencies_tree_.containsKey(parent))
                {
                    childs = (org.objectweb.ccm.util.Vector)dependencies_tree_.get(parent);
                    if (childs.indexOf(obj1)==-1)
                        childs.add(obj1);

                    // continue;
                }
                else
                {
                    childs = new org.objectweb.ccm.util.Vector();
                    childs.add(obj1);
                    dependencies_tree_.put(parent, childs);
                }

                obj1 = parent;
                parent = obj1.getParent();
            }

            if (root_dependencies.indexOf(obj1)==-1)
                root_dependencies.add(obj1);
        }

        // generate import statement
        //
        //  TODO : pbs with ::CORBA::TypeCode
        //  ::CORBA::TCKind, ::CORBA::RepositoryId, ::CORBA::Identifier
        //  ::CORBA::Visibility and ::CORBA::ValueModifier are TypeCode
        //  dependencies, so when a TypeCode is used, the corba_used_
        //  member is set to true
        //  should do something to avoid this behaviour
        //
/*
        if (typecode_used_)
            printTypeCodeImport();
        if (corba_used_)
            printCORBAImport();
        if (components_used_)
            printComponentsImport();

        if ((typecode_used_) ||
            (corba_used_) ||
            (components_used_))
            super.printImmediately("\\n");
 */

        // generate user import statement
        super.printImmediately(imported());

        // visit root dependencies
        //        System.err.println("### Visiting root dependencies ...");
        visit(root_dependencies);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    forward(org.objectweb.ccm.util.Vector vect)
    {
        org.objectweb.ccm.IDL3.Declaration[] decls = (org.objectweb.ccm.IDL3.Declaration[])
            vect.toArray(new org.objectweb.ccm.IDL3.Declaration[0]);

        for (int i=0;i<decls.length;i++)
        {
            if ((decls[i].getDeclKind() & forward_check())!=0)
                visitContained(decls[i], "_FORWARD");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    forward(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(true, forward_check());

        for (int i=0;i<objs.length;i++)
            visitContained(objs[i], "_FORWARD");

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    contents(org.objectweb.ccm.IDL3.Scope scope,
             int limited)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(true, limited);

        for (int i=0;i<objs.length;i++)
            visitContained(objs[i], "");

        return "";
   }

    /**
     **
     **/
    public java.lang.String
    printCORBAImport()
    {
        super.printImmediately("import ::CORBA;\\n");
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    printComponentsImport()
    {
        super.printImmediately("import ::Components;\\n");
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    printTypeCodeImport()
    {
        if (!corba_used_)
            super.printImmediately("import ::CORBA;\\n");
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    imported()
    {
        java.lang.String[] objs = ((IDL3_IDL3)getGenerator()).getImported();
        java.lang.String res = "";
        for (int i=0;i<objs.length;i++)
            res = res + "import "+objs[i]+";\\n";

        if (!res.equals(""))
            res = res + "\\n";

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    imported(org.objectweb.ccm.util.Vector v)
    {
        java.lang.String res = "";
        for (java.util.Enumeration e = v.elements(); e.hasMoreElements();)
        {
            java.util.StringTokenizer st = 
                new java.util.StringTokenizer((java.lang.String) e.nextElement());
            if (st.hasMoreTokens()) {
                res += "import " + st.nextToken() + ";\\n";
            }
        }

        if (!res.equals(""))
            res = res + "\\n";

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    macro(org.objectweb.ccm.IDL3.Declaration obj)
    {
        java.lang.String res = "";
        java.lang.String version = obj.getVersion();
        java.lang.String id = obj.getId();
        java.lang.String prefix = obj.getPrefix();
        if (!prefix.equals(last_prefix_))
        {
            res = res + "#pragma prefix \""+prefix+"\"\\n";
            last_prefix_ = prefix;
        }
        else
        {
            java.lang.String p_id = obj.getParent().getId();
            int idx = p_id.lastIndexOf(':');
            if (id.indexOf(p_id.substring(0, idx))==-1)
                res = res + "#pragma id "+obj.getName()+" \""+id+"\"\\n";
        }
        if (!version.equals("1.0"))
            res = res + "#pragma version "+obj.getName()+" "+version+"\\n";

        return res;
   }
    /**
     **
     **/
    public java.lang.String
    base(org.objectweb.ccm.IDL3.ComponentDecl comp)
    {
        if (comp!=null)
            return comp.getAbsoluteName();
   
        delete(3);
        return "";
   }

    /**
     **
     **/
    public java.lang.String
    base(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        if (home!=null)
            return home.getAbsoluteName();
   
        delete(3);
        return "";
   }


    /**
     **
     **/
    public java.lang.String
    supported(org.objectweb.ccm.IDL3.Declaration[] itfs)
    {
        java.lang.String res = "";
        for (int i=0;i<itfs.length;i++)
            res = res + itfs[i].getAbsoluteName() + ", ";

        if (res.equals(""))
        {
            delete(9);
            skip(2);
        }
        else
            res = res.substring(0, res.length()-2);
   
        return res;
    }

    /**
     **
     **/
    public java.lang.String
    primary_key(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        org.objectweb.ccm.IDL3.Declaration pk = home.getPrimaryKey();

        if (pk==null)
        {
            delete(11);
            skip(2);
            return "";
        }
        else
            return pk.getAbsoluteName();
   }

    /**
     **
     **/
    public java.lang.String
    access(org.objectweb.ccm.IDL3.AttributeDecl attr)
    {
        if (attr.isReadonly())
            return "readonly ";

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    type(org.objectweb.ccm.IDL3.TypeRef obj)
    {
        return getTranslator().toIDL(obj);
    }

    /**
     **
     **/
    public java.lang.String
    array(org.objectweb.ccm.IDL3.TypeRef obj)
    {
        return getTranslator().toIDLPostfix(obj);
    }

    /**
     **
     **/
    public java.lang.String
    raises(org.objectweb.ccm.IDL3.ExceptionDecl[] excs)
    {
        return raises(excs, 0);
    }

    /**
     **
     **/
    public java.lang.String
    raises(org.objectweb.ccm.IDL3.ExceptionDecl[] excs,
           int val)
    {
        java.lang.String res = "";
        for (int i=0;i<excs.length;i++)
            res = res + excs[i].getAbsoluteName() + ", ";

        if (res.equals(""))
        {
            if (val==0) delete(11); else delete(14);
            skip(1);
        }
        else
        {
            if (val==0) insert(11); else insert(14);
            res = res.substring(0, res.length()-2);
        }

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    bases(org.objectweb.ccm.IDL3.InterfaceDecl itf)
    {
        java.lang.String res = "";
        org.objectweb.ccm.IDL3.Declaration[] itfs = null;
        itfs = itf.getBaseInterfaces();

        for (int i=0;i<itfs.length;i++)
            res = res+itfs[i].getAbsoluteName()+", ";

        if (!res.equals(""))
            return res.substring(0, res.length()-2);
   
        delete(3);
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    mode(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        if (op.isOneway())
            return "oneway ";

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    toStringMode(int mode)
    {
        if (mode==org.objectweb.ccm.IDL3.OperationDecl.PARAM_IN)
            return "in ";
        else if (mode==org.objectweb.ccm.IDL3.OperationDecl.PARAM_OUT)
            return "out ";
        else
            return "inout ";
    }

    /**
     **
     **/
    public java.lang.String
    params(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        org.objectweb.ccm.IDL3.ParameterList list = null;
        list = op.getParameters();

        java.lang.String[] names = list.getParamNames();
        org.objectweb.ccm.IDL3.TypeRef[] types = list.getParamTypes();
        int[] modes = list.getParamModes();
        java.lang.String res = "";

        for (int i=0;i<names.length;i++)
            res = res + toStringMode(modes[i]) + getTranslator().toIDL(types[i]) + " " + names[i] + ", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    initializer_params(org.objectweb.ccm.IDL3.Initializer obj)
    {
        org.objectweb.ccm.IDL3.ParameterList list = null;
        list = obj.getParameters();

        java.lang.String[] names = list.getParamNames();
        org.objectweb.ccm.IDL3.TypeRef[] types = list.getParamTypes();
        int[] modes = list.getParamModes();
        java.lang.String res = "";

        for (int i=0;i<names.length;i++)
            res = res + toStringMode(modes[i]) + getTranslator().toIDL(types[i]) + " " + names[i] + ", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    contexts(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        java.lang.String[] ctxs = op.getContexts();
        java.lang.String res = "";

        for (int i=0;i<ctxs.length;i++)
            res = res + "\"" + ctxs[i] + "\", ";

        if (res.equals(""))
        {
            delete(12);
            skip(1) ;
            return "";
        }

        insert(12);
        res = res.substring(0, res.length()-2);
        return res;
    }

    /**
     **
     **/
    public java.lang.String
    value(org.objectweb.ccm.IDL3.ConstantDecl constant)
    {
        return getTranslator().toString(constant.getAnyValue(), constant.getType());
    }

    /**
     **
     **/
    public java.lang.String
    struct_members(org.objectweb.ccm.IDL3.Declaration obj,
                   org.objectweb.ccm.IDL3.StructMembers list)
    {
        java.lang.String[] names = list.getMemberNames();
        org.objectweb.ccm.IDL3.TypeRef[] types = list.getMemberTypes();
        boolean is_inner = false;
        int multiple_idx = -1;

        for (int i=0;i<names.length;i++)
        {
            // check for inner struct, union or enum members
            if ((types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_struct) ||
                (types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_union) ||
                (types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_enum))
            {
                org.objectweb.ccm.IDL3.Declaration decl = null;
                decl = (org.objectweb.ccm.IDL3.Declaration)types[i];
                if (decl.getParent().getId().equals(obj.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;


            // compute the declarator
            java.lang.String declarator = "";
            // check for multiple declarators
            //  e.g.  long l1, l2, l3;
            if ((i<names.length-1) && (types[i]==types[i+1]))
                multiple_idx = i;
            while ((i<names.length-1) && (types[i]==types[i+1]))
                i++;

            if (multiple_idx==-1)
                declarator = names[i]+getTranslator().toIDLPostfix(types[i]);
            else
            {
                for (int j=multiple_idx;j<i+1;j++)
                    declarator = declarator +
                        names[j]+getTranslator().toIDLPostfix(types[j]) +
                        ", ";

                declarator = declarator.substring(0, declarator.length()-2);
                multiple_idx = -1;
            }

            if (is_inner)
            {
                // inline struct, union or enum
                org.objectweb.ccm.IDL3.Declaration decl = null;
                java.lang.String id = "";
                
                decl = (org.objectweb.ccm.IDL3.Declaration)types[i];
                id = org.objectweb.ccm.IDL3.DeclarationKind.toString(decl.getDeclKind()).toUpperCase()+
                    "_INLINED";

                java.lang.String[] aliases = { "obj", "declarator" };
                java.lang.Object[] objs = { decl, declarator };
                super.map(id, aliases, objs);
            }
            else
            {
                java.lang.String[] aliases = { "type_spec", "declarator" };
                java.lang.Object[] objs = { getTranslator().toIDL(types[i]),
                                            declarator};
                super.map("STRUCT_MEMBER", aliases, objs);
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    union_members(org.objectweb.ccm.IDL3.UnionDecl union)
    {
        org.objectweb.ccm.IDL3.UnionMembers list = union.getMembers();
        java.lang.String[] names = list.getMemberNames();
        org.objectweb.ccm.IDL3.TypeRef[] types = list.getMemberTypes();
        org.objectweb.ccm.IDL3.AnyValue[] values = list.getMemberValues();
        int di = union.getDefaultIndex();
        boolean is_default = false;
        boolean print = true;
        boolean is_inner = false;

        for (int i=0;i<names.length;i++)
        {
            // check that the i and i+1 element spec are different.
            // else just print the label
            if ((i<names.length-1) &&
                (names[i].equals(names[i+1])))
                print = false;
            else
                print = true;

            if (di==i)
                is_default = true;
            else
                is_default = false;

            // check for inner struct, union or enum members
            if ((types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_struct) ||
                (types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_union) ||
                (types[i].getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_enum))
            {
                org.objectweb.ccm.IDL3.Declaration decl = null;
                decl = (org.objectweb.ccm.IDL3.Declaration)types[i];
                if (decl.getParent().getId().equals(union.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;

            if (!print)
            {
                // print the label
                if (is_default)
                    super.printImmediately("default :\\n");
                else
                {
                    java.lang.String value = getTranslator().toString(values[i], union.getType());
                    super.printImmediately("case "+value+" :\\n");
                }
            }
            else if (is_inner)
            {
                // print the label first
                if (is_default)
                    super.printImmediately("default :\\n");
                else
                {
                    java.lang.String value = getTranslator().toString(values[i], union.getType());
                    super.printImmediately("case "+value+" :\\n");
                }

                // inline struct, union or enum
                org.objectweb.ccm.IDL3.Declaration decl = null;
                java.lang.String id = "";

                decl = (org.objectweb.ccm.IDL3.Declaration)types[i];
                id = org.objectweb.ccm.IDL3.DeclarationKind.toString(decl.getDeclKind()).toUpperCase()+
                    "_INLINED";

                java.lang.String[] aliases2 = { "obj", "declarator" };
                java.lang.Object[] objs2 = { decl, names[i]+getTranslator().toIDLPostfix(types[i]) };
                // reajust the tabulation
                super.inc();
                super.map(id, aliases2, objs2);
                super.dec();
            }
            else
            {
                java.lang.String[] aliases = { "value", "type_spec", "declarator" };
                java.lang.Object[] objs = { getTranslator().toString(values[i], union.getType()),
                                            getTranslator().toIDL(types[i]), 
                                            names[i] + getTranslator().toIDLPostfix(types[i])};
                if (is_default)
                    super.map("UNION_DEFAULT_MEMBER", aliases, objs);
                else
                    super.map("UNION_MEMBER", aliases, objs);
            }
        }

        return "";
   }

    /**
     **
     **/
    public java.lang.String
    enum_members(org.objectweb.ccm.IDL3.EnumDecl enume)
    {
        java.lang.String[] names = enume.getMembers();
        java.lang.String res = "";

        for (int i=0;i<names.length;i++)
            res = res + names[i] + ", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);
   
        return res;
    }

    /**
     **
     **/
    public java.lang.String
    modifier(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        if (value.isCustom())
            return "custom ";
        else if (value.isAbstract())
            return "abstract ";
        else
            return "";
    }

    /**
     **
     **/
    public java.lang.String
    base_value(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.ValueDecl base = value.getBaseValue();
        if (base!=null)
        {
            if (value.isTruncatable())
                return "truncatable "+base.getAbsoluteName();
            else 
                return base.getAbsoluteName();
        }
        delete(3);
        skip(2);
        return "";
   }

    /**
     **
     **/
    public java.lang.String
    abstract_values(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.ValueDecl[] bases = value.getAbstractValues();
        java.lang.String res = "";

        for (int i=0;i<bases.length;i++)
            res = res + bases[i].getAbsoluteName() + ", ";

        if ((value.getBaseValue()==null) && (!res.equals("")))
            return " : " + res.substring(0, res.length()-2);
        else if ((value.getBaseValue()!=null) && (!res.equals("")))
            return res.substring(0, res.length()-2);
        else if ((value.getBaseValue()!=null) && (res.equals("")))
            delete(2);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    initializers(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.Initializer[] ops = value.getInitializers();
        java.lang.String res = "";

        for (int i=0;i<ops.length;i++)
            res = res + "factory " + ops[i].getName() +
                  "(" + initializer_params(ops[i]) + ");\\n";

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    visibility(org.objectweb.ccm.IDL3.ValueMemberDecl vm)
    {
        if (vm.isPrivate())
            return "private";
        else
            return "public";
    }

    /**
     **
     **/
    public java.lang.String
    multiple(org.objectweb.ccm.IDL3.UsesDecl uses)
    {
        if (uses.isMultiple())
            return "multiple ";

        return "";
    }
}
