/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.idl;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class IDLTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    IDLTranslator()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    toIDL(org.objectweb.ccm.IDL3.TypeRef typeref)
    {
        switch(typeref.getTypeKind())
        {
            case org.objectweb.ccm.IDL3.TypeKind._tk_null :
                return "null";
            case org.objectweb.ccm.IDL3.TypeKind._tk_void :
                return "void";
            case org.objectweb.ccm.IDL3.TypeKind._tk_short :
                return "short";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ushort :
                return "unsigned short";
            case org.objectweb.ccm.IDL3.TypeKind._tk_long :
                return "long";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulong :
                return "unsigned long";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longlong :
                return "long long";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulonglong :
                return "unsigned long long";
            case org.objectweb.ccm.IDL3.TypeKind._tk_float :
                return "float";
            case org.objectweb.ccm.IDL3.TypeKind._tk_double :
                return "double";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longdouble :
                return "long double";
            case org.objectweb.ccm.IDL3.TypeKind._tk_boolean :
                return "boolean";
            case org.objectweb.ccm.IDL3.TypeKind._tk_char :
                return "char";
            case org.objectweb.ccm.IDL3.TypeKind._tk_wchar :
                return "wchar";
            case org.objectweb.ccm.IDL3.TypeKind._tk_octet :
                return "octet";
            case org.objectweb.ccm.IDL3.TypeKind._tk_any :
                return "any";
            case org.objectweb.ccm.IDL3.TypeKind._tk_Principal :
                return "::CORBA::Principal";
            case org.objectweb.ccm.IDL3.TypeKind._tk_TypeCode :
                return "::CORBA::TypeCode";

            case org.objectweb.ccm.IDL3.TypeKind._tk_objref :
                return "Object";

            case org.objectweb.ccm.IDL3.TypeKind._tk_struct :
            case org.objectweb.ccm.IDL3.TypeKind._tk_union :
            case org.objectweb.ccm.IDL3.TypeKind._tk_enum :
            case org.objectweb.ccm.IDL3.TypeKind._tk_exception :
            case org.objectweb.ccm.IDL3.TypeKind._tk_alias :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value_box :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value :
            case org.objectweb.ccm.IDL3.TypeKind._tk_native :
            case org.objectweb.ccm.IDL3.TypeKind._tk_abstract_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_local_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_component :
            case org.objectweb.ccm.IDL3.TypeKind._tk_event :
            case org.objectweb.ccm.IDL3.TypeKind._tk_home :
                org.objectweb.ccm.IDL3.Declaration decl = null;
                decl = (org.objectweb.ccm.IDL3.Declaration)typeref;
                return decl.getAbsoluteName();

            case org.objectweb.ccm.IDL3.TypeKind._tk_string :
                int l = ((org.objectweb.ccm.IDL3.TypeRefImpl)typeref).getLength();
                if (l==0)
                    return "string";
                return "string<"+l+">";
            case org.objectweb.ccm.IDL3.TypeKind._tk_wstring :
                l = ((org.objectweb.ccm.IDL3.TypeRefImpl)typeref).getLength();
                if (l==0)
                    return "wstring";
                return "wstring<"+l+">";
            case org.objectweb.ccm.IDL3.TypeKind._tk_sequence :
                org.objectweb.ccm.IDL3.TypeRefImpl impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                l = impl.getLength();
                java.lang.String str = toIDL(impl.getContentType());
                if (l==0)
                    return "sequence<"+str+">";
                return "sequence<"+str+","+l+">";
            case org.objectweb.ccm.IDL3.TypeKind._tk_array :
                impl = (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                str = toIDL(impl.getContentType());
                return str;
            case org.objectweb.ccm.IDL3.TypeKind._tk_fixed :
                impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                return "fixed<"+impl.getDigits()+","+impl.getScale()+">";
            default :
                return "";
        }
    }

    /**
     **
     **/
    public java.lang.String
    toIDLPostfix(org.objectweb.ccm.IDL3.TypeRef typeref)
    {
        switch(typeref.getTypeKind())
        {
            case org.objectweb.ccm.IDL3.TypeKind._tk_array :
                org.objectweb.ccm.IDL3.TypeRefImpl impl = null;
                impl = (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                java.lang.String str = toIDLPostfix(impl.getContentType());
                int l = impl.getLength();
                return str+"["+l+"]";
            default :
                return "";
        }
    }

    /**
     **
     **/
    public java.lang.String
    toString(org.objectweb.ccm.IDL3.AnyValue anyvalue,
             org.objectweb.ccm.IDL3.TypeRef typeref)
    {
        //
        //  TODO : add escaped sequencesto strings, eg '\n' => "\\n"
        //
        if (anyvalue.isInteger())
            return java.lang.String.valueOf(anyvalue.getAsInteger());
        else if (anyvalue.isString())
            return "\""+anyvalue.getAsString()+"\"";
        else if (anyvalue.isWString())
            return "L\""+anyvalue.getAsWString()+"\"";
        else if (anyvalue.isChar())
            return "\'"+java.lang.String.valueOf(anyvalue.getAsChar())+"\'";
        else if (anyvalue.isWChar())
            return "L\'"+java.lang.String.valueOf(anyvalue.getAsWChar())+"\'";
        else if (anyvalue.isFloating())
            return java.lang.String.valueOf(anyvalue.getAsFloating());
        else if (anyvalue.isFixed())
            return anyvalue.getAsFixed();
        else if (anyvalue.isBoolean())
        {
            if (anyvalue.getAsBoolean())
                return "TRUE";

            return "FALSE";
        }
        else if (anyvalue.isEnum())
        {
            // build enum member absolute name
            java.lang.String abs_name = ((org.objectweb.ccm.IDL3.EnumDecl)typeref).getParent().getAbsoluteName();
            return abs_name + "::" + anyvalue.getAsEnum();
        }
        else
            return "";
    }
}
