/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.java;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class JavaImplPrinter
       extends JavaPrinterBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    JavaImplPrinter(Generator gen,
                    java.lang.String indent)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, '$', '#', indent);

        // set the "self" alias in root scope on this object.
        super.setAlias("self", this);

        // load the template file.
        super.loadFile("java_impl_common.template");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods inherited from Printer.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    modifyCurrentLine(java.lang.String str)
    {
        if (str.length()==0)
            return "";

        if (str.charAt(0)=='{')
            return "{\\n$self.inc()$";

        if (str.charAt(0)=='}')
            return "$self.dec()$}\\n";

        if ((str.charAt(0)!=getCallChar()) || (str.charAt(str.length()-1)!=getCallChar()))
            return str+"\\n";

        return str;
    }

    // ==================================================================
    //
    // Public methods (common).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    which()
    {
        return "ir3_jimpl";
    }

    /**
     **
     **/
    public java.lang.String
    visitModule(org.objectweb.ccm.IDL3.ModuleDecl module)
    {
        java.lang.String[] aliases = { "obj" };
        org.objectweb.ccm.IDL3.Declaration[] components = null;
        components = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_component);
        for (int i=0;i<components.length;i++)
        {
            java.lang.Object[] objs = { components[i] };
            super.map("COMPONENT_MONOLITHIC", aliases, objs);
        }

        org.objectweb.ccm.IDL3.Declaration[] homes = null;
        homes = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_home);
        for (int i=0;i<homes.length;i++)
        {
            java.lang.Object[] objs = { homes[i] };
            super.map("HOME_IMPL", aliases, objs);
        }

        org.objectweb.ccm.IDL3.Declaration[] values = null;
        org.objectweb.ccm.IDL3.ValueDecl value = null;
        values = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_event+
                                          org.objectweb.ccm.IDL3.DeclarationKind._dk_value);
        for (int i=0;i<values.length;i++)
        {
            value = (org.objectweb.ccm.IDL3.ValueDecl)values[i];
            if (value.isAbstract())
                continue ;

            java.lang.Object[] objs = { values[i] };
            super.map("VALUE_IMPL", aliases, objs);
            super.map("VALUE_FACTORY_IMPL", aliases, objs);
        }

        org.objectweb.ccm.IDL3.Declaration[] modules = null;
        modules = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_module);
        for (int i=0;i<modules.length;i++)
            visitModule((org.objectweb.ccm.IDL3.ModuleDecl)modules[i]);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    packageName(org.objectweb.ccm.IDL3.Declaration decl)
    {
        java.lang.String pack_name = getTranslator().getPackage(decl);
        if (!pack_name.equals(""))
            return pack_name;

        delete(8);
        skip(3);
        return "";
    }

    // ==================================================================
    //
    // Public methods (component related).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    businessOperations(org.objectweb.ccm.IDL3.ComponentDecl comp)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        org.objectweb.ccm.IDL3.InterfaceDecl itf = comp.getLocalMonolithicMapping();
        objs = itf.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation+
                                      org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute);

        for (int i=0;i<objs.length;i++)
        {
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute)
            {
                org.objectweb.ccm.IDL3.AttributeDecl attr = (org.objectweb.ccm.IDL3.AttributeDecl)objs[i];
                if (attr.isReadonly())
                    visitContained(attr, "_IMPL_READONLY");
                else
                    visitContained(attr, "_IMPL");
            }
            else
            {
                org.objectweb.ccm.IDL3.OperationDecl op = (org.objectweb.ccm.IDL3.OperationDecl)objs[i];
                if (op.getParent().getAbsoluteName().startsWith("::Components"))
                    continue ;
                else if (op.getType().getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_void)
                    visitContained(objs[i], "_IMPL_VOID");
                else
                    visitContained(objs[i], "_IMPL");
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    businessOperations(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        org.objectweb.ccm.IDL3.InterfaceDecl itf = home.getLocalMapping();
        objs = itf.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation);

        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.OperationDecl op = (org.objectweb.ccm.IDL3.OperationDecl)objs[i];
            if (op.getParent().getAbsoluteName().startsWith("::Components"))
                continue ;
            else if (op.getType().getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_void)
                visitContained(objs[i], "_IMPL_VOID");
            else
                visitContained(objs[i], "_IMPL");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    initMembers(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = value.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_value_member);
        java.lang.String str = "";

        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.ValueMemberDecl member = (org.objectweb.ccm.IDL3.ValueMemberDecl)objs[i];
            //
            // Keywords check added.
            //
            str = checkName(member.getName()) +
                " = " +
                nullValue(member.getType()) +
                ";\\n";
            super.printImmediately(str);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    localOperations(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = value.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation);
        java.lang.String str = "";

        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.OperationDecl op = (org.objectweb.ccm.IDL3.OperationDecl)objs[i];
            if (op.getType().getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_void)
                visitContained(objs[i], "_IMPL_VOID");
            else
                visitContained(objs[i], "_IMPL");
        }


        return "";
    }

    /**
     **
     **/
    public java.lang.String
    baseFactory(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.Initializer[] inits = value.getInitializers();

        if (inits.length==0)
            return "org.omg.CORBA.portable.ValueFactory";

        //
        // Keywords check added.
        //
        return checkName(value.getName())+"ValueFactory";
    }

    /**
     **
     **/
    public java.lang.String
    factoriesImpl(org.objectweb.ccm.IDL3.ValueDecl value)
    {
        org.objectweb.ccm.IDL3.Initializer[] inits = value.getInitializers();
        java.lang.String[] aliases = { "obj", "value" };

        for (int i=0;i<inits.length;i++)
        {
            java.lang.Object[] objs = { inits[i], value };
            super.map("INITIALIZER_IMPL", aliases, objs);
        }

        return "";
    }
}
