/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.java;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * Base class for Java printers.
 * It defines operations that have a general purpose.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class JavaPrinterBase
       extends Printer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected JavaTranslator translator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    JavaPrinterBase(Generator gen,
                    char call_char,
                    char comment_char,
                    java.lang.String indent)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, call_char, comment_char, indent);

        // init internal state.
        translator_ = new JavaTranslator();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods that have a general purpose.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    open(org.objectweb.ccm.IDL3.Declaration obj,
         java.lang.String ext)
    {
        super.setDirectory(getTranslator().getAsDirectory(obj));
        super.open(name(obj)+ext);
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    close(org.objectweb.ccm.IDL3.Declaration obj,
          java.lang.String str)
    {
        System.out.println("File \'"+getTranslator().getAsDirectory(obj)+name(obj)+str+"\' has been generated");
        super.close();
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    visitContained(org.objectweb.ccm.IDL3.Declaration contained,
                   java.lang.String extension)
    {
        java.lang.String id =
            org.objectweb.ccm.IDL3.DeclarationKind.toString(contained.getDeclKind()).toUpperCase()+
            extension;

        java.lang.String[] aliases = { "obj" };
        java.lang.Object[] objs = { contained };
        super.map(id, aliases, objs);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    visitContainedWithMapping(org.objectweb.ccm.IDL3.Declaration obj,
                              org.objectweb.ccm.IDL3.Declaration mapobj,
                              java.lang.String extension)
    {
        java.lang.String id =
            org.objectweb.ccm.IDL3.DeclarationKind.toString(mapobj.getDeclKind()).toUpperCase()+
            extension;

        java.lang.String[] aliases = { "obj" , "mapobj" };
        java.lang.Object[] objs = { obj, mapobj };
        super.map(id, aliases, objs);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    visit(org.objectweb.ccm.IDL3.Declaration obj)
    {
        if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_module)
            return visitContained(obj, "");

        java.lang.String msg =
            "error in class org.objectweb.ccm.visitorIDL3.JavaPrinterBase\n"+
            "   operation : visit\n"+
            "   message : root visited object must be a module";
        System.err.println(msg);
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    name(org.objectweb.ccm.IDL3.Declaration decl)
    {
        return getTranslator().checkKeywords(decl.getName());
    }

    /**
     **
     **/
    public java.lang.String
    checkName(java.lang.String idl_name)
    {
        return getTranslator().checkKeywords(idl_name);
    }

    /**
     **
     **/
    public java.lang.String
    type(org.objectweb.ccm.IDL3.TypeRef type)
    {
        return getTranslator().toJava(type);
    }

    /**
     **
     **/
    public java.lang.String
    isVoid(org.objectweb.ccm.IDL3.TypeRef type)
    {
        if (type.getTypeKind()!=org.objectweb.ccm.IDL3.TypeKind._tk_void)
            return "return ";

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    params(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        return params(op.getParameters(), op.getContexts());
    }

    /**
     **
     **/
    public java.lang.String
    params(org.objectweb.ccm.IDL3.Initializer init)
    {
        return params(init.getParameters(), null);
    }

    /**
     **
     **/
    public java.lang.String
    params(org.objectweb.ccm.IDL3.ParameterList list,
           java.lang.String[] ctxs)
    {
        java.lang.String[] names = list.getParamNames();
        org.objectweb.ccm.IDL3.TypeRef[] types = list.getParamTypes();
        int[] modes = list.getParamModes();
        java.lang.String res = "";

        for (int i=0;i<names.length;i++)
            res = res + getTranslator().toJava(types[i], modes[i]) + " " + names[i] + ", ";

        if ((ctxs!=null) && (ctxs.length!=0))
            res = res + "org.omg.CORBA.Context ctx, ";
        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    paramNames(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        return paramNames(op.getParameters(), op.getContexts());
    }

    /**
     **
     **/
    public java.lang.String
    paramNames(org.objectweb.ccm.IDL3.Initializer init)
    {
        return paramNames(init.getParameters(), null);
    }

    /**
     **
     **/
    public java.lang.String
    paramNames(org.objectweb.ccm.IDL3.ParameterList list,
               java.lang.String[] ctxs)
    {
        java.lang.String[] names = list.getParamNames();
        java.lang.String str = "";

        for (int i=0;i<names.length;i++)
            str = str + names[i] + ", ";

        if ((ctxs!=null) && (ctxs.length!=0))
            str = str + "ctx, ";
        if (!str.equals(""))
            str = str.substring(0, str.length()-2);

        return str;
    }

    /**
     **
     **/
    public java.lang.String
    exceptions(org.objectweb.ccm.IDL3.ExceptionDecl[] excs)
    {
        java.lang.String res = "";
        for (int i=0;i<excs.length;i++)
            res = res + getTranslator().getAbsoluteName(excs[i]) + ", ";

        if (!res.equals(""))
            return "throws " + res.substring(0, res.length()-2) + "\\n";

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    nullValue(org.objectweb.ccm.IDL3.TypeRef type)
    {
        switch(type.getTypeKind())
        {
        case org.objectweb.ccm.IDL3.TypeKind._tk_null :
            return "????";
        case org.objectweb.ccm.IDL3.TypeKind._tk_void :
            return "????";
        case org.objectweb.ccm.IDL3.TypeKind._tk_short :
        case org.objectweb.ccm.IDL3.TypeKind._tk_ushort :
            return "0";
        case org.objectweb.ccm.IDL3.TypeKind._tk_long :
        case org.objectweb.ccm.IDL3.TypeKind._tk_ulong :
            return "0";
        case org.objectweb.ccm.IDL3.TypeKind._tk_longlong :
        case org.objectweb.ccm.IDL3.TypeKind._tk_ulonglong :
            return "0";
        case org.objectweb.ccm.IDL3.TypeKind._tk_char :
        case org.objectweb.ccm.IDL3.TypeKind._tk_wchar :
            return "\'\0\'";
        case org.objectweb.ccm.IDL3.TypeKind._tk_string :
        case org.objectweb.ccm.IDL3.TypeKind._tk_wstring :
            return "\"\"";
        case org.objectweb.ccm.IDL3.TypeKind._tk_float :
            return "0f";
        case org.objectweb.ccm.IDL3.TypeKind._tk_double :
            return "0.0";
        case org.objectweb.ccm.IDL3.TypeKind._tk_boolean :
            return "true";
        case org.objectweb.ccm.IDL3.TypeKind._tk_octet :
            return "(byte)0";
            // objects case
        default :
            return "null";
        }
    }

    /**
     **
     **/
    public org.objectweb.ccm.IDL3.TypeRef
    getType(org.objectweb.ccm.IDL3.Declaration obj)
    {
        switch(obj.getDeclKind())
        {
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_alias :
                return ((org.objectweb.ccm.IDL3.AliasDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute :
                return ((org.objectweb.ccm.IDL3.AttributeDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_constant :
                return ((org.objectweb.ccm.IDL3.ConstantDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_emits :
                return ((org.objectweb.ccm.IDL3.EmitsDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes :
                return ((org.objectweb.ccm.IDL3.PublishesDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes :
                return ((org.objectweb.ccm.IDL3.ConsumesDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_operation :
                return ((org.objectweb.ccm.IDL3.OperationDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_provides :
                return ((org.objectweb.ccm.IDL3.ProvidesDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_union :
                return ((org.objectweb.ccm.IDL3.UnionDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_uses :
                return ((org.objectweb.ccm.IDL3.UsesDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_value_box :
                return ((org.objectweb.ccm.IDL3.ValueBoxDecl)obj).getType();
            case org.objectweb.ccm.IDL3.DeclarationKind._dk_value_member :
                return ((org.objectweb.ccm.IDL3.ValueMemberDecl)obj).getType();
            default :
                return null;
        }
    }

    /**
     **
     **/
    public JavaTranslator
    getTranslator()
    {
        return translator_;
    }
}
