/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc.

====================================================================*/

package org.objectweb.ccm.visitorIDL3.java;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class JavaSkeletonPrinter
       extends JavaPrinterBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    JavaSkeletonPrinter(Generator gen,
                        java.lang.String indent)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, '$', '#', indent);

        // set the "self" alias in root scope on this object.
        super.setAlias("self", this);

        // load the template file.
        //        super.loadFile("java_skeleton.template");
        super.loadFile("java_skeleton_component.template");
        super.loadFile("java_skeleton_home.template");
        super.loadFile("java_skeleton_event.template");
        super.loadFile("java_skeleton_common.template");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods inherited from Printer.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    modifyCurrentLine(java.lang.String str)
    {
        if (str.length()==0)
	    // Modified by Philippe Merle
	    // for dealing with empty line in template files.
            return "\\n";

        if (str.charAt(0)=='{')
            return "{\\n$self.inc()$";

        if (str.charAt(0)=='}')
            return "$self.dec()$}\\n";

        if ((str.charAt(0)!=getCallChar()) || (str.charAt(str.length()-1)!=getCallChar()))
            return str+"\\n";

        return str;
    }

    // ==================================================================
    //
    // Public methods (common).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    which()
    {
        return "ir3_java";
    }

    /**
     **
     **/
    public java.lang.String
    visitModule(org.objectweb.ccm.IDL3.ModuleDecl module)
    {
        org.objectweb.ccm.util.Vector ports = new org.objectweb.ccm.util.Vector();

        org.objectweb.ccm.IDL3.Declaration[] components = null;
        components = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_component);
        for (int i=0;i<components.length;i++)
        {
            org.objectweb.ccm.IDL3.ComponentDecl comp = null;
            org.objectweb.ccm.IDL3.Declaration[] comp_ports = null;

            comp = (org.objectweb.ccm.IDL3.ComponentDecl)components[i];
            visitContained(comp, "_SKELETON");
            visitContained(comp.getClientMapping(), "_SKELETON_INTERCEPTOR");
            visitContained(comp, "_MONOLITHIC_WRAPPER");
            comp_ports = comp.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                          org.objectweb.ccm.IDL3.DeclarationKind._dk_uses +
                                          org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                          org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                          org.objectweb.ccm.IDL3.DeclarationKind._dk_emits);

            for (int j=0;j<comp_ports.length;j++)
            {
                org.objectweb.ccm.IDL3.TypeRef port_type = getType(comp_ports[j]);
                if (!ports.contains(port_type))
                    ports.add(port_type);
            }
        }

        org.objectweb.ccm.IDL3.Declaration[] ports_arr = null;
        ports_arr = (org.objectweb.ccm.IDL3.Declaration[])ports.toArray(new org.objectweb.ccm.IDL3.Declaration[0]);
        for (int i=0;i<ports_arr.length;i++)
        {
            if (ports_arr[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_interface)
            {
                visitContained(ports_arr[i], "_SKELETON_INTERCEPTOR");
                visitContained(ports_arr[i], "_STUB_INTERCEPTOR");
            }
            else if (ports_arr[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_event)
            {
                org.objectweb.ccm.IDL3.EventDecl event = null;
                event = (org.objectweb.ccm.IDL3.EventDecl)ports_arr[i];
                visitContained(event.getClientMapping(),"_SKELETON_INTERCEPTOR");
                visitContained(event.getClientMapping(),"_STUB_INTERCEPTOR");
           }
        }

        org.objectweb.ccm.IDL3.Declaration[] homes = null;
        homes = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_home);
        for (int i=0;i<homes.length;i++)
        {
            visitContained(homes[i], "_SKELETON");
            visitContained(((org.objectweb.ccm.IDL3.HomeDecl)homes[i]).getClientMapping(),
                           "_SKELETON_INTERCEPTOR");
            visitContained(((org.objectweb.ccm.IDL3.HomeDecl)homes[i]).getClientMapping(),
                           "_STUB_INTERCEPTOR");
        }

        org.objectweb.ccm.IDL3.Declaration[] events = null;
        events = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_event);
        for (int i=0;i<events.length;i++)
        {
            visitContained(events[i], "");
            visitContained(events[i], "_CONSUMER_WRAPPER");
        }

        org.objectweb.ccm.IDL3.Declaration[] values = null;
        values = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_value);
        for (int i=0;i<values.length;i++)
        {
            //
            //  TODO : incomplete : should recursively check inherited values also
            //
            org.objectweb.ccm.IDL3.ValueDecl val = (org.objectweb.ccm.IDL3.ValueDecl)values[i];
            if (val.isPrimaryKey())
                visitContained(values[i], "_PK");
            else
                visitContained(values[i], "");
        }

        org.objectweb.ccm.IDL3.Declaration[] modules = null;
        modules = module.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_module);
        for (int i=0;i<modules.length;i++)
            visitContained(modules[i], "");

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    extractSuperName(org.objectweb.ccm.IDL3.Declaration obj,
                     org.objectweb.ccm.IDL3.Declaration mapobj)
    {
        java.lang.String res = mapobj.getName();
        int idx = res.indexOf(obj.getName());
        if (idx==0)
            return res;

        return res.substring(0, idx-1);
    }

    /**
     **
     **/
    public java.lang.String
    packageName(org.objectweb.ccm.IDL3.Declaration decl)
    {
        java.lang.String pack_name = getTranslator().getPackage(decl);
        if (!pack_name.equals(""))
            return pack_name;

        delete(8);
        skip(3);
        return "";
    }

    // ==================================================================
    //
    // Public methods (component related).
    //
    // ==================================================================

    // Added by Philippe Merle
    //
    public java.lang.String
    javaPackage(org.objectweb.ccm.IDL3.Declaration type)
    {
        java.lang.String packageName = getTranslator().getPackage(type);

        if (packageName.length() != 0)
            packageName = packageName + '.';

        return packageName;
    }

    // Added by Philippe Merle
    //
    public java.lang.String
    javaType(org.objectweb.ccm.IDL3.Declaration type,
             java.lang.String prefix,
             java.lang.String postfix)
    {
        return javaPackage(type) + prefix + type.getName() + postfix;
    }

    // Added by Philippe Merle
    //
    public java.lang.String
    javaCCMConsumer(org.objectweb.ccm.IDL3.Declaration type)
    {
        return javaType(type, "CCM_", "Consumer");
    }

    // Added by Philippe Merle.
    public java.lang.String
    executorWrapperAttributes(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs =
            scope.getContents(false,
                org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes);

        java.lang.String res = "";
        for (int i=0; i<objs.length; i++)
        {
            java.lang.String name = objs[i].getName();

            res = res +
                "/** Wrapper for port " + name + ". */\n    " +
                "protected " + name + "_ConsumerWrapper " +
                name + "_wrapper_ =\n        new " +
                name + "_ConsumerWrapper();\n\n";
        }

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    obtainExecutorWrapper(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes);

        java.lang.String[] aliases = { "name" , "impl" };
        for (int i=0;i<objs.length;i++)
        {
            java.lang.String impl = "";
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_provides)
                impl = "_monolithic_executor.get_"+objs[i].getName()+"()";
            else
                impl = objs[i].getName()+"_wrapper_";

            java.lang.Object[] objs2 = { objs[i].getName(), impl };
            super.map("COMPONENT_MONOLITHIC_EXECUTOR_WRAPPER", aliases, objs2);
        }

        return "";
    }

    /**
     ** Added by Philippe Merle.
     **/
    public java.lang.String
    monolithicEventWrapperClasses(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs =
            scope.getContents(false,
                org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes);

        java.lang.String res = "";
        for (int i=0; i<objs.length; i++)
        {
            org.objectweb.ccm.IDL3.InterfaceDecl itf =
                ((org.objectweb.ccm.IDL3.EventDecl)getType(objs[i])).
                                                   getLocalConsumerMapping();

            org.objectweb.ccm.IDL3.Declaration[] ops =
                itf.getContents(false,
                    org.objectweb.ccm.IDL3.DeclarationKind._dk_operation);

            for (int j=0; j<ops.length; j++)
                visitContainedWithMapping(objs[i], ops[j], 
                     "_EVENT_CONSUMER_MONOLITHIC_WRAPPER");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    portDeclarations(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        java.lang.String res = "";
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_uses +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_emits);


        for (int i=0;i<objs.length;i++)
        {
            java.lang.String port_info = "";
            org.objectweb.ccm.IDL3.Declaration type = null;
            type = (org.objectweb.ccm.IDL3.Declaration)getType(objs[i]);

            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_provides)
                port_info = "ProvidesInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+type.getId()+"\")";
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes)
            {
                org.objectweb.ccm.IDL3.EventDecl event = (org.objectweb.ccm.IDL3.EventDecl)type;
                org.objectweb.ccm.IDL3.InterfaceDecl itf = event.getClientMapping();
                port_info = "ConsumesInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+itf.getId()+"\")";
            }
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_emits)
            {
                org.objectweb.ccm.IDL3.EventDecl event = (org.objectweb.ccm.IDL3.EventDecl)type;
                org.objectweb.ccm.IDL3.InterfaceDecl itf = event.getClientMapping();
                port_info = "EmitsInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+itf.getId()+"\")";
            }
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes)
            {
                org.objectweb.ccm.IDL3.EventDecl event = (org.objectweb.ccm.IDL3.EventDecl)type;
                org.objectweb.ccm.IDL3.InterfaceDecl itf = event.getClientMapping();
                port_info = "PublishesInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+itf.getId()+"\")";
            }
            else
            {
                org.objectweb.ccm.IDL3.UsesDecl uses = (org.objectweb.ccm.IDL3.UsesDecl)objs[i];
                if (uses.isMultiple())
                    port_info = "UsesInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+type.getId()+"\", "+
                        java.lang.Integer.toString(java.lang.Integer.MAX_VALUE)+")";
                else
                    port_info = "UsesInfo(\""+objs[i].getId()+"\", \""+objs[i].getName()+"\", \""+type.getId()+"\")";
            }

            res = res+"_declare_port(new org.objectweb.ccm.Components."+port_info+");\\n";
        }

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    computeComponentOpName(org.objectweb.ccm.IDL3.Declaration obj,
                           org.objectweb.ccm.IDL3.Declaration mapobj)
    {
        java.lang.String str = "";

        if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_provides)
            str = "provide_facet";
        else if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_uses)
        {
            str = "_"+extractSuperName(obj, mapobj);
            org.objectweb.ccm.IDL3.UsesDecl uses = (org.objectweb.ccm.IDL3.UsesDecl)obj;
            if (!uses.isMultiple())
                str = str+"_simple_receptacle";
        }
        else if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes)
            str = "_get_consumer";
        else if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_emits)
            str = "_"+extractSuperName(obj, mapobj)+"_emitter";
        else if (obj.getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes)
            str = "_"+extractSuperName(obj, mapobj);

        return str;
    }

    /**
     **  Note: receptacles operations are not generated here as they are
     **  already generated for the remote interface implementation
     **  (they have the same signature).
     **/
    public java.lang.String
    contextOperations(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, 
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_emits);

        java.lang.String[] aliases = { "obj", "event_abs_name", "port_name", "event" };
        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.OperationDecl[] ops = null;
            java.lang.String id = "";

            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes)
            {
                ops = ((org.objectweb.ccm.IDL3.PublishesDecl)objs[i]).getLocalContextMapping();
                id  = "COMPONENT_CONTEXT_PUBLISHES";
            }
            else
            {
                ops = ((org.objectweb.ccm.IDL3.EmitsDecl)objs[i]).getLocalContextMapping();
                id = "COMPONENT_CONTEXT_EMITS";
            }

            for (int j=0;j<ops.length;j++)
            {
                java.lang.Object[] objs2 = { ops[j], 
                                            getTranslator().getAbsoluteName(
                                                (org.objectweb.ccm.IDL3.Declaration)getType(objs[i])),
                                            objs[i].getName(),
                                            getType(objs[i]) };
                super.map(id, aliases, objs2);
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    connectionOperations(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, 
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_uses +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_emits);

        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.OperationDecl[] ops = null;
            java.lang.String id = "";

            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_provides)
                ops = ((org.objectweb.ccm.IDL3.ProvidesDecl)objs[i]).getClientMapping();
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_uses)
                ops = ((org.objectweb.ccm.IDL3.UsesDecl)objs[i]).getClientMapping();
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes)
                ops = ((org.objectweb.ccm.IDL3.ConsumesDecl)objs[i]).getClientMapping();
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes)
                ops = ((org.objectweb.ccm.IDL3.PublishesDecl)objs[i]).getClientMapping();
            else 
                ops = ((org.objectweb.ccm.IDL3.EmitsDecl)objs[i]).getClientMapping();

            java.lang.String[] aliases = { "obj", "narrow", "super_op_name",
                                           "port_name", "param_names",
                                           "port_abs_name", "struct_abs_name" };
            for (int j=0;j<ops.length;j++)
            {
                id = "COMPONENT_CONNECTION";
                java.lang.String narrow = "";
                java.lang.String param_names = paramNames(ops[j]);
                java.lang.String port_abs_name = getTranslator().getAbsoluteName(objs[i]);
                java.lang.String struct_abs_name = "";
                if (!param_names.equals(""))
                    param_names = ", "+param_names;
                if (ops[j].getType().getTypeKind()!=org.objectweb.ccm.IDL3.TypeKind._tk_void)
                {
                    // only narrow the result if the returned type is not ::Components::Cookie
                    // or even if it's not an valuetype (more simple)
                    if (ops[j].getType().getTypeKind()!=org.objectweb.ccm.IDL3.TypeKind._tk_value)
                    {
                        id = id+"_DEFAULT";
                        narrow = type(ops[j].getType())+"Helper.narrow";
                    }
                    else
                        id = id+"_NO_NARROW";
                }
                else
                    id = id+"_VOID";

                // if uses multiple, uses another template for the get_connections_XXX operation.
                if ((objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_uses) &&
                    (ops[j].getName().startsWith("get_connections_")))
                {
                    id = "COMPONENT_CONNECTION_USES_MULTIPLE";
                    org.objectweb.ccm.IDL3.AliasDecl alias = null;
                    org.objectweb.ccm.IDL3.TypeRefImpl sequence = null;
                    org.objectweb.ccm.IDL3.StructDecl struct = null;
                    alias = (org.objectweb.ccm.IDL3.AliasDecl)ops[j].getType();
                    sequence = (org.objectweb.ccm.IDL3.TypeRefImpl)alias.getType();
                    struct = (org.objectweb.ccm.IDL3.StructDecl)sequence.getContentType();
                    struct_abs_name = getTranslator().getAbsoluteName(struct);
                    narrow = type(getType(objs[i]))+"Helper.narrow";
                }

                java.lang.Object[] objs2 = { ops[j], narrow,
                                             computeComponentOpName(objs[i], ops[j]),
                                             objs[i].getName(),
                                             param_names,
                                             port_abs_name,
                                             struct_abs_name };
                super.map(id, aliases, objs2);
            }
        }

        return "";
    }


    // ==================================================================
    //
    // Public methods (home related).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    homeHelper(org.objectweb.ccm.IDL3.TypeRef type)
    {
        java.lang.String res = getTranslator().toJava(type);
        // for primary keys
        if (type.getTypeKind()!=org.objectweb.ccm.IDL3.TypeKind._tk_interface)
            res = res + "Factory";

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    hasPK(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        if (home.getPrimaryKey()==null)
            return "false";

        return "true";
    }

    /**
     **
     **/
    public java.lang.String
    baseHome(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        if (home.getPrimaryKey()==null)
            return "CCMHomeImpl";

        return "CCMHomeWithPKImpl";
    }

    /**
     **
     **/
    public java.lang.String
    implementedHomeExecutor(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        if (home.getPrimaryKey()==null)
            return "HomeExecutor";

        return "HomeExecutorWithPK";
    }

    /**
     **
     **/
    public java.lang.String
    homeExecutorOps(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        java.lang.String id = "HOME_EXECUTOR_OPS";
        java.lang.String key_abs_name = "";
        if (home.getPrimaryKey()!=null)
        {
            id = id + "_WITH_PK";
            key_abs_name = getTranslator().getAbsoluteName(home.getPrimaryKey());
        }

        java.lang.String[] aliases = { "key_abs_name" };
        java.lang.Object[] objs = { key_abs_name };
        super.map(id, aliases, objs);


        return "";
    }
 
    /**
     **
     **/
    public java.lang.String
    primaryKeyOperations(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        org.objectweb.ccm.IDL3.InterfaceDecl itf = null;
        org.objectweb.ccm.IDL3.Declaration[] ops = null;

        itf = home.getClientImplicitMapping();
        ops = itf.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation);

        java.lang.String id = "";
        java.lang.String[] aliases = { "obj" };
        for (int i=0;i<ops.length;i++)
        {
            id = "HOME_OPERATION_SUPER";
            if (getType(ops[i]).getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_void)
                id = id + "_VOID";

            java.lang.Object[] objs = { ops[i] };
            super.map(id, aliases, objs);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    factoryFinderOperations(org.objectweb.ccm.IDL3.HomeDecl home)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;

        objs = home.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_factory +
                                       org.objectweb.ccm.IDL3.DeclarationKind._dk_finder);

        java.lang.String id = "";
        org.objectweb.ccm.IDL3.Declaration[] ops = null;
        org.objectweb.ccm.IDL3.FactoryDecl factory = null;
        org.objectweb.ccm.IDL3.FinderDecl finder = null;
        java.lang.String[] aliases = { "obj" };
        for (int i=0;i<objs.length;i++)
        {
            id = "HOME";
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_factory)
            {
                factory = (org.objectweb.ccm.IDL3.FactoryDecl)objs[i];
                ops = factory.getClientMapping();
                id = id + "_FACTORY_SUPER";
            }
            else
            {
                finder = (org.objectweb.ccm.IDL3.FinderDecl)objs[i];
                ops = finder.getClientMapping();
                id = id + "_FINDER_SUPER";
            }

            for (int j=0;j<ops.length;j++)
            {
                java.lang.Object[] objs2 = { ops[j] };
                super.map(id, aliases, objs2);
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    portInterceptors(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_emits +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_uses);

        java.lang.String res = "";
        org.objectweb.ccm.IDL3.Declaration type = null;
        java.lang.String ext = "";
        for (int i=0;i<objs.length;i++)
        {
            if ((objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_emits) ||
                (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes))
                ext = "ConsumerStub";
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes)
                ext = "ConsumerSkeleton";
            else if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_uses)
                ext = "Stub";
            else
                ext = "Skeleton";

            type = (org.objectweb.ccm.IDL3.Declaration)getType(objs[i]);
            res = "if (uid.equals(\""+objs[i].getId()+
                "\"))\\n"+
                getIndentation()+"return new "+getTranslator().getAbsoluteName(type)+ext+"Interceptor();\\n";

            super.printImmediately(res);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    portUids(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        java.lang.String res = "";
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_emits +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_uses);

        for (int i=0;i<objs.length;i++)
            res = res+"\""+objs[i].getId()+"\", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    portTypeUid(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_provides +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_emits +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_publishes +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_uses);

        java.lang.String res = "";
        org.objectweb.ccm.IDL3.Declaration type = null;
        for (int i=0;i<objs.length;i++)
        {
            type = (org.objectweb.ccm.IDL3.Declaration)getType(objs[i]);

            res = "if (port_uid.equals(\""+objs[i].getId()+
                "\"))\\n"+
                getIndentation()+"return \"";

            java.lang.String type_id = type.getId();
            if ((objs[i].getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_provides) &&
                (objs[i].getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_uses))
            {
                org.objectweb.ccm.IDL3.EventDecl event = (org.objectweb.ccm.IDL3.EventDecl)type;
                org.objectweb.ccm.IDL3.InterfaceDecl itf = event.getClientMapping();
                type_id = itf.getId();
            }

            res = res+type_id+"\";\\n";

            super.printImmediately(res);
        }

        return "";
    }

    /**
     **  Note: only generated for events.
     **/
    public java.lang.String
    executorWrappers(org.objectweb.ccm.IDL3.ComponentDecl comp)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = comp.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes);

        java.lang.String[] aliases = { "obj", "pack_name", "port_id" };
        for (int i=0;i<objs.length;i++)
        {
            org.objectweb.ccm.IDL3.ConsumesDecl cons = (org.objectweb.ccm.IDL3.ConsumesDecl)objs[i];
            org.objectweb.ccm.IDL3.EventDecl event = (org.objectweb.ccm.IDL3.EventDecl)cons.getType();

            java.lang.Object[] objs2 = { event, javaPackage(event),
                                         cons.getId() };
            super.map("COMPONENT_EVENT_WRAPPER", aliases, objs2);
        }

        return "";
    }

    // ==================================================================
    //
    // Public methods (event related).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    eventConsumerOpWrapper(org.objectweb.ccm.IDL3.EventDecl event)
    {
        org.objectweb.ccm.IDL3.InterfaceDecl itf = null;
        org.objectweb.ccm.IDL3.Declaration[] ops = null;
        itf = event.getClientMapping();
        ops = itf.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation);

        for (int i=0;i<ops.length;i++)
        {
            if (!ops[i].getName().equals("push_event"))
                visitContainedWithMapping(event, ops[i], "_EVENT_CONSUMER_WRAPPER");
        }

        return "";
    }

    // ==================================================================
    //
    // Public methods (interceptor related).
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    delegateOperations(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation +
                                        org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute);

        for (int i=0;i<objs.length;i++)
        {
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute)
            {
                org.objectweb.ccm.IDL3.AttributeDecl attr = (org.objectweb.ccm.IDL3.AttributeDecl)objs[i];
                if (attr.isReadonly())
                    visitContained(attr, "_READONLY_DELEGATE");
                else
                    visitContained(attr, "_DELEGATE");
            }
            else
                visitContained(objs[i], "_DELEGATE");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    interceptorDelegateWrapper(org.objectweb.ccm.IDL3.Declaration obj,
                               java.lang.String name)
    {
        if (obj.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_consumes)
            return name;

        return "new "+name(obj)+"EventWrapper("+name+")";
    }

    /**
     **
     **/
    public java.lang.String
    interceptorIState(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        java.lang.String str = "";
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute);

        for (int i=0;i<objs.length;i++)
        {
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute)
            {
                str = "private org.objectweb.ccm.Containers.CallCoordinator _cc_get_"+
                    objs[i].getName()+" = null;\\n\\n";

                super.printImmediately(str);

                org.objectweb.ccm.IDL3.AttributeDecl attr = (org.objectweb.ccm.IDL3.AttributeDecl)objs[i];
                if (!attr.isReadonly())
                {
                    str = "private org.objectweb.ccm.Containers.CallCoordinator _cc_set_"+
                        objs[i].getName()+" = null;\\n\\n";
                    super.printImmediately(str);
                }
            }
            else
            {
                str = "private org.objectweb.ccm.Containers.CallCoordinator _cc_"+
                    objs[i].getName()+" = null;\\n\\n";
                super.printImmediately(str);
            }

        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    interceptorOperations(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] objs = null;
        objs = scope.getContents(false, org.objectweb.ccm.IDL3.DeclarationKind._dk_operation +
                                 org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute);
        for (int i=0;i<objs.length;i++)
        {
            if (objs[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_attribute)
            {
                org.objectweb.ccm.IDL3.AttributeDecl attr = (org.objectweb.ccm.IDL3.AttributeDecl)objs[i];
                if (attr.isReadonly())
                    visitContained(attr, "_READONLY_INTERCEPTOR");
                else
                    visitContained(attr, "_INTERCEPTOR");
            }
            else 
                visitContained(objs[i], "_INTERCEPTOR");
        }

        return "";
    }
}
