/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.xmi;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class XMIPrinter
       extends Printer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected XMITranslator translator_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Table stereotypes_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Table types_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector external_refs_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Table generalizations_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Table dependencies_tree_;

    /**
     **
     **/
    protected long id_counter_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    XMIPrinter(Generator gen,
               java.lang.String indent)
    throws fr.lifl.cim.jidlscript.lang.JISError
    {
        super(gen, '$', '#', indent);

        // init internal state.
        translator_ = new XMITranslator();
        id_counter_ = java.lang.System.currentTimeMillis();
        initStereotypes();
        initTypes();
        external_refs_ = new org.objectweb.ccm.util.Vector();
        dependencies_tree_ = new org.objectweb.ccm.util.Table();
        generalizations_ = new org.objectweb.ccm.util.Table();


        // set the "self" alias in root scope on this object.
        super.setAlias("self", this);

        super.loadFile("xmi.template");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected void
    initStereotypes()
    {
        stereotypes_ = new org.objectweb.ccm.util.Table();
        stereotypes_.put("CORBAModule", "Package");
        stereotypes_.put("CORBAInterface", "Class");
        stereotypes_.put("CORBAValue", "Class");
        stereotypes_.put("CORBATypedef", "Class");
        stereotypes_.put("CORBABoxedValue", "Class");
        stereotypes_.put("CORBAConstant", "Attribute");
        stereotypes_.put("CORBAConstants", "Class");      
        stereotypes_.put("CORBAStruct", "Class");
        stereotypes_.put("CORBAUnion", "Class");
        stereotypes_.put("CORBAEnum", "Class");
        stereotypes_.put("CORBAException", "Class");
        stereotypes_.put("CORBASequence", "Class");
        stereotypes_.put("CORBAArray", "Class");
        stereotypes_.put("CORBAAlias", "Class");
        stereotypes_.put("CORBAstring", "DataType");
        stereotypes_.put("CORBAwstring", "DataType");
        stereotypes_.put("CORBAshort", "DataType");
        stereotypes_.put("CORBAlong", "DataType");
        stereotypes_.put("CORBAunsignedshort", "DataType");
        stereotypes_.put("CORBAunsignedlong", "DataType");
        stereotypes_.put("CORBAfloat", "DataType");
        stereotypes_.put("CORBAdouble", "DataType");
        stereotypes_.put("CORBAboolean", "DataType");
        stereotypes_.put("CORBAchar", "DataType");
        stereotypes_.put("CORBAwchar", "DataType");
        stereotypes_.put("CORBAoctet", "DataType");
        stereotypes_.put("CORBAany", "DataType");
        stereotypes_.put("null", "DataType");
        stereotypes_.put("void", "DataType");
        stereotypes_.put("CORBAlonglong", "DataType");
        stereotypes_.put("CORBAlongdouble", "DataType");

        //
        //  TODO : review the name and type
        //         change also in XMITranslator
        //
        stereotypes_.put("CORBATypeCode", "DataType");
        stereotypes_.put("CORBAPrincipal", "DataType");
        stereotypes_.put("CORBAObject", "DataType");
        stereotypes_.put("CORBAfixed", "DataType");
    }

    /**
     **
     **/
    protected void
    initTypes()
    {
        types_ = new org.objectweb.ccm.util.Table();
        java.util.Enumeration keys = stereotypes_.keys();
        while (keys.hasMoreElements())
        {
            types_.put(keys.nextElement(), newId());
        }
    }

    /**
     **
     **/
    protected java.lang.String
    getExternalId(org.objectweb.ccm.IDL3.Declaration decl)
    {
        //        System.err.println("$$$ Adding External ref : "+decl.getId());

        external_refs_.add(decl);
        java.lang.String id = newId(decl);

        org.objectweb.ccm.IDL3.Declaration[] depend = decl.getDependencies();
        for (int i=0;i<depend.length;i++)
        {
            //            System.err.println("    Dependency is : "+depend[i].getId());
            if (!types_.containsKey(depend[i]))
                getExternalId(depend[i]);
        }

        return id;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    modifyCurrentLine(java.lang.String str)
    {
        int l = str.length();

        if (l==0)
            return "";

        if ((l>2) &&
            (str.charAt(0)=='<') &&
            ((str.charAt(1)=='!') || (str.charAt(1)=='?')))
            return str+"\\n";

        if ((l>2) &&
            (str.charAt(l-1)=='>') &&
            (str.charAt(l-2)!='/') &&
            (str.indexOf("</")==-1))
            return str+"\\n$self.inc()$";

        if ((l>2) &&
            (str.charAt(0)=='<') &&
            (str.charAt(1)=='/'))
            return "$self.dec()$"+str+"\\n";

        if ((str.charAt(0)!=getCallChar()) ||
            (str.charAt(l-1)!=getCallChar()))
            return str+"\\n";

        return str;
    }

    /**
     **
     **/
    public XMITranslator
    getTranslator()
    {
        return translator_;
    }

    /**
     **
     **/
    public java.lang.String
    visitContained(org.objectweb.ccm.IDL3.Declaration contained,
                   java.lang.String extension)
    {
        java.lang.String id =
            org.objectweb.ccm.IDL3.DeclarationKind.toString(contained.getDeclKind()).toUpperCase()+
            extension;

        java.lang.String[] aliases = { "obj" };
        java.lang.Object[] objs = { contained };
        super.map(id, aliases, objs);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    visitModule(org.objectweb.ccm.IDL3.Declaration decl)
    {
        if (decl.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_module)
        {
            java.lang.String msg =
                "error in class org.objectweb.ccm.visitorIDL3.XMIPrinter\n"+
                "   operation : visitModule\n"+
                "   message : root visited object must be a module";
            throw new Error(msg);
        }

        java.lang.String[] aliases = { "obj" };
        java.lang.Object[] objs = { decl };
        super.map("MODULE", aliases, objs);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    newId()
    {
        id_counter_++;
        return "xmi."+java.lang.Long.toString(id_counter_);
    }

    /**
     **
     **/
    public java.lang.String
    newId(org.objectweb.ccm.IDL3.Declaration decl)
    {
        java.lang.String res = (java.lang.String)types_.get(decl.getId());
        if (res!=null)
            return res;

        res = newId();
        types_.put(decl.getId(), res);
        return res;
    }

    /**
     **
     **/
    public java.lang.String
    visit(org.objectweb.ccm.util.Vector vect)
    {
        org.objectweb.ccm.IDL3.Declaration[] decls = (org.objectweb.ccm.IDL3.Declaration[])
            vect.toArray(new org.objectweb.ccm.IDL3.Declaration[0]);

        for (int i=0;i<decls.length;i++)
        {
            //            System.err.println("!!! Visiting tree element : "+decls[i].getId());
            if (decls[i].getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_module)
            {
                java.lang.String[] aliases = { "obj", "vect" };
                java.lang.Object[] objs = { decls[i], dependencies_tree_.get(decls[i]) };
                super.map("MODULE_UNCOMPLETE", aliases, objs);
            }
            else
                visitContained(decls[i], "");
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    externalTypes()
    {
        // build dependencies tree
        org.objectweb.ccm.IDL3.Declaration[] depend = null;
        org.objectweb.ccm.IDL3.Scope parent = null;
        org.objectweb.ccm.util.Vector childs = null;
        org.objectweb.ccm.IDL3.Declaration obj1 = null;
        org.objectweb.ccm.util.Vector root_dependencies = new org.objectweb.ccm.util.Vector();

        depend = (org.objectweb.ccm.IDL3.Declaration[])external_refs_.toArray(
            new org.objectweb.ccm.IDL3.Declaration[0]);

        for (int i=0;i<depend.length;i++)
        {
            //            System.err.println("### External ref : "+depend[i].getId());
            obj1  = depend[i];
            parent = obj1.getParent();

            while (parent.getDeclKind()!=org.objectweb.ccm.IDL3.DeclarationKind._dk_repository)
            {
                if (dependencies_tree_.containsKey(parent))
                {
                    childs = (org.objectweb.ccm.util.Vector)dependencies_tree_.get(parent);
                    if (childs.indexOf(obj1)==-1)
                        childs.add(obj1);

                    // continue;
                }
                else
                {
                    childs = new org.objectweb.ccm.util.Vector();
                    childs.add(obj1);
                    dependencies_tree_.put(parent, childs);
                }

                obj1 = parent;
                parent = obj1.getParent();
            }

            if (root_dependencies.indexOf(obj1)==-1)
                root_dependencies.add(obj1);
        }

        visit(root_dependencies);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    generalizations()
    {
        java.util.Enumeration keys = generalizations_.keys();
        java.util.Enumeration values = generalizations_.elements();
        java.lang.String[] aliases = { "gen_id", "child", "parent" };
        java.lang.Object[] objs = new java.lang.Object[3];

        while (keys.hasMoreElements())
        {
            objs[1] = getXMIId((java.lang.String)keys.nextElement());
            XMIIdCouple[] elems = (XMIIdCouple[])values.nextElement();

            for (int i=0;i<elems.length;i++)
            {
                objs[0] = elems[i].gen_id;
                objs[2] = elems[i].parent_id;
                super.map("GENERALIZATION", aliases, objs);
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    corbaStereotypes()
    {
        java.util.Enumeration keys = stereotypes_.keys();
        java.util.Enumeration values = stereotypes_.elements();

        java.lang.String[] aliases = { "name", "base_class" };
        java.lang.Object[] objs = new java.lang.Object[2];
        while (keys.hasMoreElements())
        {
            objs[0] = keys.nextElement();
            objs[1] = values.nextElement();
            super.map("CORBA_STEREOTYPE", aliases, objs);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    getScopeId(org.objectweb.ccm.IDL3.Declaration decl)
    {
        java.lang.String id = "";
        if (decl.getParent().getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_module)
            id = "MODULE_SCOPE";
        else if (decl.getParent().getDeclKind()==org.objectweb.ccm.IDL3.DeclarationKind._dk_repository)
            id = "ROOT_SCOPE";
        else
            id = "NOT_MODULE_SCOPE";

        java.lang.String[] aliases = { "scope_id" };
        java.lang.Object[] objs = { getXMIId(decl.getParent().getId()) };
        super.map(id, aliases, objs);

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    getXMIId(java.lang.String stereo_name)
    {
        java.lang.String res = (java.lang.String)types_.get(stereo_name);
        if (res==null)
            res = "";

        return res;
    }

    /**
     **
     **/
    public java.lang.String
    getXMIId(org.objectweb.ccm.IDL3.TypeRef type)
    {
        if (type instanceof org.objectweb.ccm.IDL3.Declaration)
        {
            org.objectweb.ccm.IDL3.Declaration decl =
                (org.objectweb.ccm.IDL3.Declaration)type;

            java.lang.String res = getXMIId(decl.getId());
            if (res.equals(""))
                res = getExternalId(decl);

            return res;
        }

        return getXMIId(getTranslator().toXMIString(type));
    }

    /**
     **
     **/
    public java.lang.String
    contents(org.objectweb.ccm.IDL3.Scope scope)
    {
        org.objectweb.ccm.IDL3.Declaration[] decls = null;
        decls = scope.getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);

        for (int i=0;i<decls.length;i++)
            visitContained(decls[i], "");

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    baseInterfaces(org.objectweb.ccm.IDL3.InterfaceDecl itf)
    {
        org.objectweb.ccm.IDL3.InterfaceDecl[] bases = null;
        bases = itf.getBaseInterfaces();

        // generalizable element
        java.lang.String gen_id = "";
        org.objectweb.ccm.util.Vector gen_bases = new org.objectweb.ccm.util.Vector();

        java.lang.String[] aliases = {"gen_id"};
        java.lang.Object[] objs = new java.lang.Object[1];

        for (int i=0;i<bases.length;i++)
        {
            gen_id = newId();
            gen_bases.add(new XMIIdCouple(gen_id, getXMIId((org.objectweb.ccm.IDL3.TypeRef)bases[i])));
            objs[0] = gen_id;
            super.map("GENERALIZABLE_ELEMENT", aliases, objs);
        }

        generalizations_.put(itf.getId(),
                             (XMIIdCouple[])gen_bases.toArray(new XMIIdCouple[0]));

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    toStringMode(int mode)
    {
        if (mode==org.objectweb.ccm.IDL3.OperationDecl.PARAM_IN)
            return "in";

        if (mode==org.objectweb.ccm.IDL3.OperationDecl.PARAM_OUT)
            return "out";

        return "inout";
    }

    /**
     **
     **/
    public java.lang.String
    operationParams(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        org.objectweb.ccm.IDL3.ParameterList params = op.getParameters();
        org.objectweb.ccm.IDL3.TypeRef[] types = params.getParamTypes();
        java.lang.String[] names = params.getParamNames();
        int[] modes = params.getParamModes();

        java.lang.String[] aliases = {"name", "kind", "operation", "type"};
        java.lang.Object[] objs = new java.lang.Object[4];
        objs[2] = getXMIId(op.getId());

        for (int i=0;i<names.length;i++)
        {
            objs[0] = names[i];
            objs[1] = toStringMode(modes[i]);
            objs[3] = getXMIId(types[i]);

            super.map("OPERATION_PARAMETER", aliases, objs);
        }
        
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    operationExceptions(org.objectweb.ccm.IDL3.OperationDecl op)
    {
        org.objectweb.ccm.IDL3.ExceptionDecl[] excs = op.getExceptions();

        java.lang.String[] aliases = {"exception"};
        java.lang.Object[] objs = new java.lang.Object[1];

        for (int i=0;i<excs.length;i++)
        {
            //
            //  TODO : really that value ????
            //
            objs[0] = excs[i].getAbsoluteName().substring(2);
            super.map("OPERATION_EXCEPTION", aliases, objs);
        }
        
        return "";
    }

    /**
     **
     **/
    public java.lang.String
    structMembers(java.lang.String par_id,
                  org.objectweb.ccm.IDL3.StructMembers members)
    {
        org.objectweb.ccm.IDL3.TypeRef[] types = members.getMemberTypes();
        java.lang.String[] names = members.getMemberNames();

        java.lang.String[] aliases = {"name", "lower", "upper", "class_id", "type"};
        java.lang.Object[] objs = new java.lang.Object[5];
        objs[1] = "1";
        objs[2] = "1";
        objs[3] = getXMIId(par_id);

        for (int i=0;i<names.length;i++)
        {
            objs[0] = names[i];
            objs[4] = getXMIId(types[i]);
            super.map("MEMBER", aliases, objs);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    enumMembers(org.objectweb.ccm.IDL3.EnumDecl enume)
    {
        java.lang.String[] names = enume.getMembers();
        java.lang.String[] aliases = {"name", "lower", "upper", "class_id", "type"};
        java.lang.Object[] objs = new java.lang.Object[5];
        objs[1] = "1";
        objs[2] = "1";
        objs[3] = getXMIId(enume.getId());
        objs[4] = "";

        for (int i=0;i<names.length;i++)
        {
            objs[0] = names[i];
            super.map("MEMBER", aliases, objs);
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    aliasOriginal(org.objectweb.ccm.IDL3.AliasDecl alias)
    {
        // generalizable element
        java.lang.String gen_id = newId();
        XMIIdCouple[] content = { new XMIIdCouple(gen_id, getXMIId(alias.getType())) };
        generalizations_.put(alias.getId(), content);

        java.lang.String[] aliases = {"gen_id"};
        java.lang.Object[] objs = { gen_id };
        super.map("GENERALIZABLE_ELEMENT", aliases, objs);

        // content type
        org.objectweb.ccm.IDL3.TypeRef orig = alias.getType();
        java.lang.String[] aliases2 = {"tag", "value"};
        java.lang.Object[] objs2 = new java.lang.Object[2];

        if (orig.getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_sequence)
        {
            objs2[0] = "sequence";
            objs2[1] = java.lang.Integer.toString(((org.objectweb.ccm.IDL3.TypeRefImpl)orig).getLength());
            super.map("ALIAS_ORIGINAL", aliases2, objs2);
        }
        else if (orig.getTypeKind()==org.objectweb.ccm.IDL3.TypeKind._tk_string)
        {
            int l = ((org.objectweb.ccm.IDL3.TypeRefImpl)orig).getLength();
            if (l!=0)
            {
                objs2[0] = "length";
                objs2[1] = java.lang.Integer.toString(l);
                super.map("ALIAS_ORIGINAL", aliases2, objs2);
            }
        }

        return "";
    }

    /**
     **
     **/
    public java.lang.String
    a()
    {
        return "";
    }
}

/**
 **
 **/
class XMIIdCouple
{
    public java.lang.String gen_id;
    public java.lang.String parent_id;

    public 
    XMIIdCouple(java.lang.String gen_id,
                java.lang.String parent_id)
    {
        this.gen_id = gen_id;
        this.parent_id = parent_id;
    }
}
