/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Tran Huynh.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.xmi;

import org.objectweb.ccm.visitorIDL3.common.*;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class XMITranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    XMITranslator()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public java.lang.String
    toXMI(org.objectweb.ccm.IDL3.TypeRef typeref)
    {
        switch(typeref.getTypeKind())
        {
            case org.objectweb.ccm.IDL3.TypeKind._tk_null :
                return "null";
            case org.objectweb.ccm.IDL3.TypeKind._tk_void :
                return "void";
            case org.objectweb.ccm.IDL3.TypeKind._tk_short :
                return "CORBAshort";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ushort :
                return "CORBAunsignedshort";
            case org.objectweb.ccm.IDL3.TypeKind._tk_long :
                return "CORBAlong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulong :
                return "CORBAunsignedlong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longlong :
                return "CORBAlonglong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulonglong :
                return "CORBAunsignedlonglong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_float :
                return "CORBAfloat";
            case org.objectweb.ccm.IDL3.TypeKind._tk_double :
                return "CORBAdouble";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longdouble :
                return "CORBAlongdouble";
            case org.objectweb.ccm.IDL3.TypeKind._tk_boolean :
                return "CORBAboolean";
            case org.objectweb.ccm.IDL3.TypeKind._tk_char :
                return "CORBAchar";
            case org.objectweb.ccm.IDL3.TypeKind._tk_wchar :
                return "CORBAwchar";
            case org.objectweb.ccm.IDL3.TypeKind._tk_octet :
                return "CORBAoctet";
            case org.objectweb.ccm.IDL3.TypeKind._tk_any :
                return "CORBAany";
            case org.objectweb.ccm.IDL3.TypeKind._tk_Principal :
                return "CORBAPrincipal";
            case org.objectweb.ccm.IDL3.TypeKind._tk_TypeCode :
                return "CORBATypeCode";
            case org.objectweb.ccm.IDL3.TypeKind._tk_objref :
                return "Object";

            case org.objectweb.ccm.IDL3.TypeKind._tk_alias :
            case org.objectweb.ccm.IDL3.TypeKind._tk_struct :
            case org.objectweb.ccm.IDL3.TypeKind._tk_union :
            case org.objectweb.ccm.IDL3.TypeKind._tk_enum :
            case org.objectweb.ccm.IDL3.TypeKind._tk_exception :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value_box :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value :
            case org.objectweb.ccm.IDL3.TypeKind._tk_native :
            case org.objectweb.ccm.IDL3.TypeKind._tk_abstract_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_local_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_component :
            case org.objectweb.ccm.IDL3.TypeKind._tk_home :
            case org.objectweb.ccm.IDL3.TypeKind._tk_event :
                org.objectweb.ccm.IDL3.Declaration decl = (org.objectweb.ccm.IDL3.Declaration)typeref;
                return getAbsoluteName(decl);

            case org.objectweb.ccm.IDL3.TypeKind._tk_sequence :
                org.objectweb.ccm.IDL3.TypeRefImpl impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                java.lang.String str = toXMI(impl.getContentType());
                int length = impl.getLength();
                if (length==0) 
                    return "CORBAsequence<"+str+">";
                return "CORBAsequence<"+str+", "+length+">";

            case org.objectweb.ccm.IDL3.TypeKind._tk_array :
                impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                str = toXMI(impl.getContentType());
                return "CORBAarray<"+str+", "+impl.getLength()+">";

            case org.objectweb.ccm.IDL3.TypeKind._tk_string :
                impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                length = impl.getLength();
                if (length==0)
                    return "CORBAstring";
                return "CORBAstring<"+impl.getLength()+">";
    
            case org.objectweb.ccm.IDL3.TypeKind._tk_wstring :
                impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                length = impl.getLength();
                if (length==0)
                    return "CORBAwstring";
                return "CORBAwstring<"+impl.getLength()+">";
    
            case org.objectweb.ccm.IDL3.TypeKind._tk_fixed :
                impl= (org.objectweb.ccm.IDL3.TypeRefImpl)typeref;
                return "CORBAfixed<"+impl.getDigits()+", "+impl.getScale()+">";

            default :
                return "";
        }
    }

    /**
     **
     **/
    public java.lang.String
    toXMIString(org.objectweb.ccm.IDL3.TypeRef typeref)
    {
        switch(typeref.getTypeKind())
        {
            case org.objectweb.ccm.IDL3.TypeKind._tk_null :
                return "null";
            case org.objectweb.ccm.IDL3.TypeKind._tk_void :
                return "void";
            case org.objectweb.ccm.IDL3.TypeKind._tk_short :
                return "CORBAshort";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ushort :
                return "CORBAunsignedshort";
            case org.objectweb.ccm.IDL3.TypeKind._tk_long :
                return "CORBAlong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulong :
                return "CORBAunsignedlong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longlong :
                return "CORBAlonglong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_ulonglong :
                return "CORBAunsignedlonglong";
            case org.objectweb.ccm.IDL3.TypeKind._tk_float :
                return "CORBAfloat";
            case org.objectweb.ccm.IDL3.TypeKind._tk_double :
                return "CORBAdouble";
            case org.objectweb.ccm.IDL3.TypeKind._tk_longdouble :
                return "CORBAlongdouble";
            case org.objectweb.ccm.IDL3.TypeKind._tk_boolean :
                return "CORBAboolean";
            case org.objectweb.ccm.IDL3.TypeKind._tk_char :
                return "CORBAchar";
            case org.objectweb.ccm.IDL3.TypeKind._tk_wchar :
                return "CORBAwchar";
            case org.objectweb.ccm.IDL3.TypeKind._tk_octet :
                return "CORBAoctet";
            case org.objectweb.ccm.IDL3.TypeKind._tk_any :
                return "CORBAany";
            case org.objectweb.ccm.IDL3.TypeKind._tk_Principal :
                return "CORBAPrincipal";
            case org.objectweb.ccm.IDL3.TypeKind._tk_TypeCode :
                return "CORBATypeCode";
            case org.objectweb.ccm.IDL3.TypeKind._tk_objref :
                return "Object";
            case org.objectweb.ccm.IDL3.TypeKind._tk_string :
                return "CORBAstring";
            case org.objectweb.ccm.IDL3.TypeKind._tk_wstring :
                return "CORBAwstring";
            case org.objectweb.ccm.IDL3.TypeKind._tk_fixed :
                return "CORBAfixed";
            case org.objectweb.ccm.IDL3.TypeKind._tk_sequence :
                return "CORBASequence";
            case org.objectweb.ccm.IDL3.TypeKind._tk_array :
                return "CORBAArray";

            case org.objectweb.ccm.IDL3.TypeKind._tk_alias :
            case org.objectweb.ccm.IDL3.TypeKind._tk_struct :
            case org.objectweb.ccm.IDL3.TypeKind._tk_union :
            case org.objectweb.ccm.IDL3.TypeKind._tk_enum :
            case org.objectweb.ccm.IDL3.TypeKind._tk_exception :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value_box :
            case org.objectweb.ccm.IDL3.TypeKind._tk_value :
            case org.objectweb.ccm.IDL3.TypeKind._tk_native :
            case org.objectweb.ccm.IDL3.TypeKind._tk_abstract_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_local_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_interface :
            case org.objectweb.ccm.IDL3.TypeKind._tk_component :
            case org.objectweb.ccm.IDL3.TypeKind._tk_home :
            case org.objectweb.ccm.IDL3.TypeKind._tk_event :
                org.objectweb.ccm.IDL3.Declaration decl = (org.objectweb.ccm.IDL3.Declaration)typeref;
                return getAbsoluteName(decl);

            default :
                return "";
        }
    }

    /**
     **
     **/
    public java.lang.String
    getAbsoluteName(org.objectweb.ccm.IDL3.Declaration decl)
    {
        return decl.getAbsoluteName();
    }
}
