/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/**
 * AbstractStorageHomeDeclImpl is a wrapper class for
 * PSDL abstract storagehome declarations.
 *
 *
 * Inherits from:
 *
 * - StorageHomeBaseImpl as abstract storagehomes are also IDL forward scopes.
 *
 *
 * Provides:
 *
 * - The managed abstract storagetype,
 *   i.e. the setAbstractStorageType and getAbstractStorageType operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class AbstractStorageHomeDeclImpl
     extends StorageHomeBaseImpl
  implements org.objectweb.corba.ast.api.AbstractStorageHomeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The managed abstract storagetype. */
    protected AbstractStorageTypeDeclImpl managedAbstractStorageType_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    AbstractStorageHomeDeclImpl(Repository rep,
                                ScopeImpl parent)
    {
        // Call the StorageHomeBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        managedAbstractStorageType_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_abstract_storage_home;
    }

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeIDLImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for StorageHomeBaseImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Check if the declaration is valid.
     */
    public void
    check()
    {
        // Call the StorageHomeBase "check" method
        super.check(managedAbstractStorageType_);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageHomeBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AbstractStorageHomeDecl
    //
    // ==================================================================

    /**
     * Set the managed abstract storagetype.
     *
     * @param mast The managed abstract storagetype.
     */
    public void
    setAbstractStorageType(AbstractStorageTypeDecl mast)
    {
        if(mast != null) {
            managedAbstractStorageType_ = (AbstractStorageTypeDeclImpl)mast;
	}
    }

    /**
     * Obtain the managed abstract storagetype.
     *
     * @return The managed abstract storagetype.
     */
    public AbstractStorageTypeDecl
    getAbstractStorageType()
    {
        return managedAbstractStorageType_;
    }
}
