/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/**
 * AbstractStorageTypeDeclImpl is a wrapper class for
 * PSDL abstract storagetype declarations.
 *
 *
 * Inherits from:
 *
 * - StorageTypeBaseImpl as abstract storagetypes are also IDL forward scopes.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class AbstractStorageTypeDeclImpl
     extends StorageTypeBaseImpl
  implements org.objectweb.corba.ast.api.AbstractStorageTypeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    AbstractStorageTypeDeclImpl(Repository rep,
                                ScopeImpl parent)
    {
        // Call the StorageTypeBaseImpl constructor.
        super(rep, parent);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_abstract_storage_type;
    }

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeIDLImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for StorageTypeBaseImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AbstractStorageTypeDecl
    //
    // ==================================================================
}
