/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST InterfaceList. */
import org.objectweb.corba.ast.api.InterfaceList;

/** To use CORBA::InterfaceDef. */
import org.omg.CORBA.InterfaceDef;

/**
 * ComponentBaseImpl is a wrapper base class for
 * IDL component and home declarations.
 *
 *
 * Inherits from:
 *
 * - InterfaceDeclImpl as components are also interfaces.
 *
 * - ComponentBase as OMG IDL for component and home declarations.
 *
 *
 * Provides:
 *
 * - The list of supported interfaces,
 *   i.e. the getSupportedInterfaceList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class ComponentBaseImpl
              extends InterfaceDeclImpl
           implements org.objectweb.corba.ast.api.ComponentBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The supported interfaces.  */
    protected InterfaceListImpl supported_interfaces_;

    /** The client interface mapping. */
    protected InterfaceDeclImpl client_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    ComponentBaseImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the InterfaceDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        supported_interfaces_ = new InterfaceListImpl();
        client_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add an array of supported interfaces.
     *
     * @param supported_interfaces  The supported interfaces to add.
     */
    protected void
    addSupportedInterfaces(InterfaceDef[] supported_interfaces)
    {
        for (int i=0; i<supported_interfaces.length; i++)
   	{
            // WARNING: This will break consistency of array items!!!
            supported_interfaces_.add(
                (InterfaceDecl)getRepository().
                               lookupId(supported_interfaces[i].id()));
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.corba.ast.api.ComponentBase
    //
    // ==================================================================

    /**
     * Obtain the list of supported interfaces.
     *
     * @param itf The list of supported interfaces.
     */
    public InterfaceList
    getSupportedInterfaceList()
    {
        return supported_interfaces_;
    }
}
