/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST CategoryKind. */
import org.objectweb.corba.ast.api.CategoryKind;

/** To access AST HomeExecutorDecl. */
import org.objectweb.corba.ast.api.HomeExecutorDecl;

/** To access AST ProxyHomeDecl. */
import org.objectweb.corba.ast.api.ProxyHomeDecl;

/**
 * CompositionDeclImpl is a wrapper class for CIDL compsition declarations.
 *
 *
 * Inherits from:
 * - ScopeImpl as executors are also IDL scopes.
 * - CompositionDecl as OMG IDL for CIDL composition declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class CompositionDeclImpl
     extends ScopeImpl
  implements org.objectweb.corba.ast.api.CompositionDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated composition category. */
    private CategoryKind category_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    CompositionDeclImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        category_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null; 
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_composition;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
      // TODO check if the composition declaration is correct
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.CompositionDecl
    //
    // ==================================================================

    /**
     * Set the associated composition category.
     *
     * @param ash The associated composition category.
     */
    public void
    setCategoryKind(CategoryKind ck)
    {
        category_ = ck;
    }

    /**
     * Obtain the associated composition category.
     *
     * @return The associated composition category.
     */
    public CategoryKind
    getCategoryKind()
    {
        return category_;
    }

 
    /**
     * Start a new CIDL home executor declaration.
     *
     * @param name The name of the CIDL home executor declaration.
     *
     * @return The new created HomeExecutorDecl.
     */
    public HomeExecutorDecl
    startHomeExecutor(String name)
    {
        HomeExecutorDecl decl = new HomeExecutorDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new CIDL proxy home declaration.
     *
     * @param name The name of the CIDL proxy home declaration.
     *
     * @return The new created ProxyHomeDecl.
     */
    public ProxyHomeDecl
    startProxyHome(String name)
    {
        ProxyHomeDecl decl = new ProxyHomeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }
}
