/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST AnyValue. */
import org.objectweb.corba.ast.api.AnyValue;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/**
 * DeclarationWithAnyValueImpl is a wrapper class for
 * Declaration with an associated any value.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - DeclarationWithAnyValue: OMG IDL for Declaration with
 *                            an associated any value.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class DeclarationWithAnyValueImpl
       extends DeclarationImpl
       implements org.objectweb.corba.ast.api.DeclarationWithAnyValue
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The value of the constant. */
    private AnyValue value_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    DeclarationWithAnyValueImpl(Repository rep,
                                ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // init internal state.
        value_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.DeclarationWithAnyValue
    //
    // ==================================================================

    /**
     * Set the associated AnyValue.
     *
     * @param value The AnyValue to set.
     */
    public void
    setAnyValue(AnyValue value)
    {
        value_ = value;
    }

    /**
     * Obtain the associated AnyValue.
     *
     * @return The associated AnyValue.
     */
    public AnyValue
    getAnyValue()
    {
        return value_;
    }
}
