/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST PsdlTypeRef. */
import org.objectweb.corba.ast.api.PsdlTypeRef;

/**
 * DeclarationWithPsdlTypeRefImpl is a wrapper class for
 * Declaration with an associated psdl type.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - DeclarationWithPsdlTypeRef: OMG IDL for Declaration with an associated psdl type.
 *
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

abstract public
          class DeclarationWithPsdlTypeRefImpl
        extends DeclarationImpl
     implements org.objectweb.corba.ast.api.DeclarationWithPsdlTypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated PsdlTypeRef. */
    protected PsdlTypeRef type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    DeclarationWithPsdlTypeRefImpl(Repository rep,
                               ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        type_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationWithTypeRefImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.corba.ast.api.DeclarationWithPsdlTypeRef
    //
    // ==================================================================

    /**
     * Set the associated TypeRef reference.
     *
     * @param type The associated TypeRef reference.
     */
    public void
    setType(PsdlTypeRef type)
    {
       type_ = type;
    }

    /**
     * Obtain the associated TypeRef reference.
     *
     * @return The associated TypeRef reference.
     */
    public PsdlTypeRef
    getType()
    {
        return type_;
    }
}
