/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/**
 * DeclarationWithTypeRefImpl is a wrapper class for
 * Declaration with an associated type.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - DeclarationWithTypeRef: OMG IDL for Declaration with an associated type.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public
class DeclarationWithTypeRefImpl
extends DeclarationImpl
implements org.objectweb.corba.ast.api.DeclarationWithTypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated TypeRef. */
    protected TypeRef type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    DeclarationWithTypeRefImpl(Repository rep,
                               ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        type_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtain the associated CORBA::IDLType reference.
     *
     * @return The associated CORBA::IDLType reference.
     */
    protected org.omg.CORBA.IDLType
    getIDLType()
    {
        if(type_ == null)
            return null;

        return ((IDLTypeWrapper)type_).getIDLType();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.DeclarationWithTypeRef
    //
    // ==================================================================

    /**
     * Set the associated TypeRef reference.
     *
     * @param type The associated TypeRef reference.
     */
    public void
    setType(TypeRef type)
    {
       type_ = type;
    }

    /**
     * Obtain the associated TypeRef reference.
     *
     * @return The associated TypeRef reference.
     */
    public TypeRef
    getType()
    {
        return type_;
    }
}
