/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * DelegationImpl is a wrapper class for CIDL Delegation.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class DelegationImpl
     extends ObjectBase
  implements org.objectweb.corba.ast.api.Delegation
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated source name. */
    protected String source_;

    /** The associated target name. */
    protected String target_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    DelegationImpl()
    {
        // Call the second constructor.
        this(null, null);
    }

    /**
     * The constructor.
     *
     * @param source The associated source name.
     * @param target The associated target name.
     */
    public
    DelegationImpl(String source,
                   String target)
    {
        // Call the default ObjectBase constructor.
        super();

        // Init internal state.
        source_ = source;
        target_ = target;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Delegation
    //
    // ==================================================================

    /**
     * Set the associated source name.
     *
     * @param name The associated source name.
     */
    public void
    setSource(String name)
    {
        source_ = name;
    }

    /**
     * Obtain the associated source name.
     *
     * @return The associated source name.
     */
    public String
    getSource()
    {
        return source_;
    }

    /**
     * Set the associated target name.
     *
     * @param name The associated target name.
     */
    public void
    setTarget(String name)
    {
        target_ = name;
    }

    /**
     * Obtain the associated target name.
     *
     * @return The associated target name.
     */
    public String
    getTarget()
    {
        return target_;
    }
}
