/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;


/**
 * This class defines a comparator for dependencies.
 * It can be used to sort a list of dependencies.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class DependencyComparator
       implements java.util.Comparator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public 
    DependencyComparator()
    {
    }

    // ==================================================================
    //
    //  Public methods for the java.util.Comparator interface.
    //
    // ==================================================================

    /**
     * Compare two declarations.
     */
    public int
    compare(java.lang.Object o1,
            java.lang.Object o2)
    {
        Declaration decl1 = (Declaration)o1;
        Declaration decl2 = (Declaration)o2;

        Declaration[] depend = decl1.getDependencies();
        for (int i=0;i<depend.length;i++)
        {
            if (depend[i]==o2)
                return 1;
        }

        depend = decl2.getDependencies();
        for (int i=0;i<depend.length;i++)
        {
            if (depend[i]==o1)
                return -1;
        }

        return 0;
    }

    /**
     *  Is equals to another object.
     */
    public boolean
    equals(java.lang.Object o)
    {
        return (o==this);
    }
}
