/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST ExceptionDecl. */
import org.objectweb.corba.ast.api.ExceptionDecl;

/** To use CORBA::ExceptionDef */
import org.omg.CORBA.ExceptionDef;

/** To use CORBA::ExceptionDescription */
import org.omg.CORBA.ExceptionDescription;

/**
 * ExceptionListImpl is a wrapper class for IDL exception lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - ExceptionList: OMG IDL for exception lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExceptionListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.ExceptionList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public ExceptionListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::ExceptionDefSeq.
     *
     * @return Its associated CORBA::ExceptionDefSeq.
     */
    public ExceptionDef[]
    getExceptionDefSeq()
    {
        // Create an array with the same length as the list. 
        ExceptionDef[] result = new ExceptionDef[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            ExceptionDeclImpl exc = (ExceptionDeclImpl)it.next();

            // Obtain its associated CORBA::ExceptionDef.
            result[i] = exc.getExceptionDef();       
        }

        // Return the array.
        return result;
    }

    /**
     * Obtain its associated CORBA::ExceptionDescriptionSeq.
     *
     * @return Its associated CORBA::ExceptionDescriptionSeq.
     */
    public ExceptionDescription[]
    getExceptionDescriptionSeq()
    {
        // Create an array with the same length as the list. 
        ExceptionDescription[] result = new ExceptionDescription[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            ExceptionDeclImpl exc = (ExceptionDeclImpl)it.next();

            // Obtain its associated CORBA::ExceptionDescription.
            result[i] = exc.getExceptionDescription();
        }

        // Return the array.
        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ExceptionList
    //
    // ==================================================================

    /**
     * Add an ExceptionDecl to the list.
     *
     * @param exc The ExceptionDecl to add to the list.
     */
    public void
    add(ExceptionDecl exception)
    {
        super.addObject(exception);
    }

    /**
     * Obtain all the ExceptionDecl declarations added to this list.
     *
     * @return All the ExceptionDecl declarations added to this list.
     */
    public ExceptionDecl[]
    getExceptions()
    {
       return (ExceptionDecl[])super.toArray(new ExceptionDecl[0]);
    }
}
