/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DelegationList. */
import org.objectweb.corba.ast.api.DelegationList;

/** To access AST SegmentDecl. */
import org.objectweb.corba.ast.api.SegmentDecl;

/**
 * ExecutorDeclImpl is a wrapper class for CIDL executor declarations.
 *
 *
 * Inherits from:
 * - ScopeImpl as executors are also IDL scopes.
 * - ExecutorDecl as OMG IDL for CIDL executor declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class ExecutorDeclImpl
     extends ScopeImpl
  implements org.objectweb.corba.ast.api.ExecutorDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** List of feature delegations. */
    private DelegationListImpl feature_delegation_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    ExecutorDeclImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        feature_delegation_ = new DelegationListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
      // TODO
    }

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
      // TODO
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null; 
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_executor;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
      // TODO check if the executor declaration is correct
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ExecutorDecl
    //
    // ==================================================================

    /**
     * Obtain the feature delegation list.
     *
     * @return The feature delegation list.
     */
    public DelegationList
    getFeatureDelegationList()
    {
        return feature_delegation_;
    }

    /**
     * Start a new CIDL segment declaration.
     *
     * @param name The name of the CIDL segment declaration.
     *
     * @return The new created SegmentDecl.
     */
    public SegmentDecl
    startSegment(String name)
    {
        SegmentDecl decl = new SegmentDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }
}
