/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** To use CORBA::ComponentIR::FinderDef. */
import org.omg.CORBA.ComponentIR.FinderDef;
import org.omg.CORBA.ComponentIR.FinderDefHelper;

/**
 * FinderDeclImpl is a wrapper class for
 * IDL home finder operation declarations.
 *
 *
 * Inherits from:
 *
 * - HomeOperationBaseImpl as these are IDL home operations.
 *
 * - FinderDecl: OMG IDL for home finder operation declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FinderDeclImpl
       extends HomeOperationBaseImpl
       implements org.objectweb.corba.ast.api.FinderDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 FinderDef. */
    private FinderDef finder_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the finder declaration.
     */
    protected
    FinderDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the HomeOperationBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        finder_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 FinderDef.
     *
     * @param contained The FinderDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        finder_def_ = FinderDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the finder declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return finder_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the finder declaration into the IFR.
     */
    public void
    create()
    {
        finder_def_ = the_parent_.getHomeDef().
                      create_finder(getId(), getName(), getVersion(),
                                    parameters_.getParameterDescriptionSeq(),
                                    exceptions_.getExceptionDefSeq());
    }

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_finder;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientOperationMapping
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.LocalOperationMapping
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.HomeOperationBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.FinderDecl
    //
    // ==================================================================

}
