/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * FixedTypeRefImpl is a wrapper for fixed IDL type references.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *     <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class FixedTypeRefImpl
       extends TypeRefImpl
       implements org.objectweb.corba.ast.api.FixedTypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Associated digits. */
    private short digits_;

    /** Associated scale. */
    private short scale_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param type The IDLType that should be referenced.
     * @param digits The associated digits.
     * @param scale The associated scale.
     */
    protected
    FixedTypeRefImpl(org.omg.CORBA.IDLType type,
                     short digits,
                     short scale)
    {
        // Call the TypeRefImpl constructor.
        super(type, org.objectweb.corba.ast.api.TypeKind.tk_fixed);

        // Init internal state.
        digits_ = digits;
        scale_ = scale;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.FixedTypeRef
    //
    // ==================================================================

    /**
     * Set the associated digits.
     *
     * @param digits The associated digits.
     */
    public void
    setDigits(short digits)
    {
        digits_ = digits;
    }

    /**
     * Obtain the associated digits.
     *
     * @return The associated digits.
     */
    public short
    getDigits()
    {
        return digits_;
    }

    /**
     * Set the associated scale.
     *
     * @param scale The associated scale.
     */
    public void
    setScale(short scale)
    {
        scale_ = scale;
    }

    /**
     * Obtain the associated scale.
     *
     * @return The associated scale.
     */
    public short
    getScale()
    {
        return scale_;
    }
}
