/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * ForwardScopeIDLImpl is a wrapper class for Scopes
 * which can be defined as forward.
 *
 * Inherits from:
 * - ForwardScopeImpl as forward scopes are also IDL scopes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class ForwardScopeIDLImpl
              extends ForwardScopeImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the forward declaration.
     */
    protected
    ForwardScopeIDLImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the ForwardScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    abstract protected void
    createContainer();

    /**
     * Complete the container object.
     */
    abstract protected void
    completeContainer();

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    /**
     * Tests if this forward declaration is already created.
     *
     * @return True if it is already created else false.
     */
    public boolean
    isForwardCreated()
    {
        return getContainer() != null;
    }
 
    /**
     * Create thisforward declaration.
     */
    public void
    createForward()
    {
        createContainer();
    }

    /**
     * Complete this forward declaration.
     */
    public void
    completeForward()
    {
        completeContainer();

        // move it in the same container to have the good order
        getContained().move(the_parent_.getContainer(), getName(), getVersion());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================
}
