/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * ForwardScopePsdlImpl is a wrapper class for Scopes
 * which can be defined as forward.
 *
 * Inherits from:
 * - ForwardScopeImpl as forward scopes are also IDL scopes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class ForwardScopePsdlImpl
              extends ForwardScopeImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Is it created. */
    protected  boolean isCreated_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the forward declaration.
     */
    protected
    ForwardScopePsdlImpl(Repository rep,
                         ScopeImpl parent)
    {
        // Call the ForwardScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        isCreated_ = false;
        content_loaded_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        // TODO: Must be checked!!!
        return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    /**
     * Tests if this forward declaration is already created.
     *
     * @return True if it is already created else false.
     */
    public boolean
    isForwardCreated()
    {
        return isCreated_;
    }
 
    /**
     * Create this forward declaration.
     */
    public void
    createForward()
    {
        isCreated_ = true;
    }

    /**
     * Complete this forward declaration.
     */
    public void
    completeForward()
    {
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================
}
