/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST HomeDecl. */
import org.objectweb.corba.ast.api.HomeDecl;

/**
 * HomeOperationBaseImpl is a wrapper class for
 * IDL home factory and finder operation declarations.
 *
 *
 * Inherits from:
 *
 * - OperationDeclImpl as these are operations,
 *
 * - HomeOperationBase: OMG IDL for home factory and finder
 *                      operation declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class HomeOperationBaseImpl
         extends OperationDeclImpl
         implements org.objectweb.corba.ast.api.HomeOperationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The client interface operation mapping. */
    private OperationDecl[] client_mapping_;

    /** The local interface operation mapping. */
    private OperationDecl[] local_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    HomeOperationBaseImpl(Repository rep,
                          ScopeImpl parent)
    {
        // Call the OperationDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        client_mapping_ = null;
        local_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the client operation mapping.
     *
     * @return The client operation mapping.
     */
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_ == null)
        {
            // retrieve the client mapping interface.
            InterfaceDecl client = ((HomeDecl)the_parent_).
                                   getClientExplicitMapping();
            client.getContents(true, DeclarationKind.dk_null);

            client_mapping_ = new OperationDecl[1];
            client_mapping_[0] = (OperationDecl)client.find(getName());
        }
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.LocalOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the local operation mapping.
     *
     * @return The local operation mapping.
     */
    public OperationDecl[]
    getLocalMapping()
    {
        if (local_mapping_ == null)
        {
            // retrieve the local mapping interface.
            InterfaceDecl local = ((HomeDecl)the_parent_).getLocalMapping();
            local.getContents(true, DeclarationKind.dk_null);

            local_mapping_ = new OperationDecl[1];
            local_mapping_[0] = (OperationDecl)local.find(getName());
        }
        return local_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.HomeOperationBase
    //
    // ==================================================================

}
