/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST Initializer. */
import org.objectweb.corba.ast.api.Initializer;

/** To use CORBA::ExtInitializer. */
import org.omg.CORBA.ExtInitializer;

/**
 * InitializerListImpl is a wrapper class
 * for IDL valuetype initializer lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - InitializerList: OMG IDL for valuetype initializer lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class InitializerListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.InitializerList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    InitializerListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::ExtInitializerSeq.
     *
     * @return Its associated CORBA::ExtInitializerSeq.
     */
    public ExtInitializer[]
    getExtInitializerSeq()
    {
        // Create an array with the same length as the list. 
        ExtInitializer[] result = new ExtInitializer[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            InitializerImpl init = (InitializerImpl)it.next();

            // Obtain its associated CORBA::ExceptionDef.
            result[i] = init.getExtInitializer();       
        }

        // Return the array.
        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InitializerList
    //
    // ==================================================================

    /**
     * Add an Initializer to this list.
     *
     * @param init The Initializer to add to the list.
     */
    public void
    add(Initializer init)
    {
        super.addObject(init);
    }

    /**
     * Obtain all the Initializer declarations added to this list.
     *
     * @return All the Initializer declarations added to this list.
     */
    public Initializer[]
    getInitializers()
    {
       return (Initializer[])super.toArray(new Initializer[0]);
    }
}
