/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** To use CORBA::InterfaceDef. */
import org.omg.CORBA.InterfaceDef;

/**
 * InterfaceListImpl is a wrapper class for IDL interface lists.
 *
 *
 * Inherits from:
 *
 * - ScopeListImpl: The base class for all lists of AST Scope objects.
 *
 * - InterfaceList: OMG IDL for interface lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class InterfaceListImpl
       extends ScopeListImpl
       implements org.objectweb.corba.ast.api.InterfaceList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    InterfaceListImpl()
    {
        // Call the ScopeListImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::InterfaceDefSeq.
     *
     * @return Its associated CORBA::InterfaceDefSeq.
     */
    public InterfaceDef[]
    getInterfaceDefSeq()
    {
        // Create an array with the same length as the list. 
        InterfaceDef[] result = new InterfaceDef[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            InterfaceDeclImpl itf = (InterfaceDeclImpl)it.next();

            // Obtain its associated CORBA::ExceptionDef.
            result[i] = itf.getExtInterfaceDef();       
        }

        // Return the array.
        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceList
    //
    // ==================================================================

    /**
     * Add an InterfaceDecl to this list.
     *
     * @param itf The InterfaceDecl to add to the list.
     */
    public void
    add(InterfaceDecl itf)
    {
        super.addObject(itf);
    }

    /**
     * Obtain all the InterfaceDecl declarations added to this list.
     *
     * @return All the InterfaceDecl declarations added to this list.
     */
    public InterfaceDecl[]
    getInterfaces()
    {
       return (InterfaceDecl[])super.toArray(new InterfaceDecl[0]);
    }
}
