/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/**
 * InterfacePortDeclImpl is a wrapper class for
 * IDL interface port declarations.
 *
 * Inherits from:
 * - DeclarationImpl as provides are declarations,
 * - ClientOperationMapping
 *   to obtain the equivalent client operation mapping.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class InterfacePortDeclImpl
                extends DeclarationImpl
                implements org.objectweb.corba.ast.api.InterfacePortDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The port interface type. */
    protected InterfaceDeclImpl interface_;

    /** Client operation mapping. */
    protected OperationDecl[] client_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the provides declaration.
     */
    protected
    InterfacePortDeclImpl(Repository rep,
                          ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        interface_ = null;
        client_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfacePortDecl
    //
    // ==================================================================

    /**
     * Set the associated InterfaceDecl.
     *
     * @param itf The associated InterfaceDecl.
     */
    public void
    setInterface(InterfaceDecl itf)
    {
        if(itf != null)
        {
            interface_ = (InterfaceDeclImpl)itf;
        }
    }

    /**
     * Obtain the associated InterfaceDecl.
     *
     * @return The associated InterfaceDecl.
     */
    public InterfaceDecl
    getInterface()
    {
        return interface_;
    }
}
