/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * ListBaseImpl is a wrapper base class for all lists of objects.
 *
 * This class encapsulates a list containing elements
 * and provides some methods to access to the list.
 *
 * The goal of this class is to allow us in the future to change
 * the used data structure (currently java.lang.Vector) to another
 * data structure, without rewriting all subclasses.
 *
 *
 * Inherits from:
 *
 * - ObjectBase: The base class for all objects.
 *
 * - List: The base interface for all lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class ListBaseImpl
       extends ObjectBase
       implements org.objectweb.corba.ast.api.List
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The list containing elements. */
    private java.util.Vector elements_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    ListBaseImpl()
    {
        // Call the ObjectBase constructor.
        super();

        // Init internal state, i.e. create the list.
        elements_ = new java.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain an element of the list.
     *
     * @param index The index of the element.
     */
    public Object
    get(int index)
    {
        return elements_.get(index);
    };

    /**
     * Add an element if it is not null.
     *
     * @param element The element to add.
     */
    public void
    addObject(Object element)
    {
        if (element != null)
        {
          elements_.add(element);
        }
    };

    /**
     * Obtain an array containing all the added elements.
     *
     * @param emptyArray An empty array providing the .
     *
     * @return An array containing all the added elements.
     */
    public Object[]
    toArray(Object[] emptyArray)
    {
       return elements_.toArray(emptyArray);
    };

    /**
     * Obtain a vector containing all the added elements.
     *
     * @return A vector containing all the added elements.
     */
    public java.util.Vector
    toVector()
    {
       return elements_;
    };

    /**
     * Obtain a new iterator to elements of the list.
     *
     * @return The new iterator.
     */
    public java.util.Iterator
    iterator()
    {
        return elements_.iterator();
    }

    /*
     * Check if an same element of the list is present several times
     *
     * @return  : -1 if all elements are distinct
     *            the index of element present several times.
     */
    public int
    checkSameItem()
    {
        for(int i=0; i<getSize(); i++)
        {
            Object obj_i = get(i);

            for(int j=i+1; j<getSize(); j++)
            {
                Object obj_j = get(j);

                if( obj_i == obj_j )
                    return i;
            }
        }
        return -1;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.List
    //
    // ==================================================================

    /**
     * Obtain the size of the list.
     *
     * @return The size of the list.
     */
    public int
    getSize()
    {
        return elements_.size();
    };
}
