/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST ExceptionList. */
import org.objectweb.corba.ast.api.ExceptionList;

/** To access AST ParameterList. */
import org.objectweb.corba.ast.api.ParameterList;

/**
 * OperationBaseImpl is a wrapper base class for all operation declarations.
 *
 * Inherits from:
 * - DeclarationWithTypeRefImpl as operations are IDL declarations
 *   and have an associated type.
 *
 * Provides:
 * - The list of parameters,
 * - The list of exceptions.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class OperationBaseImpl
                extends DeclarationWithTypeRefImpl
                implements org.objectweb.corba.ast.api.OperationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The operation parameters. */
    protected ParameterListImpl parameters_;

    /** The operation exceptions. */
    protected ExceptionListImpl exceptions_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    OperationBaseImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        parameters_ = new ParameterListImpl();
        exceptions_ = new ExceptionListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationBase
    //
    // ==================================================================

    /**
     * Obtain the parameter list.
     *
     * @return The parameter list.
     */
    public ParameterList
    getParameterList()
    {
        return parameters_;
    }

    /**
     * Obtain the exception list.
     *
     * @return The exception list.
     */
    public ExceptionList getExceptionList()
    {
        return exceptions_;
    }
}
