/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access CORBA parameter mode. */
import org.omg.CORBA.ParameterMode;

/**
 * ParameterImpl is a wrapper class for IDL operation parameters.
 *
 * Inherits from:
 * - WithNameTypeRef for the associated name and type reference.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class ParameterImpl
       extends WithNameTypeRefImpl
       implements org.objectweb.corba.ast.api.Parameter
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated mode. */
    private ParameterMode mode_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param name The associated name.
     * @param type The associated type.
     * @param mode The associated mode.
     */
    public
    ParameterImpl(String name,
                  TypeRef type,
                  ParameterMode mode)
    {
        // Call the WithNameTypeRefImpl constructor.
        super(name, type);

        // Init internal state.
        mode_ = mode;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::ParameterDescription.
     *
     * @return Its associated CORBA::ParameterDescription.
     */
    public org.omg.CORBA.ParameterDescription
    getParameterDescription()
    {
        return new org.omg.CORBA.ParameterDescription(
                       getName(),
                       org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                       getIDLType(),
                       mode_);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Parameter
    //
    // ==================================================================

    /**
     * Obtain the parameter mode.
     *
     * @return The parameter mode.
     */
    public int
    getMode()
    {
        return mode_.value();
    }

    /**
     * Is it an in parameter?
     *
     * @return True if it is an in parameter, else false.
     */
    public boolean isIn()
    {
        return mode_ == ParameterMode.PARAM_IN;
    }

    /**
     * Is it an out parameter?
     *
     * @return True if it is an out parameter, else false.
     */
    public boolean isOut()
    {
        return mode_ == ParameterMode.PARAM_OUT;
    }

    /**
     * Is it an inout parameter?
     *
     * @return True if it is an inout parameter, else false.
     */
    public boolean isInOut()
    {
        return mode_ == ParameterMode.PARAM_INOUT;
    }
}
