/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST StringList. */
import org.objectweb.corba.ast.api.StringList;

/** To access AST DelegationList. */
import org.objectweb.corba.ast.api.DelegationList;

/**
 * ProxyHomeDeclImpl is a wrapper class for
 * CIDL segment declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - ProxyHomeDecl: OMG IDL for ProxyHomeDecl.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class ProxyHomeDeclImpl
     extends DeclarationImpl
  implements org.objectweb.corba.ast.api.ProxyHomeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The list of abstract operations. */
    private StringListImpl operations_;

    /** The list of home delegations. */
    private DelegationListImpl home_delegation_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    ProxyHomeDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        operations_ = new StringListImpl();
        home_delegation_ = new DelegationListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
      return DeclarationKind.dk_proxy_home;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ProxyHomeDecl
    //
    // ==================================================================

    /**
     * Obtain the list of abstract operations.
     *
     * @return The list of abstract operations.
     */
    public StringList
    getAbstractOperationList()
    {
        return operations_;
    }

    /**
     * Obtain the list of home delegations.
     *
     * @return The list of home delegations.
     */
    public DelegationList
    getHomeDelegationList()
    {
        return home_delegation_;
    }
}
