/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/**
 * PsdlOperationDeclImpl is a wrapper class for PSDL operation declarations.
 *
 *
 * Inherits from:
 *
 * - OperationBaseImpl as PSDL operations are like IDL operations.
 *
 * - PsdlOperationDecl: OMG IDL for  PSDL operation declarations.
 *
 *
 * Provides:
 *
 * - The 'Const' property,
 *   i.e. the setConst and isConst operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PsdlOperationDeclImpl
     extends OperationBaseImpl
  implements org.objectweb.corba.ast.api.PsdlOperationDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The 'Const' property. */
    private boolean const_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value member declaration.
     **/
    protected
    PsdlOperationDeclImpl(Repository rep,
                          ScopeImpl parent)
    {
        // Call the OperationBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
	    const_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        // TODO
        return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_psdl_operation;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.PsdlOperationDecl
    //
    // ==================================================================

    /**
     * Set as a const state operation.
     *
     * @param b True if const, or false otherwise.
     */
    public void
    setConst(boolean b)
    {
        const_ = b;
    }

    /**
     * Is it a const state operation?
     *
     * @return True if it is const state operation, else false.
     */
    public boolean
    isConst()
    {
        return const_;
    }
}
