/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** Used to access AST StorageTypeBase. */
import org.objectweb.corba.ast.api.StorageTypeBase;

/**
 * PsdlTypeRefImpl is a wrapper for psdl and idl types.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PsdlTypeRefImpl
       extends ObjectBase
       implements org.objectweb.corba.ast.api.PsdlTypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The storage type base wrapped */
    protected StorageTypeBase psdl_type_;

    /** The storage type wrapped */
    protected TypeRef idl_type_;

    /** Is this type a reference to an object ? */
    protected boolean ref_;

    /** Is this type a strong reference to an object ? */
    protected boolean strong_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor for idl types.
     *
     * @param idl_type - The IDLType that should be referenced.
     */
    protected
    PsdlTypeRefImpl(TypeRef idl_type)
    {
        // init internal states
        idl_type_ = idl_type;
        psdl_type_ = null;
        ref_ = false;
        strong_ = false;
    }

    /**
     * The constructor for psdl abstract storagetype.
     *
     * @param psdl_type - The abstract storagetype that should be referenced.
     * @param ref - Is it a reference to a storagetype?
     * @param strong - Is it a strong reference to a storagetype?
     */
    protected
    PsdlTypeRefImpl(StorageTypeBase psdl_type, boolean ref, boolean strong)
    {
        // init internal states
        idl_type_ = null;
        psdl_type_ = psdl_type;
        ref_ = ref;
        strong_ = strong;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface org.objectweb.corba.ast.api.PsdlTypeRef
    //
    // ==================================================================

    /**
     * Obtain its StorageTypeBase reference.
     *
     * @return The StorageTypeBase associated with the type reference.
     */
    public StorageTypeBase
    getStorageTypeBase()
    {
        return psdl_type_;
    }

    /**
     * Obtain its idl type reference.
     *
     * @return The idl type reference associated with the type reference.
     */
    public TypeRef
    getTypeRef()
    {
        return idl_type_;
    }

    /**
     * Is this type a reference to a psdl object ?
     *
     * @return True if it is a reference, else false.
     */
    public boolean
    isRef()
    {
        return ref_;
    }

    /**
     * Is this type a strong reference to a psdl object ?
     *
     * @return True if it is a strong reference, else false.
     */
    public boolean
    isStrong()
    {
        return strong_;
    }

}
