/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/**
 * ScopeListImpl is a wrapper class for AST Scope lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class ScopeListImpl
       extends ListBaseImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    ScopeListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Find a declaration into the list.
     *
     * @param name The name of the searched declaration.
     * @return The declaration that was searched
     *          or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
        // Iterate on all elements of the list.
        for(java.util.Iterator it = iterator(); it.hasNext(); )
        {
            // Obtain the current element.
            Scope scope = (Scope)it.next();

            // Find the named declaration.
            Declaration decl = scope.find(name);

            // If found then returns it.
            if (decl != null) return decl;
        }

        // Else not found.
        return null;
    }

    /**
     * Add a Scope to this list.
     *
     * @param scope The Scope to add to the list.
     */
    public void
    addScope(Scope scope)
    {
        super.addObject(scope);
    }

    /**
     * Obtain all the AST scopes added to this list.
     *
     * @return All the AST scopes added to this list.
     */
    public Scope[]
    getScopes()
    {
       return (Scope[])super.toArray(new Scope[0]);
    }
}
