/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/**
 * StorageHomeKeyDeclImpl is a wrapper class for
 * PSDL storagehome key declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationWithMemberListImpl as they are IDL declarations,
 *
 * - StorageHomeKeyDecl: OMG IDL for PSDL storagehome key declarations
 *   with a list of member names.
 *
 *
 * Provides:
 *
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class StorageHomeKeyDeclImpl
     extends DeclarationWithMemberListImpl
  implements org.objectweb.corba.ast.api.StorageHomeKeyDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    StorageHomeKeyDeclImpl(Repository rep,
                           ScopeImpl parent)
    {
        // Call the DeclarationWithMemberListImpl constructor.
        super(rep, parent);

        // Init internal state.
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
      return DeclarationKind.dk_storage_home_key;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.DeclarationWithMemberList
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageHomeKeyDecl
    //
    // ==================================================================

}
