/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/**
 * StorageTypeStateMemberDeclImpl is a wrapper class for
 * PSDL storagetype state member declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationWithTypeRefImpl as state members are IDL declarations
 *   and have an associated type.
 *
 * - StorageTypeStateMemberDecl: OMG IDL for PSDL storagetype state
 *   member declarations.
 *
 *
 * Provides:
 *
 * - The 'Readonly' property,
 *   i.e. the setReadonly and isReadonly operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class StorageTypeStateMemberDeclImpl
     extends DeclarationWithPsdlTypeRefImpl
  implements org.objectweb.corba.ast.api.StorageTypeStateMemberDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Readonly property. */
    private boolean readonly_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value member declaration.
     **/
    protected
    StorageTypeStateMemberDeclImpl(Repository rep,
                                   ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        readonly_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the value
     *         member declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        // TODO
        return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl;
    }

    // ==================================================================
    //
    // Internal methods for DeclarationWithTypeRefImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_storage_type_state_member;
    }

    /**
     * Create the value member declaration into the IFR.
     */
    public void
    create()
    {
        // TODO
        super.create();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.corba.ast.api.DeclarationWithTypeRef
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeStateMemberDecl
    //
    // ==================================================================

    /**
     * Set as a readonly state member.
     *
     * @param b True if readonly, or false otherwise.
     */
    public void
    setReadonly(boolean b)
    {
        readonly_ = b;
    }

    /**
     * Is it a readonly state member?
     *
     * @return True if it is readonly state member, else false.
     */
    public boolean
    isReadonly()
    {
        return readonly_;
    }
}
