/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * StringListImpl is a wrapper class for lists of strings.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - StringList: OMG IDL for lists of strings.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class StringListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.StringList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    StringListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /*
     * Check if an same element of the list is present several times
     *
     * @return  : -1 if all elements are distinct
     *            the index of element present several times.
     */
    public int
    checkSameItem()
    {
        for(int i=0; i<getSize(); i++)
        {
            String s1 = (String)get(i);

            for(int j=i+1; j<getSize(); j++)
            {
                String s2 = (String)get(j);

                if( s1.compareTo(s2) == 0 )
                    return i;
            }
        }
        return -1;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.List
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StringList
    //
    // ==================================================================

    /**
     * Add a string to this list.
     *
     * @param s The string to add to the list.
     */
    public void
    add(String s)
    {
        super.addObject(s);
    }

    /**
     * Obtain all the strings added to this list.
     *
     * @return All the strings added to this list.
     */
    public String[]
    getStrings()
    {
       return (String[])super.toArray(new String[0]);
    }
}
