/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST StructMember. */
import org.objectweb.corba.ast.api.StructMember;

/**
 * StructMemberListImpl is a wrapper class for IDL struct member lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - StructMemberList: OMG IDL for struct member lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructMemberListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.StructMemberList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    StructMemberListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::StructMemberSeq.
     *
     * @return Its associated CORBA::StructMemberSeq.
     */
    public org.omg.CORBA.StructMember[]
    getStructMemberSeq()
    {
        // Create an array with the same length as the list. 
        org.omg.CORBA.StructMember[] result =
            new org.omg.CORBA.StructMember[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            StructMemberImpl sm = (StructMemberImpl)it.next();

            // Obtain its associated CORBA::StructMember.
            result[i] = sm.getStructMember();       
        }

        // Return the array.
        return result;
    }

    /**
     * Return the index name.
     *
     * @param index The index of the member that should be returned.
     *
     * @return The name of the member at the specified index.
     */
    public String
    nameAt(int index)
    {
        return ((StructMemberImpl)super.get(index)).getName();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StructMemberList
    //
    // ==================================================================

    /**
     * Add a member.
     *
     * @param name The name of the member.
     * @param type The TypeRef of the member.
     */
    public void
    addMember(String name,
              org.objectweb.corba.ast.api.TypeRef type)
    {
        if(type != null)
        {
            super.addObject( new StructMemberImpl(name, type) );
        }
    }

    /**
     * Obtain all the struct members.
     *
     * @return All the struct members.
     */
    public StructMember[]
    getStructMembers()
    {
       return (StructMember[])super.toArray(new StructMember[0]);
    }
}
