/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to manipulate CORBA IDLType. */
import org.omg.CORBA.IDLType;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to use TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/**
 * TypeRefImpl is a wrapper to CORBA::IDLType object
 * provided by the Interface Repository.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class TypeRefImpl
       extends ObjectBase
       implements IDLTypeWrapper,
                  org.objectweb.corba.ast.api.TypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA::IDLType from the Interface Repository. */
    private IDLType type_;

    /** Associated AST TypeKind. */
    private TypeKind kind_;

    /** List of dependencies. */
    protected Declaration[] dependencies_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param type The IDLType that should be referenced.
     * @param kind The associated type kind.
     */
    protected
    TypeRefImpl(IDLType type,
                TypeKind kind)
    {
        // Init internal state.
        type_ = type;
        kind_ = kind;
        dependencies_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * TODO: Why is it needed?
     */
    protected String
    getId()
    {
        if (kind_==TypeKind.tk_objref)
            return "IDL:omg.org/CORBA/Object:1.0";
        throw new org.objectweb.corba.ast.api.ASTError("Bad TypeKind!");
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public IDLType
    getIDLType()
    {
        return type_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the external Declaration dependencies.
     *
     * @return The list of Declaration dependencies.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;
        dependencies_ = new Declaration[0];
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return kind_;
    }

    /**
     * Test if this TypeRef is also a Declaration object.
     *
     * @return True if it is a Declaration, false otherwise.
     */
    public boolean
    isDeclaration()
    {
        return false;
    }

}
