/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to reference an AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/**
 * TypeRefWithContentImpl is a wrapper for sequence and array IDL types.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class TypeRefWithContentImpl
       extends TypeRefWithLengthImpl
       implements org.objectweb.corba.ast.api.TypeRefWithContent
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Associated content for sequence and array types. */
    private TypeRef content_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param type The IDLType that should be referenced.
     * @param kind The associated type kind.
     * @param length The associated length.
     * @param content The associated content type.
     */
    protected
    TypeRefWithContentImpl(org.omg.CORBA.IDLType type,
                           org.objectweb.corba.ast.api.TypeKind kind,
                           int length,
                           TypeRef content)
    {
        // Call the TypeRefWithLengthImpl constructor.
        super(type, kind, length);

        // Init internal state.
        content_ = content;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain the external Declaration dependencies.
     *
     * @return The list of Declaration dependencies.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        if (content_.isDeclaration())
        {
            Declaration[] depend = content_.getDependencies();
            Declaration[] depend2 = new Declaration[1+depend.length];
            for (int i=0;i<depend.length;i++)
                depend2[i] = depend[i];

            depend2[depend.length] = (Declaration)content_;
            dependencies_ = depend2;
        }
        else
            dependencies_ = content_.getDependencies();

        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRefWithContent
    //
    // ==================================================================

    /**
     * Set the contained type.
     *
     * @param type The contained type.
     */
    public void
    setContentType(TypeRef content)
    {
        content_ = content;
    }

    /**
     * Obtain the contained type.
     *
     * @return The contained type.
     */
    public TypeRef
    getContentType()
    {
        return content_;
    }
}
