/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * TypeRefWithLengthImpl is a wrapper for IDL type references with an
 * associated length, i.e. string, sequence, array, and wstring types.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class TypeRefWithLengthImpl
       extends TypeRefImpl
       implements org.objectweb.corba.ast.api.TypeRefWithLength
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Associated length for string, sequence, array, and wstring types. */
    private int length_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param type The IDLType that should be referenced.
     * @param kind The associated type kind.
     * @param length The associated length.
     */
    protected
    TypeRefWithLengthImpl(org.omg.CORBA.IDLType type,
                          org.objectweb.corba.ast.api.TypeKind kind,
                          int length)
    {
        // Call the TypeRefImpl constructor.
        super(type, kind);

        // Init internal state.
        length_ = length;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRefWithLength
    //
    // ==================================================================

    /**
     * Set the associated length.
     *
     * @param length The associated length.
     */
    public void
    setLength(int length)
    {
        length_ = length;
    }

    /**
     * Obtain the associated length.
     *
     * @return The associated length.
     */
    public int
    getLength()
    {
        return length_;
    }
}
