/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * UnionMemberImpl is a wrapper class for IDL union members.
 *
 * Inherits from:
 * - WithNameTypeRef for the associated name and type reference,
 * - WithAnyValue    for the associated any value.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class UnionMemberImpl
       extends WithNameTypeRefImpl
       implements org.objectweb.corba.ast.api.UnionMember
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated AnyValue. */
    private AnyValueImpl any_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param name The associated name.
     * @param type The associated type.
     * @param value The associated value.
     */
    public
    UnionMemberImpl(String name,
                    org.objectweb.corba.ast.api.TypeRef type,
                    AnyValueImpl any)
    {
        // Call the WithNameTypeRefImpl constructor.
        super(name, type);

        // Init internal state.
        any_ = any;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::UnionMember.
     *
     * @param discriminator The discriminator TypeCode.
     *
     * @return Its associated CORBA::UnionMember.
     */
    protected org.omg.CORBA.UnionMember
    getUnionMember(org.omg.CORBA.TypeCode discriminator)
    {
        return new org.omg.CORBA.UnionMember(
                       getName(),
                       any_.computeAny(discriminator),
                       org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                       getIDLType());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithAny
    //
    // ==================================================================

    /**
     * Set the associated AnyValue.
     *
     * @param value The AnyValue to set.
     */
    public void
    setAnyValue(org.objectweb.corba.ast.api.AnyValue value)
    {
      any_ = (AnyValueImpl)value;
    }

    /**
     * Obtain the associated AnyValue.
     *
     * @return The associated AnyValue.
     */
    public org.objectweb.corba.ast.api.AnyValue
    getAnyValue()
    {
      return any_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.UnionMember
    //
    // ==================================================================
}
