/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST UnionMember. */
import org.objectweb.corba.ast.api.UnionMember;

/**
 * UnionMemberListImpl is a wrapper class for IDL union member lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - UnionMemberList: OMG IDL for union member lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UnionMemberListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.UnionMemberList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    UnionMemberListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::UnionMemberSeq.
     *
     *
     * @param discriminator The discriminator TypeCode.
     *
     * @return Its associated CORBA::UnionMemberSeq.
     **/
    public org.omg.CORBA.UnionMember[]
    getUnionMemberSeq(org.omg.CORBA.TypeCode discriminator)
    {
        // Create an array with the same length as the list. 
        org.omg.CORBA.UnionMember[] result =
            new org.omg.CORBA.UnionMember[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            UnionMemberImpl um = (UnionMemberImpl)it.next();

            // Obtain its associated CORBA::UnionMember.
            result[i] = um.getUnionMember(discriminator);       
        }

        // Return the array.
        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.UnionMemberList
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the StructMembers interface.
    //
    // ==================================================================

    /**
     * Add an union member.
     *
     * @param name The name of the union member.
     * @param type The TypeRef of the union member.
     * @param value The AnyValue of the union member.
     */
    public void
    addMember(String name,
              org.objectweb.corba.ast.api.TypeRef type,
              org.objectweb.corba.ast.api.AnyValue value)
    {
        if ((type!=null) && (name!=null) && (value!=null))
        {
            super.addObject( new UnionMemberImpl(name, type,
                                                (AnyValueImpl)value) );
        }
    }

    /**
     * Obtain all the union members.
     *
     * @return All the union members.
     */
    public UnionMember[]
    getUnionMembers()
    {
       return (UnionMember[])super.toArray(new UnionMember[0]);
    }
}
