/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST ValueDecl. */
import org.objectweb.corba.ast.api.ValueDecl;

/** To use CORBA::ValueDef. */
import org.omg.CORBA.ValueDef;

/**
 * ValueListImpl is a wrapper class for IDL valuetype lists.
 *
 *
 * Inherits from:
 *
 * - ScopeListImpl: The base class for all lists of AST Scope objects.
 *
 * - ValueList: OMG IDL for valuetype lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class ValueListImpl
       extends ScopeListImpl
       implements org.objectweb.corba.ast.api.ValueList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    ValueListImpl()
    {
        // Call the ScopeListImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::ValueDefSeq.
     *
     * @return Its associated CORBA::ValueDefSeq.
     */
    protected ValueDef[]
    getValueDefSeq()
    {
        // Create an array with the same length as the list. 
        ValueDef[] result = new ValueDef[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            ValueDeclImpl itf = (ValueDeclImpl)it.next();

            // Obtain its associated CORBA::ExtValueDef.
            result[i] = itf.getExtValueDef();       
        }

        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ValueList
    //
    // ==================================================================

    /**
     * Add a ValueDecl to this list.
     *
     * @param value The ValueDecl to add to the list.
     */
    public void
    add(ValueDecl value)
    {
        super.addObject(value);
    }

    /**
     * Obtain all the ValueDecl declarations added to this list.
     *
     * @return All the ValueDecl declarations added to this list.
     */
    public ValueDecl[]
    getValues()
    {
       return (ValueDecl[])super.toArray(new ValueDecl[0]);
    }
}
