/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to reference an AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/**
 * WithNameTypeRefImpl is a wrapper class for objects containing
 * an associated name and type reference.
 *
 * Inherits from:
 * - WithName    for the associated name,
 * - WithTypeRef for the associated type.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.1
 */

public class WithNameTypeRefImpl
       extends WithNameImpl
       implements org.objectweb.corba.ast.api.WithNameTypeRef
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated type. */
    protected TypeRef type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param name The associated name.
     * @param type The associated type.
     */
    public
    WithNameTypeRefImpl(String name,
                        TypeRef type)
    {
        // Call the WithNameImpl constructor.
        super(name);

        // Init internal state.
        type_ = type;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the associated CORBA::IDLType reference.
     *
     * @return The associated CORBA::IDLType reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ((IDLTypeWrapper)type_).getIDLType();
    }

    /**
     * Obtain its associated CORBA::StructMember.
     *
     * @return Its associated CORBA::StructMember.
     */
    public org.omg.CORBA.StructMember
    getStructMember()
    {
        return new org.omg.CORBA.StructMember(
                       getName(),
                       org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void(),
                       getIDLType());
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithNameTypeRef
    //
    // ==================================================================

    /**
     * Set the associated TypeRef reference.
     *
     * @param type The associated TypeRef reference.
     */
    public void
    setType(TypeRef type)
    {
        type_ = type;
    }

    /**
     * Obtain the associated TypeRef reference.
     *
     * @return The associated TypeRef reference.
     */
    public TypeRef
    getType()
    {
        return type_;
    }
}
