/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.api;

/**
 * This interface allows you to parse and feed an idl3 file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 1.0
 */

public interface IR3FeedOperations
         extends Command
{
    /**
     * Initialize Interface Repository and Abstract Syntax Tree.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    initRepository();

    /**
     * Preprocess the idl file.
     **/
    public void
    preprocess(String args);

    /**
     * Compile the idl file and feed the IR.
     *
     * @param inputfile - The file to be compiled.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    compile(String inputfile);

    /**
     * Get the File scope previously compiled.
     *
     * @return The AST file scope.
     **/
    public org.objectweb.corba.ast.api.FileScope
    getFileScope();

    /**
     * Add preprocessor options to the command line.
     **/
    public void
    addPreprocessorOptions();

}
