/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Demarey Christophe.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// To use Command line utilities.
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;

/**
 * This class is a base for commands.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

abstract public class CommandBase
              extends org.objectweb.ccm.CORBA.Application
           implements org.objectweb.corba.command.api.Command
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Print output messages?
     **/
    protected boolean printMsg_;

    /**
     * The command line.
     **/
    protected CommandLine cmd_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    CommandBase()
    {
        // Init internal state
        printMsg_ = true;
        cmd_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================



    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.Command interface.
    //
    // ==================================================================

    /**
     *  Print a message on the standard output if printMsg is True.
     *
     * @param text - The message to print.
     *
     **/
    public void
    message(String text)
    {
        if (printMsg_)
            System.out.println(text);
    }

    /**
     *  Display a message on the error output.
     *
     * @param text - The message to print.
     *
     **/
    public void
    error(String text)
    {
        System.err.println("[Error] "+text);
    }

}
