/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

/**
 * This class is a wrapper for Generators.
 * It defines common methods.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class GeneratorBase
              extends IR3Feed
           implements org.objectweb.corba.command.api.GeneratorBaseOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Generator Options.
     **/
    org.objectweb.util.cmdline.api.OptionArgument ofile_; // -o <file>

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    GeneratorBase()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Add common generator options to the command line.
     **/
    public void
    addGeneratorOptions()
    {
        // Specify output file
        ofile_ = new org.objectweb.util.cmdline.lib.DefaultOptionArgument(
                        new String[] { "-o" },
                        "outputfile",
                        new String[] { "Display mapping on file <outputfile>.",
                                       "outputfile is the file name without extension."
                                      },
                        "generated" );
        cmd_.addOption(ofile_);
    }

    /**
     *  Indent a file.
     *
     *  @param intputfile - The file to indent.
     *
     *  @return The status : true if ok, else false.
     **/
    public boolean
    indent(String inputfile)
    {
        org.objectweb.corba.generator.common.Indentor ind = new org.objectweb.corba.generator.common.Indentor();
        try{
            ind.indent(inputfile);
        }catch(java.io.IOException e){
            error("I/O Exception with file " + inputfile);
            return false;
        }
        return true;
    }

}
