/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// To use Command line utilities.
import org.objectweb.util.cmdline.api.*;
import org.objectweb.util.cmdline.lib.*;

/**
 * This class allows you to parse and feed an idl3 file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 1.0
 */

public class IR3Feed
     extends CommandBase
  implements org.objectweb.corba.command.api.IR3FeedOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * The Interface Repository.
     **/
    protected org.objectweb.ccm.ComponentRepository ir_;

    /**
     * The Abstract Syntax Tree.
     **/
    protected org.objectweb.corba.ast.api.AST ast_;

    /**
     * The File scope.
     **/
    protected org.objectweb.corba.ast.api.FileScope fileScope_;

    /**
     * Preprocessor Options.
     **/
    org.objectweb.corba.command.options.DNameOption dname_; // -DNAME=DEF
    org.objectweb.corba.command.options.MultipleOptionArgument uname_; // -UNAME
    org.objectweb.corba.command.options.MultipleOptionArgument idir_;   // -IDIR

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    IR3Feed()
    {
        // Call the CommandBase constructor
        super();

        // Init internal state
        ir_ = null;
        ast_ = null;
        fileScope_ = null;
        // options
        dname_ = null;
        uname_ = null;
        idir_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     **/
    public static void
    main(String[] args)
    {
        IR3Feed ir3feed = new IR3Feed();
//            for(int i=0; i< args.length; i++)
//                System.out.println("args["+i+"] = "+ args[i]);
        ir3feed.start(args);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    run(java.lang.String[] args)
    {
        boolean ok = true;
        String[] arguments = null;

        // Analyze command line
        arguments = parseArguments(args);

        // Get the repository
        ok = initRepository();
        if (!ok) return -1;

        // Compile the file
        ok = compile(arguments[0]);
        if (!ok)
        {
            // getFileScope().destroy()
            return -1;
        }

/*        String[] tmp, tmp2;
        tmp = dname_.getOptionArguments();
        tmp2 = dname_.getOptionArgumentValues();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -D. NAME="+tmp[i]+"  DEF="+tmp2[i]);
        tmp = uname_.getOptionArguments();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -U. NAME="+tmp[i]);
        tmp = idir_.getOptionArguments();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -I. NAME="+tmp[i]);
*/
        return 0;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.api.IR3FeedOperations interface.
    //
    // ==================================================================

    /**
     * Initialize Interface Repository and Abstract Syntax Tree.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    initRepository()
    {
        try {
            // Obtain the Interface Repository.
            ir_ = org.objectweb.ccm.ComponentRepositoryHelper.narrow(
                        org.objectweb.ccm.CORBA.TheInterfaceRepository.getRepository() );

            // Start the Abstract Syntax Tree
            ast_ = new org.objectweb.corba.ast.lib.Repository(ir_);
        } catch(Exception ex) {
            System.err.println("Problem with Interface Repository initialization");
            ex.printStackTrace();
            return false;
        }
        return true;
    }

    /**
     * Preprocess the idl file.
     **/
    public void
    preprocess(String args)
    {
        // org.objectweb.corba.command.PreProcessor.process(args);
    }

    /**
     * Compile the idl file and feed the IR.
     *
     * @param inputfile - The file to be compiled.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    compile(String inputfile)
    {
        org.objectweb.corba.parser.api.Parser parser = null;

        message("Compile file " + inputfile + " ...");
        // Create a new parser
        try{
            parser = new org.objectweb.corba.parser.lib.Parser(
                                                 new java.io.FileInputStream(inputfile),
                                                 ast_,
                                                 new org.objectweb.corba.parser.lib.ErrorManager(inputfile) );
        }catch(java.io.FileNotFoundException e){
            error("File " + inputfile + " not found!");
            return false;
        }

        // Parse the file
        message("Feeding the Interface Repository ...");
        fileScope_ = parser.parseCIDL();
        if (fileScope_ == null)
        {
            error("Compilation failed.");
            return false;
        }
        else
            message("Compilation completed.");
        return true;
    }

    /**
     * Get the File scope previously compiled.
     *
     * @return The AST file scope.
     **/
    public org.objectweb.corba.ast.api.FileScope
    getFileScope()
    {
        return fileScope_;
    }

    /**
     * Add preprocessor options to the command line.
     **/
    public void
    addPreprocessorOptions()
    {
        // -DNAME=DEF
        dname_ = new org.objectweb.corba.command.options.DNameOption(
                        "-D",
                        new String[] { "-DNAME", "-DNAME=DEF" },
                        "",
                        new String[] { "Define NAME as DEF, default 1."},
                        "" );
        cmd_.addOption(dname_);

        // -UNAME
        uname_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-U",
                        new String[] { "-UNAME" },
                        "",
                        new String[] { "Remove any definition for NAME."},
                        "",
                        true,
                        false );
        cmd_.addOption(uname_);

        // -IDIR
        idir_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-I",
                        new String[] { "-IDIR" },
                        "",
                        new String[] { "Put DIR in the include search path."},
                        "",
                        true,
                        false );
        cmd_.addOption(idir_);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.Command interface.
    //
    // ==================================================================

    /**
     * Analyze command line.
     *
     * @param args - Arguments of the command.
     *
     **/
    public String[] parseArguments(String[] args)
    {
        // Create a command line.
        cmd_ = new DefaultCommandLine( "ir3_feed",
                                       "idl_file",
                                       new String[] { "Feed the repository with an idl file." },
                                       true);
        // Add preprocessor options
        addPreprocessorOptions();

        // Parse the command line arguments.
        return cmd_.parse(args);
    }

}
