/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;


/**
 * This class allows you to generate idl2 for a declaration from the repository.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */


public class IR3toIDL2
     extends GeneratorBase
  implements org.objectweb.corba.command.api.IR3toIDL2Operations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Include Options.
     **/
    org.objectweb.corba.command.options.MultipleOptionArgument include_;
    org.objectweb.corba.command.options.MultipleOptionArgument userInclude_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    IR3toIDL2()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     **/
    public static void
    main(String[] args)
    {
        IR3toIDL2 ir3_idl2 = new IR3toIDL2();
//            for(int i=0; i< args.length; i++)
//                System.out.println("args["+i+"] = "+ args[i]);
        ir3_idl2.start(args);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    run(java.lang.String[] args)
    {
        boolean ok = true;
        String[] arguments = null;

        // Analyze command line
        arguments = parseArguments(args);

/*        System.out.println("output file = " + ofile_.getArgument());
        System.out.println("declaration = " + arguments[0]);
        String[] tmp = include_.getOptionArguments();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -i. include="+tmp[i]);
        tmp = userInclude_.getOptionArguments();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -ipath. user include="+tmp[i]);
*/
        // Initialize Interface Repository and Abstract Syntax Tree
        initRepository();

        // Generate idl3
        ok = generate_idl2( arguments[0],ofile_.getArgument()+".idl",
                            include_.getOptionArguments(), userInclude_.getOptionArguments() );
        if (ok) return 0;
        return -1;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.api.IR3toIDL2Operations interface.
    //
    // ==================================================================

    /**
     *  Generate idl2 for a scope.
     *
     *  @param decl_name - The scope to be generated.
     *  @param outputfile - The file name to write in.
     *  @param includes - Files to include with <...>.
     *  @param userIncludes - User files to include "...".
     *
     *  @return The status : true if ok, else false.
     **/
    public boolean
    generate_idl2( String decl_name, String outputfile,
                   String[] includes, String[] userIncludes )
    {
        // Create an idl2 repository
        org.objectweb.corba.ast.api.AST idl2_ast =
                    new org.objectweb.corba.ast.lib.Repository(ir_.as_IDL2_repository());

        // Retrieve the scope to generate
        org.objectweb.corba.ast.api.Scope scope = null;
        try{
            scope = (org.objectweb.corba.ast.api.Scope)((org.objectweb.corba.ast.lib.Repository)idl2_ast).lookup(decl_name);
        }catch(ClassCastException e){
            error(decl_name+"is not a scope");
            return false;
        }

        message("Generating idl2 for " + decl_name + " ...");

        // Launch the generator
        org.objectweb.corba.generator.idl.IDL2Generator gen =
                         new org.objectweb.corba.generator.idl.IDL2Generator(scope);
        gen.initialize(outputfile, "IR3toIDL2");

        // generate
        gen.put("includes", includes);
        gen.put("userIncludes", userIncludes);
        try{
            gen.generate(decl_name);
        }catch(org.objectweb.corba.generator.common.GenerationException ex){
            System.err.println(ex.getMessage());
            return false;
        }

        // Get an idl3 repository
        ir_.as_IDL3_repository();

        // Indent the output file
        if (!indent(outputfile))
            return false;

        message("Idl2 generated in " + outputfile + ".");
        return true;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.Command interface.
    //
    // ==================================================================

    /**
     * Analyze command line.
     *
     * @param args - Arguments of the command.
     *
     **/
    public String[] parseArguments(String[] args)
    {
        // Create a command line.
        cmd_ = new org.objectweb.util.cmdline.lib.DefaultCommandLine(
                            "ir3_idl2",
                            "declaration",
                            new String[] { "Generate idl2 for declaration from the repository." },
                            true );

        // Add common generator options
        addGeneratorOptions();

        // Add include option
        include_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-i",
                        new String[] { "-i" },
                        "file",
                        new String[] { "Add file in an include statement." },
                        "",
                        true );
        cmd_.addOption(include_);

        // Add user include option
        userInclude_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-ipath",
                        new String[] { "-ipath" },
                        "path",
                        new String[] { "Add an user file in an include statement.",
                                       "path must be the complete name of the file." },
                        "",
                        true );
        cmd_.addOption(userInclude_);

        // Parse the command line arguments.
        return cmd_.parse(args);
    }
}
