/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

/**
 * This class allows you to generate idl3 for a declaration from the repository.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class IR3toIDL3
     extends GeneratorBase
  implements org.objectweb.corba.command.api.IR3toIDL3Operations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Import Option.
     **/
    org.objectweb.corba.command.options.MultipleOptionArgument import_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    IR3toIDL3()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     **/
    public static void
    main(String[] args)
    {
        IR3toIDL3 ir3_idl3 = new IR3toIDL3();
//            for(int i=0; i< args.length; i++)
//                System.out.println("args["+i+"] = "+ args[i]);
        ir3_idl3.start(args);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    run(java.lang.String[] args)
    {
        boolean ok = true;
        String[] arguments = null;

        // Analyze command line
        arguments = parseArguments(args);

/*        System.out.println("output file = " + ofile_.getArgument());
        System.out.println("declaration = " + arguments[0]);
        String[] tmp = import_.getOptionArguments();
        for(int i=0;i<tmp.length;i++)
            System.out.println("Option -i. IR_OBJECT="+tmp[i]);
*/
        // Initialize Interface Repository and Abstract Syntax Tree
        initRepository();

        // Generate idl3 (decl, outputfile, imports)
        ok = generate_idl3(arguments[0],ofile_.getArgument()+".idl", import_.getOptionArguments());
        if (ok) return 0;
        return -1;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.api.IR3toIDL3Operations interface.
    //
    // ==================================================================

    /**
     * Generate idl3 for a scope.
     * IR must be initialized first with method "initRepository()".
     *
     * @param decl_name - The scope to be generated.
     * @param outputfile - The file name to write in.
     * @param imports - IR declarations to import.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    generate_idl3(String decl_name, String outputfile, String[] imports)
    {
        // Retrieve the scope to generate
        org.objectweb.corba.ast.api.Scope scope = null;
        try{
            scope = (org.objectweb.corba.ast.api.Scope)((org.objectweb.corba.ast.api.Scope)ast_).lookup(decl_name);
        }catch(ClassCastException e){
            error(decl_name+"is not a scope");
            return false;
        }

        message("Generating idl3 for " + decl_name + " ...");

        // Launch the generator
        org.objectweb.corba.generator.idl.IDL3Generator gen =
                         new org.objectweb.corba.generator.idl.IDL3Generator(scope);
        gen.initialize(outputfile, "IR3toIDL3");

        // generate
        gen.put("imports", imports);
        try{
            gen.generate(decl_name);
        }catch(org.objectweb.corba.generator.common.GenerationException ex){
            System.err.println(ex.getMessage());
            return false;
        }

        // Indent the output file
        if (!indent(outputfile))
            return false;

        message("Idl3 generated in " + outputfile + ".");
        return true;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.Command interface.
    //
    // ==================================================================

    /**
     * Analyze command line.
     *
     * @param args - Arguments of the command.
     *
     **/
    public String[] parseArguments(String[] args)
    {
        // Create a command line.
        cmd_ = new org.objectweb.util.cmdline.lib.DefaultCommandLine(
                            "ir3_idl3",
                            "declaration",
                            new String[] { "Generate idl3 for declaration from the repository." },
                            true );

        // Add common generator options
        addGeneratorOptions();

        // Add import option
        import_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-i",
                        new String[] { "-i" },
                        "ir_object",
                        new String[] { "Add ir_object in an import statement." },
                        "",
                        true );
        cmd_.addOption(import_);

        // Parse the command line arguments.
        return cmd_.parse(args);
    }

}
