/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

/**
 * This class allows you to generate psdl from a psdl file.
 *
 * @author <a href="mailto:Chrsitophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PSDLtoPSDL
     extends IR3toIDL3
  implements org.objectweb.corba.command.api.PSDLtoPSDLOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    PSDLtoPSDL()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** The main bootstrap method.
     **/
    public static void
    main(String[] args)
    {
        PSDLtoPSDL psdl = new PSDLtoPSDL();
//            for(int i=0; i< args.length; i++)
//                System.out.println("args["+i+"] = "+ args[i]);
        psdl.start(args);
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.ccm.CORBA.Application class.
    //
    // ==================================================================

    /**
     **  Run the application.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    run(java.lang.String[] args)
    {
        boolean ok = true;
        String[] arguments = null;
        String inputfile = null;

        // Analyze command line
        arguments = parseArguments(args);
        inputfile = arguments[0];

        // Get the repository
        ok = initRepository();
        if (!ok) return -1;

        // Compile the file
        ok = compile(inputfile);
        if (!ok)
        {
            // getFileScope().destroy()
            return -1;
        }

        // Generate psdl (inputfile, outputfile)
        ok = generate_psdl(inputfile, ofile_.getArgument()+".psdl");
        if (!ok) return -1;

        return 0;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.api.IR3toPSDLOperations interface.
    //
    // ==================================================================

    /**
     * Generate psdl3 from AST.
     * IR must be initialized first with method "initRepository()".
     *
     * @param inputfile - The source file.
     * @param outputfile - The file name to write in.
     *
     * @return The status : true if ok, else false.
     **/
    public boolean
    generate_psdl(String inputfile, String outputfile)
    {
        // Launch the generator
        org.objectweb.corba.generator.psdl.PSDLGenerator gen =
                         new org.objectweb.corba.generator.psdl.PSDLGenerator(getFileScope());
        gen.initialize(outputfile, "PSDLtoPSDL");

        // generate
        message("Generating psdl from " + inputfile + " ...");
//        gen.put("imports", outputfile+".idl");
        try{
            gen.generate( org.objectweb.corba.ast.api.DeclarationKind.dk_psdl);
        }catch(org.objectweb.corba.generator.common.GenerationException ex){
            System.err.println(ex.getMessage());
            return false;
        }

        // Indent the output file
        if (!indent(outputfile))
            return false;

        message("Psdl generated in " + outputfile + ".");
        return true;
    }

    // ==================================================================
    //
    //  Methods for the org.objectweb.corba.command.Command interface.
    //
    // ==================================================================

    /**
     * Analyze command line.
     *
     * @param args - Arguments of the command.
     *
     **/
    public String[] parseArguments(String[] args)
    {
        // Create a command line.
        cmd_ = new org.objectweb.util.cmdline.lib.DefaultCommandLine(
                            "my_psdl",
                            "file",
                            new String[] { "Generate psdl from a file." },
                            true );

        // Add common generator options
        addGeneratorOptions();

        // Add import option
/*        import_ = new org.objectweb.corba.command.options.MultipleOptionArgument(
                        "-i",
                        new String[] { "-i" },
                        "ir_object",
                        new String[] { "Add ir_object in an import statement." },
                        "",
                        true );
        cmd_.addOption(import_);
*/
        // Parse the command line arguments.
        return cmd_.parse(args);
    }

}
