/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.command.options;

/**
 * This is an implementation for -D preprocessor option.
 * ex: -DNAME=TOTO => option = -D
 *                 => argument = NAME
 *                 => value = TOTO
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class DNameOption
     extends MultipleOptionArgument
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The option associated value. */
    protected String value_;

    /** List of associated values for this option. */
    private java.util.Vector values_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param id The option identifier.
     * @param value The argument associated value.
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param argument The initial argument value.
     */
    public
    DNameOption( String id,
                 String[] labels,
                 String arguments,
                 String[] description,
                 String argument)
    {
        super(id, labels, arguments, description, argument, true, false);
        values_ = new java.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    /**
     * Appends internal state representation into the given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     */
    protected void
    appendInternalState(StringBuffer sb)
    {
        super.appendInternalState(sb);
        append(sb, "value", getValue());
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Checks the current command line argument.
     *
     * @param current The current command line argument.
     *
     * @return true if the option accepts this argument.
     */
    public boolean
    check(String current)
    {
        if ( (current.length()>id_.length()) && (current.startsWith(id_)) )
        {
            setValue(current.substring(2));
            return true;
        }
        return false;
    }

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        String tmp = getValue();
        int index;

        // Get the argument name and the associated value
        index = tmp.indexOf("=");
        if (index != -1)
        {
            setArgument(tmp.substring(0,index));
            setValue(tmp.substring(index+1, tmp.length()));
        }
        else
        {
            setArgument(tmp);
            setValue("1");
        }
        arguments_.add( getArgument() );
        values_.add( getValue() );
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================

    /**
     * Gets the command line option argument associated value.
     *
     * @return The command line option argument associated value.
     */
    public String
    getValue()
    {
        return value_;
    }

    /**
     * Sets the command line option argument associated value.
     *
     * @param argument The command line argument associated option value.
     */
    public void
    setValue(String value)
    {
        value_ = value;
    }

    /**
     * Get all associated values for this option.
     *
     * @return An array of values.
     */
    public String[]
    getOptionArgumentValues()
    {
        return (String[])values_.toArray(new String[0]);
    }

}
