/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.command.options;

/**
 * This is a default implementation for options that can be specified more
 * that once. Moreover this class can consume one or two arguments of the
 * command line.
 * ex: -UNAME => Remove any definition for NAME
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class MultipleOptionArgument
     extends org.objectweb.util.cmdline.lib.DefaultOptionArgument
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The option identifier. */
    protected String id_;

    /** Consume another argument? */
    protected boolean consume_;

    /** Can one have twice the same option? */
    protected boolean multiple_;

    /** List of arguments declared with this option. */
    protected java.util.Vector arguments_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param id - The option identifier.
     * @param labels - The usage labels.
     * @param arguments - The usage arguments.
     * @param description - The usage description.
     * @param argument - The initial argument value.
     * @param multiple - Can one have twice the same option?
     */
    public
    MultipleOptionArgument( String id,
                            String[] labels,
                            String arguments,
                            String[] description,
                            String argument,
                            boolean multiple )
    {
        super(labels, arguments, description, argument);
        id_ = id;
        multiple_ = multiple;
        consume_ = true;
        arguments_ = new java.util.Vector();
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param id - The option identifier.
     * @param labels - The usage labels.
     * @param arguments - The usage arguments.
     * @param description - The usage description.
     * @param argument - The initial argument value.
     * @param multiple - Can one have twice the same option?
     * @param consume - Consume one more argument from the command line?
     */
    public
    MultipleOptionArgument( String id,
                            String[] labels,
                            String arguments,
                            String[] description,
                            String argument,
                            boolean multiple,
                            boolean consume )
    {
        super(labels, arguments, description, argument);
        id_ = id;
        multiple_ = multiple;
        consume_ = consume;
        arguments_ = new java.util.Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Checks the current command line argument.
     *
     * @param current The current command line argument.
     *
     * @return true if the option accepts this argument.
     */
    public boolean
    check(String current)
    {
        if ( current.equals(id_) )
            return true;

        if ( (current.length()>id_.length()) && (current.startsWith(id_)) && (!consume_))
        {
            setArgument(current.substring(2));
            arguments_.add( getArgument() );
            return true;
        }
        return false;
    }

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        if (!multiple_)
            checkAlreadySet(iterator);
        if (consume_)
            arguments_.add(consumeArgument(iterator));
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================

    /**
     * Get all arguments declared with this option.
     *
     * @return An array of arguments.
     */
    public String[]
    getOptionArguments()
    {
        return (String[])arguments_.toArray(new String[0]);
    }

}
