/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.generator.idl;

import org.objectweb.corba.ast.api.TypeKind;
import org.objectweb.corba.ast.api.Declaration;

/**
 * .
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.5
 */

public class IDLTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    IDLTranslator()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Translate a type into its idl format.
     *
     * @param typeref - The type reference to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDL(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        TypeKind tk = typeref.getTypeKind();

        if (tk == TypeKind.tk_null)
            return "null";
        else if (tk == TypeKind.tk_void)
            return "void";
        else if (tk ==  TypeKind.tk_short)
            return "short";
        else if (tk == TypeKind.tk_ushort)
            return "unsigned short";
        else if (tk == TypeKind.tk_long)
            return "long";
        else if (tk == TypeKind.tk_ulong)
            return "unsigned long";
        else if (tk == TypeKind.tk_longlong)
            return "long long";
        else if (tk == TypeKind.tk_ulonglong)
            return "unsigned long long";
        else if (tk == TypeKind.tk_float)
            return "float";
        else if (tk == TypeKind.tk_double)
            return "double";
        else if (tk == TypeKind.tk_longdouble)
            return "long double";
        else if (tk == TypeKind.tk_boolean)
            return "boolean";
        else if (tk == TypeKind.tk_char)
            return "char";
        else if (tk == TypeKind.tk_wchar)
            return "wchar";
        else if (tk == TypeKind.tk_octet)
            return "octet";
        else if (tk == TypeKind.tk_any)
            return "any";
        else if (tk == TypeKind.tk_Principal)
            return "::CORBA::Principal";
        else if (tk == TypeKind.tk_TypeCode)
            return "::CORBA::TypeCode";

        else if (tk == TypeKind.tk_objref)
            return "Object";

        else if ( (tk == TypeKind.tk_struct) ||
                  (tk == TypeKind.tk_union) ||
                  (tk == TypeKind.tk_enum) ||
                  (tk == TypeKind.tk_exception) ||
                  (tk == TypeKind.tk_alias) ||
                  (tk == TypeKind.tk_value_box) ||
                  (tk == TypeKind.tk_value) ||
                  (tk == TypeKind.tk_native) ||
                  (tk == TypeKind.tk_abstract_interface) ||
                  (tk == TypeKind.tk_local_interface) ||
                  (tk == TypeKind.tk_interface) ||
                  (tk == TypeKind.tk_component) ||
                  (tk == TypeKind.tk_event) ||
                  (tk == TypeKind.tk_home) )
        {
                Declaration decl = (Declaration)typeref;
                return decl.getAbsoluteName();
        }
        else if (tk == TypeKind.tk_string)
        {
                int l = ((org.objectweb.corba.ast.api.TypeRefWithLength)typeref).getLength();
                if (l==0)
                    return "string";
                return "string<"+l+">";
        }
        else if (tk == TypeKind.tk_wstring)
        {
                int l = ((org.objectweb.corba.ast.api.TypeRefWithLength)typeref).getLength();
                if (l==0)
                    return "wstring";
                return "wstring<"+l+">";
        }
        else if (tk == TypeKind.tk_sequence)
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                int l = type.getLength();
                String str = toIDL(type.getContentType());
                if (l==0)
                    return "sequence<"+str+">";
                return "sequence<"+str+","+l+">";
        }
        else if (tk == TypeKind.tk_array)
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                String str = toIDL(type.getContentType());
                return str;
        }
        else if (tk == TypeKind.tk_fixed)
        {
                org.objectweb.corba.ast.api.FixedTypeRef type = (org.objectweb.corba.ast.api.FixedTypeRef)typeref;
                return "fixed<"+type.getDigits()+","+type.getScale()+">";
        }
        else
                return "";
    }

    /**
     * Translate an array into its idl format.
     *
     * @param typeref - The array to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDLPostfix(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        String res = "";

        if ( typeref.getTypeKind() == TypeKind.tk_array )
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = null;
                type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                res = toIDLPostfix(type.getContentType())+"["+type.getLength()+"]";
        }
        return res;
    }

    /**
     * Get the string value of an Any.
     *
     * @param anyvalue - The Any to translate.
     * @param typeref - The value's type.
     *
     * @return The resulting string.
     **/
    public String
    toString(org.objectweb.corba.ast.api.AnyValue anyvalue,
             org.objectweb.corba.ast.api.TypeRef typeref)
    {
        //
        //  TODO : add escaped sequencesto strings, eg '\n' => "\\n"
        //
        if (anyvalue.isInteger())
            return java.lang.String.valueOf(anyvalue.getAsInteger());
        else if (anyvalue.isString())
            return "\""+anyvalue.getAsString()+"\"";
        else if (anyvalue.isWString())
            return "L\""+anyvalue.getAsWString()+"\"";
        else if (anyvalue.isChar())
            return "\'"+java.lang.String.valueOf(anyvalue.getAsChar())+"\'";
        else if (anyvalue.isWChar())
            return "L\'"+java.lang.String.valueOf(anyvalue.getAsWChar())+"\'";
        else if (anyvalue.isFloating())
            return java.lang.String.valueOf(anyvalue.getAsFloating());
        else if (anyvalue.isFixed())
            return anyvalue.getAsFixed();
        else if (anyvalue.isBoolean())
        {
            if (anyvalue.getAsBoolean())
                return "TRUE";
            else
                return "FALSE";
        }
        else if (anyvalue.isEnum())
        {
            // build enum member absolute name
            String abs_name = ((org.objectweb.corba.ast.api.EnumDecl)typeref).getParent().getAbsoluteName();
            return abs_name + "::" + anyvalue.getAsEnum();
        }
        else
            return "";
    }

}
