/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.psdl;

/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;


public class PSDLGenerator
     extends org.objectweb.corba.generator.idl.IDL3Generator
{

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param root - The file scope containing all declarations.
     */

    public PSDLGenerator(org.objectweb.corba.ast.api.Scope root)
    {
        // Call the IDL3Generator constructor
        super(root);

        // Init internal states
        translator_ = new org.objectweb.corba.generator.psdl.PSDLTranslator();
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     **/
    public void
    initialize(String outputfile, String app_name)
    {
        java.util.Vector vect = new java.util.Vector();

        /** Initialize environment **/
    // TO DO : retrieve elements below from xml properties file

        // The ressource path
        vect.add(System.getProperty("ccm.templates"));
        setRessourcePath(vect);

        // Templates to use.
        vect.clear();
        vect.add("idl3.vm");
        vect.add("psdl.vm");
        setLibrary(vect);

        // The output file
        setOutputName(outputfile);

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
        put("gen", this);
        put("translator", translator_);
        put("declarator", "");
        put("limited_types" , "dk_psdl");
    }

    // ===========================================================
    //
    // Useful methods for PSDL Mapping.
    //
    // ===========================================================

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        return ( DeclarationKind.dk_abstract_storage_home +
                 DeclarationKind.dk_abstract_storage_type +
                 DeclarationKind.dk_storage_type );
    }


    // ===========================================================
    //
    // Methods for PSDL Mapping.
    //
    // ===========================================================

}
