/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.util;

/**
 * BasicLoggerImpl is a basic logger implementation.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 *
 * TODO: The implementation must use MonoLog!!!
 */

public class BasicLoggerImpl
  implements Logger
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * This boolean indicates if trace logging is active or not.
     */
    public static final boolean isTraceLogged = false; // true;

    /**
     * This boolean indicates if error logging is active or not.
     */
    public static final boolean isErrorLogged = false; // true;

    /**
     * Output where trace and error messages are logged.
     */
    protected java.io.PrintStream output_ = System.err;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    BasicLoggerImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for Logger
    //
    // ==================================================================

    /**
     * Logs a trace message.
     *
     * @param caller The caller object.
     * @param message The trace message to log.
     *
     * TODO: The implementation must use MonoLog!!!
     */
    public
    void trace(Object caller,
               String message)
    {
        if(isTraceLogged) {
            output_.println(caller + ": " + message + '.');
	}
    }

    /**
     * Logs an error message.
     *
     * @param caller The caller object.
     * @param message The error message to log.
     *
     * TODO: The implementation must use MonoLog!!!
     */
    public
    void error(Object caller,
               String message)
    {
        if(isErrorLogged) {
            try {
                throw new Error(message);
            } catch (Error exc) {
                output_.print(caller + ": " + message + "!!!");
                exc.printStackTrace();
            }
	}
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
}
