/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.api;

/**
 * This interface represents command lines.
 *
 * Command lines have an associated usage.
 *
 * This provides the following properties:
 *
 * - Options: The command line options.
 *
 * This provides the following methods:
 *
 * - addOption: Adds a command line option.
 * - parse:     Parses command line arguments.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public interface CommandLine
         extends Usage
{
    /**
     * Gets the command line options.
     *
     * @return The command line options.
     */
    public Option[]
    getOptions();

    /**
     * Adds a command line option.
     *
     * @param option The command line option object.
     */
    public void
    addOption(Option option);

    /**
     * Parses given command line arguments and 
     * feeds the command line option objects.
     *
     * @param args The command line arguments.
     *
     * @return The command line arguments not parsed.
     */
    public String[]
    parse(String[] args);
}
