/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.api;

/**
 * This interface provides an iterator on command line arguments.
 *
 * This provides the following properties:
 *
 * - CommandLine: The associated command line.
 * - Current: The current command line argument.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public interface Iterator
{
    /**
     * Gets the associated command line.
     *
     * @return The associated command line.
     */
    public CommandLine
    getCommandLine();

    /**
     * Gets the current argument.
     *
     * @return The current argument in the iteration.
     */
    public String
    getCurrent();

    /**
     * Indicates if the iterator has more arguments.
     *
     * @return true if the iteration has more arguments.
     *  (In other words, returns true if next would return an
     *  argument rather than throwing an exception.)
     */
    public boolean
    hasNext();

    /**
     * Obtains the next argument in the iteration.
     *
     * @return The next argument in the iteration.
     *
     * @exception java.util.NoSuchElementException
     *            Thrown when iteration has no more arguments.
     */
    public String
    next();

    /**
     * Remove from the underlying collection the last argument
     * returned by the iterator.
     */
    public void
    remove();
}
