/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.api;

/**
 * The Option interface offers an abstraction of command line options.
 *
 * Command line options have an associated usage.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public interface Option
         extends Usage
{
    /**
     * Is it a mandatory option?
     *
     * @return true if it is a mandatory option.
     */
    public boolean
    isMandatory();

    /**
     * Sets the mandatory option status.
     *
     * @param mandatory true if it is a mandatory option.
     */
    public void
    setMandatory(boolean mandatory);

    /**
     * Obtains the Set status of the option.
     *
     * @return true if the option was set during
     *         the command line argument parsing.
     */
    public boolean
    isSet();

    /**
     * Checks the current command line argument.
     *
     * @param current The current command line argument.
     *
     * @return true if the option accepts this argument.
     */
    public boolean
    check(String current);

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(Iterator iterator);
}
