/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

import org.objectweb.util.cmdline.api.CommandLine;

/**
 * This interface provides an iterator on command line arguments.
 *
 * This provides the following properties:
 *
 * - CommandLine: The associated command line.
 * - Current: The current command line argument.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultIterator 
  implements org.objectweb.util.cmdline.api.Iterator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated command line. */
    protected CommandLine commandLine_;

    /** The current argument. */
    protected String current_ = null;

    /** To store command line arguments. */
    private java.util.Vector arguments_;

    /** To store the command line arguments iterator. */
    protected java.util.Iterator iterator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param args The command line arguments.
     * @param commandLine The associated command line.
     */
    public
    DefaultIterator(String[] args,
	            CommandLine commandLine)
    {
        commandLine_ = commandLine;

        current_ = null;

        arguments_ = new java.util.Vector();
        for(int i=0; i<args.length; i++)
          arguments_.addElement(args[i]);

        iterator_ = arguments_.iterator();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Gets all the command line options.
     *
     * @return All the command line options.
     */
    protected String[]
    getUnparsedArguments()
    {
        return ((String[])(arguments_.toArray(new String[0])));
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Iterator
    //
    // ==================================================================

    /**
     * Gets the associated command line.
     *
     * @return The associated command line.
     */
    public CommandLine
    getCommandLine()
    {
        return commandLine_;
    }

    /**
     * Gets the current argument.
     *
     * @return The current argument in the iteration.
     */
    public String
    getCurrent()
    {
        return current_;
    }

    /**
     * Indicates if the iterator has more arguments.
     *
     * @return true if the iteration has more arguments.
     *  (In other words, returns true if next would return an
     *  argument rather than throwing an exception.)
     */
    public boolean
    hasNext()
    {
        return iterator_.hasNext();
    }

    /**
     * Obtains the next argument in the iteration.
     *
     * @return The next element in the iteration.
     *
     * @exception java.util.NoSuchElementException
     *            Thrown when iteration has no more elements.
     */
    public String
    next()
    {
        current_ = (String)iterator_.next();
        return current_;
    }

    /**
     * Remove from the underlying collection the last argument
     * returned by the iterator.
     */
    public void
    remove()
    {
        current_ = null;
        iterator_.remove();
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
