/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

/**
 * This is a default implementation of the
 * org.objectweb.util.cmdline.api.OptionArgument interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultOptionArgument
     extends DefaultOptionBase
  implements org.objectweb.util.cmdline.api.OptionArgument
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The option argument. */
    protected String argument_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * It inits the Labels, Arguments, and Description usage properties
     * with an empty string.
     */
    public
    DefaultOptionArgument()
    {
        this("", "", (String[])null, null);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionArgument(String label,
                          String arguments,
                          String description,
                          String argument)
    {
        this(new String[] { label }, arguments,
             new String[] { description }, argument);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionArgument(String[] labels,
                          String arguments,
                          String description,
                          String argument)
    {
        this(labels, arguments, new String[] { description }, argument);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionArgument(String label,
                          String arguments,
                          String[] description,
                          String argument)
    {
        this(new String[] { label }, arguments, description, argument);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionArgument(String[] labels,
                          String arguments,
                          String[] description,
                          String argument)
    {
        super(labels, arguments, description);
        setArgument(argument);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    /**
     * Appends internal state representation into the given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     */
    protected void
    appendInternalState(StringBuffer sb)
    {
        super.appendInternalState(sb);
        append(sb, "argument", getArgument());
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        checkAlreadySet(iterator);
        setArgument(consumeArgument(iterator));
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.OptionArgument
    //
    // ==================================================================

    /**
     * Gets the command line option argument.
     *
     * @return The command line option argument.
     */
    public String
    getArgument()
    {
        return argument_;
    }

    /**
     * Sets the command line option argument.
     *
     * @param argument The command line option argument.
     */
    public void
    setArgument(String argument)
    {
        argument_ = argument;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
