/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

/**
 * This is a default implementation of the
 * org.objectweb.util.cmdline.api.OptionFlag interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultOptionFlag
     extends DefaultOptionBase
  implements org.objectweb.util.cmdline.api.OptionFlag
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The option flag. */
    protected boolean flag_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * It inits the Labels, Arguments, and Description usage properties
     * with an empty string.
     */
    public
    DefaultOptionFlag()
    {
        this("", (String[])null, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionFlag(String label,
                      String description,
                      boolean flag)
    {
        this(new String[] { label }, description, flag);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionFlag(String[] labels,
                      String description,
                      boolean flag)
    {
        this(labels, new String[] { description }, flag);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionFlag(String label,
                      String[] description,
                      boolean flag)
    {
        this(new String[] { label }, description, flag);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param description The usage description.
     * @param flag The initial flag value.
     */
    public
    DefaultOptionFlag(String[] labels,
                      String[] description,
                      boolean flag)
    {
        super(labels, new String[0], description);
        setFlag(flag);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    /**
     * Appends internal state representation into the given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     */
    protected void
    appendInternalState(StringBuffer sb)
    {
        super.appendInternalState(sb);
        append(sb, "flag", getFlag());
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        checkAlreadySet(iterator);
        flag_ = true;
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.OptionFlag
    //
    // ==================================================================

    /**
     * Gets the flag status.
     *
     * @return The flag status.
     */
    public boolean
    getFlag()
    {
        return flag_;
    }

    /**
     * Sets the flag status.
     *
     * @param flag The flag status.
     */
    public void
    setFlag(boolean flag)
    {
        flag_ = flag;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
