/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

/**
 * This is a default implementation of the
 * org.objectweb.util.cmdline.api.Usage interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultUsage
     extends org.objectweb.util.lib.DefaultPrintableBase
  implements org.objectweb.util.cmdline.api.Usage
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The usage labels. */
    protected String[] labels_;

    /** The usage arguments. */
    protected String[] arguments_;

    /** The usage description. */
    protected String[] description_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * It inits the Labels, Arguments, and Description properties
     * with an empty string.
     */
    public
    DefaultUsage()
    {
        this(new String[] { "" }, new String[0], null);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description properties.
     *
     * @param label The usage label.
     * @param argument The usage argument.
     * @param description The usage description.
     */
    public
    DefaultUsage(String label,
                 String argument,
                 String[] description)
    {
        this(new String[] { label }, new String[] { argument }, description);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     */
    public
    DefaultUsage(String label,
                 String[] arguments,
                 String[] description)
    {
        this(new String[] { label }, arguments, description);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description properties.
     *
     * @param labels The usage labels.
     * @param argument The usage argument.
     * @param description The usage description.
     */
    public
    DefaultUsage(String[] labels,
                 String argument,
                 String[] description)
    {
        this(labels, new String[] { argument }, description);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     */
    public
    DefaultUsage(String[] labels,
                 String[] arguments,
                 String[] description)
    {
        setLabels(labels);
        setArguments(arguments);
        setDescription(description);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    /**
     * Appends internal state representation into the given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     */
    protected void
    appendInternalState(StringBuffer sb)
    {
        append(sb, "labels", getLabels());
        append(sb, "arguments", getArguments());
        append(sb, "description", getDescription());
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Usage
    //
    // ==================================================================

    /**
     * Gets the usage labels.
     *
     * @return The usage labels.
     */
    public String[]
    getLabels()
    {
        return labels_;
    }

    /**
     * Sets the usage labels.
     *
     * @param labels The usage labels.
     */
    public void
    setLabels(String[] labels)
    {
        labels_ = labels;
    }

    /**
     * Gets the usage arguments.
     *
     * @return The usage arguments.
     */
    public String[]
    getArguments()
    {
        return arguments_;
    }

    /**
     * Sets the usage arguments.
     *
     * @param arguments The usage arguments.
     */
    public void
    setArguments(String[] arguments)
    {
        arguments_ = arguments;
    }

    /**
     * Gets the usage description.
     *
     * @return The usage description.
     */
    public String[]
    getDescription()
    {
        return description_;
    }

    /**
     * Sets the usage description.
     *
     * @param description The usage description.
     */
    public void
    setDescription(String[] description)
    {
        description_ = description;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
