/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

import org.objectweb.util.lib.StringBufferHelper;

import org.objectweb.util.cmdline.api.Usage;

/**
 * Some helper functions for the interface
 * org.objectweb.util.cmdline.api.Usage.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class UsageHelper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * Gets the given usage labels as string.
     *
     * @param usage The given usage.
     * @param separator The given separator.
     *
     * @return The usage labels as string.
     */
    static public String
    usageLabelsAsString(Usage usage,
                        String separator)
    {
        StringBuffer sb = new StringBuffer();
        StringBufferHelper.append(sb, usage.getLabels(), separator);
        return sb.toString();
    }

    /**
     * Gets the given usage arguments as string.
     *
     * @param usage The given usage.
     * @param separator The given separator.
     *
     * @return The usage arguments as string.
     */
    static public String
    usageArgumentsAsString(Usage usage,
                           String separator)
    {
        StringBuffer sb = new StringBuffer();
        StringBufferHelper.append(sb, usage.getArguments(), separator);
        return sb.toString();
    }

    /**
     * Gets the given usage description as string.
     *
     * @param usage The given usage.
     * @param separator The given separator.
     *
     * @return The usage description as string.
     */
    static public String
    usageDescriptionAsString(Usage usage,
                             String separator)
    {
        StringBuffer sb = new StringBuffer();
        StringBufferHelper.append(sb, usage.getDescription(), separator);
        return sb.toString();
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
