/*====================================================================

ObjectWeb Util Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.lib;

/**
 * This is an abstract default implementation of the interface
 * org.objectweb.util.api.StringBufferizable.
 *
 * Moreover, this implements the java.lang.Object.toString method.
 *
 * Subclasses must implement the appendInternalState method.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class DefaultStringBufferizableBase
              extends StringBufferHelper
           implements org.objectweb.util.api.StringBufferizable
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    DefaultStringBufferizableBase()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Appends internal state representation into the given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     */
    abstract protected void
    appendInternalState(StringBuffer sb);

    // ==================================================================
    //
    // Public methods for class java.lang.Object
    //
    // ==================================================================

    /**
     * Returns a string representation of the target object.
     *
     * By default:
     * getClass().getName() + '[' + { name=value } + ']'
     *
     * @return A string representation of the target object.
     */
    public String
    toString()
    {
        // The string buffer into the string representation is appended.
        StringBuffer sb = new StringBuffer();
        toStringBuffer(sb);
        return sb.toString();
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.api.StringBufferizable
    //
    // ==================================================================

    /**
     * Puts the target object to a java.lang.StringBuffer buffer.
     *
     * By default:
     * getClass().getName() + '[' + { name=value } + ']'
     *
     * @param sb The java.lang.StringBuffer buffer.
     */
    public void
    toStringBuffer(java.lang.StringBuffer sb)
    {
        // Appends the full class name.
        sb.append(getClass().getName());
        sb.append('[');

        // Appends internal state to the string buffer.
        appendInternalState(sb);

        sb.append(']');
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
