/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Romain Rouvoy_______________________________.

====================================================================*/

package org.objectweb.ccm.demo1;

/**
 * This is the implementation of the OMG IDL3 demo1::Client component type.
 *
 * This class inherits from the local CCM_Client interface
 * generated by the OpenCCM's IDL3 to IDL2 mapping generator.
 *
 * This class also implements the java.awt.event.ActionListener to manage
 * users' interactions on the Swing GUI associated to this component.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</A>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</A>
 */

public class ClientImpl
     extends org.omg.CORBA.LocalObject
  implements CCM_Client,
             org.omg.Components.SessionComponent,
             java.awt.event.ActionListener
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The name of the component. */ 
    private String name_;

    /** To refer to the GUI frame. */
    private javax.swing.JFrame frame_;

    /** To refer to the user's input field. */
    private javax.swing.JTextField text_;

    /** To refer to the component context. */
    private CCM_Client_Context the_context_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ClientImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // Checks if the configuration is completed.
	if(name_ == null)
            throw new org.omg.Components.InvalidConfiguration();

	// Check if the connection to the server is set.
	if(the_context_.get_connection_to_server() == null)
            throw new org.omg.Components.InvalidConfiguration();

        // Instantiating the GUI.
				
        // Creates a Swing Frame.
        frame_ = new javax.swing.JFrame(name_ + "'s Client GUI");
        // Sets its size.
        frame_.setSize(400, 300);

        // Creates a text field for user's inputs.
        text_ = new javax.swing.JTextField("", 1);

        // Creates a button to invoke the connected server component.
        javax.swing.JButton button = new javax.swing.JButton("Print Text");
        button.addActionListener(this);

        // Constructs and shows the GUI.
        javax.swing.JPanel panel = new javax.swing.JPanel(
                                           new java.awt.BorderLayout());
        frame_.getContentPane().add(panel);
        panel.add(new javax.swing.JScrollPane(text_),
                  java.awt.BorderLayout.CENTER);
        panel.add(button, java.awt.BorderLayout.SOUTH);
        frame_.pack();
        frame_.show();
    }
		
    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Set the session component context.
     *
     * @param context The session component context.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    set_session_context(org.omg.Components.SessionContext context)
    throws org.omg.Components.CCMException
    {
        the_context_ = (CCM_Client_Context)context;
    }

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove() 
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        frame_.dispose();
	frame_ = null;
    }				

    // ==================================================================
    //
    // Methods for the OMG IDL demo1::CCM_Client
    //
    // ==================================================================

    /**
     * The mutator method for the attribute name.
     *
     * @param n The name.
     */
    public void
    the_name(String n)
    {
        name_ = n;

        if (frame_ != null)
            frame_.setTitle(name_ + "'s Client GUI");
    }

    /**
     * The accessor method for the attribute name.
     *
     * @return The name.
     */
    public String
    the_name()
    {
        return name_;
    }

    // ==================================================================
    //
    // Methods for interface java.awt.event.ActionListener
    //
    // ==================================================================

    /**
     * When the button is selectionned.
     *
     * @param e The associate ActionEvent.
     */
    public void
    actionPerformed(java.awt.event.ActionEvent e)
    {
        // Obtain the object reference associated to the
        // 'to_server' receptacle.
        Display to_server = the_context_.get_connection_to_server();

	// Check if the connection is available.
	if(to_server == null)
	{
	    System.err.println("The demo1::Client::to_server receptacle is not set!");
	    return;
	}

        // Calls the print service.
        to_server.print(name_ + ":" + text_.getText());
    }
}
