/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * This is the implementation of the OMG IDL3
 * DiningPhilosophers::ForkManager component type.
 *
 * This class inherits from the local CCM_ForkManager interface
 * generated by the OpenCCM's IDL3 to IDL2 mapping generator.
 *
 * The provided the_fork facet is directly implemented by the
 * component class by implementing the DiningPhilosophers::Fork interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 */

public class ForkManagerImpl
     extends org.omg.CORBA.LocalObject
  implements CCM_ForkManager,
             CCM_Fork,
             org.omg.Components.SessionComponent
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Tells if the fork is available. */
    private boolean available_;

    /** The fork manager CCM context. */
    private CCM_ForkManager_Context the_context_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ForkManagerImpl()
    {
        available_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.     
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // Nothing to do.
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Setting the session component context.
     *
     * @param context The session component context.
     */
    public void
    set_session_context(org.omg.Components.SessionContext context)
    throws org.omg.Components.CCMException
    {
        the_context_ = (CCM_ForkManager_Context)context;
    }

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove() 
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    // ==================================================================
    //
    // Methods for OMG IDL DiningPhilosophers::CCM_ForkManager
    //
    // ==================================================================

    /**
     * The component must provide the executor object implementing
     * the the_fork Facet.
     */
    public CCM_Fork
    get_the_fork()
    {
        // Returns the component because it implements the Fork facet.
        return this;
    }

    // ==================================================================
    //
    // Methods for OMG IDL DiningPhilosophers::Fork
    //
    // ==================================================================

    /**
     * Obtain the fork.
     *
     * @exception InUse
     *            Thrown if the fork has already used by another philosopher.
     */
    public void
    get() 
    throws InUse
    {
        // Check if there is no current philosopher.
        if (! available_) 
        {
            throw new InUse();
        }

        // The philosopher is now the current fork user.
        available_ = false;
    }

    /**
     * Release the fork.
     */
    public void
    release()
    {
        // Check if the philosopher is the current one.
        if (available_) 
        {
            return;
        }

        // There is no current fork user.
        available_ = true;        
    }
}
