/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This class is a basic implementation of the CCMHome interface.
 * It also implements all generic operations for CCM homes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class CCMHomeBase
extends org.omg.CORBA.LocalObject
implements org.omg.Components.CCMHomeOperations,
	   org.omg.Components.HomeConfigurationOperations,
           org.objectweb.ccm.Deployment.CCMHomeLocal
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected org.objectweb.ccm.Deployment.listener.CCMHomeListener listener_; 

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    protected
    CCMHomeBase()
    {
				listener_ = null;
				org.objectweb.ccm.Deployment.listener.TheCCMHomeConfigurator.get_instance().configure_ccm_home(this);
    }

    // ==================================================================
    //
    // Internal methods that must be implemented by subclasses.
    //
    // ==================================================================

    /**
     **
     **/
    protected abstract java.lang.String
    _component_uid_();

    /**
     **
     **/
    protected abstract java.lang.String
    _home_uid_();

    /**
     **
     **/
    protected abstract org.objectweb.ccm.Containers.HomeServant
    _home_servant_();

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Create a CCM component reference.
     **
     **  @return The CCM component reference.
     **/
    protected final org.omg.Components.CCMObject
    _create_with_instance(org.omg.Components.EnterpriseComponent comp)
    {
        org.omg.Components.CCMObject ref = null;

        // create the reference
        ref = _home_servant_().the_PCA().create_component_reference_with_instance(_home_servant_(), comp);
				listener_.on_create(ref); 
        return ref;
    }

    /**
     **  Create a CCM component reference.
     **
     **  @return The CCM component reference.
     **/
    protected final org.omg.Components.CCMObject
    _find_with_instance(org.omg.Components.EnterpriseComponent comp)
    {
        org.omg.Components.CCMObject ref = null;

        // find the reference
        ref = _home_servant_().the_PCA().find_component_reference_with_instance(_home_servant_(), comp);
        return ref;
    }

    // ==================================================================
    //
    // Public methods.  
    //
    // ==================================================================
		/**
		 ** Add a new plugin to the list of deployment plugins
		 **
		 ** @param listener the plugin listener to add 
		 **/
    public void
    add_listener (org.objectweb.ccm.Deployment.listener.CCMHomeListener listener)
    {	
				if (listener_ == null)
						listener_ = listener;
				else
						listener_.set_delegate(listener);
    }

    // ==================================================================
    //
    // Methods for the Components::CCMHome interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/CCMHome/get_component_def:1.0
    //
    /**
     **  Obtains the component definition into the Interface Repository.
     **
     **  @return The reference to the component definition.
     **/
    public final org.omg.CORBA.IRObject
    get_component_def()
    {
        // Obtain the Interface Repository.
        org.omg.CORBA.Repository ir =
						org.objectweb.ccm.CORBA.TheInterfaceRepository.getRepository();
				
				// Check if the Interface Repository is known.
        if(ir != null)
				{
						throw new org.omg.CORBA.INTF_REPOS();
				}

				// Lookup the managed component repository Id.
				return ir.lookup_id(_component_uid_());
    }

    //
    // IDL:omg.org/Components/CCMHome/get_home_def:1.0
    //
    /**
     **  Obtains the home definition into the Interface Repository.
     **
     **  @return The reference to the home definition.
     **/
    public final org.omg.CORBA.IRObject
    get_home_def()
    {
        // Obtain the Interface Repository.
        org.omg.CORBA.Repository ir =
						org.objectweb.ccm.CORBA.TheInterfaceRepository.getRepository();
				
				// Check if the Interface Repository is known.
        if(ir != null)
				{
						throw new org.omg.CORBA.INTF_REPOS();
				}
				
				// Lookup the home repository Id.
				return ir.lookup_id(_home_uid_());
    }

    //
    // IDL:omg.org/Components/CCMHome/remove_home:1.0
    //
    /**
     **
     **/
    public void
    remove_home()
    throws org.omg.Components.RemoveFailure
    {
        _home_servant_().remove();
    }

    // ==================================================================
    //
    // Methods for the Components::HomeConfiguration interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/HomeConfiguration/set_configurator:1.0
    //
    /**
     **  Sets the configurator associated to this home.
     **
     **  @param cfg The configurator.
     **/
    public final void
    set_configurator(org.omg.Components.Configurator cfg)
    {
				// TODO: Must be implemented!
				throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/HomeConfiguration/set_configuration_values:1.0
    //
    /**
     **  Sets the configuration values.
     **
     **  @param config The set of configuration values.
     **/
    public final void
    set_configuration_values(org.omg.Components.ConfigValue[] config)
    {
				// TODO: Must be implemented!
				throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/HomeConfiguration/complete_component_configuration:1.0
    //
    /**
     **  Completes component configuration.
     **
     **  @param b Complete or not component configuration.
     **/
    public final void
    complete_component_configuration(boolean b)
    {
				// TODO: Must be implemented!
				throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/HomeConfiguration/disable_home_configuration:1.0
    //
    /**
     **  Disables home configuration.
     **/
    public final void
    disable_home_configuration()
    {
				// TODO: Must be implemented!
				throw new org.omg.CORBA.NO_IMPLEMENT();
    }
}










