/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * This interface ??????.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class CallContextBase
                extends org.omg.CORBA.LocalObject
                implements CallContext
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected HomeServant the_home_servant_;

    /**
     **
     **/
    protected java.lang.String the_uid_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    CallContextBase(HomeServant servant,
                    java.lang.String uid)
    {
        the_home_servant_ = servant;
        the_uid_ = uid;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected abstract SystemConfiguration
    getConfiguration();

    // ==================================================================
    //
    // Public methods for the CallContext interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallContext/get_call_coordinator:1.0
    //
    /**
     **
     **/
    public CallCoordinator
    get_call_coordinator(java.lang.String op_name)
    {
        SystemConfiguration config = getConfiguration();

        if (config == null)
          return new org.objectweb.ccm.Containers.Plugins.EmptyCoordinator();

        SystemHomeFinder finder = the_home_servant_.the_home_finder();
        CallCoordinator coord = config.create_call_coordinator(the_uid_, op_name, finder);
        CallController[] crtls = config.create_call_controllers(the_uid_, op_name, finder);
        for (int i=0;i<crtls.length;i++)
            coord.add(crtls[i]);

        return coord;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallContext/duplicate:1.0
    //
    /**
     **
     **/
    public CallContext
    duplicate(java.lang.String op_name)
    {
        CallContext child = new OperationCallContext(this, op_name);
        return child;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallContext/uid:1.0
    //
    /**
     **
     **/
    public java.lang.String
    uid()
    {
        return the_uid_;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallContext/operation_name:1.0
    //
    /**
     **
     **/
    public java.lang.String
    operation_name()
    {
        // not implemented by this class
        // call this operation only on a duplicated context.
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }
}
