/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * This interface ??????.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentServantImpl
       extends org.omg.CORBA.LocalObject
       implements ComponentServant,
                  ActivationContext
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected HomeServant the_home_servant_;

    /**
     **
     **/
    protected ComponentExecutor the_component_executor_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.WeakTable active_objects_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.WeakTable activation_coords_;

    /**
     **
     **/
    protected org.omg.CORBA.Object[] the_port_references_;

    /**
     **
     **/
    protected org.omg.Components.CCMObject the_component_ref_;

    /**
     **
     **/
    protected org.omg.Components.EnterpriseComponent the_component_instance_;

    /**
     **
     **/
    protected short state_;

    /**
     **
     **/
    protected org.omg.Components.PrimaryKeyBase the_primary_key_;

    /**
     **
     **/
    protected ServantLocatorImpl the_servant_locator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    ComponentServantImpl(ServantLocatorImpl locator,
                         HomeServant home_servant,
                         org.omg.Components.EnterpriseComponent instance,
                         org.omg.Components.PrimaryKeyBase key)
    {
        the_servant_locator_ = locator;
        the_home_servant_ = home_servant;
        the_primary_key_ = key;
        the_component_executor_ = null;
        the_port_references_ = new org.omg.CORBA.Object[home_servant.the_home_executor()._the_port_uids().length];
        for (int i=0;i<the_port_references_.length;i++)
            the_port_references_[i] = null;
        the_component_ref_ = null;
        active_objects_ = null;
        activation_coords_ = null;
        state_ = ActivationContext.DEACTIVATED;
        initializeComponentExecutor(instance);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Cast an EntrepriseComponent executor to a SessionComponent executor.
     *
     * @param executor The EntrepriseComponent executor.
     *
     * @return As a SessionComponent or null.
     */
    protected org.omg.Components.SessionComponent
    castToSessionComponent(org.omg.Components.EnterpriseComponent executor)
    {
        return (executor instanceof org.omg.Components.SessionComponent)?
	  (org.omg.Components.SessionComponent)executor : null;
    }

    /**
     **
     **/
    protected void
    initializeComponentExecutor(org.omg.Components.EnterpriseComponent instance)
    {
        if (instance==null)
            return ;

        // create the interceptor hashtable.
        active_objects_ = new org.objectweb.ccm.util.WeakTable();

        // create the activation coordinators hashtable.
        activation_coords_ = new org.objectweb.ccm.util.WeakTable();

        HomeExecutorBase home = the_home_servant_.the_home_executor();

        // create the executor.
        the_component_executor_ = home._get_component_executor();

        // set the executor as the context for the enterprise component.
        org.omg.Components.SessionComponent sc = castToSessionComponent(instance);
	if(sc != null)
	{
            try
            {
                sc.set_session_context((org.omg.Components.SessionContext)the_component_executor_);
            } catch(org.omg.Components.CCMException ex) {
              //
              //  TODO : what ????
              //
                throw new Error("Activation failed");
            }
	}

        // check if the instance is a ExecutorLocator.
        org.omg.Components.ExecutorLocator locator = null;
        if (instance instanceof org.omg.Components.ExecutorLocator)
            locator = (org.omg.Components.ExecutorLocator)instance;
        // else create a wrapper.
        else
            locator = home._get_executor_locator(instance);

        // set the infos needed by the component executor.
        the_component_executor_._executor_locator(locator);
        the_component_executor_._component_servant(this);
    }


    // ==================================================================
    //
    // Public methods for the ComponentServant interface.
    //
    // ==================================================================

    /**
     **
     **/
    public org.omg.CORBA.Object
    get_reference(java.lang.String uid)
    {
        HomeExecutorBase home = the_home_servant_.the_home_executor();
        java.lang.String comp_uid = home._the_component_uid();
        if (comp_uid.equals(uid))
            return the_component_ref_;

        java.lang.String[] uids = home._the_port_uids();
        int i=0;
        while (!uids[i].equals(uid)) 
            i++;

        if (the_port_references_[i]==null)
            the_port_references_[i] = the_servant_locator_.createPortReference(this, uid);

        return the_port_references_[i];
    }

    /**
     **
     **/
    public Interceptor
    get_native_interface(java.lang.String uid)
    {
        HomeExecutorBase home = the_home_servant_.the_home_executor();

        // Pass through the activation interception first.
        // Check if the coordinator has already been created.
/*
 * TODO : think more about this point
 *
        ActivationCoordinator coord = (ActivationCoordinator)activation_coords_.get(uid);
        if (coord==null)
        {
            ActivationController[] ctrls = null;
            SystemConfiguration comp_config = the_home_servant_.the_component_configuration();
            coord = comp_config.create_activation_coordinator(uid, the_home_servant_.the_home_finder());
            ctrls = comp_config.create_activation_controllers(uid, the_home_servant_.the_home_finder());
            for (int i=0;i<ctrls.length;i++)
                coord.add(ctrls[i]);

            // store the coordinator.
            activation_coords_.put(uid, coord);
        }

        // call the preinvoke operation
        coord.preinvoke(this);
 */

        // check if the associated interceptor is not activated
        Interceptor inter = (Interceptor)active_objects_.get(uid);

        if (inter==null)
        {
            // retrieve the interceptor
            inter = home._get_interceptor(uid);

            // create the call context for the interceptor and set the needed infos
            CallContext context = new ComponentCallContext(the_home_servant_, uid);
            inter._call_context(context);

            // store the interceptor
            active_objects_.put(uid, inter);
        }

        // retrieve the name.
        int idx = uid.lastIndexOf('/');
        if (idx==-1) idx = uid.indexOf(':');
        int idx2 = uid.lastIndexOf(':');
        java.lang.String name = uid.substring(idx+1, idx2);

        // obtain the executor.
        try
        {
            org.omg.CORBA.Object exe =
                 the_component_executor_._executor_locator().obtain_executor(name);

            // special case for the component itself
            if (uid.equals(home._the_component_uid()))
            {
                the_component_executor_._delegate((org.omg.Components.EnterpriseComponent)exe);
                inter._delegate((java.lang.Object)the_component_executor_);
            }
            else
            {
                inter._delegate(home._get_executor_wrapper(exe, uid));
            }
        }
        catch(org.omg.Components.CCMException ex)
        {
            //
            //  TODO : what ???
            //
        }

        return inter;
    }

    /**
     **
     **/
    public void
    release_native_interface(Interceptor inter)
    {
        HomeExecutorBase home = the_home_servant_.the_home_executor();        
        java.lang.String uid = inter._call_context().uid();

        // Pass through the activation interception first.
        // The coordinator should always exist.
/*
 * TODO : think more about this point
 *
        ActivationCoordinator coord = (ActivationCoordinator)activation_coords_.get(uid);

        // call the postinvoke operation
        coord.postinvoke(this);
*/

        try
        {
            org.omg.CORBA.Object exe = null;
            // special case for the component itself
            if (uid.equals(home._the_component_uid()))
            {
                exe = (org.omg.CORBA.Object)the_component_executor_._delegate();
                the_component_executor_._delegate(null);
            }
            else
            {
                exe = (org.omg.CORBA.Object)inter._delegate();
            }
            the_component_executor_._executor_locator().release_executor(exe);
            inter._delegate(null);

        }
        catch(org.omg.Components.CCMException ex)
        {
             //
             //  TODO : what ???
             //
        }
    }

    /**
     **
     **/
    public ActivationContext
    activation_context()
    {
        return this;
    }

    /**
     **
     **/
    public HomeServant
    the_home_servant()
    {
        return the_home_servant_;
    }

    /**
     **
     **/
    public ComponentExecutor
    the_component_executor()
    {
        return the_component_executor_;
    }

    /**
     **
     **/
    public org.omg.Components.CCMObject
    the_component_ref()
    {
        return the_component_ref_;
    }

    /**
     **
     **/
    public org.omg.Components.PrimaryKeyBase
    the_registration_info()
    {
        return the_primary_key_;
    }

    // ==================================================================
    //
    // Public methods for the ActivationContext interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/ActivationContext/state:1.0
    //
    /**
     **
     **/
    public void
    state(short new_state)
    {
        // if no state changes then do nothing.
        if (state_==new_state)
            return ;

        //
        //  TODO : change to be poolable also
        //  at this time, only instance mode is supported.
        //
        switch (new_state)
        {
            case DEACTIVATED :
                // release the executor.
                the_component_executor_ = null;
                // the_component_instance_ = null;
                active_objects_ = null;
                activation_coords_ = null;
                break;
            case ACTIVATED :
                 // check if we need to create a new executor.
                if ((the_component_executor_==null) && (the_component_instance_==null))
                {
                    // create a new component instance.
                    HomeExecutorBase home = null;
                    home = the_home_servant_.the_home_executor();

                    // create an instance.
                    try
                    {
                        if (!home._has_primary_key())
                            the_component_instance_ = ((HomeExecutor)home)._get_enterprise_component();
                        else
                            the_component_instance_ = ((HomeExecutorWithPK)home)._get_enterprise_component(the_primary_key_);
                    }
                    catch(org.omg.Components.CCMException ex)
                    {
                        //
                        //  TODO : what ????
                        //
                        throw new Error("Creation of instance failed !");
                    }

                    // create the executor.
                    initializeComponentExecutor(the_component_instance_);
                }
                else if (the_component_executor_==null)
                {
                    // create the executor.
                    initializeComponentExecutor(the_component_instance_);
                }
                break;
            default :
                break;
        }

        // change the state.
        state_ = new_state;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/ActivationContext/state:1.0
    //
    /**
     **
     **/
    public short
    state()
    {
        return state_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setComponentRef(org.omg.Components.CCMObject ref)
    {
        the_component_ref_ = ref;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    getReferenceWithInstance(org.omg.Components.EnterpriseComponent instance)
    {
        if (the_component_instance_==instance)
            return the_component_ref_;

        return null;
    }

    /**
     ** Remove the associated component.
     **/
    public void
    remove()
        throws org.omg.Components.RemoveFailure
    {
        the_servant_locator_.removeReference(the_component_ref_);

        org.omg.Components.EnterpriseComponent delegate = the_component_executor_._delegate();

	org.omg.Components.SessionComponent sc = castToSessionComponent(delegate);
        if(sc != null)
        {
            try
            {
                sc.ccm_remove();
            } catch (org.omg.Components.CCMException ex) {
	      // TODO: log exception.
	      //       set the reason field of the RemoveFailure exception.
                throw new org.omg.Components.RemoveFailure();
            }
        }
    }
}
