/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Containers;

/**
 * Implementation class of the ::OpenCCM::Containers::PCA and 
 * ::OpenCCM::Containers::PCAHooks interfaces.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class PCAImpl
       extends org.omg.CORBA.LocalObject
       implements PCA,
                  PCAHooks
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **
     **/
    protected java.lang.String the_name_;

    /**
     **
     **/
    protected PCA the_parent_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector children_ ;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector policies_;

    /**
     **
     **/
    protected org.omg.PortableServer.POA the_poa_;

    /**
     **
     **/
    protected ServantLocatorImpl the_servant_locator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    protected
    PCAImpl(java.lang.String name,
            PCA parent)
    {
        the_name_ = name;
        the_parent_ = parent;
        the_poa_ = null;
        the_servant_locator_ = null;
        policies_ = new org.objectweb.ccm.util.Vector();
        children_ = new org.objectweb.ccm.util.Vector();
    }

    // ==================================================================
    //
    // Public methods for the PCA interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/create_PCA:1.0
    //
    /**
     **
     **/
    public PCA
    create_PCA(java.lang.String name,
               PCAPolicy[] policies)
    throws CreationFailed
    {
        PCAImpl pca_ = null;
        try
        {
            // create the new PCA
            pca_ = new PCAImpl(name, this);

            // create and add default CORBA policies
            org.omg.PortableServer.POA rootPOA = org.objectweb.ccm.CORBA.TheRootPOA.getRootPOA();
            pca_.policies_.add(rootPOA.create_thread_policy(org.omg.PortableServer.ThreadPolicyValue.ORB_CTRL_MODEL));
            pca_.policies_.add(rootPOA.create_lifespan_policy(org.omg.PortableServer.LifespanPolicyValue.TRANSIENT));
            pca_.policies_.add(rootPOA.create_id_uniqueness_policy(org.omg.PortableServer.IdUniquenessPolicyValue.MULTIPLE_ID));
            pca_.policies_.add(rootPOA.create_id_assignment_policy(org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));
            pca_.policies_.add(rootPOA.create_implicit_activation_policy(
                org.omg.PortableServer.ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION));
            pca_.policies_.add(rootPOA.create_servant_retention_policy(org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN));
            pca_.policies_.add(rootPOA.create_request_processing_policy(
                org.omg.PortableServer.RequestProcessingPolicyValue.USE_SERVANT_MANAGER));

            // create the generic ::PortableServer::ServantLocator
            pca_.the_servant_locator_ = new ServantLocatorImpl();

            // apply the configured policies
            for (int i=0;i<policies.length;i++)
                policies[i].apply(pca_);

            // create the ::PortableServer::POA and activate it
            org.omg.CORBA.Policy[] corba_policies = (org.omg.CORBA.Policy[])pca_.policies_.toArray(new org.omg.CORBA.Policy[0]);
            pca_.the_poa_ = the_poa_.create_POA(name, null, corba_policies);
            pca_.the_servant_locator_.setPOA(pca_.the_poa_);
            pca_.the_poa_.set_servant_manager(pca_.the_servant_locator_);
            pca_.the_poa_.the_POAManager().activate();
        }
        catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
        {
            throw new CreationFailed(ex.getMessage());
        }
        catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
        {
            throw new CreationFailed(ex.getMessage());
        }
        catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
        {
            throw new CreationFailed(ex.getMessage());
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
        {
            throw new CreationFailed(ex.getMessage());
        }

        children_.add(pca_);
        return pca_;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/create_component_reference:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMObject
    create_component_reference(HomeServant home_servant)
    {
        org.omg.Components.CCMObject ref = null;
        ref = org.omg.Components.CCMObjectHelper.narrow(the_servant_locator_.createReference(home_servant));
        return ref;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/create_component_reference_with_instance:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMObject
    create_component_reference_with_instance(HomeServant home_servant,
                                             org.omg.Components.EnterpriseComponent instance)
    {
        org.omg.Components.CCMObject ref = null;
        ref = org.omg.Components.CCMObjectHelper.narrow(the_servant_locator_.createReference(home_servant, instance));
        return ref;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/find_component_reference_with_instance:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMObject
    find_component_reference_with_instance(HomeServant home_servant,
                                           org.omg.Components.EnterpriseComponent instance)
    {
        org.omg.Components.CCMObject ref = null;
        ref = org.omg.Components.CCMObjectHelper.narrow(the_servant_locator_.findReference(home_servant, instance));
        return ref;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/create_registered_component_reference:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMObject
    create_registered_component_reference(HomeServant home_servant,
                                          org.omg.Components.PrimaryKeyBase key)
    {
        org.omg.Components.CCMObject ref = null;
        ref = org.omg.Components.CCMObjectHelper.narrow(the_servant_locator_.createReference(home_servant, key));
        return ref;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/activate_home:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMHome
    activate_home(HomeServant home)
    {
        throw new java.lang.Error("Only the RootPCA can activate homes");
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/deactivate_home:1.0
    //
    /**
     **
     **/
    public void
    deactivate_home(HomeServant home)
    {
        throw new java.lang.Error("Only the RootPCA can deactivate homes");
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/the_name:1.0
    //
    /**
     **
     **/
    public java.lang.String
    the_name()
    {
        return the_name_;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/the_parent:1.0
    //
    /**
     **
     **/
    public PCA
    the_parent()
    {
        return the_parent_;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCA/children:1.0
    //
    /**
     **
     **/
    public PCA[]
    children()
    {
        return (PCA[])children_.toArray(new PCA[0]);
    }

    // ==================================================================
    //
    // Public methods for the PCAHooks interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/PCAHooks/add_native_policy:1.0
    //
    /**
     **
     **/
    public void
    add_native_policy(NativePolicy policy)
    {
        policies_.add(policy);
    }
}
