/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Containers.Plugins;

/**
 * This interface ??????.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class PortSpecificConfiguration
       extends org.omg.CORBA.LocalObject
       implements org.objectweb.ccm.Containers.SystemConfiguration
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Associations of (port_uid, system home class)
     **/
    protected org.objectweb.ccm.util.Table call_coordinator_sh_;

    /**
     **  Assocations of (port_uid, array of system home class)
     **/
    protected org.objectweb.ccm.util.Table call_controllers_sh_;

    /**
     **  Associations of (port_uid, property set)
     **/
    protected org.objectweb.ccm.util.Table call_coordinator_ps_;

    /**
     **  Associations of (port_uid, array of property set)
     **/
    protected org.objectweb.ccm.util.Table call_controllers_ps_;

    /**
     **  Associations of (port_uid, system home class)
     **/
    protected org.objectweb.ccm.util.Table act_coordinator_sh_;

    /**
     **  Assocations of (port_uid, array of system home class)
     **/
    protected org.objectweb.ccm.util.Table act_controllers_sh_;

    /**
     **  Associations of (port_uid, property set)
     **/
    protected org.objectweb.ccm.util.Table act_coordinator_ps_;

    /**
     **  Associations of (port_uid, array of property set)
     **/
    protected org.objectweb.ccm.util.Table act_controllers_ps_;

    /**
     **  The policies for the component.
     **/
    protected java.lang.String[] policies_;

    /**
     **  Reference to the parent configuration.
     **/
    protected org.objectweb.ccm.Containers.SystemConfiguration parent_configuration_;

    /**
     **  The ::CORBA::RepositoryId of the component without the version field.
     **/
    protected java.lang.String the_component_base_uid_;

    /**
     **  The version of the component.
     **/
    protected java.lang.String the_component_version_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    PortSpecificConfiguration()
    {
        call_coordinator_sh_ = new org.objectweb.ccm.util.Table();
        call_controllers_sh_ = new org.objectweb.ccm.util.Table();
        call_coordinator_ps_ = new org.objectweb.ccm.util.Table();
        call_controllers_ps_ = new org.objectweb.ccm.util.Table();
        act_coordinator_sh_ = new org.objectweb.ccm.util.Table();
        act_controllers_sh_ = new org.objectweb.ccm.util.Table();
        act_coordinator_ps_ = new org.objectweb.ccm.util.Table();
        act_controllers_ps_ = new org.objectweb.ccm.util.Table();
        policies_ = new java.lang.String[0];
        parent_configuration_ = null;
        the_component_base_uid_ = "";
        the_component_version_ = "1.0";
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected java.lang.String
    getCallCoordinatorSystemHome(java.lang.String uid)
    {
        return (java.lang.String)call_coordinator_sh_.get(uid);
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.PropertySet
    getCallCoordinatorPropertySet(java.lang.String uid)
    {
        return (org.objectweb.ccm.Containers.PropertySet)call_coordinator_ps_.get(uid);
    }

    /**
     **
     **/
    protected java.lang.String[]
    getCallControllersSystemHome(java.lang.String uid)
    {
        return (java.lang.String[])call_controllers_sh_.get(uid);
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.PropertySet[]
    getCallControllersPropertySet(java.lang.String uid)
    {
        return (org.objectweb.ccm.Containers.PropertySet[])call_controllers_ps_.get(uid);
    }

    /**
     **
     **/
    protected java.lang.String
    getActCoordinatorSystemHome(java.lang.String uid)
    {
        return (java.lang.String)act_coordinator_sh_.get(uid);
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.PropertySet
    getActCoordinatorPropertySet(java.lang.String uid)
    {
        return (org.objectweb.ccm.Containers.PropertySet)act_coordinator_ps_.get(uid);
    }

    /**
     **
     **/
    protected java.lang.String[]
    getActControllersSystemHome(java.lang.String uid)
    {
        return (java.lang.String[])act_controllers_sh_.get(uid);
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.PropertySet[]
    getActControllersPropertySet(java.lang.String uid)
    {
        return (org.objectweb.ccm.Containers.PropertySet[])act_controllers_ps_.get(uid);
    }

    /**
     **
     **/
    protected java.lang.String[]
    getPCAPolicies()
    {
        return policies_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    setComponentUId(java.lang.String comp_uid)
    {
        int idx = comp_uid.lastIndexOf(':');
        the_component_base_uid_ = comp_uid.substring(0, idx);        
        the_component_version_ = comp_uid.substring(idx+1);
    }

    /**
     **
     **/
    public void
    setCallCoordinator(java.lang.String port_name,
                       java.lang.String home_alias)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        call_coordinator_sh_.put(uid, home_alias);
    }

    /**
     **
     **/
    public void
    setCallCoordinatorConfig(java.lang.String port_name,
                             org.objectweb.ccm.Containers.PropertySet config)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        call_coordinator_ps_.put(uid, config);
    }

    /**
     **
     **/
    public void
    setCallControllers(java.lang.String port_name,
                       java.lang.String[] homes_alias)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        call_controllers_sh_.put(uid, homes_alias);
    }

    /**
     **
     **/
    public void
    setCallControllersConfig(java.lang.String port_name,
                             org.objectweb.ccm.Containers.PropertySet[] configs)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        call_controllers_ps_.put(uid, configs);
    }

    /**
     **
     **/
    public void
    setActCoordinator(java.lang.String port_name,
                      java.lang.String home_alias)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        act_coordinator_sh_.put(uid, home_alias);
    }

    /**
     **
     **/
    public void
    setActCoordinatorConfig(java.lang.String port_name,
                            org.objectweb.ccm.Containers.PropertySet config)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        act_coordinator_ps_.put(uid, config);
    }

    /**
     **
     **/
    public void
    setActControllers(java.lang.String port_name,
                      java.lang.String[] homes_alias)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        act_controllers_sh_.put(uid, homes_alias);
    }

    /**
     **
     **/
    public void
    setActControllersConfig(java.lang.String port_name,
                            org.objectweb.ccm.Containers.PropertySet[] configs)
    {
        // Note that the component version is assumed.
        java.lang.String uid = the_component_base_uid_ + '/' + port_name + ':' + the_component_version_;
        act_controllers_ps_.put(uid, configs);
    }

    /**
     **
     **/
    public void
    setPCAPolicies(java.lang.String[] policies)
    {
        policies_ = policies;
    }

    // ==================================================================
    //
    // Public methods for the SystemConfiguration interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/SystemConfiguration/create_call_coordinator:1.0
    //
    /**
     **
     **/
    public org.objectweb.ccm.Containers.CallCoordinator
    create_call_coordinator(java.lang.String uid,
                            java.lang.String op_name,
                            org.objectweb.ccm.Containers.SystemHomeFinder finder)
    {
        java.lang.String home_alias = null;
        org.objectweb.ccm.Containers.PropertySet config = null;

        home_alias = getCallCoordinatorSystemHome(uid);
        config = getCallCoordinatorPropertySet(uid);

        if ((home_alias==null) && (parent_configuration_!=null))
            return parent_configuration_.create_call_coordinator(uid, op_name, finder);
        else if (home_alias==null)
            home_alias = "EmptyCoordinatorHome";

        try
        {
            org.objectweb.ccm.Containers.SystemHome home = null;
            org.objectweb.ccm.Containers.CallCoordinator result = null;

            home = finder.find_system_home(home_alias);
            result = (org.objectweb.ccm.Containers.CallCoordinator)home.create_component(config);
            result.configure(config);
            return result;
        }
        catch(org.objectweb.ccm.Containers.HomeNotFound ex)
        {
            throw new Error("System home : "+home_alias+" was not found");
        }
        catch(org.objectweb.ccm.Containers.CreationFailed ex)
        {
            throw new Error("Creation of system component for system home : "+home_alias+" failed");
        }
        catch(org.objectweb.ccm.Containers.ConfigurationFailed ex)
        {
            throw new Error("Configuration of system component for system home : "+home_alias+" failed");
        }
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/SystemConfiguration/create_call_controllers:1.0
    //
    /**
     **
     **/
    public org.objectweb.ccm.Containers.CallController[]
    create_call_controllers(java.lang.String uid,
                            java.lang.String op_name,
                            org.objectweb.ccm.Containers.SystemHomeFinder finder)
    {
        org.objectweb.ccm.Containers.CallController[] result = null;
        java.lang.String[] homes_alias = null;
        org.objectweb.ccm.Containers.PropertySet[] configs = null;

        homes_alias = getCallControllersSystemHome(uid);
        configs = getCallControllersPropertySet(uid);

        if ((homes_alias==null) && (parent_configuration_!=null))
            return parent_configuration_.create_call_controllers(uid, op_name, finder);
        else if (homes_alias==null)
	{
            homes_alias = new java.lang.String[1];
            homes_alias[0] = "EmptyControllerHome";
            configs = new org.objectweb.ccm.Containers.PropertySet[1];
            configs[0] = null;
        }

        int i=0;

        try
        {
            result = new org.objectweb.ccm.Containers.CallController[homes_alias.length];
            org.objectweb.ccm.Containers.SystemHome home = null;
            for (;i<homes_alias.length;i++)
	    {
                home = finder.find_system_home(homes_alias[i]);
                result[i] = (org.objectweb.ccm.Containers.CallController)home.create_component(configs[i]);
                result[i].configure(configs[i]);
	    }

            if (parent_configuration_!=null)
            {
                org.objectweb.ccm.Containers.CallController[] tmp1 = null;
                org.objectweb.ccm.Containers.CallController[] tmp2 = null;
                tmp1 = parent_configuration_.create_call_controllers(uid, op_name, finder);
                if (tmp1!=null)
                {
                    int l = tmp1.length + result.length;
                    tmp2 = new org.objectweb.ccm.Containers.CallController[l];
                    java.lang.System.arraycopy(result, 0, tmp2, 0, result.length);
                    java.lang.System.arraycopy(tmp1, 0, tmp2, result.length, tmp1.length);
                    return tmp2;
                }
            }

            return result;
        }
        catch(org.objectweb.ccm.Containers.HomeNotFound ex)
        {
            throw new Error("System home :"+homes_alias[i]+" was not found");
        }
        catch(org.objectweb.ccm.Containers.CreationFailed ex)
        {
            throw new Error("Creation of system component for system home :"+homes_alias[i]+" failed");
        }
        catch(org.objectweb.ccm.Containers.ConfigurationFailed ex)
        {
            throw new Error("Configuration of system component for system home :"+homes_alias[i]+" failed");
        }
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/Container/SystemConfiguration/create_activation_coordinator:1.0
    //
    /**
     **
     **/
    public org.objectweb.ccm.Containers.ActivationCoordinator
    create_activation_coordinator(java.lang.String uid,
                                  org.objectweb.ccm.Containers.SystemHomeFinder finder)
    {
        java.lang.String home_alias = null;
        org.objectweb.ccm.Containers.PropertySet config = null;

        home_alias = getActCoordinatorSystemHome(uid);
        config = getActCoordinatorPropertySet(uid);

        if ((home_alias==null) && (parent_configuration_!=null))
            return parent_configuration_.create_activation_coordinator(uid, finder);
        else if (home_alias==null)
            // should never be called !!!
            throw new java.lang.Error("Can\'t use an empty coordinator for activation");


        try
        {
            org.objectweb.ccm.Containers.SystemHome home = null;
            org.objectweb.ccm.Containers.ActivationCoordinator result = null;

            home = finder.find_system_home(home_alias);
            result = (org.objectweb.ccm.Containers.ActivationCoordinator)home.create_component(config);
            result.configure(config);
            return result;
        }
        catch(org.objectweb.ccm.Containers.HomeNotFound ex)
        {
            throw new Error("System home : "+home_alias+" was not found");
        }
        catch(org.objectweb.ccm.Containers.CreationFailed ex)
        {
            throw new Error("Creation of system component for system home : "+home_alias+" failed");
        }
        catch(org.objectweb.ccm.Containers.ConfigurationFailed ex)
        {
            throw new Error("Configuration of system component for system home : "+home_alias+" failed");
        }
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/Container/SystemConfiguration/create_activation_controllers:1.0
    //
    /**
     **
     **/
    public org.objectweb.ccm.Containers.ActivationController[]
    create_activation_controllers(java.lang.String uid,
                                  org.objectweb.ccm.Containers.SystemHomeFinder finder)
    {
        org.objectweb.ccm.Containers.ActivationController[] result = null;
        org.objectweb.ccm.Containers.ActivationController[] tmp = null;
        java.lang.String[] homes_alias = null;
        org.objectweb.ccm.Containers.PropertySet[] configs = null;

        homes_alias = getActControllersSystemHome(uid);
        configs = getActControllersPropertySet(uid);

        if ((homes_alias==null) && (parent_configuration_!=null))
            return parent_configuration_.create_activation_controllers(uid, finder);
        else if (homes_alias==null)
            // should never be called !!!
            throw new java.lang.Error("Can\'t use an empty controller for activation");

        int i=0;

        try
        {
            result = new org.objectweb.ccm.Containers.ActivationController[homes_alias.length];
            org.objectweb.ccm.Containers.SystemHome home = null;
            for (;i<homes_alias.length;i++)
	    {
                home = finder.find_system_home(homes_alias[i]);
                result[i] = (org.objectweb.ccm.Containers.ActivationController)home.create_component(configs[i]);
                result[i].configure(configs[i]);
	    }

            if (parent_configuration_!=null)
            {
                org.objectweb.ccm.Containers.ActivationController[] tmp1 = null;
                org.objectweb.ccm.Containers.ActivationController[] tmp2 = null;
                tmp1 = parent_configuration_.create_activation_controllers(uid, finder);
                if (tmp1!=null)
                {
                    int l = tmp1.length + result.length;
                    tmp2 = new org.objectweb.ccm.Containers.ActivationController[l];
                    java.lang.System.arraycopy(result, 0, tmp2, 0, result.length);
                    java.lang.System.arraycopy(tmp1, 0, tmp2, result.length, tmp1.length);
                    return tmp2;
                }
            }

            return result;
        }
        catch(org.objectweb.ccm.Containers.HomeNotFound ex)
        {
            throw new Error("System home :"+homes_alias[i]+" was not found");
        }
        catch(org.objectweb.ccm.Containers.CreationFailed ex)
        {
            throw new Error("Creation of system component for system home :"+homes_alias[i]+" failed");
        }
        catch(org.objectweb.ccm.Containers.ConfigurationFailed ex)
        {
            throw new Error("Configuration of system component for system home :"+homes_alias[i]+" failed");
        }
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/SystemConfiguration/create_PCAPolicies:1.0
    //
    /**
     **
     **/
    public org.objectweb.ccm.Containers.PCAPolicy[]
    create_PCAPolicies(java.lang.String uid,
                       org.objectweb.ccm.Containers.SystemHomeFinder finder)
    {
        org.objectweb.ccm.Containers.PCAPolicy[] result = null;
        java.lang.String[] homes_alias = null;

        homes_alias = getPCAPolicies();
        int i=0;
        try
        {
            result = new org.objectweb.ccm.Containers.PCAPolicy[homes_alias.length];
            org.objectweb.ccm.Containers.SystemHome home = null;
            for (;i<homes_alias.length;i++)
	    {
                home = finder.find_system_home(homes_alias[i]);
                result[i] = (org.objectweb.ccm.Containers.PCAPolicy)home.create_component(null);
                result[i].configure(null);
	    }
        }
        catch(org.objectweb.ccm.Containers.HomeNotFound ex)
        {
            throw new Error("System home :"+homes_alias[i]+" was not found");
        }
        catch(org.objectweb.ccm.Containers.CreationFailed ex)
        {
            throw new Error("Creation of system component for system home :"+homes_alias[i]+" failed");
        }
        catch(org.objectweb.ccm.Containers.ConfigurationFailed ex)
        {
            throw new Error("Configuration of system component for system home :"+homes_alias[i]+" failed");
        }

        if ((result.length==0) && (parent_configuration_!=null))
            return parent_configuration_.create_PCAPolicies(uid, finder);
        else if (result.length==0)
            return new org.objectweb.ccm.Containers.PCAPolicy[0];

        return result;
    }

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/SystemConfiguration/set_parent_configuration:1.0
    //
    /**
     **
     **/
    public void
    set_parent_configuration(org.objectweb.ccm.Containers.SystemConfiguration config)
    {
        parent_configuration_ = config;
    }

    // ==================================================================
    //
    // Public methods for the SystemComponent interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for the Configuration interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/Configuration/configure:1.0
    //
    /**
     **
     **/
    public void
    configure(org.objectweb.ccm.Containers.PropertySet config)
    throws org.objectweb.ccm.Containers.ConfigurationFailed
    {
        //
        // TODO : do nothing ???
        //
        throw new org.objectweb.ccm.Containers.ConfigurationFailed("not implemented !");
    }
}
