/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment ;


/**
 * This class implements the OMG IDL
 * ::Deployment::ComponentServerLocal interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class ComponentServerLocalImpl
extends org.omg.CORBA.LocalObject
implements ComponentServerLocal
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /**
     * Reference to the component server implementation
     */
    protected ComponentServerBase the_component_server_ ; 
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    /**
     **  The constructor.
     **
     **  @param component_server reference to the component server implementation.
     **/
    public
    ComponentServerLocalImpl(ComponentServerBase component_server)
    {
        the_component_server_ = component_server; 
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** Configure the listener for the current component server
     **
     ** @param listener plugin to install on the component server
     **/
    public void
    add_listener (org.objectweb.ccm.Deployment.listener.ComponentServerListener listener)
    {
        the_component_server_.add_listener (listener); 
    }
    

    /**
     ** Add the container to the component server list of available containers 
     **
     ** @param ref reference of the container to register
     **/
    public void
    add_container (org.omg.Components.Deployment.Container ref)
    {
        the_component_server_.add_container (ref); 
    }
    
    
    // ==================================================================
    //
    // Methods for the ::plugins::deployment::api::ComponentServerLocal interface.
    //
    // ==================================================================

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ComponentServerLocal/remove_container:1.0
    //
    /**
     **  To remove a container from the component server references.
     **
     ** @param ref Container reference to unregistered
     **/
    public void
    remove_container(org.omg.Components.Deployment.Container ref)
    throws org.omg.Components.RemoveFailure
    {
        the_component_server_.remove_container (ref); 
    }

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ComponentServerLocal/get_containers:1.0
    //
    /**
     **  To obtain all created containers references.
     **
     **/
    public org.omg.Components.Deployment.Container[]
    get_containers()
    {
        return the_component_server_.get_containers (); 
    }

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ComponentServerLocal/remove:1.0
    //
    /**
     **  To remove this component server.
     **
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        the_component_server_.remove (); 
    }
} 
