/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment ;


/**
 * This class implements the OMG IDL
 * ::Deployment::ContainerLocal interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class ContainerLocalImpl
extends org.omg.CORBA.LocalObject
implements ContainerLocal
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * reference to the container implementation
     */
    protected ContainerBase the_container_;

    /**
     * container reference
     */
    protected org.omg.Components.Deployment.Container the_container_ref;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param container reference to the container implementation.
     **/
    public
    ContainerLocalImpl(ContainerBase container)
    {
        the_container_ = container; 
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** Configure the listener for the current container
     **
     ** @param listener plugin to install on the container
     **/
    public void
    add_listener (org.objectweb.ccm.Deployment.listener.ContainerListener listener)
    {
        the_container_.add_listener (listener); 
    }

    /**
     ** Add the home to thecontainer list of available homes 
     **
     ** @param ref reference of the CCMHome to register
     **/
    public void
    add_home(org.omg.Components.CCMHome ref)
    {
        the_container_.add_home (ref); 
    }
    
    // ==================================================================
    //
    // Methods for the Components::Deployment::Container interface.
    //
    // ==================================================================

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ContainerLocal/remove_home:1.0
    //
    /**
     ** remove the home from the container references.
     **
     ** @param ref CCMHome reference to unregistered
     **/
    public void
    remove_home(org.omg.Components.CCMHome ref)
    throws org.omg.Components.RemoveFailure
    {
        the_container_.remove_home (ref); 
    }

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ContainerLocal/get_homes:1.0
    //
    /**
     ** To obtain all created containers references.
     **
     ** @return list of available CCMhomes
     **/
    public org.omg.Components.CCMHome[]
    get_homes()
    {
        return the_container_.get_homes (); 
    }

    //
    //  IDL:objectweb.org/ccm/plugins/deployment/api/ContainerLocal/remove:1.0
    //
    /**
     ** Destroy the current container
     **
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        the_container_.remove (); 
    }
} 



