/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.
====================================================================*/

package org.objectweb.ccm.Deployment.listener ; 

import org.objectweb.ccm.Deployment.CCMHomeLocal ;

/**
 * Partial implementation for the 
 * Deployment::listener::CCMHomeListener interface
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public abstract class CCMHomeListenerBase
extends org.omg.CORBA.LocalObject
implements CCMHomeListener
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /**
     * The delegate listener
     */ 
    protected CCMHomeListener delegate_;

    /**
     * The event source
     */ 
    protected CCMHomeLocal source_; 

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    /**
     * Constructor with parameter
     *
     * @param source the event source
     */ 
    protected
    CCMHomeListenerBase(CCMHomeLocal source)
    {
        source_ = source; 
        delegate_ = null; 
    }

    /**
     * Default Constructor
     *
     */ 
    protected
    CCMHomeListenerBase()
    {
        this(null); 
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    /**
     * Provide a new instance of the class
     *
     * @return new instance
     */
    protected abstract CCMHomeListener
    _new();  
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the ::plugins::deployment::CCMHomeListener interface.
    //
    // ==================================================================
    /**
     * Set the delegate listener
     *
     * @param listener the delegate listener
     */ 
    public void
    set_delegate(CCMHomeListener listener)
    {
        if (delegate_ == null)
        { 
            delegate_ = listener;
            source_ = listener.get_source();
        }
        else
            delegate_.set_delegate (listener);  
    }

    /**
     * Provide the interface for interacting with the Home
     *
     * @return the interface of manipulation
     */ 
    public CCMHomeLocal
    get_source()
    {
        return source_;
    }
    
    /**
     * Set the interface for interacting with the Home
     *
     * @param home the interface of manipulation
     */ 
    public void
    set_source(CCMHomeLocal home)
    {
        source_ = home; 
    }
     
    /**
     * Configure the specified component server with the current listener
     *
     * @param home the component server to configure
     */  
    public void
    configure_ccm_home(CCMHomeLocal home)
    {
	home.add_listener(_new());
    }
    
    // ==================================================================
    //
    // Methods for the ::plugins::deployment::CCMHomeEvents interface.
    //
    // ==================================================================
    /**
     * Event sent after the create function is called on CCMHome object
     *
     * @param ref the object created
     */ 
    public abstract void
    on_create (org.omg.Components.CCMObject ref); 
    
    /**
     * Event sent after the add function is called on CCMHome object
     *
     * @param ref the object added
     */ 
    public abstract void
    on_add (org.omg.Components.CCMObject ref ); 
    
    /**
     * Event sent before the remove function is called on CCMHome object
     *
     * @param ref the object removed
     */ 
    public abstract void
    on_remove (org.omg.Components.CCMObject ref); 
        
    /**
     * Event sent before the destruction of the related CCMHome object 
     *
     */ 
    public abstract void
    on_destroy ();  
}



