/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment.listener ; 

import org.objectweb.ccm.Deployment.listener.ComponentServerListener ;
import org.objectweb.ccm.Deployment.ComponentServerLocal ;

/**
 * Partial implementation for the
 * ::plugins::deployment::ComponentServerListener interface
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public abstract class ComponentServerListenerBase
		extends org.omg.CORBA.LocalObject
		implements ComponentServerListener
{ 
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /**
     * The delegate listener
     */ 
    protected ComponentServerListener delegate_;
    
    /**
     * The event source
     */ 
    protected ComponentServerLocal source_; 
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    /**
     * Constructor with parameter
     *
     * @param source the event source
     */ 
    protected
		ComponentServerListenerBase(ComponentServerLocal source)
    {
				delegate_ = null;
				source_ = source; 
    }

    /**
     * Default Constructor
     *
     */ 
    protected
    ComponentServerListenerBase()
    {
				this(null) ; 
    }     
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
  	/**
     * Provide a new instance of the class
     *
     * @return new instance
		 */
		protected abstract ComponentServerListener
		_new();  

    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the ::plugins::deployment::ComponentServerListener interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerListener/set_delegate:1.0
    //
    /**
     * Set the delegate listener
     *
     * @param listener the delegate listener     
     */ 
    public void
		set_delegate (ComponentServerListener listener)
    {
				set_source(listener.get_source());
				if (delegate_ == null)
						{
								delegate_ = listener;
						}
				else
						delegate_.set_delegate (listener);
    } 

    //
    //  IDL:omg.org/plugins/deployment/ComponentServerListener/get_source:1.0
    //
    /**
     * Provide the interface for interacting with the component server
     *
     * @return the interface of manipulation
     */
    public ComponentServerLocal
    get_source()
    {
				return source_;
    } 
 
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerListener/set_source:1.0
    //
    /**
     * Set the interface for interacting with the Container
     *
     * @param home the interface of manipulation
     */ 
    public void
    set_source(ComponentServerLocal home)
    {
				source_ = home; 
    }

    //
    //  IDL:omg.org/plugins/deployment/ComponentServerListener/configure_component_server:1.0
    //
    /**
     * Configure the specified component server with the current listener
     *
     * @param component_server the component server to configure
     */  
    public void
    configure_component_server(ComponentServerLocal component_server)
    {
				component_server.add_listener(_new());
    }

    //
    //  IDL:omg.org/plugins/deployment/ComponentServerListener/get_sub_listener:1.0
    //
    /**
     * Provide the equivalent Container listener for the current listener
     *
     * @return the equivalent Container listener
     */
    public abstract ContainerListener 
  	get_sub_listener();

  
    // ==================================================================
    //
    // Methods for the ::plugins::deployment::ComponentServerEvents interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_create:1.0
    //
		/**
		 * Event sent after the create function is called on ComponentServer object
		 *
		 * @param ref the object created
		 */
		public abstract void 
    on_create (org.omg.Components.Deployment.Container cont_ref) ;  
		
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_add:1.0
    //
		/**
		 * Event sent after the add function is called on ComponentServer object
		 *
		 * @param ref the object added
		 */
		public abstract void 
    on_add (org.omg.Components.Deployment.Container cont_ref) ;  
		
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_remove:1.0
    //
		/**
		 * Event sent before the remove function is called on ComponentServer object
		 *
		 * @param ref the object removed
		 */ 
		public abstract void 
    on_remove (org.omg.Components.Deployment.Container cont_ref); 
		
    //
    //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_destroy:1.0
    //
		/**
		 * Event sent before the destruction of the related ComponentServer object 
		 *
		 */ 
		public abstract void 
    on_destroy ();  
}







