/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment.listener ; 


import org.objectweb.ccm.Deployment.ContainerLocal ;

/**
 * Partial implementation for the
 * ::plugins::deployment::ContainerListener interface
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public abstract class ContainerListenerBase
extends org.omg.CORBA.LocalObject
implements ContainerListener
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /**
     * The delegate listener
     */ 
    protected ContainerListener delegate_;

    /**
     * The event source
     */ 
    protected ContainerLocal source_; 
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    /**
     * Constructor with parameter
     *
     * @param source the event source
     */ 
    protected
	  ContainerListenerBase(ContainerLocal source)
    {
        delegate_ = null;
        source_ = source; 
    }

    /**
     * Default Constructor
     *
     */ 
    protected
    ContainerListenerBase()
    {
        this(null) ; 
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    /**
     * Provide a new instance of the class
     *
     * @return new instance
     */
    protected abstract ContainerListener
	  _new();  

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the ::plugins::deployment::ContainerListener interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/plugins/deployment/ContainerResource/set_delegate:1.0
    //
    /**
     * Set the delegate listener
     *
     * @param listener the delegate listener
     */ 
    public void
    set_delegate (ContainerListener listener)
    {
				set_source(listener.get_source());
				if (delegate_ == null)
						delegate_ = listener;
				else
						delegate_.set_delegate (listener);
    }

    //
    //  IDL:omg.org/plugins/deployment/ContainerResource/get_source:1.0
    //
    /**
     * Provide the interface for interacting with the container
     *
     * @return the interface of manipulation
     */       
    public ContainerLocal
    get_source()
    {
        return source_; 
    }
    
    //
    //  IDL:omg.org/plugins/deployment/ContainerResource/set_source:1.0
    //
    /**
     * Set the interface for interacting with the Container
     *
     * @param home the interface of manipulation
     */ 
    public void
    set_source(ContainerLocal home)
    {
        source_ = home; 
    }
    
    //
    //  IDL:omg.org/plugins/deployment/ContainerResource/configure_container:1.0
    //
    /**
     * Configure the specified container with the current listener
     *
     * @param container the component server to configure
     */  
    public void
  	configure_container(ContainerLocal container)
    {
				container.add_listener(_new());
    }
    
    //
    //  IDL:omg.org/plugins/deployment/ContainerResource/get_sub_listener:1.0
    //
    /**
     * Provide the equivalent CCMHome listener for the current listener
     *
     * @return the equivalent CCMHome listener
     */
    public abstract CCMHomeListener 
	  get_sub_listener();

    // ==================================================================
    //
    // Methods for the ::plugins::deployment::ContainerEvents interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/plugins/deployment/ContainerEvents/on_create:1.0
    //
    /**
     * Event sent after the create function is called on Container object
     *
     * @param ref the object created
     */ 
    public abstract void 
	  on_create (org.omg.Components.CCMHome ref) ;  

    //
    //  IDL:omg.org/plugins/deployment/ContainerEvents/on_add:1.0
    //
    /**
     * Event sent after the add function is called on Container object
     *
     * @param ref the object added
     */ 
    public abstract void 
    on_add (org.omg.Components.CCMHome ref) ;  

    //
    //  IDL:omg.org/plugins/deployment/ContainerEvents/on_remove:1.0
    //
    /**
     * Event sent before the remove function is called on Container object
     *
     * @param ref the object removed
     */ 
    public abstract void 
    on_remove (org.omg.Components.CCMHome ref) ; 

    //
    //  IDL:omg.org/plugins/deployment/ContainerEvents/on_destroy:1.0
    //
    /**
     * Event sent before the destruction of the related Container object 
     *
     */ 
    public abstract void 
    on_destroy () ;  
}


