/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.controller;

import org.objectweb.ccm.CORBA.TheORB;

/**
 * This class implements an transactionnal profile for the
 * OpenCCM::Containers::CallController interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public abstract class WithCurrentController
		extends org.omg.CORBA.LocalObject
		implements org.objectweb.ccm.Containers.CallController
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Reference to the transaction service current interface
     *
     */
    protected org.omg.CosTransactions.Current current_;

		/**
		 * Determine if a transaction has been started by the controller
		 *
		 */
		protected boolean private_transaction_;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **/
    public
    WithCurrentController()
    {
				current_ = null;
				private_transaction_ = false;
				try
						{
								org.omg.CORBA.Object _obj = TheORB.getORB().resolve_initial_references("TransactionCurrent");
								current_ = org.omg.CosTransactions.CurrentHelper.narrow( _obj );
						}
				catch (org.omg.CORBA.ORBPackage.InvalidName ex)
						{
						}
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
		
		/**
		 ** tell if there is no transaction started
		 **
		 ** @return true -> there is no active transaction
		 **/
		protected boolean
		no_transaction(org.objectweb.ccm.Containers.CallContext ctx)
		{
				return current_.get_status() == org.omg.CosTransactions.Status.StatusNoTransaction;
		}
		
		/**
		 ** begin a new transaction
		 **
		 **/
		protected void
		begin(org.objectweb.ccm.Containers.CallContext ctx)
		{
				try
						{
								current_.begin();
								private_transaction_ = true;
						}
				catch (org.omg.CosTransactions.SubtransactionsUnavailable ex)
						{
						}		
				// TODO :
				//
				// Wrapp the exception
		}

		/**
		 ** Commit a transaction
		 ** <i> Only if it has been started by the current controller </i>
		 **
		 **/
		protected void
		commit(org.objectweb.ccm.Containers.CallContext ctx)
		{
				if (private_transaction_)
						{
								try
										{
												current_.commit(false);
												private_transaction_ = false;
										} catch(org.omg.CosTransactions.NoTransaction ex) {
										} catch(org.omg.CosTransactions.HeuristicMixed ex) {
										} catch(org.omg.CosTransactions.HeuristicHazard ex) {
										}		
						}		
				// TODO :
				//
				// Wrapp the exceptions
		}

    // ==================================================================
    //
    // Public methods for the CallController interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for the CallProtocol interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallProtocol/preinvoke:1.0
    //
    /**
     **
     **/
    public abstract void
		preinvoke(org.objectweb.ccm.Containers.CallContext ctx);

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/CallProtocol/postinvoke:1.0
    //
    /**
     **
     **/
    public abstract void
		postinvoke(org.objectweb.ccm.Containers.CallContext ctx);

    // ==================================================================
    //
    // Public methods for the SystemComponent interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for the Configuration interface.
    //
    // ==================================================================

    //
    //  IDL:goal.lifl.fr/OpenCCM/Containers/Configuration/configure:1.0
    //
    /**
     **
     **/
    public abstract void
	  configure(org.objectweb.ccm.Containers.PropertySet config)
				throws org.objectweb.ccm.Containers.ConfigurationFailed;
}
