/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment;

import org.objectweb.ccm.Deployment.CCMHomeLocal ;
import org.omg.Components.CCMObject ;
import org.omg.CosTransactions.Coordinator ;

/**
 * Transaction implementation for the
 * ::plugins::transaction::deployment::CCMHomeResource interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class CCMHomeResourceImpl
extends org.omg.CORBA.LocalObject 
implements CCMHomeResource
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * Reference on the related home
   */
  protected CCMHomeLocal the_home_; 
  
  /**
   * Reference on the coordinator of the current resource
   */
  protected Coordinator the_coordinator_; 
  
  /**
   * Reference to the resource manager
   *
   */
  protected CCMHomeListenerTransaction the_listener_; 
  
  /**
   * Reference to the resource manager
   *
   */
  protected ResourceManagerLocal the_resource_manager_; 
  
  /**
   * List of not committed objects
   */
  protected org.objectweb.ccm.util.Vector objects_;
  

  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  
  /**
   **  The constructor.
   ** 
   ** @param home reference to the home
   ** @param coordinator reference to the current coordinator
   ** @param resource reference to the ResourceManager of the current transaction
   **
   **/
  public 
  CCMHomeResourceImpl(CCMHomeListenerTransaction listener,
                      CCMHomeLocal home,
                      Coordinator coordinator,
                      ResourceManagerLocal manager)
  {
    the_home_ = home;
    objects_ = new org.objectweb.ccm.util.Vector (); 
    register(listener, coordinator, manager);
  }
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  protected void
  register(CCMHomeListenerTransaction listener,
           Coordinator coordinator,
           ResourceManagerLocal resource_manager)
  {
    the_listener_ = listener;
    the_coordinator_ = coordinator;
    the_resource_manager_ = resource_manager;
    
    the_listener_.add_resource(this);
    the_resource_manager_.add_home(this);				
  }
  
  
  protected void
  unregister( )
  {   
    try {
      the_coordinator_ = null ;
      the_listener_.remove_resource(this);
      the_resource_manager_.remove_home (this); 
    } catch ( org.omg.Components.RemoveFailure ex ) { }
  }
  
  
  /**
   * apply the modification to the associated home
   *
   */
  protected void
  commit_add ()
  {
    for (int i = 0; i < objects_.size(); i++) {
      CCMObject obj_ref = (CCMObject) objects_.elementAt(i) ;
      //the_home_.add_object (obj_ref);
    }
  }
  
  /**
   * Deactivate the current resource
   *
   */
  protected void
  removeItself()
  {
    unregister();
  }  
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::CCMHomeEvents interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/Deployment/listener/CCMHomeEvents/on_create:1.0
  //
  /**
   * Event sent after the create function is called on CCMHome object
   *
   * @param ref the object created
   */ 
  public void
  on_create (CCMObject ref)
  {
    objects_.addElement (ref);
  }
  
  //
  //  IDL:omg.org/Deployment/listener/CCMHomeEvents/on_add:1.0
  //
  /**
   * Event sent after the add function is called on CCMHome object
   *
   * @param ref the object added
   */ 
  public void
  on_add (CCMObject ref )
  {
  }
  
  //
  //  IDL:omg.org/Deployment/listener/CCMHomeEvents/on_remove:1.0
  //
  /**
   * Event sent before the remove function is called on CCMHome object
   *
   * @param ref the object removed
   */ 
  public void
  on_remove (CCMObject ref)
  {
  }
  
  //
  //  IDL:omg.org/Deployment/listener/CCMHomeEvents/on_destroy:1.0
  //
  /**
   * Event sent before the destruction of the related CCMHome object 
   *
   */ 
  public void
  on_destroy ()
  {
  }
  
  // ==================================================================
  //
  // Methods for the CosTransactions::Resource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/CosTransaction/Resource/prepare:1.0
  //
  /**
   * prepare the commit phase : first part of the 2-commit phase 
   *
   * @return a vote determining the strategy to apply (Commit, Rollback or Read-Only)
   *
   */
  public org.omg.CosTransactions.Vote
  prepare()
    throws org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    return org.omg.CosTransactions.Vote.VoteCommit; 
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/rollback:1.0
  //
  /**
   * rollback the transaction started : second part of the 2-commit-phase
   *
   */
  public void
  rollback()
    throws org.omg.CosTransactions.HeuristicCommit,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    for (int i = 0; i < objects_.size(); i++) {
      CCMObject obj_ref = (CCMObject) objects_.elementAt(i) ;
      try {
        obj_ref.remove();
      } catch (org.omg.Components.RemoveFailure ex) { }
    }
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit:1.0
  //
  /**
   * Commit the the transaction started : second part of the 2-commit-phase
   *
   */
  public void
  commit()
    throws org.omg.CosTransactions.NotPrepared,
    org.omg.CosTransactions.HeuristicRollback,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    commit_add();
    removeItself();
  }
  
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit_one_phase:1.0
  //
  /**
   * Commit the the transaction started : 1-commit phase
   *
   */
  public void
  commit_one_phase()
    throws org.omg.CosTransactions.HeuristicHazard
  {
    commit_add();
    removeItself();
  }
  
  
  //
  //  IDL:omg.org/CosTransaction/Resource/forget:1.0
  //
  /**
   * Cancel the transaction started
   *
   */
  public void
  forget()
  {
    removeItself();
  }
  
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::CCMHomeResource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/Deployment/listener/transaction/CCMHomeResource/get_home:1.0
  //
  /**
   * provide the reference of the associated home
   *
   * @return reference to the home
   *
   */
  public org.objectweb.ccm.Deployment.CCMHomeLocal
  get_home()
  {
    return the_home_; 
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::CosCoordinator interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/Deployment/listener/transaction/CCMHomeResource/coordinator:1.0
  //
  /**
   * Provide a reference to the coordinator of the associated transaction
   *
   * @return specific coordinator
   */
  public Coordinator
  coordinator()
  {
    return the_coordinator_;
  }    
}




