/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment ;

import org.objectweb.ccm.Deployment.ComponentServerLocal ;
import org.omg.Components.Deployment.Container ;
import org.omg.CosTransactions.Coordinator ;

/**
 * This class implements the OMG IDL
 * ::plugins::deployment::transaction::ComponentServerResource interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class ComponentServerResourceImpl
extends org.omg.CORBA.LocalObject
implements ComponentServerResource
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * Reference on the related component server
   */
  protected ComponentServerLocal the_component_server_; 
  
  /**
   * Reference on the coordinator of the current resource
   */
  protected Coordinator the_coordinator_;
  
  /**
   * Reference on the listener
   */
  protected ComponentServerListenerTransaction the_listener_;
   
  /**
   * Reference to the resource manager
   */
  protected ResourceManagerLocal the_resource_manager_; 
  
  /**
   * The list of the sub resources
   */
  protected org.objectweb.ccm.util.Vector containers_;
  
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   ** The constructor.
   **
   ** @param component_server reference to the component server
   ** @param coordinator reference to the current coordinator
   ** @param resource reference to the ResourceManager of the current transaction
   **
   **/ 
  public
  ComponentServerResourceImpl(ComponentServerListenerTransaction listener,
                              ComponentServerLocal component_server,
                              Coordinator coordinator,  
                              ResourceManagerLocal resource_manager)
  {
    the_component_server_ = component_server; 
    containers_ = new org.objectweb.ccm.util.Vector();
    register(listener, coordinator, resource_manager);
  }
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  protected void
  register(ComponentServerListenerTransaction listener,
           Coordinator coordinator,
           ResourceManagerLocal resource_manager)
  {
    the_listener_ = listener;
    the_coordinator_ = coordinator;
    the_resource_manager_ = resource_manager;
    
    the_listener_.add_resource(this);
    the_resource_manager_.add_component_server(this);
  }


  protected void
  unregister( )
  {
    try {
      the_coordinator_ = null;
      the_listener_.remove_resource(this);
      the_resource_manager_.remove_component_server(this);
    } catch ( org.omg.Components.RemoveFailure ex ) { }
  }
  
  
  /**
   * Add the list of current Container to the componentserver
   *  
   */ 
  protected void
  commit_add ()
  {
    for (int i=0 ; i<containers_.size() ; i++) {
      Container cont_ref = (Container) containers_.elementAt(i);
      the_component_server_.add_container (cont_ref);
    }
  }
  
  /**
   * Desactivate the current resource
   *
   */ 
  protected void
  removeItself()
  {
    unregister();
  } 
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ComponentServerEvents interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_create:1.0
  //
  /**
   * Event sent after the create function is called on ComponentServer object
   *
   * @param ref the object created
   */
  public void
  on_create (Container cont_ref)
  {
    containers_.addElement (cont_ref); 
  }
  
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_add:1.0
  //
  /**
   * Event sent after the add function is called on ComponentServer object
   *
   * @param ref the object added
   */
  public void
  on_add (Container cont_ref) {}
  
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_remove:1.0
  //
  /**
   * Event sent before the remove function is called on ComponentServer object
   *
   * @param ref the object removed
   */ 
  public void
  on_remove (Container cont_ref) {}
  
  //
  //  IDL:omg.org/plugins/deployment/ComponentServerEvents/on_destroy:1.0
  //
  /**
   * Event sent before the destruction of the related ComponentServer object 
   *
   */ 
  public void
  on_destroy () {}
  
  // ==================================================================
  //
  // Methods for the CosTransactions::Resource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/CosTransaction/Resource/prepare:1.0
  //
  /**
   * Vote request preparing the commit phase
   *
   * @return the vote for the resource
   */ 
  public org.omg.CosTransactions.Vote
  prepare()
    throws org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    return org.omg.CosTransactions.Vote.VoteCommit;
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/rollback:1.0
  //
  /**
   * Rollback all operations registered by the resource
   * 
   */ 
  public void
  rollback()
    throws org.omg.CosTransactions.HeuristicCommit,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    for (int i=0 ; i < containers_.size() ; i++) {
      Container cont_ref = (Container) containers_.elementAt(i);
      try {
        cont_ref.remove();
      } catch (org.omg.Components.RemoveFailure ex) { }
    }
    removeItself();
  }

  //
  //  IDL:omg.org/CosTransaction/Resource/commit:1.0
  //
  /**
   * Commit all operations registered by the resource
   * 
   */ 
  public void
  commit()
    throws org.omg.CosTransactions.NotPrepared,
    org.omg.CosTransactions.HeuristicRollback,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    commit_add ();
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit_one_phase:1.0
  //
  /**
   * One phase commit of all operations registered by the resource
   *
   */ 
  public void
  commit_one_phase()
    throws org.omg.CosTransactions.HeuristicHazard
  {
    commit_add (); 
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/forget:1.0
  //
  /**
   * Delete the current resource from the resource manager
   *
   */ 
  public void
  forget()
  { 
    removeItself();
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::ComponentServerResource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/transaction/ComponentServerResource/get_component_server:1.0
  //
  /**
   * Provide the reference of the related component server
   *
   * @return reference to the component server
   */ 
  public org.objectweb.ccm.Deployment.ComponentServerLocal
  get_component_server ()
  {
    return the_component_server_;
  }
  
  //     //
  //     //  IDL:omg.org/plugins/deployment/transaction/ComponentServerResource/remove_container_resource:1.0
  //     //
  //     /**
  //      * Unregister a sub resource from the component server resource
  //      *
  //      * @param resource the resource to unregister
  //      */
  //     public void
  //     remove_container_resource(ContainerResource resource)
  //         throws org.omg.Components.RemoveFailure
  //     {
  //         containers_.removeElement (resource); 
  //     }
    
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::CosCoordinator interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/transaction/ComponentServerResource/coordinator:1.0
  //
  /**
   * Provide the reference of the transaction coordinator
   *
   * @return reference on the coordinator
   */ 
  public Coordinator
  coordinator()
  {
    return the_coordinator_;
  }
} 










