/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment ;

import org.objectweb.ccm.Deployment.ContainerLocal ;
import org.omg.Components.CCMHome;
import org.omg.CosTransactions.Coordinator;

/**
 * This class implements the OMG IDL
 * ::plugins::deployment::transaction::ContainerResource interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */ 
public class ContainerResourceImpl
extends org.omg.CORBA.LocalObject
implements ContainerResource 
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * Reference on the related container
   */
  protected ContainerLocal the_container_; 
  
  /**
   * Reference on the coordinator of the current resource
   */
  protected Coordinator the_coordinator_; 
  
  /**
   * Reference to the associated listener
   *
   */
  protected ContainerListenerTransaction the_listener_;
  
  /**
   * Reference to the resource manager
   *
   */
  protected ResourceManagerLocal the_resource_manager_; 
  
  /**
   * List of not committed homes
   */
  protected org.objectweb.ccm.util.Vector homes_;
  
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   *  The constructor.
   *
   ** @param component_server reference to the component server
   ** @param coordinator reference to the current coordinator
   ** @param resource reference to the ResourceManager of the current transaction
   */
  public
  ContainerResourceImpl (ContainerListenerTransaction listener,
                         ContainerLocal container,
                         Coordinator coordinator,
                         ResourceManagerLocal resource_manager)
  {
    the_container_ = container ;
    homes_ = new org.objectweb.ccm.util.Vector();
    register(listener, coordinator, resource_manager);
  }
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  protected void
  register(ContainerListenerTransaction listener,
           Coordinator coordinator,
           ResourceManagerLocal resource_manager)
  {
    the_listener_ = listener;
    the_coordinator_ = coordinator;
    the_resource_manager_ = resource_manager;
    
    the_listener_.add_resource(this);
    the_resource_manager_.add_container(this);				
  }
  
  
  protected void
  unregister( )
  {   
    try {
      the_coordinator_ = null ;
      the_listener_.remove_resource(this);
      the_resource_manager_.remove_container (this); 
    } catch ( org.omg.Components.RemoveFailure ex ) { }
  }
  

  
  /**
   * Add the list of current Container to the componentserver
   *  
   */ 
  protected void
  commit_add ()
  {
    for (int i=0 ; i < homes_.size() ; i++) {
      CCMHome home_ref = (CCMHome) homes_.elementAt(i) ;
      the_container_.add_home (home_ref);
    }
  }
  
  /**
   * Desactivate the current resource
   *
   */ 
  protected void
  removeItself()
  {
    unregister();
  } 
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::ContainerEvents interface.
  //
  // ==================================================================
  
  //
  //  IDL:omg.org/plugins/deployment/ContainerEvents/on_create:1.0
  //
  /**
   * Event sent after the create function is called on Container object
   *
   * @param ref the object created
   */ 
  public void
  on_create (CCMHome ref)
  {
    homes_.addElement (ref);
  }
  
  //
  //  IDL:omg.org/plugins/deployment/ContainerEvents/on_add:1.0
  //
  /**
   * Event sent after the add function is called on Container object
   *
   * @param ref the object added
   */ 
  public void
  on_add (CCMHome ref) { }
  
  //
  //  IDL:omg.org/plugins/deployment/ContainerEvents/on_remove:1.0
  //
  /**
   * Event sent before the remove function is called on Container object
   *
   * @param ref the object removed
   */ 
  public void
  on_remove (CCMHome ref) { }
    
  //
  //  IDL:omg.org/plugins/deployment/ContainerEvents/on_destroy:1.0
  //
  /**
   * Event sent before the destruction of the related Container object 
   *
   */ 
  public void
  on_destroy () { } 

  // ==================================================================
  //
  // Methods for the CosTransactions::Resource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/CosTransaction/Resource/prepare:1.0
  //
  /**
   * Vote request preparing the commit phase
   *
   * @return the vote for the resource
   */ 
  public org.omg.CosTransactions.Vote
  prepare()
    throws org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    return org.omg.CosTransactions.Vote.VoteCommit; 
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/rollback:1.0
  //
  /**
   * Rollback all operations registered by the resource
   * 
   */ 
  public void
  rollback()
    throws org.omg.CosTransactions.HeuristicCommit,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    for (int i=0 ; i<homes_.size() ; i++) {
      CCMHome home_ref = (CCMHome) homes_.elementAt(i) ;
      try	{
        home_ref.remove_home();
      } catch (org.omg.Components.RemoveFailure ex) { }
    }
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit:1.0
  //
  /**
   * Commit all operations registered by the resource
   * 
   */ 
  public void
  commit()
    throws org.omg.CosTransactions.NotPrepared,
    org.omg.CosTransactions.HeuristicRollback,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    commit_add (); 
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit_one_phase:1.0
  //
  /**
   * One phase commit of all operations registered by the resource
   *
   */ 
  public void
  commit_one_phase()
    throws org.omg.CosTransactions.HeuristicHazard
  {
    commit_add (); 
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/forget:1.0
  //
  /**
   * Delete the current resource from the resource manager
   *
   */ 
  public void 
  forget()
  {
    removeItself();
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::ContainerResource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/transaction/ContainerResource/get_container:1.0
  //
  /**
   * Provide the reference of the related container
   *
   * @return reference to the container
   */ 
  public org.objectweb.ccm.Deployment.ContainerLocal
  get_container ()
  {
    return the_container_; 
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::CosCoordinator interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/transaction/ContainerResource/coordinator:1.0
  //
  /**
   * Provide the reference of the transaction coordinator
   *
   * @return reference on the coordinator
   */ 
  public Coordinator
    coordinator()
  {
    return the_coordinator_;
  }
} 







